/* Mapping tables for JIS0208 handling.
   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
   This file is part of the GNU C Library.
   Contributed by Ulrich Drepper <drepper@cygnus.com>, 1997.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdint.h>

#include "jis0208.h"


/* We use one big table for the mapping.  We know that the first byte
   is in range 0x21 to 0xea.  The second byte never is in the ranges
   0x00 to 0x20 and 0x7f to 0xff.  Compacting the table by excluding
   these ranges we get a table with only this gaps:

   at	0x222e	0x2241	0x2250	0x226a	0x2279	0x2321	0x2339	0x235a
   len	12	9	12	8	5	15	8	7

   at	0x237a	0x2473	0x2576	0x2638	0x2658	0x2741	0x2771	0x2840
   len	4	11	8	9	38	16	13	62

   at	0x4f53
   len	43

   It's not worth trying to use these gaps.  The table can be generated
   using

   egrep '^0x' < .../eastasia/jis/jis0208.txt |
   perl tab.pl

   with tab.pl containing:
   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   $n=0;
   while (<>) {
     local($sjis, $jis, $ucs4, %rest) = split;
     local($u)=hex($ucs4);
     local($s)=hex($jis);
     printf ("\n ") if ($n % 4 eq 0);
     ++$n;
     printf (" [0x%04x] = 0x%04x,",
	     int(($s - 0x2121) / 256) * 94 + (($s - 0x2121) & 0xff), $u);
   }
   printf ("\n");
   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
const uint16_t __jis0208_to_ucs[0x1e80] =
{
  [0x0000] = 0x3000, [0x0001] = 0x3001, [0x0002] = 0x3002, [0x0003] = 0xff0c,
  [0x0004] = 0xff0e, [0x0005] = 0x30fb, [0x0006] = 0xff1a, [0x0007] = 0xff1b,
  [0x0008] = 0xff1f, [0x0009] = 0xff01, [0x000a] = 0x309b, [0x000b] = 0x309c,
  [0x000c] = 0x00b4, [0x000d] = 0xff40, [0x000e] = 0x00a8, [0x000f] = 0xff3e,
  [0x0010] = 0xffe3, [0x0011] = 0xff3f, [0x0012] = 0x30fd, [0x0013] = 0x30fe,
  [0x0014] = 0x309d, [0x0015] = 0x309e, [0x0016] = 0x3003, [0x0017] = 0x4edd,
  [0x0018] = 0x3005, [0x0019] = 0x3006, [0x001a] = 0x3007, [0x001b] = 0x30fc,
  [0x001c] = 0x2015, [0x001d] = 0x2010, [0x001e] = 0xff0f, [0x001f] = 0x005c,
  [0x0020] = 0x301c, [0x0021] = 0x2016, [0x0022] = 0xff5c, [0x0023] = 0x2026,
  [0x0024] = 0x2025, [0x0025] = 0x2018, [0x0026] = 0x2019, [0x0027] = 0x201c,
  [0x0028] = 0x201d, [0x0029] = 0xff08, [0x002a] = 0xff09, [0x002b] = 0x3014,
  [0x002c] = 0x3015, [0x002d] = 0xff3b, [0x002e] = 0xff3d, [0x002f] = 0xff5b,
  [0x0030] = 0xff5d, [0x0031] = 0x3008, [0x0032] = 0x3009, [0x0033] = 0x300a,
  [0x0034] = 0x300b, [0x0035] = 0x300c, [0x0036] = 0x300d, [0x0037] = 0x300e,
  [0x0038] = 0x300f, [0x0039] = 0x3010, [0x003a] = 0x3011, [0x003b] = 0xff0b,
  [0x003c] = 0x2212, [0x003d] = 0x00b1, [0x003e] = 0x00d7, [0x003f] = 0x00f7,
  [0x0040] = 0xff1d, [0x0041] = 0x2260, [0x0042] = 0xff1c, [0x0043] = 0xff1e,
  [0x0044] = 0x2266, [0x0045] = 0x2267, [0x0046] = 0x221e, [0x0047] = 0x2234,
  [0x0048] = 0x2642, [0x0049] = 0x2640, [0x004a] = 0x00b0, [0x004b] = 0x2032,
  [0x004c] = 0x2033, [0x004d] = 0x2103, [0x004e] = 0xffe5, [0x004f] = 0xff04,
  [0x0050] = 0x00a2, [0x0051] = 0x00a3, [0x0052] = 0xff05, [0x0053] = 0xff03,
  [0x0054] = 0xff06, [0x0055] = 0xff0a, [0x0056] = 0xff20, [0x0057] = 0x00a7,
  [0x0058] = 0x2606, [0x0059] = 0x2605, [0x005a] = 0x25cb, [0x005b] = 0x25cf,
  [0x005c] = 0x25ce, [0x005d] = 0x25c7, [0x005e] = 0x25c6, [0x005f] = 0x25a1,
  [0x0060] = 0x25a0, [0x0061] = 0x25b3, [0x0062] = 0x25b2, [0x0063] = 0x25bd,
  [0x0064] = 0x25bc, [0x0065] = 0x203b, [0x0066] = 0x3012, [0x0067] = 0x2192,
  [0x0068] = 0x2190, [0x0069] = 0x2191, [0x006a] = 0x2193, [0x006b] = 0x3013,
  [0x0077] = 0x2208, [0x0078] = 0x220b, [0x0079] = 0x2286, [0x007a] = 0x2287,
  [0x007b] = 0x2282, [0x007c] = 0x2283, [0x007d] = 0x222a, [0x007e] = 0x2229,
  [0x0087] = 0x2227, [0x0088] = 0x2228, [0x0089] = 0x00ac, [0x008a] = 0x21d2,
  [0x008b] = 0x21d4, [0x008c] = 0x2200, [0x008d] = 0x2203, [0x0099] = 0x2220,
  [0x009a] = 0x22a5, [0x009b] = 0x2312, [0x009c] = 0x2202, [0x009d] = 0x2207,
  [0x009e] = 0x2261, [0x009f] = 0x2252, [0x00a0] = 0x226a, [0x00a1] = 0x226b,
  [0x00a2] = 0x221a, [0x00a3] = 0x223d, [0x00a4] = 0x221d, [0x00a5] = 0x2235,
  [0x00a6] = 0x222b, [0x00a7] = 0x222c, [0x00af] = 0x212b, [0x00b0] = 0x2030,
  [0x00b1] = 0x266f, [0x00b2] = 0x266d, [0x00b3] = 0x266a, [0x00b4] = 0x2020,
  [0x00b5] = 0x2021, [0x00b6] = 0x00b6, [0x00bb] = 0x25ef, [0x00cb] = 0xff10,
  [0x00cc] = 0xff11, [0x00cd] = 0xff12, [0x00ce] = 0xff13, [0x00cf] = 0xff14,
  [0x00d0] = 0xff15, [0x00d1] = 0xff16, [0x00d2] = 0xff17, [0x00d3] = 0xff18,
  [0x00d4] = 0xff19, [0x00dc] = 0xff21, [0x00dd] = 0xff22, [0x00de] = 0xff23,
  [0x00df] = 0xff24, [0x00e0] = 0xff25, [0x00e1] = 0xff26, [0x00e2] = 0xff27,
  [0x00e3] = 0xff28, [0x00e4] = 0xff29, [0x00e5] = 0xff2a, [0x00e6] = 0xff2b,
  [0x00e7] = 0xff2c, [0x00e8] = 0xff2d, [0x00e9] = 0xff2e, [0x00ea] = 0xff2f,
  [0x00eb] = 0xff30, [0x00ec] = 0xff31, [0x00ed] = 0xff32, [0x00ee] = 0xff33,
  [0x00ef] = 0xff34, [0x00f0] = 0xff35, [0x00f1] = 0xff36, [0x00f2] = 0xff37,
  [0x00f3] = 0xff38, [0x00f4] = 0xff39, [0x00f5] = 0xff3a, [0x00fc] = 0xff41,
  [0x00fd] = 0xff42, [0x00fe] = 0xff43, [0x00ff] = 0xff44, [0x0100] = 0xff45,
  [0x0101] = 0xff46, [0x0102] = 0xff47, [0x0103] = 0xff48, [0x0104] = 0xff49,
  [0x0105] = 0xff4a, [0x0106] = 0xff4b, [0x0107] = 0xff4c, [0x0108] = 0xff4d,
  [0x0109] = 0xff4e, [0x010a] = 0xff4f, [0x010b] = 0xff50, [0x010c] = 0xff51,
  [0x010d] = 0xff52, [0x010e] = 0xff53, [0x010f] = 0xff54, [0x0110] = 0xff55,
  [0x0111] = 0xff56, [0x0112] = 0xff57, [0x0113] = 0xff58, [0x0114] = 0xff59,
  [0x0115] = 0xff5a, [0x011a] = 0x3041, [0x011b] = 0x3042, [0x011c] = 0x3043,
  [0x011d] = 0x3044, [0x011e] = 0x3045, [0x011f] = 0x3046, [0x0120] = 0x3047,
  [0x0121] = 0x3048, [0x0122] = 0x3049, [0x0123] = 0x304a, [0x0124] = 0x304b,
  [0x0125] = 0x304c, [0x0126] = 0x304d, [0x0127] = 0x304e, [0x0128] = 0x304f,
  [0x0129] = 0x3050, [0x012a] = 0x3051, [0x012b] = 0x3052, [0x012c] = 0x3053,
  [0x012d] = 0x3054, [0x012e] = 0x3055, [0x012f] = 0x3056, [0x0130] = 0x3057,
  [0x0131] = 0x3058, [0x0132] = 0x3059, [0x0133] = 0x305a, [0x0134] = 0x305b,
  [0x0135] = 0x305c, [0x0136] = 0x305d, [0x0137] = 0x305e, [0x0138] = 0x305f,
  [0x0139] = 0x3060, [0x013a] = 0x3061, [0x013b] = 0x3062, [0x013c] = 0x3063,
  [0x013d] = 0x3064, [0x013e] = 0x3065, [0x013f] = 0x3066, [0x0140] = 0x3067,
  [0x0141] = 0x3068, [0x0142] = 0x3069, [0x0143] = 0x306a, [0x0144] = 0x306b,
  [0x0145] = 0x306c, [0x0146] = 0x306d, [0x0147] = 0x306e, [0x0148] = 0x306f,
  [0x0149] = 0x3070, [0x014a] = 0x3071, [0x014b] = 0x3072, [0x014c] = 0x3073,
  [0x014d] = 0x3074, [0x014e] = 0x3075, [0x014f] = 0x3076, [0x0150] = 0x3077,
  [0x0151] = 0x3078, [0x0152] = 0x3079, [0x0153] = 0x307a, [0x0154] = 0x307b,
  [0x0155] = 0x307c, [0x0156] = 0x307d, [0x0157] = 0x307e, [0x0158] = 0x307f,
  [0x0159] = 0x3080, [0x015a] = 0x3081, [0x015b] = 0x3082, [0x015c] = 0x3083,
  [0x015d] = 0x3084, [0x015e] = 0x3085, [0x015f] = 0x3086, [0x0160] = 0x3087,
  [0x0161] = 0x3088, [0x0162] = 0x3089, [0x0163] = 0x308a, [0x0164] = 0x308b,
  [0x0165] = 0x308c, [0x0166] = 0x308d, [0x0167] = 0x308e, [0x0168] = 0x308f,
  [0x0169] = 0x3090, [0x016a] = 0x3091, [0x016b] = 0x3092, [0x016c] = 0x3093,
  [0x0178] = 0x30a1, [0x0179] = 0x30a2, [0x017a] = 0x30a3, [0x017b] = 0x30a4,
  [0x017c] = 0x30a5, [0x017d] = 0x30a6, [0x017e] = 0x30a7, [0x017f] = 0x30a8,
  [0x0180] = 0x30a9, [0x0181] = 0x30aa, [0x0182] = 0x30ab, [0x0183] = 0x30ac,
  [0x0184] = 0x30ad, [0x0185] = 0x30ae, [0x0186] = 0x30af, [0x0187] = 0x30b0,
  [0x0188] = 0x30b1, [0x0189] = 0x30b2, [0x018a] = 0x30b3, [0x018b] = 0x30b4,
  [0x018c] = 0x30b5, [0x018d] = 0x30b6, [0x018e] = 0x30b7, [0x018f] = 0x30b8,
  [0x0190] = 0x30b9, [0x0191] = 0x30ba, [0x0192] = 0x30bb, [0x0193] = 0x30bc,
  [0x0194] = 0x30bd, [0x0195] = 0x30be, [0x0196] = 0x30bf, [0x0197] = 0x30c0,
  [0x0198] = 0x30c1, [0x0199] = 0x30c2, [0x019a] = 0x30c3, [0x019b] = 0x30c4,
  [0x019c] = 0x30c5, [0x019d] = 0x30c6, [0x019e] = 0x30c7, [0x019f] = 0x30c8,
  [0x01a0] = 0x30c9, [0x01a1] = 0x30ca, [0x01a2] = 0x30cb, [0x01a3] = 0x30cc,
  [0x01a4] = 0x30cd, [0x01a5] = 0x30ce, [0x01a6] = 0x30cf, [0x01a7] = 0x30d0,
  [0x01a8] = 0x30d1, [0x01a9] = 0x30d2, [0x01aa] = 0x30d3, [0x01ab] = 0x30d4,
  [0x01ac] = 0x30d5, [0x01ad] = 0x30d6, [0x01ae] = 0x30d7, [0x01af] = 0x30d8,
  [0x01b0] = 0x30d9, [0x01b1] = 0x30da, [0x01b2] = 0x30db, [0x01b3] = 0x30dc,
  [0x01b4] = 0x30dd, [0x01b5] = 0x30de, [0x01b6] = 0x30df, [0x01b7] = 0x30e0,
  [0x01b8] = 0x30e1, [0x01b9] = 0x30e2, [0x01ba] = 0x30e3, [0x01bb] = 0x30e4,
  [0x01bc] = 0x30e5, [0x01bd] = 0x30e6, [0x01be] = 0x30e7, [0x01bf] = 0x30e8,
  [0x01c0] = 0x30e9, [0x01c1] = 0x30ea, [0x01c2] = 0x30eb, [0x01c3] = 0x30ec,
  [0x01c4] = 0x30ed, [0x01c5] = 0x30ee, [0x01c6] = 0x30ef, [0x01c7] = 0x30f0,
  [0x01c8] = 0x30f1, [0x01c9] = 0x30f2, [0x01ca] = 0x30f3, [0x01cb] = 0x30f4,
  [0x01cc] = 0x30f5, [0x01cd] = 0x30f6, [0x01d6] = 0x0391, [0x01d7] = 0x0392,
  [0x01d8] = 0x0393, [0x01d9] = 0x0394, [0x01da] = 0x0395, [0x01db] = 0x0396,
  [0x01dc] = 0x0397, [0x01dd] = 0x0398, [0x01de] = 0x0399, [0x01df] = 0x039a,
  [0x01e0] = 0x039b, [0x01e1] = 0x039c, [0x01e2] = 0x039d, [0x01e3] = 0x039e,
  [0x01e4] = 0x039f, [0x01e5] = 0x03a0, [0x01e6] = 0x03a1, [0x01e7] = 0x03a3,
  [0x01e8] = 0x03a4, [0x01e9] = 0x03a5, [0x01ea] = 0x03a6, [0x01eb] = 0x03a7,
  [0x01ec] = 0x03a8, [0x01ed] = 0x03a9, [0x01f6] = 0x03b1, [0x01f7] = 0x03b2,
  [0x01f8] = 0x03b3, [0x01f9] = 0x03b4, [0x01fa] = 0x03b5, [0x01fb] = 0x03b6,
  [0x01fc] = 0x03b7, [0x01fd] = 0x03b8, [0x01fe] = 0x03b9, [0x01ff] = 0x03ba,
  [0x0200] = 0x03bb, [0x0201] = 0x03bc, [0x0202] = 0x03bd, [0x0203] = 0x03be,
  [0x0204] = 0x03bf, [0x0205] = 0x03c0, [0x0206] = 0x03c1, [0x0207] = 0x03c3,
  [0x0208] = 0x03c4, [0x0209] = 0x03c5, [0x020a] = 0x03c6, [0x020b] = 0x03c7,
  [0x020c] = 0x03c8, [0x020d] = 0x03c9, [0x0234] = 0x0410, [0x0235] = 0x0411,
  [0x0236] = 0x0412, [0x0237] = 0x0413, [0x0238] = 0x0414, [0x0239] = 0x0415,
  [0x023a] = 0x0401, [0x023b] = 0x0416, [0x023c] = 0x0417, [0x023d] = 0x0418,
  [0x023e] = 0x0419, [0x023f] = 0x041a, [0x0240] = 0x041b, [0x0241] = 0x041c,
  [0x0242] = 0x041d, [0x0243] = 0x041e, [0x0244] = 0x041f, [0x0245] = 0x0420,
  [0x0246] = 0x0421, [0x0247] = 0x0422, [0x0248] = 0x0423, [0x0249] = 0x0424,
  [0x024a] = 0x0425, [0x024b] = 0x0426, [0x024c] = 0x0427, [0x024d] = 0x0428,
  [0x024e] = 0x0429, [0x024f] = 0x042a, [0x0250] = 0x042b, [0x0251] = 0x042c,
  [0x0252] = 0x042d, [0x0253] = 0x042e, [0x0254] = 0x042f, [0x0264] = 0x0430,
  [0x0265] = 0x0431, [0x0266] = 0x0432, [0x0267] = 0x0433, [0x0268] = 0x0434,
  [0x0269] = 0x0435, [0x026a] = 0x0451, [0x026b] = 0x0436, [0x026c] = 0x0437,
  [0x026d] = 0x0438, [0x026e] = 0x0439, [0x026f] = 0x043a, [0x0270] = 0x043b,
  [0x0271] = 0x043c, [0x0272] = 0x043d, [0x0273] = 0x043e, [0x0274] = 0x043f,
  [0x0275] = 0x0440, [0x0276] = 0x0441, [0x0277] = 0x0442, [0x0278] = 0x0443,
  [0x0279] = 0x0444, [0x027a] = 0x0445, [0x027b] = 0x0446, [0x027c] = 0x0447,
  [0x027d] = 0x0448, [0x027e] = 0x0449, [0x027f] = 0x044a, [0x0280] = 0x044b,
  [0x0281] = 0x044c, [0x0282] = 0x044d, [0x0283] = 0x044e, [0x0284] = 0x044f,
  [0x0292] = 0x2500, [0x0293] = 0x2502, [0x0294] = 0x250c, [0x0295] = 0x2510,
  [0x0296] = 0x2518, [0x0297] = 0x2514, [0x0298] = 0x251c, [0x0299] = 0x252c,
  [0x029a] = 0x2524, [0x029b] = 0x2534, [0x029c] = 0x253c, [0x029d] = 0x2501,
  [0x029e] = 0x2503, [0x029f] = 0x250f, [0x02a0] = 0x2513, [0x02a1] = 0x251b,
  [0x02a2] = 0x2517, [0x02a3] = 0x2523, [0x02a4] = 0x2533, [0x02a5] = 0x252b,
  [0x02a6] = 0x253b, [0x02a7] = 0x254b, [0x02a8] = 0x2520, [0x02a9] = 0x252f,
  [0x02aa] = 0x2528, [0x02ab] = 0x2537, [0x02ac] = 0x253f, [0x02ad] = 0x251d,
  [0x02ae] = 0x2530, [0x02af] = 0x2525, [0x02b0] = 0x2538, [0x02b1] = 0x2542,
  [0x0582] = 0x4e9c, [0x0583] = 0x5516, [0x0584] = 0x5a03, [0x0585] = 0x963f,
  [0x0586] = 0x54c0, [0x0587] = 0x611b, [0x0588] = 0x6328, [0x0589] = 0x59f6,
  [0x058a] = 0x9022, [0x058b] = 0x8475, [0x058c] = 0x831c, [0x058d] = 0x7a50,
  [0x058e] = 0x60aa, [0x058f] = 0x63e1, [0x0590] = 0x6e25, [0x0591] = 0x65ed,
  [0x0592] = 0x8466, [0x0593] = 0x82a6, [0x0594] = 0x9bf5, [0x0595] = 0x6893,
  [0x0596] = 0x5727, [0x0597] = 0x65a1, [0x0598] = 0x6271, [0x0599] = 0x5b9b,
  [0x059a] = 0x59d0, [0x059b] = 0x867b, [0x059c] = 0x98f4, [0x059d] = 0x7d62,
  [0x059e] = 0x7dbe, [0x059f] = 0x9b8e, [0x05a0] = 0x6216, [0x05a1] = 0x7c9f,
  [0x05a2] = 0x88b7, [0x05a3] = 0x5b89, [0x05a4] = 0x5eb5, [0x05a5] = 0x6309,
  [0x05a6] = 0x6697, [0x05a7] = 0x6848, [0x05a8] = 0x95c7, [0x05a9] = 0x978d,
  [0x05aa] = 0x674f, [0x05ab] = 0x4ee5, [0x05ac] = 0x4f0a, [0x05ad] = 0x4f4d,
  [0x05ae] = 0x4f9d, [0x05af] = 0x5049, [0x05b0] = 0x56f2, [0x05b1] = 0x5937,
  [0x05b2] = 0x59d4, [0x05b3] = 0x5a01, [0x05b4] = 0x5c09, [0x05b5] = 0x60df,
  [0x05b6] = 0x610f, [0x05b7] = 0x6170, [0x05b8] = 0x6613, [0x05b9] = 0x6905,
  [0x05ba] = 0x70ba, [0x05bb] = 0x754f, [0x05bc] = 0x7570, [0x05bd] = 0x79fb,
  [0x05be] = 0x7dad, [0x05bf] = 0x7def, [0x05c0] = 0x80c3, [0x05c1] = 0x840e,
  [0x05c2] = 0x8863, [0x05c3] = 0x8b02, [0x05c4] = 0x9055, [0x05c5] = 0x907a,
  [0x05c6] = 0x533b, [0x05c7] = 0x4e95, [0x05c8] = 0x4ea5, [0x05c9] = 0x57df,
  [0x05ca] = 0x80b2, [0x05cb] = 0x90c1, [0x05cc] = 0x78ef, [0x05cd] = 0x4e00,
  [0x05ce] = 0x58f1, [0x05cf] = 0x6ea2, [0x05d0] = 0x9038, [0x05d1] = 0x7a32,
  [0x05d2] = 0x8328, [0x05d3] = 0x828b, [0x05d4] = 0x9c2f, [0x05d5] = 0x5141,
  [0x05d6] = 0x5370, [0x05d7] = 0x54bd, [0x05d8] = 0x54e1, [0x05d9] = 0x56e0,
  [0x05da] = 0x59fb, [0x05db] = 0x5f15, [0x05dc] = 0x98f2, [0x05dd] = 0x6deb,
  [0x05de] = 0x80e4, [0x05df] = 0x852d, [0x05e0] = 0x9662, [0x05e1] = 0x9670,
  [0x05e2] = 0x96a0, [0x05e3] = 0x97fb, [0x05e4] = 0x540b, [0x05e5] = 0x53f3,
  [0x05e6] = 0x5b87, [0x05e7] = 0x70cf, [0x05e8] = 0x7fbd, [0x05e9] = 0x8fc2,
  [0x05ea] = 0x96e8, [0x05eb] = 0x536f, [0x05ec] = 0x9d5c, [0x05ed] = 0x7aba,
  [0x05ee] = 0x4e11, [0x05ef] = 0x7893, [0x05f0] = 0x81fc, [0x05f1] = 0x6e26,
  [0x05f2] = 0x5618, [0x05f3] = 0x5504, [0x05f4] = 0x6b1d, [0x05f5] = 0x851a,
  [0x05f6] = 0x9c3b, [0x05f7] = 0x59e5, [0x05f8] = 0x53a9, [0x05f9] = 0x6d66,
  [0x05fa] = 0x74dc, [0x05fb] = 0x958f, [0x05fc] = 0x5642, [0x05fd] = 0x4e91,
  [0x05fe] = 0x904b, [0x05ff] = 0x96f2, [0x0600] = 0x834f, [0x0601] = 0x990c,
  [0x0602] = 0x53e1, [0x0603] = 0x55b6, [0x0604] = 0x5b30, [0x0605] = 0x5f71,
  [0x0606] = 0x6620, [0x0607] = 0x66f3, [0x0608] = 0x6804, [0x0609] = 0x6c38,
  [0x060a] = 0x6cf3, [0x060b] = 0x6d29, [0x060c] = 0x745b, [0x060d] = 0x76c8,
  [0x060e] = 0x7a4e, [0x060f] = 0x9834, [0x0610] = 0x82f1, [0x0611] = 0x885b,
  [0x0612] = 0x8a60, [0x0613] = 0x92ed, [0x0614] = 0x6db2, [0x0615] = 0x75ab,
  [0x0616] = 0x76ca, [0x0617] = 0x99c5, [0x0618] = 0x60a6, [0x0619] = 0x8b01,
  [0x061a] = 0x8d8a, [0x061b] = 0x95b2, [0x061c] = 0x698e, [0x061d] = 0x53ad,
  [0x061e] = 0x5186, [0x061f] = 0x5712, [0x0620] = 0x5830, [0x0621] = 0x5944,
  [0x0622] = 0x5bb4, [0x0623] = 0x5ef6, [0x0624] = 0x6028, [0x0625] = 0x63a9,
  [0x0626] = 0x63f4, [0x0627] = 0x6cbf, [0x0628] = 0x6f14, [0x0629] = 0x708e,
  [0x062a] = 0x7114, [0x062b] = 0x7159, [0x062c] = 0x71d5, [0x062d] = 0x733f,
  [0x062e] = 0x7e01, [0x062f] = 0x8276, [0x0630] = 0x82d1, [0x0631] = 0x8597,
  [0x0632] = 0x9060, [0x0633] = 0x925b, [0x0634] = 0x9d1b, [0x0635] = 0x5869,
  [0x0636] = 0x65bc, [0x0637] = 0x6c5a, [0x0638] = 0x7525, [0x0639] = 0x51f9,
  [0x063a] = 0x592e, [0x063b] = 0x5965, [0x063c] = 0x5f80, [0x063d] = 0x5fdc,
  [0x063e] = 0x62bc, [0x063f] = 0x65fa, [0x0640] = 0x6a2a, [0x0641] = 0x6b27,
  [0x0642] = 0x6bb4, [0x0643] = 0x738b, [0x0644] = 0x7fc1, [0x0645] = 0x8956,
  [0x0646] = 0x9d2c, [0x0647] = 0x9d0e, [0x0648] = 0x9ec4, [0x0649] = 0x5ca1,
  [0x064a] = 0x6c96, [0x064b] = 0x837b, [0x064c] = 0x5104, [0x064d] = 0x5c4b,
  [0x064e] = 0x61b6, [0x064f] = 0x81c6, [0x0650] = 0x6876, [0x0651] = 0x7261,
  [0x0652] = 0x4e59, [0x0653] = 0x4ffa, [0x0654] = 0x5378, [0x0655] = 0x6069,
  [0x0656] = 0x6e29, [0x0657] = 0x7a4f, [0x0658] = 0x97f3, [0x0659] = 0x4e0b,
  [0x065a] = 0x5316, [0x065b] = 0x4eee, [0x065c] = 0x4f55, [0x065d] = 0x4f3d,
  [0x065e] = 0x4fa1, [0x065f] = 0x4f73, [0x0660] = 0x52a0, [0x0661] = 0x53ef,
  [0x0662] = 0x5609, [0x0663] = 0x590f, [0x0664] = 0x5ac1, [0x0665] = 0x5bb6,
  [0x0666] = 0x5be1, [0x0667] = 0x79d1, [0x0668] = 0x6687, [0x0669] = 0x679c,
  [0x066a] = 0x67b6, [0x066b] = 0x6b4c, [0x066c] = 0x6cb3, [0x066d] = 0x706b,
  [0x066e] = 0x73c2, [0x066f] = 0x798d, [0x0670] = 0x79be, [0x0671] = 0x7a3c,
  [0x0672] = 0x7b87, [0x0673] = 0x82b1, [0x0674] = 0x82db, [0x0675] = 0x8304,
  [0x0676] = 0x8377, [0x0677] = 0x83ef, [0x0678] = 0x83d3, [0x0679] = 0x8766,
  [0x067a] = 0x8ab2, [0x067b] = 0x5629, [0x067c] = 0x8ca8, [0x067d] = 0x8fe6,
  [0x067e] = 0x904e, [0x067f] = 0x971e, [0x0680] = 0x868a, [0x0681] = 0x4fc4,
  [0x0682] = 0x5ce8, [0x0683] = 0x6211, [0x0684] = 0x7259, [0x0685] = 0x753b,
  [0x0686] = 0x81e5, [0x0687] = 0x82bd, [0x0688] = 0x86fe, [0x0689] = 0x8cc0,
  [0x068a] = 0x96c5, [0x068b] = 0x9913, [0x068c] = 0x99d5, [0x068d] = 0x4ecb,
  [0x068e] = 0x4f1a, [0x068f] = 0x89e3, [0x0690] = 0x56de, [0x0691] = 0x584a,
  [0x0692] = 0x58ca, [0x0693] = 0x5efb, [0x0694] = 0x5feb, [0x0695] = 0x602a,
  [0x0696] = 0x6094, [0x0697] = 0x6062, [0x0698] = 0x61d0, [0x0699] = 0x6212,
  [0x069a] = 0x62d0, [0x069b] = 0x6539, [0x069c] = 0x9b41, [0x069d] = 0x6666,
  [0x069e] = 0x68b0, [0x069f] = 0x6d77, [0x06a0] = 0x7070, [0x06a1] = 0x754c,
  [0x06a2] = 0x7686, [0x06a3] = 0x7d75, [0x06a4] = 0x82a5, [0x06a5] = 0x87f9,
  [0x06a6] = 0x958b, [0x06a7] = 0x968e, [0x06a8] = 0x8c9d, [0x06a9] = 0x51f1,
  [0x06aa] = 0x52be, [0x06ab] = 0x5916, [0x06ac] = 0x54b3, [0x06ad] = 0x5bb3,
  [0x06ae] = 0x5d16, [0x06af] = 0x6168, [0x06b0] = 0x6982, [0x06b1] = 0x6daf,
  [0x06b2] = 0x788d, [0x06b3] = 0x84cb, [0x06b4] = 0x8857, [0x06b5] = 0x8a72,
  [0x06b6] = 0x93a7, [0x06b7] = 0x9ab8, [0x06b8] = 0x6d6c, [0x06b9] = 0x99a8,
  [0x06ba] = 0x86d9, [0x06bb] = 0x57a3, [0x06bc] = 0x67ff, [0x06bd] = 0x86ce,
  [0x06be] = 0x920e, [0x06bf] = 0x5283, [0x06c0] = 0x5687, [0x06c1] = 0x5404,
  [0x06c2] = 0x5ed3, [0x06c3] = 0x62e1, [0x06c4] = 0x64b9, [0x06c5] = 0x683c,
  [0x06c6] = 0x6838, [0x06c7] = 0x6bbb, [0x06c8] = 0x7372, [0x06c9] = 0x78ba,
  [0x06ca] = 0x7a6b, [0x06cb] = 0x899a, [0x06cc] = 0x89d2, [0x06cd] = 0x8d6b,
  [0x06ce] = 0x8f03, [0x06cf] = 0x90ed, [0x06d0] = 0x95a3, [0x06d1] = 0x9694,
  [0x06d2] = 0x9769, [0x06d3] = 0x5b66, [0x06d4] = 0x5cb3, [0x06d5] = 0x697d,
  [0x06d6] = 0x984d, [0x06d7] = 0x984e, [0x06d8] = 0x639b, [0x06d9] = 0x7b20,
  [0x06da] = 0x6a2b, [0x06db] = 0x6a7f, [0x06dc] = 0x68b6, [0x06dd] = 0x9c0d,
  [0x06de] = 0x6f5f, [0x06df] = 0x5272, [0x06e0] = 0x559d, [0x06e1] = 0x6070,
  [0x06e2] = 0x62ec, [0x06e3] = 0x6d3b, [0x06e4] = 0x6e07, [0x06e5] = 0x6ed1,
  [0x06e6] = 0x845b, [0x06e7] = 0x8910, [0x06e8] = 0x8f44, [0x06e9] = 0x4e14,
  [0x06ea] = 0x9c39, [0x06eb] = 0x53f6, [0x06ec] = 0x691b, [0x06ed] = 0x6a3a,
  [0x06ee] = 0x9784, [0x06ef] = 0x682a, [0x06f0] = 0x515c, [0x06f1] = 0x7ac3,
  [0x06f2] = 0x84b2, [0x06f3] = 0x91dc, [0x06f4] = 0x938c, [0x06f5] = 0x565b,
  [0x06f6] = 0x9d28, [0x06f7] = 0x6822, [0x06f8] = 0x8305, [0x06f9] = 0x8431,
  [0x06fa] = 0x7ca5, [0x06fb] = 0x5208, [0x06fc] = 0x82c5, [0x06fd] = 0x74e6,
  [0x06fe] = 0x4e7e, [0x06ff] = 0x4f83, [0x0700] = 0x51a0, [0x0701] = 0x5bd2,
  [0x0702] = 0x520a, [0x0703] = 0x52d8, [0x0704] = 0x52e7, [0x0705] = 0x5dfb,
  [0x0706] = 0x559a, [0x0707] = 0x582a, [0x0708] = 0x59e6, [0x0709] = 0x5b8c,
  [0x070a] = 0x5b98, [0x070b] = 0x5bdb, [0x070c] = 0x5e72, [0x070d] = 0x5e79,
  [0x070e] = 0x60a3, [0x070f] = 0x611f, [0x0710] = 0x6163, [0x0711] = 0x61be,
  [0x0712] = 0x63db, [0x0713] = 0x6562, [0x0714] = 0x67d1, [0x0715] = 0x6853,
  [0x0716] = 0x68fa, [0x0717] = 0x6b3e, [0x0718] = 0x6b53, [0x0719] = 0x6c57,
  [0x071a] = 0x6f22, [0x071b] = 0x6f97, [0x071c] = 0x6f45, [0x071d] = 0x74b0,
  [0x071e] = 0x7518, [0x071f] = 0x76e3, [0x0720] = 0x770b, [0x0721] = 0x7aff,
  [0x0722] = 0x7ba1, [0x0723] = 0x7c21, [0x0724] = 0x7de9, [0x0725] = 0x7f36,
  [0x0726] = 0x7ff0, [0x0727] = 0x809d, [0x0728] = 0x8266, [0x0729] = 0x839e,
  [0x072a] = 0x89b3, [0x072b] = 0x8acc, [0x072c] = 0x8cab, [0x072d] = 0x9084,
  [0x072e] = 0x9451, [0x072f] = 0x9593, [0x0730] = 0x9591, [0x0731] = 0x95a2,
  [0x0732] = 0x9665, [0x0733] = 0x97d3, [0x0734] = 0x9928, [0x0735] = 0x8218,
  [0x0736] = 0x4e38, [0x0737] = 0x542b, [0x0738] = 0x5cb8, [0x0739] = 0x5dcc,
  [0x073a] = 0x73a9, [0x073b] = 0x764c, [0x073c] = 0x773c, [0x073d] = 0x5ca9,
  [0x073e] = 0x7feb, [0x073f] = 0x8d0b, [0x0740] = 0x96c1, [0x0741] = 0x9811,
  [0x0742] = 0x9854, [0x0743] = 0x9858, [0x0744] = 0x4f01, [0x0745] = 0x4f0e,
  [0x0746] = 0x5371, [0x0747] = 0x559c, [0x0748] = 0x5668, [0x0749] = 0x57fa,
  [0x074a] = 0x5947, [0x074b] = 0x5b09, [0x074c] = 0x5bc4, [0x074d] = 0x5c90,
  [0x074e] = 0x5e0c, [0x074f] = 0x5e7e, [0x0750] = 0x5fcc, [0x0751] = 0x63ee,
  [0x0752] = 0x673a, [0x0753] = 0x65d7, [0x0754] = 0x65e2, [0x0755] = 0x671f,
  [0x0756] = 0x68cb, [0x0757] = 0x68c4, [0x0758] = 0x6a5f, [0x0759] = 0x5e30,
  [0x075a] = 0x6bc5, [0x075b] = 0x6c17, [0x075c] = 0x6c7d, [0x075d] = 0x757f,
  [0x075e] = 0x7948, [0x075f] = 0x5b63, [0x0760] = 0x7a00, [0x0761] = 0x7d00,
  [0x0762] = 0x5fbd, [0x0763] = 0x898f, [0x0764] = 0x8a18, [0x0765] = 0x8cb4,
  [0x0766] = 0x8d77, [0x0767] = 0x8ecc, [0x0768] = 0x8f1d, [0x0769] = 0x98e2,
  [0x076a] = 0x9a0e, [0x076b] = 0x9b3c, [0x076c] = 0x4e80, [0x076d] = 0x507d,
  [0x076e] = 0x5100, [0x076f] = 0x5993, [0x0770] = 0x5b9c, [0x0771] = 0x622f,
  [0x0772] = 0x6280, [0x0773] = 0x64ec, [0x0774] = 0x6b3a, [0x0775] = 0x72a0,
  [0x0776] = 0x7591, [0x0777] = 0x7947, [0x0778] = 0x7fa9, [0x0779] = 0x87fb,
  [0x077a] = 0x8abc, [0x077b] = 0x8b70, [0x077c] = 0x63ac, [0x077d] = 0x83ca,
  [0x077e] = 0x97a0, [0x077f] = 0x5409, [0x0780] = 0x5403, [0x0781] = 0x55ab,
  [0x0782] = 0x6854, [0x0783] = 0x6a58, [0x0784] = 0x8a70, [0x0785] = 0x7827,
  [0x0786] = 0x6775, [0x0787] = 0x9ecd, [0x0788] = 0x5374, [0x0789] = 0x5ba2,
  [0x078a] = 0x811a, [0x078b] = 0x8650, [0x078c] = 0x9006, [0x078d] = 0x4e18,
  [0x078e] = 0x4e45, [0x078f] = 0x4ec7, [0x0790] = 0x4f11, [0x0791] = 0x53ca,
  [0x0792] = 0x5438, [0x0793] = 0x5bae, [0x0794] = 0x5f13, [0x0795] = 0x6025,
  [0x0796] = 0x6551, [0x0797] = 0x673d, [0x0798] = 0x6c42, [0x0799] = 0x6c72,
  [0x079a] = 0x6ce3, [0x079b] = 0x7078, [0x079c] = 0x7403, [0x079d] = 0x7a76,
  [0x079e] = 0x7aae, [0x079f] = 0x7b08, [0x07a0] = 0x7d1a, [0x07a1] = 0x7cfe,
  [0x07a2] = 0x7d66, [0x07a3] = 0x65e7, [0x07a4] = 0x725b, [0x07a5] = 0x53bb,
  [0x07a6] = 0x5c45, [0x07a7] = 0x5de8, [0x07a8] = 0x62d2, [0x07a9] = 0x62e0,
  [0x07aa] = 0x6319, [0x07ab] = 0x6e20, [0x07ac] = 0x865a, [0x07ad] = 0x8a31,
  [0x07ae] = 0x8ddd, [0x07af] = 0x92f8, [0x07b0] = 0x6f01, [0x07b1] = 0x79a6,
  [0x07b2] = 0x9b5a, [0x07b3] = 0x4ea8, [0x07b4] = 0x4eab, [0x07b5] = 0x4eac,
  [0x07b6] = 0x4f9b, [0x07b7] = 0x4fa0, [0x07b8] = 0x50d1, [0x07b9] = 0x5147,
  [0x07ba] = 0x7af6, [0x07bb] = 0x5171, [0x07bc] = 0x51f6, [0x07bd] = 0x5354,
  [0x07be] = 0x5321, [0x07bf] = 0x537f, [0x07c0] = 0x53eb, [0x07c1] = 0x55ac,
  [0x07c2] = 0x5883, [0x07c3] = 0x5ce1, [0x07c4] = 0x5f37, [0x07c5] = 0x5f4a,
  [0x07c6] = 0x602f, [0x07c7] = 0x6050, [0x07c8] = 0x606d, [0x07c9] = 0x631f,
  [0x07ca] = 0x6559, [0x07cb] = 0x6a4b, [0x07cc] = 0x6cc1, [0x07cd] = 0x72c2,
  [0x07ce] = 0x72ed, [0x07cf] = 0x77ef, [0x07d0] = 0x80f8, [0x07d1] = 0x8105,
  [0x07d2] = 0x8208, [0x07d3] = 0x854e, [0x07d4] = 0x90f7, [0x07d5] = 0x93e1,
  [0x07d6] = 0x97ff, [0x07d7] = 0x9957, [0x07d8] = 0x9a5a, [0x07d9] = 0x4ef0,
  [0x07da] = 0x51dd, [0x07db] = 0x5c2d, [0x07dc] = 0x6681, [0x07dd] = 0x696d,
  [0x07de] = 0x5c40, [0x07df] = 0x66f2, [0x07e0] = 0x6975, [0x07e1] = 0x7389,
  [0x07e2] = 0x6850, [0x07e3] = 0x7c81, [0x07e4] = 0x50c5, [0x07e5] = 0x52e4,
  [0x07e6] = 0x5747, [0x07e7] = 0x5dfe, [0x07e8] = 0x9326, [0x07e9] = 0x65a4,
  [0x07ea] = 0x6b23, [0x07eb] = 0x6b3d, [0x07ec] = 0x7434, [0x07ed] = 0x7981,
  [0x07ee] = 0x79bd, [0x07ef] = 0x7b4b, [0x07f0] = 0x7dca, [0x07f1] = 0x82b9,
  [0x07f2] = 0x83cc, [0x07f3] = 0x887f, [0x07f4] = 0x895f, [0x07f5] = 0x8b39,
  [0x07f6] = 0x8fd1, [0x07f7] = 0x91d1, [0x07f8] = 0x541f, [0x07f9] = 0x9280,
  [0x07fa] = 0x4e5d, [0x07fb] = 0x5036, [0x07fc] = 0x53e5, [0x07fd] = 0x533a,
  [0x07fe] = 0x72d7, [0x07ff] = 0x7396, [0x0800] = 0x77e9, [0x0801] = 0x82e6,
  [0x0802] = 0x8eaf, [0x0803] = 0x99c6, [0x0804] = 0x99c8, [0x0805] = 0x99d2,
  [0x0806] = 0x5177, [0x0807] = 0x611a, [0x0808] = 0x865e, [0x0809] = 0x55b0,
  [0x080a] = 0x7a7a, [0x080b] = 0x5076, [0x080c] = 0x5bd3, [0x080d] = 0x9047,
  [0x080e] = 0x9685, [0x080f] = 0x4e32, [0x0810] = 0x6adb, [0x0811] = 0x91e7,
  [0x0812] = 0x5c51, [0x0813] = 0x5c48, [0x0814] = 0x6398, [0x0815] = 0x7a9f,
  [0x0816] = 0x6c93, [0x0817] = 0x9774, [0x0818] = 0x8f61, [0x0819] = 0x7aaa,
  [0x081a] = 0x718a, [0x081b] = 0x9688, [0x081c] = 0x7c82, [0x081d] = 0x6817,
  [0x081e] = 0x7e70, [0x081f] = 0x6851, [0x0820] = 0x936c, [0x0821] = 0x52f2,
  [0x0822] = 0x541b, [0x0823] = 0x85ab, [0x0824] = 0x8a13, [0x0825] = 0x7fa4,
  [0x0826] = 0x8ecd, [0x0827] = 0x90e1, [0x0828] = 0x5366, [0x0829] = 0x8888,
  [0x082a] = 0x7941, [0x082b] = 0x4fc2, [0x082c] = 0x50be, [0x082d] = 0x5211,
  [0x082e] = 0x5144, [0x082f] = 0x5553, [0x0830] = 0x572d, [0x0831] = 0x73ea,
  [0x0832] = 0x578b, [0x0833] = 0x5951, [0x0834] = 0x5f62, [0x0835] = 0x5f84,
  [0x0836] = 0x6075, [0x0837] = 0x6176, [0x0838] = 0x6167, [0x0839] = 0x61a9,
  [0x083a] = 0x63b2, [0x083b] = 0x643a, [0x083c] = 0x656c, [0x083d] = 0x666f,
  [0x083e] = 0x6842, [0x083f] = 0x6e13, [0x0840] = 0x7566, [0x0841] = 0x7a3d,
  [0x0842] = 0x7cfb, [0x0843] = 0x7d4c, [0x0844] = 0x7d99, [0x0845] = 0x7e4b,
  [0x0846] = 0x7f6b, [0x0847] = 0x830e, [0x0848] = 0x834a, [0x0849] = 0x86cd,
  [0x084a] = 0x8a08, [0x084b] = 0x8a63, [0x084c] = 0x8b66, [0x084d] = 0x8efd,
  [0x084e] = 0x981a, [0x084f] = 0x9d8f, [0x0850] = 0x82b8, [0x0851] = 0x8fce,
  [0x0852] = 0x9be8, [0x0853] = 0x5287, [0x0854] = 0x621f, [0x0855] = 0x6483,
  [0x0856] = 0x6fc0, [0x0857] = 0x9699, [0x0858] = 0x6841, [0x0859] = 0x5091,
  [0x085a] = 0x6b20, [0x085b] = 0x6c7a, [0x085c] = 0x6f54, [0x085d] = 0x7a74,
  [0x085e] = 0x7d50, [0x085f] = 0x8840, [0x0860] = 0x8a23, [0x0861] = 0x6708,
  [0x0862] = 0x4ef6, [0x0863] = 0x5039, [0x0864] = 0x5026, [0x0865] = 0x5065,
  [0x0866] = 0x517c, [0x0867] = 0x5238, [0x0868] = 0x5263, [0x0869] = 0x55a7,
  [0x086a] = 0x570f, [0x086b] = 0x5805, [0x086c] = 0x5acc, [0x086d] = 0x5efa,
  [0x086e] = 0x61b2, [0x086f] = 0x61f8, [0x0870] = 0x62f3, [0x0871] = 0x6372,
  [0x0872] = 0x691c, [0x0873] = 0x6a29, [0x0874] = 0x727d, [0x0875] = 0x72ac,
  [0x0876] = 0x732e, [0x0877] = 0x7814, [0x0878] = 0x786f, [0x0879] = 0x7d79,
  [0x087a] = 0x770c, [0x087b] = 0x80a9, [0x087c] = 0x898b, [0x087d] = 0x8b19,
  [0x087e] = 0x8ce2, [0x087f] = 0x8ed2, [0x0880] = 0x9063, [0x0881] = 0x9375,
  [0x0882] = 0x967a, [0x0883] = 0x9855, [0x0884] = 0x9a13, [0x0885] = 0x9e78,
  [0x0886] = 0x5143, [0x0887] = 0x539f, [0x0888] = 0x53b3, [0x0889] = 0x5e7b,
  [0x088a] = 0x5f26, [0x088b] = 0x6e1b, [0x088c] = 0x6e90, [0x088d] = 0x7384,
  [0x088e] = 0x73fe, [0x088f] = 0x7d43, [0x0890] = 0x8237, [0x0891] = 0x8a00,
  [0x0892] = 0x8afa, [0x0893] = 0x9650, [0x0894] = 0x4e4e, [0x0895] = 0x500b,
  [0x0896] = 0x53e4, [0x0897] = 0x547c, [0x0898] = 0x56fa, [0x0899] = 0x59d1,
  [0x089a] = 0x5b64, [0x089b] = 0x5df1, [0x089c] = 0x5eab, [0x089d] = 0x5f27,
  [0x089e] = 0x6238, [0x089f] = 0x6545, [0x08a0] = 0x67af, [0x08a1] = 0x6e56,
  [0x08a2] = 0x72d0, [0x08a3] = 0x7cca, [0x08a4] = 0x88b4, [0x08a5] = 0x80a1,
  [0x08a6] = 0x80e1, [0x08a7] = 0x83f0, [0x08a8] = 0x864e, [0x08a9] = 0x8a87,
  [0x08aa] = 0x8de8, [0x08ab] = 0x9237, [0x08ac] = 0x96c7, [0x08ad] = 0x9867,
  [0x08ae] = 0x9f13, [0x08af] = 0x4e94, [0x08b0] = 0x4e92, [0x08b1] = 0x4f0d,
  [0x08b2] = 0x5348, [0x08b3] = 0x5449, [0x08b4] = 0x543e, [0x08b5] = 0x5a2f,
  [0x08b6] = 0x5f8c, [0x08b7] = 0x5fa1, [0x08b8] = 0x609f, [0x08b9] = 0x68a7,
  [0x08ba] = 0x6a8e, [0x08bb] = 0x745a, [0x08bc] = 0x7881, [0x08bd] = 0x8a9e,
  [0x08be] = 0x8aa4, [0x08bf] = 0x8b77, [0x08c0] = 0x9190, [0x08c1] = 0x4e5e,
  [0x08c2] = 0x9bc9, [0x08c3] = 0x4ea4, [0x08c4] = 0x4f7c, [0x08c5] = 0x4faf,
  [0x08c6] = 0x5019, [0x08c7] = 0x5016, [0x08c8] = 0x5149, [0x08c9] = 0x516c,
  [0x08ca] = 0x529f, [0x08cb] = 0x52b9, [0x08cc] = 0x52fe, [0x08cd] = 0x539a,
  [0x08ce] = 0x53e3, [0x08cf] = 0x5411, [0x08d0] = 0x540e, [0x08d1] = 0x5589,
  [0x08d2] = 0x5751, [0x08d3] = 0x57a2, [0x08d4] = 0x597d, [0x08d5] = 0x5b54,
  [0x08d6] = 0x5b5d, [0x08d7] = 0x5b8f, [0x08d8] = 0x5de5, [0x08d9] = 0x5de7,
  [0x08da] = 0x5df7, [0x08db] = 0x5e78, [0x08dc] = 0x5e83, [0x08dd] = 0x5e9a,
  [0x08de] = 0x5eb7, [0x08df] = 0x5f18, [0x08e0] = 0x6052, [0x08e1] = 0x614c,
  [0x08e2] = 0x6297, [0x08e3] = 0x62d8, [0x08e4] = 0x63a7, [0x08e5] = 0x653b,
  [0x08e6] = 0x6602, [0x08e7] = 0x6643, [0x08e8] = 0x66f4, [0x08e9] = 0x676d,
  [0x08ea] = 0x6821, [0x08eb] = 0x6897, [0x08ec] = 0x69cb, [0x08ed] = 0x6c5f,
  [0x08ee] = 0x6d2a, [0x08ef] = 0x6d69, [0x08f0] = 0x6e2f, [0x08f1] = 0x6e9d,
  [0x08f2] = 0x7532, [0x08f3] = 0x7687, [0x08f4] = 0x786c, [0x08f5] = 0x7a3f,
  [0x08f6] = 0x7ce0, [0x08f7] = 0x7d05, [0x08f8] = 0x7d18, [0x08f9] = 0x7d5e,
  [0x08fa] = 0x7db1, [0x08fb] = 0x8015, [0x08fc] = 0x8003, [0x08fd] = 0x80af,
  [0x08fe] = 0x80b1, [0x08ff] = 0x8154, [0x0900] = 0x818f, [0x0901] = 0x822a,
  [0x0902] = 0x8352, [0x0903] = 0x884c, [0x0904] = 0x8861, [0x0905] = 0x8b1b,
  [0x0906] = 0x8ca2, [0x0907] = 0x8cfc, [0x0908] = 0x90ca, [0x0909] = 0x9175,
  [0x090a] = 0x9271, [0x090b] = 0x783f, [0x090c] = 0x92fc, [0x090d] = 0x95a4,
  [0x090e] = 0x964d, [0x090f] = 0x9805, [0x0910] = 0x9999, [0x0911] = 0x9ad8,
  [0x0912] = 0x9d3b, [0x0913] = 0x525b, [0x0914] = 0x52ab, [0x0915] = 0x53f7,
  [0x0916] = 0x5408, [0x0917] = 0x58d5, [0x0918] = 0x62f7, [0x0919] = 0x6fe0,
  [0x091a] = 0x8c6a, [0x091b] = 0x8f5f, [0x091c] = 0x9eb9, [0x091d] = 0x514b,
  [0x091e] = 0x523b, [0x091f] = 0x544a, [0x0920] = 0x56fd, [0x0921] = 0x7a40,
  [0x0922] = 0x9177, [0x0923] = 0x9d60, [0x0924] = 0x9ed2, [0x0925] = 0x7344,
  [0x0926] = 0x6f09, [0x0927] = 0x8170, [0x0928] = 0x7511, [0x0929] = 0x5ffd,
  [0x092a] = 0x60da, [0x092b] = 0x9aa8, [0x092c] = 0x72db, [0x092d] = 0x8fbc,
  [0x092e] = 0x6b64, [0x092f] = 0x9803, [0x0930] = 0x4eca, [0x0931] = 0x56f0,
  [0x0932] = 0x5764, [0x0933] = 0x58be, [0x0934] = 0x5a5a, [0x0935] = 0x6068,
  [0x0936] = 0x61c7, [0x0937] = 0x660f, [0x0938] = 0x6606, [0x0939] = 0x6839,
  [0x093a] = 0x68b1, [0x093b] = 0x6df7, [0x093c] = 0x75d5, [0x093d] = 0x7d3a,
  [0x093e] = 0x826e, [0x093f] = 0x9b42, [0x0940] = 0x4e9b, [0x0941] = 0x4f50,
  [0x0942] = 0x53c9, [0x0943] = 0x5506, [0x0944] = 0x5d6f, [0x0945] = 0x5de6,
  [0x0946] = 0x5dee, [0x0947] = 0x67fb, [0x0948] = 0x6c99, [0x0949] = 0x7473,
  [0x094a] = 0x7802, [0x094b] = 0x8a50, [0x094c] = 0x9396, [0x094d] = 0x88df,
  [0x094e] = 0x5750, [0x094f] = 0x5ea7, [0x0950] = 0x632b, [0x0951] = 0x50b5,
  [0x0952] = 0x50ac, [0x0953] = 0x518d, [0x0954] = 0x6700, [0x0955] = 0x54c9,
  [0x0956] = 0x585e, [0x0957] = 0x59bb, [0x0958] = 0x5bb0, [0x0959] = 0x5f69,
  [0x095a] = 0x624d, [0x095b] = 0x63a1, [0x095c] = 0x683d, [0x095d] = 0x6b73,
  [0x095e] = 0x6e08, [0x095f] = 0x707d, [0x0960] = 0x91c7, [0x0961] = 0x7280,
  [0x0962] = 0x7815, [0x0963] = 0x7826, [0x0964] = 0x796d, [0x0965] = 0x658e,
  [0x0966] = 0x7d30, [0x0967] = 0x83dc, [0x0968] = 0x88c1, [0x0969] = 0x8f09,
  [0x096a] = 0x969b, [0x096b] = 0x5264, [0x096c] = 0x5728, [0x096d] = 0x6750,
  [0x096e] = 0x7f6a, [0x096f] = 0x8ca1, [0x0970] = 0x51b4, [0x0971] = 0x5742,
  [0x0972] = 0x962a, [0x0973] = 0x583a, [0x0974] = 0x698a, [0x0975] = 0x80b4,
  [0x0976] = 0x54b2, [0x0977] = 0x5d0e, [0x0978] = 0x57fc, [0x0979] = 0x7895,
  [0x097a] = 0x9dfa, [0x097b] = 0x4f5c, [0x097c] = 0x524a, [0x097d] = 0x548b,
  [0x097e] = 0x643e, [0x097f] = 0x6628, [0x0980] = 0x6714, [0x0981] = 0x67f5,
  [0x0982] = 0x7a84, [0x0983] = 0x7b56, [0x0984] = 0x7d22, [0x0985] = 0x932f,
  [0x0986] = 0x685c, [0x0987] = 0x9bad, [0x0988] = 0x7b39, [0x0989] = 0x5319,
  [0x098a] = 0x518a, [0x098b] = 0x5237, [0x098c] = 0x5bdf, [0x098d] = 0x62f6,
  [0x098e] = 0x64ae, [0x098f] = 0x64e6, [0x0990] = 0x672d, [0x0991] = 0x6bba,
  [0x0992] = 0x85a9, [0x0993] = 0x96d1, [0x0994] = 0x7690, [0x0995] = 0x9bd6,
  [0x0996] = 0x634c, [0x0997] = 0x9306, [0x0998] = 0x9bab, [0x0999] = 0x76bf,
  [0x099a] = 0x6652, [0x099b] = 0x4e09, [0x099c] = 0x5098, [0x099d] = 0x53c2,
  [0x099e] = 0x5c71, [0x099f] = 0x60e8, [0x09a0] = 0x6492, [0x09a1] = 0x6563,
  [0x09a2] = 0x685f, [0x09a3] = 0x71e6, [0x09a4] = 0x73ca, [0x09a5] = 0x7523,
  [0x09a6] = 0x7b97, [0x09a7] = 0x7e82, [0x09a8] = 0x8695, [0x09a9] = 0x8b83,
  [0x09aa] = 0x8cdb, [0x09ab] = 0x9178, [0x09ac] = 0x9910, [0x09ad] = 0x65ac,
  [0x09ae] = 0x66ab, [0x09af] = 0x6b8b, [0x09b0] = 0x4ed5, [0x09b1] = 0x4ed4,
  [0x09b2] = 0x4f3a, [0x09b3] = 0x4f7f, [0x09b4] = 0x523a, [0x09b5] = 0x53f8,
  [0x09b6] = 0x53f2, [0x09b7] = 0x55e3, [0x09b8] = 0x56db, [0x09b9] = 0x58eb,
  [0x09ba] = 0x59cb, [0x09bb] = 0x59c9, [0x09bc] = 0x59ff, [0x09bd] = 0x5b50,
  [0x09be] = 0x5c4d, [0x09bf] = 0x5e02, [0x09c0] = 0x5e2b, [0x09c1] = 0x5fd7,
  [0x09c2] = 0x601d, [0x09c3] = 0x6307, [0x09c4] = 0x652f, [0x09c5] = 0x5b5c,
  [0x09c6] = 0x65af, [0x09c7] = 0x65bd, [0x09c8] = 0x65e8, [0x09c9] = 0x679d,
  [0x09ca] = 0x6b62, [0x09cb] = 0x6b7b, [0x09cc] = 0x6c0f, [0x09cd] = 0x7345,
  [0x09ce] = 0x7949, [0x09cf] = 0x79c1, [0x09d0] = 0x7cf8, [0x09d1] = 0x7d19,
  [0x09d2] = 0x7d2b, [0x09d3] = 0x80a2, [0x09d4] = 0x8102, [0x09d5] = 0x81f3,
  [0x09d6] = 0x8996, [0x09d7] = 0x8a5e, [0x09d8] = 0x8a69, [0x09d9] = 0x8a66,
  [0x09da] = 0x8a8c, [0x09db] = 0x8aee, [0x09dc] = 0x8cc7, [0x09dd] = 0x8cdc,
  [0x09de] = 0x96cc, [0x09df] = 0x98fc, [0x09e0] = 0x6b6f, [0x09e1] = 0x4e8b,
  [0x09e2] = 0x4f3c, [0x09e3] = 0x4f8d, [0x09e4] = 0x5150, [0x09e5] = 0x5b57,
  [0x09e6] = 0x5bfa, [0x09e7] = 0x6148, [0x09e8] = 0x6301, [0x09e9] = 0x6642,
  [0x09ea] = 0x6b21, [0x09eb] = 0x6ecb, [0x09ec] = 0x6cbb, [0x09ed] = 0x723e,
  [0x09ee] = 0x74bd, [0x09ef] = 0x75d4, [0x09f0] = 0x78c1, [0x09f1] = 0x793a,
  [0x09f2] = 0x800c, [0x09f3] = 0x8033, [0x09f4] = 0x81ea, [0x09f5] = 0x8494,
  [0x09f6] = 0x8f9e, [0x09f7] = 0x6c50, [0x09f8] = 0x9e7f, [0x09f9] = 0x5f0f,
  [0x09fa] = 0x8b58, [0x09fb] = 0x9d2b, [0x09fc] = 0x7afa, [0x09fd] = 0x8ef8,
  [0x09fe] = 0x5b8d, [0x09ff] = 0x96eb, [0x0a00] = 0x4e03, [0x0a01] = 0x53f1,
  [0x0a02] = 0x57f7, [0x0a03] = 0x5931, [0x0a04] = 0x5ac9, [0x0a05] = 0x5ba4,
  [0x0a06] = 0x6089, [0x0a07] = 0x6e7f, [0x0a08] = 0x6f06, [0x0a09] = 0x75be,
  [0x0a0a] = 0x8cea, [0x0a0b] = 0x5b9f, [0x0a0c] = 0x8500, [0x0a0d] = 0x7be0,
  [0x0a0e] = 0x5072, [0x0a0f] = 0x67f4, [0x0a10] = 0x829d, [0x0a11] = 0x5c61,
  [0x0a12] = 0x854a, [0x0a13] = 0x7e1e, [0x0a14] = 0x820e, [0x0a15] = 0x5199,
  [0x0a16] = 0x5c04, [0x0a17] = 0x6368, [0x0a18] = 0x8d66, [0x0a19] = 0x659c,
  [0x0a1a] = 0x716e, [0x0a1b] = 0x793e, [0x0a1c] = 0x7d17, [0x0a1d] = 0x8005,
  [0x0a1e] = 0x8b1d, [0x0a1f] = 0x8eca, [0x0a20] = 0x906e, [0x0a21] = 0x86c7,
  [0x0a22] = 0x90aa, [0x0a23] = 0x501f, [0x0a24] = 0x52fa, [0x0a25] = 0x5c3a,
  [0x0a26] = 0x6753, [0x0a27] = 0x707c, [0x0a28] = 0x7235, [0x0a29] = 0x914c,
  [0x0a2a] = 0x91c8, [0x0a2b] = 0x932b, [0x0a2c] = 0x82e5, [0x0a2d] = 0x5bc2,
  [0x0a2e] = 0x5f31, [0x0a2f] = 0x60f9, [0x0a30] = 0x4e3b, [0x0a31] = 0x53d6,
  [0x0a32] = 0x5b88, [0x0a33] = 0x624b, [0x0a34] = 0x6731, [0x0a35] = 0x6b8a,
  [0x0a36] = 0x72e9, [0x0a37] = 0x73e0, [0x0a38] = 0x7a2e, [0x0a39] = 0x816b,
  [0x0a3a] = 0x8da3, [0x0a3b] = 0x9152, [0x0a3c] = 0x9996, [0x0a3d] = 0x5112,
  [0x0a3e] = 0x53d7, [0x0a3f] = 0x546a, [0x0a40] = 0x5bff, [0x0a41] = 0x6388,
  [0x0a42] = 0x6a39, [0x0a43] = 0x7dac, [0x0a44] = 0x9700, [0x0a45] = 0x56da,
  [0x0a46] = 0x53ce, [0x0a47] = 0x5468, [0x0a48] = 0x5b97, [0x0a49] = 0x5c31,
  [0x0a4a] = 0x5dde, [0x0a4b] = 0x4fee, [0x0a4c] = 0x6101, [0x0a4d] = 0x62fe,
  [0x0a4e] = 0x6d32, [0x0a4f] = 0x79c0, [0x0a50] = 0x79cb, [0x0a51] = 0x7d42,
  [0x0a52] = 0x7e4d, [0x0a53] = 0x7fd2, [0x0a54] = 0x81ed, [0x0a55] = 0x821f,
  [0x0a56] = 0x8490, [0x0a57] = 0x8846, [0x0a58] = 0x8972, [0x0a59] = 0x8b90,
  [0x0a5a] = 0x8e74, [0x0a5b] = 0x8f2f, [0x0a5c] = 0x9031, [0x0a5d] = 0x914b,
  [0x0a5e] = 0x916c, [0x0a5f] = 0x96c6, [0x0a60] = 0x919c, [0x0a61] = 0x4ec0,
  [0x0a62] = 0x4f4f, [0x0a63] = 0x5145, [0x0a64] = 0x5341, [0x0a65] = 0x5f93,
  [0x0a66] = 0x620e, [0x0a67] = 0x67d4, [0x0a68] = 0x6c41, [0x0a69] = 0x6e0b,
  [0x0a6a] = 0x7363, [0x0a6b] = 0x7e26, [0x0a6c] = 0x91cd, [0x0a6d] = 0x9283,
  [0x0a6e] = 0x53d4, [0x0a6f] = 0x5919, [0x0a70] = 0x5bbf, [0x0a71] = 0x6dd1,
  [0x0a72] = 0x795d, [0x0a73] = 0x7e2e, [0x0a74] = 0x7c9b, [0x0a75] = 0x587e,
  [0x0a76] = 0x719f, [0x0a77] = 0x51fa, [0x0a78] = 0x8853, [0x0a79] = 0x8ff0,
  [0x0a7a] = 0x4fca, [0x0a7b] = 0x5cfb, [0x0a7c] = 0x6625, [0x0a7d] = 0x77ac,
  [0x0a7e] = 0x7ae3, [0x0a7f] = 0x821c, [0x0a80] = 0x99ff, [0x0a81] = 0x51c6,
  [0x0a82] = 0x5faa, [0x0a83] = 0x65ec, [0x0a84] = 0x696f, [0x0a85] = 0x6b89,
  [0x0a86] = 0x6df3, [0x0a87] = 0x6e96, [0x0a88] = 0x6f64, [0x0a89] = 0x76fe,
  [0x0a8a] = 0x7d14, [0x0a8b] = 0x5de1, [0x0a8c] = 0x9075, [0x0a8d] = 0x9187,
  [0x0a8e] = 0x9806, [0x0a8f] = 0x51e6, [0x0a90] = 0x521d, [0x0a91] = 0x6240,
  [0x0a92] = 0x6691, [0x0a93] = 0x66d9, [0x0a94] = 0x6e1a, [0x0a95] = 0x5eb6,
  [0x0a96] = 0x7dd2, [0x0a97] = 0x7f72, [0x0a98] = 0x66f8, [0x0a99] = 0x85af,
  [0x0a9a] = 0x85f7, [0x0a9b] = 0x8af8, [0x0a9c] = 0x52a9, [0x0a9d] = 0x53d9,
  [0x0a9e] = 0x5973, [0x0a9f] = 0x5e8f, [0x0aa0] = 0x5f90, [0x0aa1] = 0x6055,
  [0x0aa2] = 0x92e4, [0x0aa3] = 0x9664, [0x0aa4] = 0x50b7, [0x0aa5] = 0x511f,
  [0x0aa6] = 0x52dd, [0x0aa7] = 0x5320, [0x0aa8] = 0x5347, [0x0aa9] = 0x53ec,
  [0x0aaa] = 0x54e8, [0x0aab] = 0x5546, [0x0aac] = 0x5531, [0x0aad] = 0x5617,
  [0x0aae] = 0x5968, [0x0aaf] = 0x59be, [0x0ab0] = 0x5a3c, [0x0ab1] = 0x5bb5,
  [0x0ab2] = 0x5c06, [0x0ab3] = 0x5c0f, [0x0ab4] = 0x5c11, [0x0ab5] = 0x5c1a,
  [0x0ab6] = 0x5e84, [0x0ab7] = 0x5e8a, [0x0ab8] = 0x5ee0, [0x0ab9] = 0x5f70,
  [0x0aba] = 0x627f, [0x0abb] = 0x6284, [0x0abc] = 0x62db, [0x0abd] = 0x638c,
  [0x0abe] = 0x6377, [0x0abf] = 0x6607, [0x0ac0] = 0x660c, [0x0ac1] = 0x662d,
  [0x0ac2] = 0x6676, [0x0ac3] = 0x677e, [0x0ac4] = 0x68a2, [0x0ac5] = 0x6a1f,
  [0x0ac6] = 0x6a35, [0x0ac7] = 0x6cbc, [0x0ac8] = 0x6d88, [0x0ac9] = 0x6e09,
  [0x0aca] = 0x6e58, [0x0acb] = 0x713c, [0x0acc] = 0x7126, [0x0acd] = 0x7167,
  [0x0ace] = 0x75c7, [0x0acf] = 0x7701, [0x0ad0] = 0x785d, [0x0ad1] = 0x7901,
  [0x0ad2] = 0x7965, [0x0ad3] = 0x79f0, [0x0ad4] = 0x7ae0, [0x0ad5] = 0x7b11,
  [0x0ad6] = 0x7ca7, [0x0ad7] = 0x7d39, [0x0ad8] = 0x8096, [0x0ad9] = 0x83d6,
  [0x0ada] = 0x848b, [0x0adb] = 0x8549, [0x0adc] = 0x885d, [0x0add] = 0x88f3,
  [0x0ade] = 0x8a1f, [0x0adf] = 0x8a3c, [0x0ae0] = 0x8a54, [0x0ae1] = 0x8a73,
  [0x0ae2] = 0x8c61, [0x0ae3] = 0x8cde, [0x0ae4] = 0x91a4, [0x0ae5] = 0x9266,
  [0x0ae6] = 0x937e, [0x0ae7] = 0x9418, [0x0ae8] = 0x969c, [0x0ae9] = 0x9798,
  [0x0aea] = 0x4e0a, [0x0aeb] = 0x4e08, [0x0aec] = 0x4e1e, [0x0aed] = 0x4e57,
  [0x0aee] = 0x5197, [0x0aef] = 0x5270, [0x0af0] = 0x57ce, [0x0af1] = 0x5834,
  [0x0af2] = 0x58cc, [0x0af3] = 0x5b22, [0x0af4] = 0x5e38, [0x0af5] = 0x60c5,
  [0x0af6] = 0x64fe, [0x0af7] = 0x6761, [0x0af8] = 0x6756, [0x0af9] = 0x6d44,
  [0x0afa] = 0x72b6, [0x0afb] = 0x7573, [0x0afc] = 0x7a63, [0x0afd] = 0x84b8,
  [0x0afe] = 0x8b72, [0x0aff] = 0x91b8, [0x0b00] = 0x9320, [0x0b01] = 0x5631,
  [0x0b02] = 0x57f4, [0x0b03] = 0x98fe, [0x0b04] = 0x62ed, [0x0b05] = 0x690d,
  [0x0b06] = 0x6b96, [0x0b07] = 0x71ed, [0x0b08] = 0x7e54, [0x0b09] = 0x8077,
  [0x0b0a] = 0x8272, [0x0b0b] = 0x89e6, [0x0b0c] = 0x98df, [0x0b0d] = 0x8755,
  [0x0b0e] = 0x8fb1, [0x0b0f] = 0x5c3b, [0x0b10] = 0x4f38, [0x0b11] = 0x4fe1,
  [0x0b12] = 0x4fb5, [0x0b13] = 0x5507, [0x0b14] = 0x5a20, [0x0b15] = 0x5bdd,
  [0x0b16] = 0x5be9, [0x0b17] = 0x5fc3, [0x0b18] = 0x614e, [0x0b19] = 0x632f,
  [0x0b1a] = 0x65b0, [0x0b1b] = 0x664b, [0x0b1c] = 0x68ee, [0x0b1d] = 0x699b,
  [0x0b1e] = 0x6d78, [0x0b1f] = 0x6df1, [0x0b20] = 0x7533, [0x0b21] = 0x75b9,
  [0x0b22] = 0x771f, [0x0b23] = 0x795e, [0x0b24] = 0x79e6, [0x0b25] = 0x7d33,
  [0x0b26] = 0x81e3, [0x0b27] = 0x82af, [0x0b28] = 0x85aa, [0x0b29] = 0x89aa,
  [0x0b2a] = 0x8a3a, [0x0b2b] = 0x8eab, [0x0b2c] = 0x8f9b, [0x0b2d] = 0x9032,
  [0x0b2e] = 0x91dd, [0x0b2f] = 0x9707, [0x0b30] = 0x4eba, [0x0b31] = 0x4ec1,
  [0x0b32] = 0x5203, [0x0b33] = 0x5875, [0x0b34] = 0x58ec, [0x0b35] = 0x5c0b,
  [0x0b36] = 0x751a, [0x0b37] = 0x5c3d, [0x0b38] = 0x814e, [0x0b39] = 0x8a0a,
  [0x0b3a] = 0x8fc5, [0x0b3b] = 0x9663, [0x0b3c] = 0x976d, [0x0b3d] = 0x7b25,
  [0x0b3e] = 0x8acf, [0x0b3f] = 0x9808, [0x0b40] = 0x9162, [0x0b41] = 0x56f3,
  [0x0b42] = 0x53a8, [0x0b43] = 0x9017, [0x0b44] = 0x5439, [0x0b45] = 0x5782,
  [0x0b46] = 0x5e25, [0x0b47] = 0x63a8, [0x0b48] = 0x6c34, [0x0b49] = 0x708a,
  [0x0b4a] = 0x7761, [0x0b4b] = 0x7c8b, [0x0b4c] = 0x7fe0, [0x0b4d] = 0x8870,
  [0x0b4e] = 0x9042, [0x0b4f] = 0x9154, [0x0b50] = 0x9310, [0x0b51] = 0x9318,
  [0x0b52] = 0x968f, [0x0b53] = 0x745e, [0x0b54] = 0x9ac4, [0x0b55] = 0x5d07,
  [0x0b56] = 0x5d69, [0x0b57] = 0x6570, [0x0b58] = 0x67a2, [0x0b59] = 0x8da8,
  [0x0b5a] = 0x96db, [0x0b5b] = 0x636e, [0x0b5c] = 0x6749, [0x0b5d] = 0x6919,
  [0x0b5e] = 0x83c5, [0x0b5f] = 0x9817, [0x0b60] = 0x96c0, [0x0b61] = 0x88fe,
  [0x0b62] = 0x6f84, [0x0b63] = 0x647a, [0x0b64] = 0x5bf8, [0x0b65] = 0x4e16,
  [0x0b66] = 0x702c, [0x0b67] = 0x755d, [0x0b68] = 0x662f, [0x0b69] = 0x51c4,
  [0x0b6a] = 0x5236, [0x0b6b] = 0x52e2, [0x0b6c] = 0x59d3, [0x0b6d] = 0x5f81,
  [0x0b6e] = 0x6027, [0x0b6f] = 0x6210, [0x0b70] = 0x653f, [0x0b71] = 0x6574,
  [0x0b72] = 0x661f, [0x0b73] = 0x6674, [0x0b74] = 0x68f2, [0x0b75] = 0x6816,
  [0x0b76] = 0x6b63, [0x0b77] = 0x6e05, [0x0b78] = 0x7272, [0x0b79] = 0x751f,
  [0x0b7a] = 0x76db, [0x0b7b] = 0x7cbe, [0x0b7c] = 0x8056, [0x0b7d] = 0x58f0,
  [0x0b7e] = 0x88fd, [0x0b7f] = 0x897f, [0x0b80] = 0x8aa0, [0x0b81] = 0x8a93,
  [0x0b82] = 0x8acb, [0x0b83] = 0x901d, [0x0b84] = 0x9192, [0x0b85] = 0x9752,
  [0x0b86] = 0x9759, [0x0b87] = 0x6589, [0x0b88] = 0x7a0e, [0x0b89] = 0x8106,
  [0x0b8a] = 0x96bb, [0x0b8b] = 0x5e2d, [0x0b8c] = 0x60dc, [0x0b8d] = 0x621a,
  [0x0b8e] = 0x65a5, [0x0b8f] = 0x6614, [0x0b90] = 0x6790, [0x0b91] = 0x77f3,
  [0x0b92] = 0x7a4d, [0x0b93] = 0x7c4d, [0x0b94] = 0x7e3e, [0x0b95] = 0x810a,
  [0x0b96] = 0x8cac, [0x0b97] = 0x8d64, [0x0b98] = 0x8de1, [0x0b99] = 0x8e5f,
  [0x0b9a] = 0x78a9, [0x0b9b] = 0x5207, [0x0b9c] = 0x62d9, [0x0b9d] = 0x63a5,
  [0x0b9e] = 0x6442, [0x0b9f] = 0x6298, [0x0ba0] = 0x8a2d, [0x0ba1] = 0x7a83,
  [0x0ba2] = 0x7bc0, [0x0ba3] = 0x8aac, [0x0ba4] = 0x96ea, [0x0ba5] = 0x7d76,
  [0x0ba6] = 0x820c, [0x0ba7] = 0x8749, [0x0ba8] = 0x4ed9, [0x0ba9] = 0x5148,
  [0x0baa] = 0x5343, [0x0bab] = 0x5360, [0x0bac] = 0x5ba3, [0x0bad] = 0x5c02,
  [0x0bae] = 0x5c16, [0x0baf] = 0x5ddd, [0x0bb0] = 0x6226, [0x0bb1] = 0x6247,
  [0x0bb2] = 0x64b0, [0x0bb3] = 0x6813, [0x0bb4] = 0x6834, [0x0bb5] = 0x6cc9,
  [0x0bb6] = 0x6d45, [0x0bb7] = 0x6d17, [0x0bb8] = 0x67d3, [0x0bb9] = 0x6f5c,
  [0x0bba] = 0x714e, [0x0bbb] = 0x717d, [0x0bbc] = 0x65cb, [0x0bbd] = 0x7a7f,
  [0x0bbe] = 0x7bad, [0x0bbf] = 0x7dda, [0x0bc0] = 0x7e4a, [0x0bc1] = 0x7fa8,
  [0x0bc2] = 0x817a, [0x0bc3] = 0x821b, [0x0bc4] = 0x8239, [0x0bc5] = 0x85a6,
  [0x0bc6] = 0x8a6e, [0x0bc7] = 0x8cce, [0x0bc8] = 0x8df5, [0x0bc9] = 0x9078,
  [0x0bca] = 0x9077, [0x0bcb] = 0x92ad, [0x0bcc] = 0x9291, [0x0bcd] = 0x9583,
  [0x0bce] = 0x9bae, [0x0bcf] = 0x524d, [0x0bd0] = 0x5584, [0x0bd1] = 0x6f38,
  [0x0bd2] = 0x7136, [0x0bd3] = 0x5168, [0x0bd4] = 0x7985, [0x0bd5] = 0x7e55,
  [0x0bd6] = 0x81b3, [0x0bd7] = 0x7cce, [0x0bd8] = 0x564c, [0x0bd9] = 0x5851,
  [0x0bda] = 0x5ca8, [0x0bdb] = 0x63aa, [0x0bdc] = 0x66fe, [0x0bdd] = 0x66fd,
  [0x0bde] = 0x695a, [0x0bdf] = 0x72d9, [0x0be0] = 0x758f, [0x0be1] = 0x758e,
  [0x0be2] = 0x790e, [0x0be3] = 0x7956, [0x0be4] = 0x79df, [0x0be5] = 0x7c97,
  [0x0be6] = 0x7d20, [0x0be7] = 0x7d44, [0x0be8] = 0x8607, [0x0be9] = 0x8a34,
  [0x0bea] = 0x963b, [0x0beb] = 0x9061, [0x0bec] = 0x9f20, [0x0bed] = 0x50e7,
  [0x0bee] = 0x5275, [0x0bef] = 0x53cc, [0x0bf0] = 0x53e2, [0x0bf1] = 0x5009,
  [0x0bf2] = 0x55aa, [0x0bf3] = 0x58ee, [0x0bf4] = 0x594f, [0x0bf5] = 0x723d,
  [0x0bf6] = 0x5b8b, [0x0bf7] = 0x5c64, [0x0bf8] = 0x531d, [0x0bf9] = 0x60e3,
  [0x0bfa] = 0x60f3, [0x0bfb] = 0x635c, [0x0bfc] = 0x6383, [0x0bfd] = 0x633f,
  [0x0bfe] = 0x63bb, [0x0bff] = 0x64cd, [0x0c00] = 0x65e9, [0x0c01] = 0x66f9,
  [0x0c02] = 0x5de3, [0x0c03] = 0x69cd, [0x0c04] = 0x69fd, [0x0c05] = 0x6f15,
  [0x0c06] = 0x71e5, [0x0c07] = 0x4e89, [0x0c08] = 0x75e9, [0x0c09] = 0x76f8,
  [0x0c0a] = 0x7a93, [0x0c0b] = 0x7cdf, [0x0c0c] = 0x7dcf, [0x0c0d] = 0x7d9c,
  [0x0c0e] = 0x8061, [0x0c0f] = 0x8349, [0x0c10] = 0x8358, [0x0c11] = 0x846c,
  [0x0c12] = 0x84bc, [0x0c13] = 0x85fb, [0x0c14] = 0x88c5, [0x0c15] = 0x8d70,
  [0x0c16] = 0x9001, [0x0c17] = 0x906d, [0x0c18] = 0x9397, [0x0c19] = 0x971c,
  [0x0c1a] = 0x9a12, [0x0c1b] = 0x50cf, [0x0c1c] = 0x5897, [0x0c1d] = 0x618e,
  [0x0c1e] = 0x81d3, [0x0c1f] = 0x8535, [0x0c20] = 0x8d08, [0x0c21] = 0x9020,
  [0x0c22] = 0x4fc3, [0x0c23] = 0x5074, [0x0c24] = 0x5247, [0x0c25] = 0x5373,
  [0x0c26] = 0x606f, [0x0c27] = 0x6349, [0x0c28] = 0x675f, [0x0c29] = 0x6e2c,
  [0x0c2a] = 0x8db3, [0x0c2b] = 0x901f, [0x0c2c] = 0x4fd7, [0x0c2d] = 0x5c5e,
  [0x0c2e] = 0x8cca, [0x0c2f] = 0x65cf, [0x0c30] = 0x7d9a, [0x0c31] = 0x5352,
  [0x0c32] = 0x8896, [0x0c33] = 0x5176, [0x0c34] = 0x63c3, [0x0c35] = 0x5b58,
  [0x0c36] = 0x5b6b, [0x0c37] = 0x5c0a, [0x0c38] = 0x640d, [0x0c39] = 0x6751,
  [0x0c3a] = 0x905c, [0x0c3b] = 0x4ed6, [0x0c3c] = 0x591a, [0x0c3d] = 0x592a,
  [0x0c3e] = 0x6c70, [0x0c3f] = 0x8a51, [0x0c40] = 0x553e, [0x0c41] = 0x5815,
  [0x0c42] = 0x59a5, [0x0c43] = 0x60f0, [0x0c44] = 0x6253, [0x0c45] = 0x67c1,
  [0x0c46] = 0x8235, [0x0c47] = 0x6955, [0x0c48] = 0x9640, [0x0c49] = 0x99c4,
  [0x0c4a] = 0x9a28, [0x0c4b] = 0x4f53, [0x0c4c] = 0x5806, [0x0c4d] = 0x5bfe,
  [0x0c4e] = 0x8010, [0x0c4f] = 0x5cb1, [0x0c50] = 0x5e2f, [0x0c51] = 0x5f85,
  [0x0c52] = 0x6020, [0x0c53] = 0x614b, [0x0c54] = 0x6234, [0x0c55] = 0x66ff,
  [0x0c56] = 0x6cf0, [0x0c57] = 0x6ede, [0x0c58] = 0x80ce, [0x0c59] = 0x817f,
  [0x0c5a] = 0x82d4, [0x0c5b] = 0x888b, [0x0c5c] = 0x8cb8, [0x0c5d] = 0x9000,
  [0x0c5e] = 0x902e, [0x0c5f] = 0x968a, [0x0c60] = 0x9edb, [0x0c61] = 0x9bdb,
  [0x0c62] = 0x4ee3, [0x0c63] = 0x53f0, [0x0c64] = 0x5927, [0x0c65] = 0x7b2c,
  [0x0c66] = 0x918d, [0x0c67] = 0x984c, [0x0c68] = 0x9df9, [0x0c69] = 0x6edd,
  [0x0c6a] = 0x7027, [0x0c6b] = 0x5353, [0x0c6c] = 0x5544, [0x0c6d] = 0x5b85,
  [0x0c6e] = 0x6258, [0x0c6f] = 0x629e, [0x0c70] = 0x62d3, [0x0c71] = 0x6ca2,
  [0x0c72] = 0x6fef, [0x0c73] = 0x7422, [0x0c74] = 0x8a17, [0x0c75] = 0x9438,
  [0x0c76] = 0x6fc1, [0x0c77] = 0x8afe, [0x0c78] = 0x8338, [0x0c79] = 0x51e7,
  [0x0c7a] = 0x86f8, [0x0c7b] = 0x53ea, [0x0c7c] = 0x53e9, [0x0c7d] = 0x4f46,
  [0x0c7e] = 0x9054, [0x0c7f] = 0x8fb0, [0x0c80] = 0x596a, [0x0c81] = 0x8131,
  [0x0c82] = 0x5dfd, [0x0c83] = 0x7aea, [0x0c84] = 0x8fbf, [0x0c85] = 0x68da,
  [0x0c86] = 0x8c37, [0x0c87] = 0x72f8, [0x0c88] = 0x9c48, [0x0c89] = 0x6a3d,
  [0x0c8a] = 0x8ab0, [0x0c8b] = 0x4e39, [0x0c8c] = 0x5358, [0x0c8d] = 0x5606,
  [0x0c8e] = 0x5766, [0x0c8f] = 0x62c5, [0x0c90] = 0x63a2, [0x0c91] = 0x65e6,
  [0x0c92] = 0x6b4e, [0x0c93] = 0x6de1, [0x0c94] = 0x6e5b, [0x0c95] = 0x70ad,
  [0x0c96] = 0x77ed, [0x0c97] = 0x7aef, [0x0c98] = 0x7baa, [0x0c99] = 0x7dbb,
  [0x0c9a] = 0x803d, [0x0c9b] = 0x80c6, [0x0c9c] = 0x86cb, [0x0c9d] = 0x8a95,
  [0x0c9e] = 0x935b, [0x0c9f] = 0x56e3, [0x0ca0] = 0x58c7, [0x0ca1] = 0x5f3e,
  [0x0ca2] = 0x65ad, [0x0ca3] = 0x6696, [0x0ca4] = 0x6a80, [0x0ca5] = 0x6bb5,
  [0x0ca6] = 0x7537, [0x0ca7] = 0x8ac7, [0x0ca8] = 0x5024, [0x0ca9] = 0x77e5,
  [0x0caa] = 0x5730, [0x0cab] = 0x5f1b, [0x0cac] = 0x6065, [0x0cad] = 0x667a,
  [0x0cae] = 0x6c60, [0x0caf] = 0x75f4, [0x0cb0] = 0x7a1a, [0x0cb1] = 0x7f6e,
  [0x0cb2] = 0x81f4, [0x0cb3] = 0x8718, [0x0cb4] = 0x9045, [0x0cb5] = 0x99b3,
  [0x0cb6] = 0x7bc9, [0x0cb7] = 0x755c, [0x0cb8] = 0x7af9, [0x0cb9] = 0x7b51,
  [0x0cba] = 0x84c4, [0x0cbb] = 0x9010, [0x0cbc] = 0x79e9, [0x0cbd] = 0x7a92,
  [0x0cbe] = 0x8336, [0x0cbf] = 0x5ae1, [0x0cc0] = 0x7740, [0x0cc1] = 0x4e2d,
  [0x0cc2] = 0x4ef2, [0x0cc3] = 0x5b99, [0x0cc4] = 0x5fe0, [0x0cc5] = 0x62bd,
  [0x0cc6] = 0x663c, [0x0cc7] = 0x67f1, [0x0cc8] = 0x6ce8, [0x0cc9] = 0x866b,
  [0x0cca] = 0x8877, [0x0ccb] = 0x8a3b, [0x0ccc] = 0x914e, [0x0ccd] = 0x92f3,
  [0x0cce] = 0x99d0, [0x0ccf] = 0x6a17, [0x0cd0] = 0x7026, [0x0cd1] = 0x732a,
  [0x0cd2] = 0x82e7, [0x0cd3] = 0x8457, [0x0cd4] = 0x8caf, [0x0cd5] = 0x4e01,
  [0x0cd6] = 0x5146, [0x0cd7] = 0x51cb, [0x0cd8] = 0x558b, [0x0cd9] = 0x5bf5,
  [0x0cda] = 0x5e16, [0x0cdb] = 0x5e33, [0x0cdc] = 0x5e81, [0x0cdd] = 0x5f14,
  [0x0cde] = 0x5f35, [0x0cdf] = 0x5f6b, [0x0ce0] = 0x5fb4, [0x0ce1] = 0x61f2,
  [0x0ce2] = 0x6311, [0x0ce3] = 0x66a2, [0x0ce4] = 0x671d, [0x0ce5] = 0x6f6e,
  [0x0ce6] = 0x7252, [0x0ce7] = 0x753a, [0x0ce8] = 0x773a, [0x0ce9] = 0x8074,
  [0x0cea] = 0x8139, [0x0ceb] = 0x8178, [0x0cec] = 0x8776, [0x0ced] = 0x8abf,
  [0x0cee] = 0x8adc, [0x0cef] = 0x8d85, [0x0cf0] = 0x8df3, [0x0cf1] = 0x929a,
  [0x0cf2] = 0x9577, [0x0cf3] = 0x9802, [0x0cf4] = 0x9ce5, [0x0cf5] = 0x52c5,
  [0x0cf6] = 0x6357, [0x0cf7] = 0x76f4, [0x0cf8] = 0x6715, [0x0cf9] = 0x6c88,
  [0x0cfa] = 0x73cd, [0x0cfb] = 0x8cc3, [0x0cfc] = 0x93ae, [0x0cfd] = 0x9673,
  [0x0cfe] = 0x6d25, [0x0cff] = 0x589c, [0x0d00] = 0x690e, [0x0d01] = 0x69cc,
  [0x0d02] = 0x8ffd, [0x0d03] = 0x939a, [0x0d04] = 0x75db, [0x0d05] = 0x901a,
  [0x0d06] = 0x585a, [0x0d07] = 0x6802, [0x0d08] = 0x63b4, [0x0d09] = 0x69fb,
  [0x0d0a] = 0x4f43, [0x0d0b] = 0x6f2c, [0x0d0c] = 0x67d8, [0x0d0d] = 0x8fbb,
  [0x0d0e] = 0x8526, [0x0d0f] = 0x7db4, [0x0d10] = 0x9354, [0x0d11] = 0x693f,
  [0x0d12] = 0x6f70, [0x0d13] = 0x576a, [0x0d14] = 0x58f7, [0x0d15] = 0x5b2c,
  [0x0d16] = 0x7d2c, [0x0d17] = 0x722a, [0x0d18] = 0x540a, [0x0d19] = 0x91e3,
  [0x0d1a] = 0x9db4, [0x0d1b] = 0x4ead, [0x0d1c] = 0x4f4e, [0x0d1d] = 0x505c,
  [0x0d1e] = 0x5075, [0x0d1f] = 0x5243, [0x0d20] = 0x8c9e, [0x0d21] = 0x5448,
  [0x0d22] = 0x5824, [0x0d23] = 0x5b9a, [0x0d24] = 0x5e1d, [0x0d25] = 0x5e95,
  [0x0d26] = 0x5ead, [0x0d27] = 0x5ef7, [0x0d28] = 0x5f1f, [0x0d29] = 0x608c,
  [0x0d2a] = 0x62b5, [0x0d2b] = 0x633a, [0x0d2c] = 0x63d0, [0x0d2d] = 0x68af,
  [0x0d2e] = 0x6c40, [0x0d2f] = 0x7887, [0x0d30] = 0x798e, [0x0d31] = 0x7a0b,
  [0x0d32] = 0x7de0, [0x0d33] = 0x8247, [0x0d34] = 0x8a02, [0x0d35] = 0x8ae6,
  [0x0d36] = 0x8e44, [0x0d37] = 0x9013, [0x0d38] = 0x90b8, [0x0d39] = 0x912d,
  [0x0d3a] = 0x91d8, [0x0d3b] = 0x9f0e, [0x0d3c] = 0x6ce5, [0x0d3d] = 0x6458,
  [0x0d3e] = 0x64e2, [0x0d3f] = 0x6575, [0x0d40] = 0x6ef4, [0x0d41] = 0x7684,
  [0x0d42] = 0x7b1b, [0x0d43] = 0x9069, [0x0d44] = 0x93d1, [0x0d45] = 0x6eba,
  [0x0d46] = 0x54f2, [0x0d47] = 0x5fb9, [0x0d48] = 0x64a4, [0x0d49] = 0x8f4d,
  [0x0d4a] = 0x8fed, [0x0d4b] = 0x9244, [0x0d4c] = 0x5178, [0x0d4d] = 0x586b,
  [0x0d4e] = 0x5929, [0x0d4f] = 0x5c55, [0x0d50] = 0x5e97, [0x0d51] = 0x6dfb,
  [0x0d52] = 0x7e8f, [0x0d53] = 0x751c, [0x0d54] = 0x8cbc, [0x0d55] = 0x8ee2,
  [0x0d56] = 0x985b, [0x0d57] = 0x70b9, [0x0d58] = 0x4f1d, [0x0d59] = 0x6bbf,
  [0x0d5a] = 0x6fb1, [0x0d5b] = 0x7530, [0x0d5c] = 0x96fb, [0x0d5d] = 0x514e,
  [0x0d5e] = 0x5410, [0x0d5f] = 0x5835, [0x0d60] = 0x5857, [0x0d61] = 0x59ac,
  [0x0d62] = 0x5c60, [0x0d63] = 0x5f92, [0x0d64] = 0x6597, [0x0d65] = 0x675c,
  [0x0d66] = 0x6e21, [0x0d67] = 0x767b, [0x0d68] = 0x83df, [0x0d69] = 0x8ced,
  [0x0d6a] = 0x9014, [0x0d6b] = 0x90fd, [0x0d6c] = 0x934d, [0x0d6d] = 0x7825,
  [0x0d6e] = 0x783a, [0x0d6f] = 0x52aa, [0x0d70] = 0x5ea6, [0x0d71] = 0x571f,
  [0x0d72] = 0x5974, [0x0d73] = 0x6012, [0x0d74] = 0x5012, [0x0d75] = 0x515a,
  [0x0d76] = 0x51ac, [0x0d77] = 0x51cd, [0x0d78] = 0x5200, [0x0d79] = 0x5510,
  [0x0d7a] = 0x5854, [0x0d7b] = 0x5858, [0x0d7c] = 0x5957, [0x0d7d] = 0x5b95,
  [0x0d7e] = 0x5cf6, [0x0d7f] = 0x5d8b, [0x0d80] = 0x60bc, [0x0d81] = 0x6295,
  [0x0d82] = 0x642d, [0x0d83] = 0x6771, [0x0d84] = 0x6843, [0x0d85] = 0x68bc,
  [0x0d86] = 0x68df, [0x0d87] = 0x76d7, [0x0d88] = 0x6dd8, [0x0d89] = 0x6e6f,
  [0x0d8a] = 0x6d9b, [0x0d8b] = 0x706f, [0x0d8c] = 0x71c8, [0x0d8d] = 0x5f53,
  [0x0d8e] = 0x75d8, [0x0d8f] = 0x7977, [0x0d90] = 0x7b49, [0x0d91] = 0x7b54,
  [0x0d92] = 0x7b52, [0x0d93] = 0x7cd6, [0x0d94] = 0x7d71, [0x0d95] = 0x5230,
  [0x0d96] = 0x8463, [0x0d97] = 0x8569, [0x0d98] = 0x85e4, [0x0d99] = 0x8a0e,
  [0x0d9a] = 0x8b04, [0x0d9b] = 0x8c46, [0x0d9c] = 0x8e0f, [0x0d9d] = 0x9003,
  [0x0d9e] = 0x900f, [0x0d9f] = 0x9419, [0x0da0] = 0x9676, [0x0da1] = 0x982d,
  [0x0da2] = 0x9a30, [0x0da3] = 0x95d8, [0x0da4] = 0x50cd, [0x0da5] = 0x52d5,
  [0x0da6] = 0x540c, [0x0da7] = 0x5802, [0x0da8] = 0x5c0e, [0x0da9] = 0x61a7,
  [0x0daa] = 0x649e, [0x0dab] = 0x6d1e, [0x0dac] = 0x77b3, [0x0dad] = 0x7ae5,
  [0x0dae] = 0x80f4, [0x0daf] = 0x8404, [0x0db0] = 0x9053, [0x0db1] = 0x9285,
  [0x0db2] = 0x5ce0, [0x0db3] = 0x9d07, [0x0db4] = 0x533f, [0x0db5] = 0x5f97,
  [0x0db6] = 0x5fb3, [0x0db7] = 0x6d9c, [0x0db8] = 0x7279, [0x0db9] = 0x7763,
  [0x0dba] = 0x79bf, [0x0dbb] = 0x7be4, [0x0dbc] = 0x6bd2, [0x0dbd] = 0x72ec,
  [0x0dbe] = 0x8aad, [0x0dbf] = 0x6803, [0x0dc0] = 0x6a61, [0x0dc1] = 0x51f8,
  [0x0dc2] = 0x7a81, [0x0dc3] = 0x6934, [0x0dc4] = 0x5c4a, [0x0dc5] = 0x9cf6,
  [0x0dc6] = 0x82eb, [0x0dc7] = 0x5bc5, [0x0dc8] = 0x9149, [0x0dc9] = 0x701e,
  [0x0dca] = 0x5678, [0x0dcb] = 0x5c6f, [0x0dcc] = 0x60c7, [0x0dcd] = 0x6566,
  [0x0dce] = 0x6c8c, [0x0dcf] = 0x8c5a, [0x0dd0] = 0x9041, [0x0dd1] = 0x9813,
  [0x0dd2] = 0x5451, [0x0dd3] = 0x66c7, [0x0dd4] = 0x920d, [0x0dd5] = 0x5948,
  [0x0dd6] = 0x90a3, [0x0dd7] = 0x5185, [0x0dd8] = 0x4e4d, [0x0dd9] = 0x51ea,
  [0x0dda] = 0x8599, [0x0ddb] = 0x8b0e, [0x0ddc] = 0x7058, [0x0ddd] = 0x637a,
  [0x0dde] = 0x934b, [0x0ddf] = 0x6962, [0x0de0] = 0x99b4, [0x0de1] = 0x7e04,
  [0x0de2] = 0x7577, [0x0de3] = 0x5357, [0x0de4] = 0x6960, [0x0de5] = 0x8edf,
  [0x0de6] = 0x96e3, [0x0de7] = 0x6c5d, [0x0de8] = 0x4e8c, [0x0de9] = 0x5c3c,
  [0x0dea] = 0x5f10, [0x0deb] = 0x8fe9, [0x0dec] = 0x5302, [0x0ded] = 0x8cd1,
  [0x0dee] = 0x8089, [0x0def] = 0x8679, [0x0df0] = 0x5eff, [0x0df1] = 0x65e5,
  [0x0df2] = 0x4e73, [0x0df3] = 0x5165, [0x0df4] = 0x5982, [0x0df5] = 0x5c3f,
  [0x0df6] = 0x97ee, [0x0df7] = 0x4efb, [0x0df8] = 0x598a, [0x0df9] = 0x5fcd,
  [0x0dfa] = 0x8a8d, [0x0dfb] = 0x6fe1, [0x0dfc] = 0x79b0, [0x0dfd] = 0x7962,
  [0x0dfe] = 0x5be7, [0x0dff] = 0x8471, [0x0e00] = 0x732b, [0x0e01] = 0x71b1,
  [0x0e02] = 0x5e74, [0x0e03] = 0x5ff5, [0x0e04] = 0x637b, [0x0e05] = 0x649a,
  [0x0e06] = 0x71c3, [0x0e07] = 0x7c98, [0x0e08] = 0x4e43, [0x0e09] = 0x5efc,
  [0x0e0a] = 0x4e4b, [0x0e0b] = 0x57dc, [0x0e0c] = 0x56a2, [0x0e0d] = 0x60a9,
  [0x0e0e] = 0x6fc3, [0x0e0f] = 0x7d0d, [0x0e10] = 0x80fd, [0x0e11] = 0x8133,
  [0x0e12] = 0x81bf, [0x0e13] = 0x8fb2, [0x0e14] = 0x8997, [0x0e15] = 0x86a4,
  [0x0e16] = 0x5df4, [0x0e17] = 0x628a, [0x0e18] = 0x64ad, [0x0e19] = 0x8987,
  [0x0e1a] = 0x6777, [0x0e1b] = 0x6ce2, [0x0e1c] = 0x6d3e, [0x0e1d] = 0x7436,
  [0x0e1e] = 0x7834, [0x0e1f] = 0x5a46, [0x0e20] = 0x7f75, [0x0e21] = 0x82ad,
  [0x0e22] = 0x99ac, [0x0e23] = 0x4ff3, [0x0e24] = 0x5ec3, [0x0e25] = 0x62dd,
  [0x0e26] = 0x6392, [0x0e27] = 0x6557, [0x0e28] = 0x676f, [0x0e29] = 0x76c3,
  [0x0e2a] = 0x724c, [0x0e2b] = 0x80cc, [0x0e2c] = 0x80ba, [0x0e2d] = 0x8f29,
  [0x0e2e] = 0x914d, [0x0e2f] = 0x500d, [0x0e30] = 0x57f9, [0x0e31] = 0x5a92,
  [0x0e32] = 0x6885, [0x0e33] = 0x6973, [0x0e34] = 0x7164, [0x0e35] = 0x72fd,
  [0x0e36] = 0x8cb7, [0x0e37] = 0x58f2, [0x0e38] = 0x8ce0, [0x0e39] = 0x966a,
  [0x0e3a] = 0x9019, [0x0e3b] = 0x877f, [0x0e3c] = 0x79e4, [0x0e3d] = 0x77e7,
  [0x0e3e] = 0x8429, [0x0e3f] = 0x4f2f, [0x0e40] = 0x5265, [0x0e41] = 0x535a,
  [0x0e42] = 0x62cd, [0x0e43] = 0x67cf, [0x0e44] = 0x6cca, [0x0e45] = 0x767d,
  [0x0e46] = 0x7b94, [0x0e47] = 0x7c95, [0x0e48] = 0x8236, [0x0e49] = 0x8584,
  [0x0e4a] = 0x8feb, [0x0e4b] = 0x66dd, [0x0e4c] = 0x6f20, [0x0e4d] = 0x7206,
  [0x0e4e] = 0x7e1b, [0x0e4f] = 0x83ab, [0x0e50] = 0x99c1, [0x0e51] = 0x9ea6,
  [0x0e52] = 0x51fd, [0x0e53] = 0x7bb1, [0x0e54] = 0x7872, [0x0e55] = 0x7bb8,
  [0x0e56] = 0x8087, [0x0e57] = 0x7b48, [0x0e58] = 0x6ae8, [0x0e59] = 0x5e61,
  [0x0e5a] = 0x808c, [0x0e5b] = 0x7551, [0x0e5c] = 0x7560, [0x0e5d] = 0x516b,
  [0x0e5e] = 0x9262, [0x0e5f] = 0x6e8c, [0x0e60] = 0x767a, [0x0e61] = 0x9197,
  [0x0e62] = 0x9aea, [0x0e63] = 0x4f10, [0x0e64] = 0x7f70, [0x0e65] = 0x629c,
  [0x0e66] = 0x7b4f, [0x0e67] = 0x95a5, [0x0e68] = 0x9ce9, [0x0e69] = 0x567a,
  [0x0e6a] = 0x5859, [0x0e6b] = 0x86e4, [0x0e6c] = 0x96bc, [0x0e6d] = 0x4f34,
  [0x0e6e] = 0x5224, [0x0e6f] = 0x534a, [0x0e70] = 0x53cd, [0x0e71] = 0x53db,
  [0x0e72] = 0x5e06, [0x0e73] = 0x642c, [0x0e74] = 0x6591, [0x0e75] = 0x677f,
  [0x0e76] = 0x6c3e, [0x0e77] = 0x6c4e, [0x0e78] = 0x7248, [0x0e79] = 0x72af,
  [0x0e7a] = 0x73ed, [0x0e7b] = 0x7554, [0x0e7c] = 0x7e41, [0x0e7d] = 0x822c,
  [0x0e7e] = 0x85e9, [0x0e7f] = 0x8ca9, [0x0e80] = 0x7bc4, [0x0e81] = 0x91c6,
  [0x0e82] = 0x7169, [0x0e83] = 0x9812, [0x0e84] = 0x98ef, [0x0e85] = 0x633d,
  [0x0e86] = 0x6669, [0x0e87] = 0x756a, [0x0e88] = 0x76e4, [0x0e89] = 0x78d0,
  [0x0e8a] = 0x8543, [0x0e8b] = 0x86ee, [0x0e8c] = 0x532a, [0x0e8d] = 0x5351,
  [0x0e8e] = 0x5426, [0x0e8f] = 0x5983, [0x0e90] = 0x5e87, [0x0e91] = 0x5f7c,
  [0x0e92] = 0x60b2, [0x0e93] = 0x6249, [0x0e94] = 0x6279, [0x0e95] = 0x62ab,
  [0x0e96] = 0x6590, [0x0e97] = 0x6bd4, [0x0e98] = 0x6ccc, [0x0e99] = 0x75b2,
  [0x0e9a] = 0x76ae, [0x0e9b] = 0x7891, [0x0e9c] = 0x79d8, [0x0e9d] = 0x7dcb,
  [0x0e9e] = 0x7f77, [0x0e9f] = 0x80a5, [0x0ea0] = 0x88ab, [0x0ea1] = 0x8ab9,
  [0x0ea2] = 0x8cbb, [0x0ea3] = 0x907f, [0x0ea4] = 0x975e, [0x0ea5] = 0x98db,
  [0x0ea6] = 0x6a0b, [0x0ea7] = 0x7c38, [0x0ea8] = 0x5099, [0x0ea9] = 0x5c3e,
  [0x0eaa] = 0x5fae, [0x0eab] = 0x6787, [0x0eac] = 0x6bd8, [0x0ead] = 0x7435,
  [0x0eae] = 0x7709, [0x0eaf] = 0x7f8e, [0x0eb0] = 0x9f3b, [0x0eb1] = 0x67ca,
  [0x0eb2] = 0x7a17, [0x0eb3] = 0x5339, [0x0eb4] = 0x758b, [0x0eb5] = 0x9aed,
  [0x0eb6] = 0x5f66, [0x0eb7] = 0x819d, [0x0eb8] = 0x83f1, [0x0eb9] = 0x8098,
  [0x0eba] = 0x5f3c, [0x0ebb] = 0x5fc5, [0x0ebc] = 0x7562, [0x0ebd] = 0x7b46,
  [0x0ebe] = 0x903c, [0x0ebf] = 0x6867, [0x0ec0] = 0x59eb, [0x0ec1] = 0x5a9b,
  [0x0ec2] = 0x7d10, [0x0ec3] = 0x767e, [0x0ec4] = 0x8b2c, [0x0ec5] = 0x4ff5,
  [0x0ec6] = 0x5f6a, [0x0ec7] = 0x6a19, [0x0ec8] = 0x6c37, [0x0ec9] = 0x6f02,
  [0x0eca] = 0x74e2, [0x0ecb] = 0x7968, [0x0ecc] = 0x8868, [0x0ecd] = 0x8a55,
  [0x0ece] = 0x8c79, [0x0ecf] = 0x5edf, [0x0ed0] = 0x63cf, [0x0ed1] = 0x75c5,
  [0x0ed2] = 0x79d2, [0x0ed3] = 0x82d7, [0x0ed4] = 0x9328, [0x0ed5] = 0x92f2,
  [0x0ed6] = 0x849c, [0x0ed7] = 0x86ed, [0x0ed8] = 0x9c2d, [0x0ed9] = 0x54c1,
  [0x0eda] = 0x5f6c, [0x0edb] = 0x658c, [0x0edc] = 0x6d5c, [0x0edd] = 0x7015,
  [0x0ede] = 0x8ca7, [0x0edf] = 0x8cd3, [0x0ee0] = 0x983b, [0x0ee1] = 0x654f,
  [0x0ee2] = 0x74f6, [0x0ee3] = 0x4e0d, [0x0ee4] = 0x4ed8, [0x0ee5] = 0x57e0,
  [0x0ee6] = 0x592b, [0x0ee7] = 0x5a66, [0x0ee8] = 0x5bcc, [0x0ee9] = 0x51a8,
  [0x0eea] = 0x5e03, [0x0eeb] = 0x5e9c, [0x0eec] = 0x6016, [0x0eed] = 0x6276,
  [0x0eee] = 0x6577, [0x0eef] = 0x65a7, [0x0ef0] = 0x666e, [0x0ef1] = 0x6d6e,
  [0x0ef2] = 0x7236, [0x0ef3] = 0x7b26, [0x0ef4] = 0x8150, [0x0ef5] = 0x819a,
  [0x0ef6] = 0x8299, [0x0ef7] = 0x8b5c, [0x0ef8] = 0x8ca0, [0x0ef9] = 0x8ce6,
  [0x0efa] = 0x8d74, [0x0efb] = 0x961c, [0x0efc] = 0x9644, [0x0efd] = 0x4fae,
  [0x0efe] = 0x64ab, [0x0eff] = 0x6b66, [0x0f00] = 0x821e, [0x0f01] = 0x8461,
  [0x0f02] = 0x856a, [0x0f03] = 0x90e8, [0x0f04] = 0x5c01, [0x0f05] = 0x6953,
  [0x0f06] = 0x98a8, [0x0f07] = 0x847a, [0x0f08] = 0x8557, [0x0f09] = 0x4f0f,
  [0x0f0a] = 0x526f, [0x0f0b] = 0x5fa9, [0x0f0c] = 0x5e45, [0x0f0d] = 0x670d,
  [0x0f0e] = 0x798f, [0x0f0f] = 0x8179, [0x0f10] = 0x8907, [0x0f11] = 0x8986,
  [0x0f12] = 0x6df5, [0x0f13] = 0x5f17, [0x0f14] = 0x6255, [0x0f15] = 0x6cb8,
  [0x0f16] = 0x4ecf, [0x0f17] = 0x7269, [0x0f18] = 0x9b92, [0x0f19] = 0x5206,
  [0x0f1a] = 0x543b, [0x0f1b] = 0x5674, [0x0f1c] = 0x58b3, [0x0f1d] = 0x61a4,
  [0x0f1e] = 0x626e, [0x0f1f] = 0x711a, [0x0f20] = 0x596e, [0x0f21] = 0x7c89,
  [0x0f22] = 0x7cde, [0x0f23] = 0x7d1b, [0x0f24] = 0x96f0, [0x0f25] = 0x6587,
  [0x0f26] = 0x805e, [0x0f27] = 0x4e19, [0x0f28] = 0x4f75, [0x0f29] = 0x5175,
  [0x0f2a] = 0x5840, [0x0f2b] = 0x5e63, [0x0f2c] = 0x5e73, [0x0f2d] = 0x5f0a,
  [0x0f2e] = 0x67c4, [0x0f2f] = 0x4e26, [0x0f30] = 0x853d, [0x0f31] = 0x9589,
  [0x0f32] = 0x965b, [0x0f33] = 0x7c73, [0x0f34] = 0x9801, [0x0f35] = 0x50fb,
  [0x0f36] = 0x58c1, [0x0f37] = 0x7656, [0x0f38] = 0x78a7, [0x0f39] = 0x5225,
  [0x0f3a] = 0x77a5, [0x0f3b] = 0x8511, [0x0f3c] = 0x7b86, [0x0f3d] = 0x504f,
  [0x0f3e] = 0x5909, [0x0f3f] = 0x7247, [0x0f40] = 0x7bc7, [0x0f41] = 0x7de8,
  [0x0f42] = 0x8fba, [0x0f43] = 0x8fd4, [0x0f44] = 0x904d, [0x0f45] = 0x4fbf,
  [0x0f46] = 0x52c9, [0x0f47] = 0x5a29, [0x0f48] = 0x5f01, [0x0f49] = 0x97ad,
  [0x0f4a] = 0x4fdd, [0x0f4b] = 0x8217, [0x0f4c] = 0x92ea, [0x0f4d] = 0x5703,
  [0x0f4e] = 0x6355, [0x0f4f] = 0x6b69, [0x0f50] = 0x752b, [0x0f51] = 0x88dc,
  [0x0f52] = 0x8f14, [0x0f53] = 0x7a42, [0x0f54] = 0x52df, [0x0f55] = 0x5893,
  [0x0f56] = 0x6155, [0x0f57] = 0x620a, [0x0f58] = 0x66ae, [0x0f59] = 0x6bcd,
  [0x0f5a] = 0x7c3f, [0x0f5b] = 0x83e9, [0x0f5c] = 0x5023, [0x0f5d] = 0x4ff8,
  [0x0f5e] = 0x5305, [0x0f5f] = 0x5446, [0x0f60] = 0x5831, [0x0f61] = 0x5949,
  [0x0f62] = 0x5b9d, [0x0f63] = 0x5cf0, [0x0f64] = 0x5cef, [0x0f65] = 0x5d29,
  [0x0f66] = 0x5e96, [0x0f67] = 0x62b1, [0x0f68] = 0x6367, [0x0f69] = 0x653e,
  [0x0f6a] = 0x65b9, [0x0f6b] = 0x670b, [0x0f6c] = 0x6cd5, [0x0f6d] = 0x6ce1,
  [0x0f6e] = 0x70f9, [0x0f6f] = 0x7832, [0x0f70] = 0x7e2b, [0x0f71] = 0x80de,
  [0x0f72] = 0x82b3, [0x0f73] = 0x840c, [0x0f74] = 0x84ec, [0x0f75] = 0x8702,
  [0x0f76] = 0x8912, [0x0f77] = 0x8a2a, [0x0f78] = 0x8c4a, [0x0f79] = 0x90a6,
  [0x0f7a] = 0x92d2, [0x0f7b] = 0x98fd, [0x0f7c] = 0x9cf3, [0x0f7d] = 0x9d6c,
  [0x0f7e] = 0x4e4f, [0x0f7f] = 0x4ea1, [0x0f80] = 0x508d, [0x0f81] = 0x5256,
  [0x0f82] = 0x574a, [0x0f83] = 0x59a8, [0x0f84] = 0x5e3d, [0x0f85] = 0x5fd8,
  [0x0f86] = 0x5fd9, [0x0f87] = 0x623f, [0x0f88] = 0x66b4, [0x0f89] = 0x671b,
  [0x0f8a] = 0x67d0, [0x0f8b] = 0x68d2, [0x0f8c] = 0x5192, [0x0f8d] = 0x7d21,
  [0x0f8e] = 0x80aa, [0x0f8f] = 0x81a8, [0x0f90] = 0x8b00, [0x0f91] = 0x8c8c,
  [0x0f92] = 0x8cbf, [0x0f93] = 0x927e, [0x0f94] = 0x9632, [0x0f95] = 0x5420,
  [0x0f96] = 0x982c, [0x0f97] = 0x5317, [0x0f98] = 0x50d5, [0x0f99] = 0x535c,
  [0x0f9a] = 0x58a8, [0x0f9b] = 0x64b2, [0x0f9c] = 0x6734, [0x0f9d] = 0x7267,
  [0x0f9e] = 0x7766, [0x0f9f] = 0x7a46, [0x0fa0] = 0x91e6, [0x0fa1] = 0x52c3,
  [0x0fa2] = 0x6ca1, [0x0fa3] = 0x6b86, [0x0fa4] = 0x5800, [0x0fa5] = 0x5e4c,
  [0x0fa6] = 0x5954, [0x0fa7] = 0x672c, [0x0fa8] = 0x7ffb, [0x0fa9] = 0x51e1,
  [0x0faa] = 0x76c6, [0x0fab] = 0x6469, [0x0fac] = 0x78e8, [0x0fad] = 0x9b54,
  [0x0fae] = 0x9ebb, [0x0faf] = 0x57cb, [0x0fb0] = 0x59b9, [0x0fb1] = 0x6627,
  [0x0fb2] = 0x679a, [0x0fb3] = 0x6bce, [0x0fb4] = 0x54e9, [0x0fb5] = 0x69d9,
  [0x0fb6] = 0x5e55, [0x0fb7] = 0x819c, [0x0fb8] = 0x6795, [0x0fb9] = 0x9baa,
  [0x0fba] = 0x67fe, [0x0fbb] = 0x9c52, [0x0fbc] = 0x685d, [0x0fbd] = 0x4ea6,
  [0x0fbe] = 0x4fe3, [0x0fbf] = 0x53c8, [0x0fc0] = 0x62b9, [0x0fc1] = 0x672b,
  [0x0fc2] = 0x6cab, [0x0fc3] = 0x8fc4, [0x0fc4] = 0x4fad, [0x0fc5] = 0x7e6d,
  [0x0fc6] = 0x9ebf, [0x0fc7] = 0x4e07, [0x0fc8] = 0x6162, [0x0fc9] = 0x6e80,
  [0x0fca] = 0x6f2b, [0x0fcb] = 0x8513, [0x0fcc] = 0x5473, [0x0fcd] = 0x672a,
  [0x0fce] = 0x9b45, [0x0fcf] = 0x5df3, [0x0fd0] = 0x7b95, [0x0fd1] = 0x5cac,
  [0x0fd2] = 0x5bc6, [0x0fd3] = 0x871c, [0x0fd4] = 0x6e4a, [0x0fd5] = 0x84d1,
  [0x0fd6] = 0x7a14, [0x0fd7] = 0x8108, [0x0fd8] = 0x5999, [0x0fd9] = 0x7c8d,
  [0x0fda] = 0x6c11, [0x0fdb] = 0x7720, [0x0fdc] = 0x52d9, [0x0fdd] = 0x5922,
  [0x0fde] = 0x7121, [0x0fdf] = 0x725f, [0x0fe0] = 0x77db, [0x0fe1] = 0x9727,
  [0x0fe2] = 0x9d61, [0x0fe3] = 0x690b, [0x0fe4] = 0x5a7f, [0x0fe5] = 0x5a18,
  [0x0fe6] = 0x51a5, [0x0fe7] = 0x540d, [0x0fe8] = 0x547d, [0x0fe9] = 0x660e,
  [0x0fea] = 0x76df, [0x0feb] = 0x8ff7, [0x0fec] = 0x9298, [0x0fed] = 0x9cf4,
  [0x0fee] = 0x59ea, [0x0fef] = 0x725d, [0x0ff0] = 0x6ec5, [0x0ff1] = 0x514d,
  [0x0ff2] = 0x68c9, [0x0ff3] = 0x7dbf, [0x0ff4] = 0x7dec, [0x0ff5] = 0x9762,
  [0x0ff6] = 0x9eba, [0x0ff7] = 0x6478, [0x0ff8] = 0x6a21, [0x0ff9] = 0x8302,
  [0x0ffa] = 0x5984, [0x0ffb] = 0x5b5f, [0x0ffc] = 0x6bdb, [0x0ffd] = 0x731b,
  [0x0ffe] = 0x76f2, [0x0fff] = 0x7db2, [0x1000] = 0x8017, [0x1001] = 0x8499,
  [0x1002] = 0x5132, [0x1003] = 0x6728, [0x1004] = 0x9ed9, [0x1005] = 0x76ee,
  [0x1006] = 0x6762, [0x1007] = 0x52ff, [0x1008] = 0x9905, [0x1009] = 0x5c24,
  [0x100a] = 0x623b, [0x100b] = 0x7c7e, [0x100c] = 0x8cb0, [0x100d] = 0x554f,
  [0x100e] = 0x60b6, [0x100f] = 0x7d0b, [0x1010] = 0x9580, [0x1011] = 0x5301,
  [0x1012] = 0x4e5f, [0x1013] = 0x51b6, [0x1014] = 0x591c, [0x1015] = 0x723a,
  [0x1016] = 0x8036, [0x1017] = 0x91ce, [0x1018] = 0x5f25, [0x1019] = 0x77e2,
  [0x101a] = 0x5384, [0x101b] = 0x5f79, [0x101c] = 0x7d04, [0x101d] = 0x85ac,
  [0x101e] = 0x8a33, [0x101f] = 0x8e8d, [0x1020] = 0x9756, [0x1021] = 0x67f3,
  [0x1022] = 0x85ae, [0x1023] = 0x9453, [0x1024] = 0x6109, [0x1025] = 0x6108,
  [0x1026] = 0x6cb9, [0x1027] = 0x7652, [0x1028] = 0x8aed, [0x1029] = 0x8f38,
  [0x102a] = 0x552f, [0x102b] = 0x4f51, [0x102c] = 0x512a, [0x102d] = 0x52c7,
  [0x102e] = 0x53cb, [0x102f] = 0x5ba5, [0x1030] = 0x5e7d, [0x1031] = 0x60a0,
  [0x1032] = 0x6182, [0x1033] = 0x63d6, [0x1034] = 0x6709, [0x1035] = 0x67da,
  [0x1036] = 0x6e67, [0x1037] = 0x6d8c, [0x1038] = 0x7336, [0x1039] = 0x7337,
  [0x103a] = 0x7531, [0x103b] = 0x7950, [0x103c] = 0x88d5, [0x103d] = 0x8a98,
  [0x103e] = 0x904a, [0x103f] = 0x9091, [0x1040] = 0x90f5, [0x1041] = 0x96c4,
  [0x1042] = 0x878d, [0x1043] = 0x5915, [0x1044] = 0x4e88, [0x1045] = 0x4f59,
  [0x1046] = 0x4e0e, [0x1047] = 0x8a89, [0x1048] = 0x8f3f, [0x1049] = 0x9810,
  [0x104a] = 0x50ad, [0x104b] = 0x5e7c, [0x104c] = 0x5996, [0x104d] = 0x5bb9,
  [0x104e] = 0x5eb8, [0x104f] = 0x63da, [0x1050] = 0x63fa, [0x1051] = 0x64c1,
  [0x1052] = 0x66dc, [0x1053] = 0x694a, [0x1054] = 0x69d8, [0x1055] = 0x6d0b,
  [0x1056] = 0x6eb6, [0x1057] = 0x7194, [0x1058] = 0x7528, [0x1059] = 0x7aaf,
  [0x105a] = 0x7f8a, [0x105b] = 0x8000, [0x105c] = 0x8449, [0x105d] = 0x84c9,
  [0x105e] = 0x8981, [0x105f] = 0x8b21, [0x1060] = 0x8e0a, [0x1061] = 0x9065,
  [0x1062] = 0x967d, [0x1063] = 0x990a, [0x1064] = 0x617e, [0x1065] = 0x6291,
  [0x1066] = 0x6b32, [0x1067] = 0x6c83, [0x1068] = 0x6d74, [0x1069] = 0x7fcc,
  [0x106a] = 0x7ffc, [0x106b] = 0x6dc0, [0x106c] = 0x7f85, [0x106d] = 0x87ba,
  [0x106e] = 0x88f8, [0x106f] = 0x6765, [0x1070] = 0x83b1, [0x1071] = 0x983c,
  [0x1072] = 0x96f7, [0x1073] = 0x6d1b, [0x1074] = 0x7d61, [0x1075] = 0x843d,
  [0x1076] = 0x916a, [0x1077] = 0x4e71, [0x1078] = 0x5375, [0x1079] = 0x5d50,
  [0x107a] = 0x6b04, [0x107b] = 0x6feb, [0x107c] = 0x85cd, [0x107d] = 0x862d,
  [0x107e] = 0x89a7, [0x107f] = 0x5229, [0x1080] = 0x540f, [0x1081] = 0x5c65,
  [0x1082] = 0x674e, [0x1083] = 0x68a8, [0x1084] = 0x7406, [0x1085] = 0x7483,
  [0x1086] = 0x75e2, [0x1087] = 0x88cf, [0x1088] = 0x88e1, [0x1089] = 0x91cc,
  [0x108a] = 0x96e2, [0x108b] = 0x9678, [0x108c] = 0x5f8b, [0x108d] = 0x7387,
  [0x108e] = 0x7acb, [0x108f] = 0x844e, [0x1090] = 0x63a0, [0x1091] = 0x7565,
  [0x1092] = 0x5289, [0x1093] = 0x6d41, [0x1094] = 0x6e9c, [0x1095] = 0x7409,
  [0x1096] = 0x7559, [0x1097] = 0x786b, [0x1098] = 0x7c92, [0x1099] = 0x9686,
  [0x109a] = 0x7adc, [0x109b] = 0x9f8d, [0x109c] = 0x4fb6, [0x109d] = 0x616e,
  [0x109e] = 0x65c5, [0x109f] = 0x865c, [0x10a0] = 0x4e86, [0x10a1] = 0x4eae,
  [0x10a2] = 0x50da, [0x10a3] = 0x4e21, [0x10a4] = 0x51cc, [0x10a5] = 0x5bee,
  [0x10a6] = 0x6599, [0x10a7] = 0x6881, [0x10a8] = 0x6dbc, [0x10a9] = 0x731f,
  [0x10aa] = 0x7642, [0x10ab] = 0x77ad, [0x10ac] = 0x7a1c, [0x10ad] = 0x7ce7,
  [0x10ae] = 0x826f, [0x10af] = 0x8ad2, [0x10b0] = 0x907c, [0x10b1] = 0x91cf,
  [0x10b2] = 0x9675, [0x10b3] = 0x9818, [0x10b4] = 0x529b, [0x10b5] = 0x7dd1,
  [0x10b6] = 0x502b, [0x10b7] = 0x5398, [0x10b8] = 0x6797, [0x10b9] = 0x6dcb,
  [0x10ba] = 0x71d0, [0x10bb] = 0x7433, [0x10bc] = 0x81e8, [0x10bd] = 0x8f2a,
  [0x10be] = 0x96a3, [0x10bf] = 0x9c57, [0x10c0] = 0x9e9f, [0x10c1] = 0x7460,
  [0x10c2] = 0x5841, [0x10c3] = 0x6d99, [0x10c4] = 0x7d2f, [0x10c5] = 0x985e,
  [0x10c6] = 0x4ee4, [0x10c7] = 0x4f36, [0x10c8] = 0x4f8b, [0x10c9] = 0x51b7,
  [0x10ca] = 0x52b1, [0x10cb] = 0x5dba, [0x10cc] = 0x601c, [0x10cd] = 0x73b2,
  [0x10ce] = 0x793c, [0x10cf] = 0x82d3, [0x10d0] = 0x9234, [0x10d1] = 0x96b7,
  [0x10d2] = 0x96f6, [0x10d3] = 0x970a, [0x10d4] = 0x9e97, [0x10d5] = 0x9f62,
  [0x10d6] = 0x66a6, [0x10d7] = 0x6b74, [0x10d8] = 0x5217, [0x10d9] = 0x52a3,
  [0x10da] = 0x70c8, [0x10db] = 0x88c2, [0x10dc] = 0x5ec9, [0x10dd] = 0x604b,
  [0x10de] = 0x6190, [0x10df] = 0x6f23, [0x10e0] = 0x7149, [0x10e1] = 0x7c3e,
  [0x10e2] = 0x7df4, [0x10e3] = 0x806f, [0x10e4] = 0x84ee, [0x10e5] = 0x9023,
  [0x10e6] = 0x932c, [0x10e7] = 0x5442, [0x10e8] = 0x9b6f, [0x10e9] = 0x6ad3,
  [0x10ea] = 0x7089, [0x10eb] = 0x8cc2, [0x10ec] = 0x8def, [0x10ed] = 0x9732,
  [0x10ee] = 0x52b4, [0x10ef] = 0x5a41, [0x10f0] = 0x5eca, [0x10f1] = 0x5f04,
  [0x10f2] = 0x6717, [0x10f3] = 0x697c, [0x10f4] = 0x6994, [0x10f5] = 0x6d6a,
  [0x10f6] = 0x6f0f, [0x10f7] = 0x7262, [0x10f8] = 0x72fc, [0x10f9] = 0x7bed,
  [0x10fa] = 0x8001, [0x10fb] = 0x807e, [0x10fc] = 0x874b, [0x10fd] = 0x90ce,
  [0x10fe] = 0x516d, [0x10ff] = 0x9e93, [0x1100] = 0x7984, [0x1101] = 0x808b,
  [0x1102] = 0x9332, [0x1103] = 0x8ad6, [0x1104] = 0x502d, [0x1105] = 0x548c,
  [0x1106] = 0x8a71, [0x1107] = 0x6b6a, [0x1108] = 0x8cc4, [0x1109] = 0x8107,
  [0x110a] = 0x60d1, [0x110b] = 0x67a0, [0x110c] = 0x9df2, [0x110d] = 0x4e99,
  [0x110e] = 0x4e98, [0x110f] = 0x9c10, [0x1110] = 0x8a6b, [0x1111] = 0x85c1,
  [0x1112] = 0x8568, [0x1113] = 0x6900, [0x1114] = 0x6e7e, [0x1115] = 0x7897,
  [0x1116] = 0x8155, [0x1142] = 0x5f0c, [0x1143] = 0x4e10, [0x1144] = 0x4e15,
  [0x1145] = 0x4e2a, [0x1146] = 0x4e31, [0x1147] = 0x4e36, [0x1148] = 0x4e3c,
  [0x1149] = 0x4e3f, [0x114a] = 0x4e42, [0x114b] = 0x4e56, [0x114c] = 0x4e58,
  [0x114d] = 0x4e82, [0x114e] = 0x4e85, [0x114f] = 0x8c6b, [0x1150] = 0x4e8a,
  [0x1151] = 0x8212, [0x1152] = 0x5f0d, [0x1153] = 0x4e8e, [0x1154] = 0x4e9e,
  [0x1155] = 0x4e9f, [0x1156] = 0x4ea0, [0x1157] = 0x4ea2, [0x1158] = 0x4eb0,
  [0x1159] = 0x4eb3, [0x115a] = 0x4eb6, [0x115b] = 0x4ece, [0x115c] = 0x4ecd,
  [0x115d] = 0x4ec4, [0x115e] = 0x4ec6, [0x115f] = 0x4ec2, [0x1160] = 0x4ed7,
  [0x1161] = 0x4ede, [0x1162] = 0x4eed, [0x1163] = 0x4edf, [0x1164] = 0x4ef7,
  [0x1165] = 0x4f09, [0x1166] = 0x4f5a, [0x1167] = 0x4f30, [0x1168] = 0x4f5b,
  [0x1169] = 0x4f5d, [0x116a] = 0x4f57, [0x116b] = 0x4f47, [0x116c] = 0x4f76,
  [0x116d] = 0x4f88, [0x116e] = 0x4f8f, [0x116f] = 0x4f98, [0x1170] = 0x4f7b,
  [0x1171] = 0x4f69, [0x1172] = 0x4f70, [0x1173] = 0x4f91, [0x1174] = 0x4f6f,
  [0x1175] = 0x4f86, [0x1176] = 0x4f96, [0x1177] = 0x5118, [0x1178] = 0x4fd4,
  [0x1179] = 0x4fdf, [0x117a] = 0x4fce, [0x117b] = 0x4fd8, [0x117c] = 0x4fdb,
  [0x117d] = 0x4fd1, [0x117e] = 0x4fda, [0x117f] = 0x4fd0, [0x1180] = 0x4fe4,
  [0x1181] = 0x4fe5, [0x1182] = 0x501a, [0x1183] = 0x5028, [0x1184] = 0x5014,
  [0x1185] = 0x502a, [0x1186] = 0x5025, [0x1187] = 0x5005, [0x1188] = 0x4f1c,
  [0x1189] = 0x4ff6, [0x118a] = 0x5021, [0x118b] = 0x5029, [0x118c] = 0x502c,
  [0x118d] = 0x4ffe, [0x118e] = 0x4fef, [0x118f] = 0x5011, [0x1190] = 0x5006,
  [0x1191] = 0x5043, [0x1192] = 0x5047, [0x1193] = 0x6703, [0x1194] = 0x5055,
  [0x1195] = 0x5050, [0x1196] = 0x5048, [0x1197] = 0x505a, [0x1198] = 0x5056,
  [0x1199] = 0x506c, [0x119a] = 0x5078, [0x119b] = 0x5080, [0x119c] = 0x509a,
  [0x119d] = 0x5085, [0x119e] = 0x50b4, [0x119f] = 0x50b2, [0x11a0] = 0x50c9,
  [0x11a1] = 0x50ca, [0x11a2] = 0x50b3, [0x11a3] = 0x50c2, [0x11a4] = 0x50d6,
  [0x11a5] = 0x50de, [0x11a6] = 0x50e5, [0x11a7] = 0x50ed, [0x11a8] = 0x50e3,
  [0x11a9] = 0x50ee, [0x11aa] = 0x50f9, [0x11ab] = 0x50f5, [0x11ac] = 0x5109,
  [0x11ad] = 0x5101, [0x11ae] = 0x5102, [0x11af] = 0x5116, [0x11b0] = 0x5115,
  [0x11b1] = 0x5114, [0x11b2] = 0x511a, [0x11b3] = 0x5121, [0x11b4] = 0x513a,
  [0x11b5] = 0x5137, [0x11b6] = 0x513c, [0x11b7] = 0x513b, [0x11b8] = 0x513f,
  [0x11b9] = 0x5140, [0x11ba] = 0x5152, [0x11bb] = 0x514c, [0x11bc] = 0x5154,
  [0x11bd] = 0x5162, [0x11be] = 0x7af8, [0x11bf] = 0x5169, [0x11c0] = 0x516a,
  [0x11c1] = 0x516e, [0x11c2] = 0x5180, [0x11c3] = 0x5182, [0x11c4] = 0x56d8,
  [0x11c5] = 0x518c, [0x11c6] = 0x5189, [0x11c7] = 0x518f, [0x11c8] = 0x5191,
  [0x11c9] = 0x5193, [0x11ca] = 0x5195, [0x11cb] = 0x5196, [0x11cc] = 0x51a4,
  [0x11cd] = 0x51a6, [0x11ce] = 0x51a2, [0x11cf] = 0x51a9, [0x11d0] = 0x51aa,
  [0x11d1] = 0x51ab, [0x11d2] = 0x51b3, [0x11d3] = 0x51b1, [0x11d4] = 0x51b2,
  [0x11d5] = 0x51b0, [0x11d6] = 0x51b5, [0x11d7] = 0x51bd, [0x11d8] = 0x51c5,
  [0x11d9] = 0x51c9, [0x11da] = 0x51db, [0x11db] = 0x51e0, [0x11dc] = 0x8655,
  [0x11dd] = 0x51e9, [0x11de] = 0x51ed, [0x11df] = 0x51f0, [0x11e0] = 0x51f5,
  [0x11e1] = 0x51fe, [0x11e2] = 0x5204, [0x11e3] = 0x520b, [0x11e4] = 0x5214,
  [0x11e5] = 0x520e, [0x11e6] = 0x5227, [0x11e7] = 0x522a, [0x11e8] = 0x522e,
  [0x11e9] = 0x5233, [0x11ea] = 0x5239, [0x11eb] = 0x524f, [0x11ec] = 0x5244,
  [0x11ed] = 0x524b, [0x11ee] = 0x524c, [0x11ef] = 0x525e, [0x11f0] = 0x5254,
  [0x11f1] = 0x526a, [0x11f2] = 0x5274, [0x11f3] = 0x5269, [0x11f4] = 0x5273,
  [0x11f5] = 0x527f, [0x11f6] = 0x527d, [0x11f7] = 0x528d, [0x11f8] = 0x5294,
  [0x11f9] = 0x5292, [0x11fa] = 0x5271, [0x11fb] = 0x5288, [0x11fc] = 0x5291,
  [0x11fd] = 0x8fa8, [0x11fe] = 0x8fa7, [0x11ff] = 0x52ac, [0x1200] = 0x52ad,
  [0x1201] = 0x52bc, [0x1202] = 0x52b5, [0x1203] = 0x52c1, [0x1204] = 0x52cd,
  [0x1205] = 0x52d7, [0x1206] = 0x52de, [0x1207] = 0x52e3, [0x1208] = 0x52e6,
  [0x1209] = 0x98ed, [0x120a] = 0x52e0, [0x120b] = 0x52f3, [0x120c] = 0x52f5,
  [0x120d] = 0x52f8, [0x120e] = 0x52f9, [0x120f] = 0x5306, [0x1210] = 0x5308,
  [0x1211] = 0x7538, [0x1212] = 0x530d, [0x1213] = 0x5310, [0x1214] = 0x530f,
  [0x1215] = 0x5315, [0x1216] = 0x531a, [0x1217] = 0x5323, [0x1218] = 0x532f,
  [0x1219] = 0x5331, [0x121a] = 0x5333, [0x121b] = 0x5338, [0x121c] = 0x5340,
  [0x121d] = 0x5346, [0x121e] = 0x5345, [0x121f] = 0x4e17, [0x1220] = 0x5349,
  [0x1221] = 0x534d, [0x1222] = 0x51d6, [0x1223] = 0x535e, [0x1224] = 0x5369,
  [0x1225] = 0x536e, [0x1226] = 0x5918, [0x1227] = 0x537b, [0x1228] = 0x5377,
  [0x1229] = 0x5382, [0x122a] = 0x5396, [0x122b] = 0x53a0, [0x122c] = 0x53a6,
  [0x122d] = 0x53a5, [0x122e] = 0x53ae, [0x122f] = 0x53b0, [0x1230] = 0x53b6,
  [0x1231] = 0x53c3, [0x1232] = 0x7c12, [0x1233] = 0x96d9, [0x1234] = 0x53df,
  [0x1235] = 0x66fc, [0x1236] = 0x71ee, [0x1237] = 0x53ee, [0x1238] = 0x53e8,
  [0x1239] = 0x53ed, [0x123a] = 0x53fa, [0x123b] = 0x5401, [0x123c] = 0x543d,
  [0x123d] = 0x5440, [0x123e] = 0x542c, [0x123f] = 0x542d, [0x1240] = 0x543c,
  [0x1241] = 0x542e, [0x1242] = 0x5436, [0x1243] = 0x5429, [0x1244] = 0x541d,
  [0x1245] = 0x544e, [0x1246] = 0x548f, [0x1247] = 0x5475, [0x1248] = 0x548e,
  [0x1249] = 0x545f, [0x124a] = 0x5471, [0x124b] = 0x5477, [0x124c] = 0x5470,
  [0x124d] = 0x5492, [0x124e] = 0x547b, [0x124f] = 0x5480, [0x1250] = 0x5476,
  [0x1251] = 0x5484, [0x1252] = 0x5490, [0x1253] = 0x5486, [0x1254] = 0x54c7,
  [0x1255] = 0x54a2, [0x1256] = 0x54b8, [0x1257] = 0x54a5, [0x1258] = 0x54ac,
  [0x1259] = 0x54c4, [0x125a] = 0x54c8, [0x125b] = 0x54a8, [0x125c] = 0x54ab,
  [0x125d] = 0x54c2, [0x125e] = 0x54a4, [0x125f] = 0x54be, [0x1260] = 0x54bc,
  [0x1261] = 0x54d8, [0x1262] = 0x54e5, [0x1263] = 0x54e6, [0x1264] = 0x550f,
  [0x1265] = 0x5514, [0x1266] = 0x54fd, [0x1267] = 0x54ee, [0x1268] = 0x54ed,
  [0x1269] = 0x54fa, [0x126a] = 0x54e2, [0x126b] = 0x5539, [0x126c] = 0x5540,
  [0x126d] = 0x5563, [0x126e] = 0x554c, [0x126f] = 0x552e, [0x1270] = 0x555c,
  [0x1271] = 0x5545, [0x1272] = 0x5556, [0x1273] = 0x5557, [0x1274] = 0x5538,
  [0x1275] = 0x5533, [0x1276] = 0x555d, [0x1277] = 0x5599, [0x1278] = 0x5580,
  [0x1279] = 0x54af, [0x127a] = 0x558a, [0x127b] = 0x559f, [0x127c] = 0x557b,
  [0x127d] = 0x557e, [0x127e] = 0x5598, [0x127f] = 0x559e, [0x1280] = 0x55ae,
  [0x1281] = 0x557c, [0x1282] = 0x5583, [0x1283] = 0x55a9, [0x1284] = 0x5587,
  [0x1285] = 0x55a8, [0x1286] = 0x55da, [0x1287] = 0x55c5, [0x1288] = 0x55df,
  [0x1289] = 0x55c4, [0x128a] = 0x55dc, [0x128b] = 0x55e4, [0x128c] = 0x55d4,
  [0x128d] = 0x5614, [0x128e] = 0x55f7, [0x128f] = 0x5616, [0x1290] = 0x55fe,
  [0x1291] = 0x55fd, [0x1292] = 0x561b, [0x1293] = 0x55f9, [0x1294] = 0x564e,
  [0x1295] = 0x5650, [0x1296] = 0x71df, [0x1297] = 0x5634, [0x1298] = 0x5636,
  [0x1299] = 0x5632, [0x129a] = 0x5638, [0x129b] = 0x566b, [0x129c] = 0x5664,
  [0x129d] = 0x562f, [0x129e] = 0x566c, [0x129f] = 0x566a, [0x12a0] = 0x5686,
  [0x12a1] = 0x5680, [0x12a2] = 0x568a, [0x12a3] = 0x56a0, [0x12a4] = 0x5694,
  [0x12a5] = 0x568f, [0x12a6] = 0x56a5, [0x12a7] = 0x56ae, [0x12a8] = 0x56b6,
  [0x12a9] = 0x56b4, [0x12aa] = 0x56c2, [0x12ab] = 0x56bc, [0x12ac] = 0x56c1,
  [0x12ad] = 0x56c3, [0x12ae] = 0x56c0, [0x12af] = 0x56c8, [0x12b0] = 0x56ce,
  [0x12b1] = 0x56d1, [0x12b2] = 0x56d3, [0x12b3] = 0x56d7, [0x12b4] = 0x56ee,
  [0x12b5] = 0x56f9, [0x12b6] = 0x5700, [0x12b7] = 0x56ff, [0x12b8] = 0x5704,
  [0x12b9] = 0x5709, [0x12ba] = 0x5708, [0x12bb] = 0x570b, [0x12bc] = 0x570d,
  [0x12bd] = 0x5713, [0x12be] = 0x5718, [0x12bf] = 0x5716, [0x12c0] = 0x55c7,
  [0x12c1] = 0x571c, [0x12c2] = 0x5726, [0x12c3] = 0x5737, [0x12c4] = 0x5738,
  [0x12c5] = 0x574e, [0x12c6] = 0x573b, [0x12c7] = 0x5740, [0x12c8] = 0x574f,
  [0x12c9] = 0x5769, [0x12ca] = 0x57c0, [0x12cb] = 0x5788, [0x12cc] = 0x5761,
  [0x12cd] = 0x577f, [0x12ce] = 0x5789, [0x12cf] = 0x5793, [0x12d0] = 0x57a0,
  [0x12d1] = 0x57b3, [0x12d2] = 0x57a4, [0x12d3] = 0x57aa, [0x12d4] = 0x57b0,
  [0x12d5] = 0x57c3, [0x12d6] = 0x57c6, [0x12d7] = 0x57d4, [0x12d8] = 0x57d2,
  [0x12d9] = 0x57d3, [0x12da] = 0x580a, [0x12db] = 0x57d6, [0x12dc] = 0x57e3,
  [0x12dd] = 0x580b, [0x12de] = 0x5819, [0x12df] = 0x581d, [0x12e0] = 0x5872,
  [0x12e1] = 0x5821, [0x12e2] = 0x5862, [0x12e3] = 0x584b, [0x12e4] = 0x5870,
  [0x12e5] = 0x6bc0, [0x12e6] = 0x5852, [0x12e7] = 0x583d, [0x12e8] = 0x5879,
  [0x12e9] = 0x5885, [0x12ea] = 0x58b9, [0x12eb] = 0x589f, [0x12ec] = 0x58ab,
  [0x12ed] = 0x58ba, [0x12ee] = 0x58de, [0x12ef] = 0x58bb, [0x12f0] = 0x58b8,
  [0x12f1] = 0x58ae, [0x12f2] = 0x58c5, [0x12f3] = 0x58d3, [0x12f4] = 0x58d1,
  [0x12f5] = 0x58d7, [0x12f6] = 0x58d9, [0x12f7] = 0x58d8, [0x12f8] = 0x58e5,
  [0x12f9] = 0x58dc, [0x12fa] = 0x58e4, [0x12fb] = 0x58df, [0x12fc] = 0x58ef,
  [0x12fd] = 0x58fa, [0x12fe] = 0x58f9, [0x12ff] = 0x58fb, [0x1300] = 0x58fc,
  [0x1301] = 0x58fd, [0x1302] = 0x5902, [0x1303] = 0x590a, [0x1304] = 0x5910,
  [0x1305] = 0x591b, [0x1306] = 0x68a6, [0x1307] = 0x5925, [0x1308] = 0x592c,
  [0x1309] = 0x592d, [0x130a] = 0x5932, [0x130b] = 0x5938, [0x130c] = 0x593e,
  [0x130d] = 0x7ad2, [0x130e] = 0x5955, [0x130f] = 0x5950, [0x1310] = 0x594e,
  [0x1311] = 0x595a, [0x1312] = 0x5958, [0x1313] = 0x5962, [0x1314] = 0x5960,
  [0x1315] = 0x5967, [0x1316] = 0x596c, [0x1317] = 0x5969, [0x1318] = 0x5978,
  [0x1319] = 0x5981, [0x131a] = 0x599d, [0x131b] = 0x4f5e, [0x131c] = 0x4fab,
  [0x131d] = 0x59a3, [0x131e] = 0x59b2, [0x131f] = 0x59c6, [0x1320] = 0x59e8,
  [0x1321] = 0x59dc, [0x1322] = 0x598d, [0x1323] = 0x59d9, [0x1324] = 0x59da,
  [0x1325] = 0x5a25, [0x1326] = 0x5a1f, [0x1327] = 0x5a11, [0x1328] = 0x5a1c,
  [0x1329] = 0x5a09, [0x132a] = 0x5a1a, [0x132b] = 0x5a40, [0x132c] = 0x5a6c,
  [0x132d] = 0x5a49, [0x132e] = 0x5a35, [0x132f] = 0x5a36, [0x1330] = 0x5a62,
  [0x1331] = 0x5a6a, [0x1332] = 0x5a9a, [0x1333] = 0x5abc, [0x1334] = 0x5abe,
  [0x1335] = 0x5acb, [0x1336] = 0x5ac2, [0x1337] = 0x5abd, [0x1338] = 0x5ae3,
  [0x1339] = 0x5ad7, [0x133a] = 0x5ae6, [0x133b] = 0x5ae9, [0x133c] = 0x5ad6,
  [0x133d] = 0x5afa, [0x133e] = 0x5afb, [0x133f] = 0x5b0c, [0x1340] = 0x5b0b,
  [0x1341] = 0x5b16, [0x1342] = 0x5b32, [0x1343] = 0x5ad0, [0x1344] = 0x5b2a,
  [0x1345] = 0x5b36, [0x1346] = 0x5b3e, [0x1347] = 0x5b43, [0x1348] = 0x5b45,
  [0x1349] = 0x5b40, [0x134a] = 0x5b51, [0x134b] = 0x5b55, [0x134c] = 0x5b5a,
  [0x134d] = 0x5b5b, [0x134e] = 0x5b65, [0x134f] = 0x5b69, [0x1350] = 0x5b70,
  [0x1351] = 0x5b73, [0x1352] = 0x5b75, [0x1353] = 0x5b78, [0x1354] = 0x6588,
  [0x1355] = 0x5b7a, [0x1356] = 0x5b80, [0x1357] = 0x5b83, [0x1358] = 0x5ba6,
  [0x1359] = 0x5bb8, [0x135a] = 0x5bc3, [0x135b] = 0x5bc7, [0x135c] = 0x5bc9,
  [0x135d] = 0x5bd4, [0x135e] = 0x5bd0, [0x135f] = 0x5be4, [0x1360] = 0x5be6,
  [0x1361] = 0x5be2, [0x1362] = 0x5bde, [0x1363] = 0x5be5, [0x1364] = 0x5beb,
  [0x1365] = 0x5bf0, [0x1366] = 0x5bf6, [0x1367] = 0x5bf3, [0x1368] = 0x5c05,
  [0x1369] = 0x5c07, [0x136a] = 0x5c08, [0x136b] = 0x5c0d, [0x136c] = 0x5c13,
  [0x136d] = 0x5c20, [0x136e] = 0x5c22, [0x136f] = 0x5c28, [0x1370] = 0x5c38,
  [0x1371] = 0x5c39, [0x1372] = 0x5c41, [0x1373] = 0x5c46, [0x1374] = 0x5c4e,
  [0x1375] = 0x5c53, [0x1376] = 0x5c50, [0x1377] = 0x5c4f, [0x1378] = 0x5b71,
  [0x1379] = 0x5c6c, [0x137a] = 0x5c6e, [0x137b] = 0x4e62, [0x137c] = 0x5c76,
  [0x137d] = 0x5c79, [0x137e] = 0x5c8c, [0x137f] = 0x5c91, [0x1380] = 0x5c94,
  [0x1381] = 0x599b, [0x1382] = 0x5cab, [0x1383] = 0x5cbb, [0x1384] = 0x5cb6,
  [0x1385] = 0x5cbc, [0x1386] = 0x5cb7, [0x1387] = 0x5cc5, [0x1388] = 0x5cbe,
  [0x1389] = 0x5cc7, [0x138a] = 0x5cd9, [0x138b] = 0x5ce9, [0x138c] = 0x5cfd,
  [0x138d] = 0x5cfa, [0x138e] = 0x5ced, [0x138f] = 0x5d8c, [0x1390] = 0x5cea,
  [0x1391] = 0x5d0b, [0x1392] = 0x5d15, [0x1393] = 0x5d17, [0x1394] = 0x5d5c,
  [0x1395] = 0x5d1f, [0x1396] = 0x5d1b, [0x1397] = 0x5d11, [0x1398] = 0x5d14,
  [0x1399] = 0x5d22, [0x139a] = 0x5d1a, [0x139b] = 0x5d19, [0x139c] = 0x5d18,
  [0x139d] = 0x5d4c, [0x139e] = 0x5d52, [0x139f] = 0x5d4e, [0x13a0] = 0x5d4b,
  [0x13a1] = 0x5d6c, [0x13a2] = 0x5d73, [0x13a3] = 0x5d76, [0x13a4] = 0x5d87,
  [0x13a5] = 0x5d84, [0x13a6] = 0x5d82, [0x13a7] = 0x5da2, [0x13a8] = 0x5d9d,
  [0x13a9] = 0x5dac, [0x13aa] = 0x5dae, [0x13ab] = 0x5dbd, [0x13ac] = 0x5d90,
  [0x13ad] = 0x5db7, [0x13ae] = 0x5dbc, [0x13af] = 0x5dc9, [0x13b0] = 0x5dcd,
  [0x13b1] = 0x5dd3, [0x13b2] = 0x5dd2, [0x13b3] = 0x5dd6, [0x13b4] = 0x5ddb,
  [0x13b5] = 0x5deb, [0x13b6] = 0x5df2, [0x13b7] = 0x5df5, [0x13b8] = 0x5e0b,
  [0x13b9] = 0x5e1a, [0x13ba] = 0x5e19, [0x13bb] = 0x5e11, [0x13bc] = 0x5e1b,
  [0x13bd] = 0x5e36, [0x13be] = 0x5e37, [0x13bf] = 0x5e44, [0x13c0] = 0x5e43,
  [0x13c1] = 0x5e40, [0x13c2] = 0x5e4e, [0x13c3] = 0x5e57, [0x13c4] = 0x5e54,
  [0x13c5] = 0x5e5f, [0x13c6] = 0x5e62, [0x13c7] = 0x5e64, [0x13c8] = 0x5e47,
  [0x13c9] = 0x5e75, [0x13ca] = 0x5e76, [0x13cb] = 0x5e7a, [0x13cc] = 0x9ebc,
  [0x13cd] = 0x5e7f, [0x13ce] = 0x5ea0, [0x13cf] = 0x5ec1, [0x13d0] = 0x5ec2,
  [0x13d1] = 0x5ec8, [0x13d2] = 0x5ed0, [0x13d3] = 0x5ecf, [0x13d4] = 0x5ed6,
  [0x13d5] = 0x5ee3, [0x13d6] = 0x5edd, [0x13d7] = 0x5eda, [0x13d8] = 0x5edb,
  [0x13d9] = 0x5ee2, [0x13da] = 0x5ee1, [0x13db] = 0x5ee8, [0x13dc] = 0x5ee9,
  [0x13dd] = 0x5eec, [0x13de] = 0x5ef1, [0x13df] = 0x5ef3, [0x13e0] = 0x5ef0,
  [0x13e1] = 0x5ef4, [0x13e2] = 0x5ef8, [0x13e3] = 0x5efe, [0x13e4] = 0x5f03,
  [0x13e5] = 0x5f09, [0x13e6] = 0x5f5d, [0x13e7] = 0x5f5c, [0x13e8] = 0x5f0b,
  [0x13e9] = 0x5f11, [0x13ea] = 0x5f16, [0x13eb] = 0x5f29, [0x13ec] = 0x5f2d,
  [0x13ed] = 0x5f38, [0x13ee] = 0x5f41, [0x13ef] = 0x5f48, [0x13f0] = 0x5f4c,
  [0x13f1] = 0x5f4e, [0x13f2] = 0x5f2f, [0x13f3] = 0x5f51, [0x13f4] = 0x5f56,
  [0x13f5] = 0x5f57, [0x13f6] = 0x5f59, [0x13f7] = 0x5f61, [0x13f8] = 0x5f6d,
  [0x13f9] = 0x5f73, [0x13fa] = 0x5f77, [0x13fb] = 0x5f83, [0x13fc] = 0x5f82,
  [0x13fd] = 0x5f7f, [0x13fe] = 0x5f8a, [0x13ff] = 0x5f88, [0x1400] = 0x5f91,
  [0x1401] = 0x5f87, [0x1402] = 0x5f9e, [0x1403] = 0x5f99, [0x1404] = 0x5f98,
  [0x1405] = 0x5fa0, [0x1406] = 0x5fa8, [0x1407] = 0x5fad, [0x1408] = 0x5fbc,
  [0x1409] = 0x5fd6, [0x140a] = 0x5ffb, [0x140b] = 0x5fe4, [0x140c] = 0x5ff8,
  [0x140d] = 0x5ff1, [0x140e] = 0x5fdd, [0x140f] = 0x60b3, [0x1410] = 0x5fff,
  [0x1411] = 0x6021, [0x1412] = 0x6060, [0x1413] = 0x6019, [0x1414] = 0x6010,
  [0x1415] = 0x6029, [0x1416] = 0x600e, [0x1417] = 0x6031, [0x1418] = 0x601b,
  [0x1419] = 0x6015, [0x141a] = 0x602b, [0x141b] = 0x6026, [0x141c] = 0x600f,
  [0x141d] = 0x603a, [0x141e] = 0x605a, [0x141f] = 0x6041, [0x1420] = 0x606a,
  [0x1421] = 0x6077, [0x1422] = 0x605f, [0x1423] = 0x604a, [0x1424] = 0x6046,
  [0x1425] = 0x604d, [0x1426] = 0x6063, [0x1427] = 0x6043, [0x1428] = 0x6064,
  [0x1429] = 0x6042, [0x142a] = 0x606c, [0x142b] = 0x606b, [0x142c] = 0x6059,
  [0x142d] = 0x6081, [0x142e] = 0x608d, [0x142f] = 0x60e7, [0x1430] = 0x6083,
  [0x1431] = 0x609a, [0x1432] = 0x6084, [0x1433] = 0x609b, [0x1434] = 0x6096,
  [0x1435] = 0x6097, [0x1436] = 0x6092, [0x1437] = 0x60a7, [0x1438] = 0x608b,
  [0x1439] = 0x60e1, [0x143a] = 0x60b8, [0x143b] = 0x60e0, [0x143c] = 0x60d3,
  [0x143d] = 0x60b4, [0x143e] = 0x5ff0, [0x143f] = 0x60bd, [0x1440] = 0x60c6,
  [0x1441] = 0x60b5, [0x1442] = 0x60d8, [0x1443] = 0x614d, [0x1444] = 0x6115,
  [0x1445] = 0x6106, [0x1446] = 0x60f6, [0x1447] = 0x60f7, [0x1448] = 0x6100,
  [0x1449] = 0x60f4, [0x144a] = 0x60fa, [0x144b] = 0x6103, [0x144c] = 0x6121,
  [0x144d] = 0x60fb, [0x144e] = 0x60f1, [0x144f] = 0x610d, [0x1450] = 0x610e,
  [0x1451] = 0x6147, [0x1452] = 0x613e, [0x1453] = 0x6128, [0x1454] = 0x6127,
  [0x1455] = 0x614a, [0x1456] = 0x613f, [0x1457] = 0x613c, [0x1458] = 0x612c,
  [0x1459] = 0x6134, [0x145a] = 0x613d, [0x145b] = 0x6142, [0x145c] = 0x6144,
  [0x145d] = 0x6173, [0x145e] = 0x6177, [0x145f] = 0x6158, [0x1460] = 0x6159,
  [0x1461] = 0x615a, [0x1462] = 0x616b, [0x1463] = 0x6174, [0x1464] = 0x616f,
  [0x1465] = 0x6165, [0x1466] = 0x6171, [0x1467] = 0x615f, [0x1468] = 0x615d,
  [0x1469] = 0x6153, [0x146a] = 0x6175, [0x146b] = 0x6199, [0x146c] = 0x6196,
  [0x146d] = 0x6187, [0x146e] = 0x61ac, [0x146f] = 0x6194, [0x1470] = 0x619a,
  [0x1471] = 0x618a, [0x1472] = 0x6191, [0x1473] = 0x61ab, [0x1474] = 0x61ae,
  [0x1475] = 0x61cc, [0x1476] = 0x61ca, [0x1477] = 0x61c9, [0x1478] = 0x61f7,
  [0x1479] = 0x61c8, [0x147a] = 0x61c3, [0x147b] = 0x61c6, [0x147c] = 0x61ba,
  [0x147d] = 0x61cb, [0x147e] = 0x7f79, [0x147f] = 0x61cd, [0x1480] = 0x61e6,
  [0x1481] = 0x61e3, [0x1482] = 0x61f6, [0x1483] = 0x61fa, [0x1484] = 0x61f4,
  [0x1485] = 0x61ff, [0x1486] = 0x61fd, [0x1487] = 0x61fc, [0x1488] = 0x61fe,
  [0x1489] = 0x6200, [0x148a] = 0x6208, [0x148b] = 0x6209, [0x148c] = 0x620d,
  [0x148d] = 0x620c, [0x148e] = 0x6214, [0x148f] = 0x621b, [0x1490] = 0x621e,
  [0x1491] = 0x6221, [0x1492] = 0x622a, [0x1493] = 0x622e, [0x1494] = 0x6230,
  [0x1495] = 0x6232, [0x1496] = 0x6233, [0x1497] = 0x6241, [0x1498] = 0x624e,
  [0x1499] = 0x625e, [0x149a] = 0x6263, [0x149b] = 0x625b, [0x149c] = 0x6260,
  [0x149d] = 0x6268, [0x149e] = 0x627c, [0x149f] = 0x6282, [0x14a0] = 0x6289,
  [0x14a1] = 0x627e, [0x14a2] = 0x6292, [0x14a3] = 0x6293, [0x14a4] = 0x6296,
  [0x14a5] = 0x62d4, [0x14a6] = 0x6283, [0x14a7] = 0x6294, [0x14a8] = 0x62d7,
  [0x14a9] = 0x62d1, [0x14aa] = 0x62bb, [0x14ab] = 0x62cf, [0x14ac] = 0x62ff,
  [0x14ad] = 0x62c6, [0x14ae] = 0x64d4, [0x14af] = 0x62c8, [0x14b0] = 0x62dc,
  [0x14b1] = 0x62cc, [0x14b2] = 0x62ca, [0x14b3] = 0x62c2, [0x14b4] = 0x62c7,
  [0x14b5] = 0x629b, [0x14b6] = 0x62c9, [0x14b7] = 0x630c, [0x14b8] = 0x62ee,
  [0x14b9] = 0x62f1, [0x14ba] = 0x6327, [0x14bb] = 0x6302, [0x14bc] = 0x6308,
  [0x14bd] = 0x62ef, [0x14be] = 0x62f5, [0x14bf] = 0x6350, [0x14c0] = 0x633e,
  [0x14c1] = 0x634d, [0x14c2] = 0x641c, [0x14c3] = 0x634f, [0x14c4] = 0x6396,
  [0x14c5] = 0x638e, [0x14c6] = 0x6380, [0x14c7] = 0x63ab, [0x14c8] = 0x6376,
  [0x14c9] = 0x63a3, [0x14ca] = 0x638f, [0x14cb] = 0x6389, [0x14cc] = 0x639f,
  [0x14cd] = 0x63b5, [0x14ce] = 0x636b, [0x14cf] = 0x6369, [0x14d0] = 0x63be,
  [0x14d1] = 0x63e9, [0x14d2] = 0x63c0, [0x14d3] = 0x63c6, [0x14d4] = 0x63e3,
  [0x14d5] = 0x63c9, [0x14d6] = 0x63d2, [0x14d7] = 0x63f6, [0x14d8] = 0x63c4,
  [0x14d9] = 0x6416, [0x14da] = 0x6434, [0x14db] = 0x6406, [0x14dc] = 0x6413,
  [0x14dd] = 0x6426, [0x14de] = 0x6436, [0x14df] = 0x651d, [0x14e0] = 0x6417,
  [0x14e1] = 0x6428, [0x14e2] = 0x640f, [0x14e3] = 0x6467, [0x14e4] = 0x646f,
  [0x14e5] = 0x6476, [0x14e6] = 0x644e, [0x14e7] = 0x652a, [0x14e8] = 0x6495,
  [0x14e9] = 0x6493, [0x14ea] = 0x64a5, [0x14eb] = 0x64a9, [0x14ec] = 0x6488,
  [0x14ed] = 0x64bc, [0x14ee] = 0x64da, [0x14ef] = 0x64d2, [0x14f0] = 0x64c5,
  [0x14f1] = 0x64c7, [0x14f2] = 0x64bb, [0x14f3] = 0x64d8, [0x14f4] = 0x64c2,
  [0x14f5] = 0x64f1, [0x14f6] = 0x64e7, [0x14f7] = 0x8209, [0x14f8] = 0x64e0,
  [0x14f9] = 0x64e1, [0x14fa] = 0x62ac, [0x14fb] = 0x64e3, [0x14fc] = 0x64ef,
  [0x14fd] = 0x652c, [0x14fe] = 0x64f6, [0x14ff] = 0x64f4, [0x1500] = 0x64f2,
  [0x1501] = 0x64fa, [0x1502] = 0x6500, [0x1503] = 0x64fd, [0x1504] = 0x6518,
  [0x1505] = 0x651c, [0x1506] = 0x6505, [0x1507] = 0x6524, [0x1508] = 0x6523,
  [0x1509] = 0x652b, [0x150a] = 0x6534, [0x150b] = 0x6535, [0x150c] = 0x6537,
  [0x150d] = 0x6536, [0x150e] = 0x6538, [0x150f] = 0x754b, [0x1510] = 0x6548,
  [0x1511] = 0x6556, [0x1512] = 0x6555, [0x1513] = 0x654d, [0x1514] = 0x6558,
  [0x1515] = 0x655e, [0x1516] = 0x655d, [0x1517] = 0x6572, [0x1518] = 0x6578,
  [0x1519] = 0x6582, [0x151a] = 0x6583, [0x151b] = 0x8b8a, [0x151c] = 0x659b,
  [0x151d] = 0x659f, [0x151e] = 0x65ab, [0x151f] = 0x65b7, [0x1520] = 0x65c3,
  [0x1521] = 0x65c6, [0x1522] = 0x65c1, [0x1523] = 0x65c4, [0x1524] = 0x65cc,
  [0x1525] = 0x65d2, [0x1526] = 0x65db, [0x1527] = 0x65d9, [0x1528] = 0x65e0,
  [0x1529] = 0x65e1, [0x152a] = 0x65f1, [0x152b] = 0x6772, [0x152c] = 0x660a,
  [0x152d] = 0x6603, [0x152e] = 0x65fb, [0x152f] = 0x6773, [0x1530] = 0x6635,
  [0x1531] = 0x6636, [0x1532] = 0x6634, [0x1533] = 0x661c, [0x1534] = 0x664f,
  [0x1535] = 0x6644, [0x1536] = 0x6649, [0x1537] = 0x6641, [0x1538] = 0x665e,
  [0x1539] = 0x665d, [0x153a] = 0x6664, [0x153b] = 0x6667, [0x153c] = 0x6668,
  [0x153d] = 0x665f, [0x153e] = 0x6662, [0x153f] = 0x6670, [0x1540] = 0x6683,
  [0x1541] = 0x6688, [0x1542] = 0x668e, [0x1543] = 0x6689, [0x1544] = 0x6684,
  [0x1545] = 0x6698, [0x1546] = 0x669d, [0x1547] = 0x66c1, [0x1548] = 0x66b9,
  [0x1549] = 0x66c9, [0x154a] = 0x66be, [0x154b] = 0x66bc, [0x154c] = 0x66c4,
  [0x154d] = 0x66b8, [0x154e] = 0x66d6, [0x154f] = 0x66da, [0x1550] = 0x66e0,
  [0x1551] = 0x663f, [0x1552] = 0x66e6, [0x1553] = 0x66e9, [0x1554] = 0x66f0,
  [0x1555] = 0x66f5, [0x1556] = 0x66f7, [0x1557] = 0x670f, [0x1558] = 0x6716,
  [0x1559] = 0x671e, [0x155a] = 0x6726, [0x155b] = 0x6727, [0x155c] = 0x9738,
  [0x155d] = 0x672e, [0x155e] = 0x673f, [0x155f] = 0x6736, [0x1560] = 0x6741,
  [0x1561] = 0x6738, [0x1562] = 0x6737, [0x1563] = 0x6746, [0x1564] = 0x675e,
  [0x1565] = 0x6760, [0x1566] = 0x6759, [0x1567] = 0x6763, [0x1568] = 0x6764,
  [0x1569] = 0x6789, [0x156a] = 0x6770, [0x156b] = 0x67a9, [0x156c] = 0x677c,
  [0x156d] = 0x676a, [0x156e] = 0x678c, [0x156f] = 0x678b, [0x1570] = 0x67a6,
  [0x1571] = 0x67a1, [0x1572] = 0x6785, [0x1573] = 0x67b7, [0x1574] = 0x67ef,
  [0x1575] = 0x67b4, [0x1576] = 0x67ec, [0x1577] = 0x67b3, [0x1578] = 0x67e9,
  [0x1579] = 0x67b8, [0x157a] = 0x67e4, [0x157b] = 0x67de, [0x157c] = 0x67dd,
  [0x157d] = 0x67e2, [0x157e] = 0x67ee, [0x157f] = 0x67b9, [0x1580] = 0x67ce,
  [0x1581] = 0x67c6, [0x1582] = 0x67e7, [0x1583] = 0x6a9c, [0x1584] = 0x681e,
  [0x1585] = 0x6846, [0x1586] = 0x6829, [0x1587] = 0x6840, [0x1588] = 0x684d,
  [0x1589] = 0x6832, [0x158a] = 0x684e, [0x158b] = 0x68b3, [0x158c] = 0x682b,
  [0x158d] = 0x6859, [0x158e] = 0x6863, [0x158f] = 0x6877, [0x1590] = 0x687f,
  [0x1591] = 0x689f, [0x1592] = 0x688f, [0x1593] = 0x68ad, [0x1594] = 0x6894,
  [0x1595] = 0x689d, [0x1596] = 0x689b, [0x1597] = 0x6883, [0x1598] = 0x6aae,
  [0x1599] = 0x68b9, [0x159a] = 0x6874, [0x159b] = 0x68b5, [0x159c] = 0x68a0,
  [0x159d] = 0x68ba, [0x159e] = 0x690f, [0x159f] = 0x688d, [0x15a0] = 0x687e,
  [0x15a1] = 0x6901, [0x15a2] = 0x68ca, [0x15a3] = 0x6908, [0x15a4] = 0x68d8,
  [0x15a5] = 0x6922, [0x15a6] = 0x6926, [0x15a7] = 0x68e1, [0x15a8] = 0x690c,
  [0x15a9] = 0x68cd, [0x15aa] = 0x68d4, [0x15ab] = 0x68e7, [0x15ac] = 0x68d5,
  [0x15ad] = 0x6936, [0x15ae] = 0x6912, [0x15af] = 0x6904, [0x15b0] = 0x68d7,
  [0x15b1] = 0x68e3, [0x15b2] = 0x6925, [0x15b3] = 0x68f9, [0x15b4] = 0x68e0,
  [0x15b5] = 0x68ef, [0x15b6] = 0x6928, [0x15b7] = 0x692a, [0x15b8] = 0x691a,
  [0x15b9] = 0x6923, [0x15ba] = 0x6921, [0x15bb] = 0x68c6, [0x15bc] = 0x6979,
  [0x15bd] = 0x6977, [0x15be] = 0x695c, [0x15bf] = 0x6978, [0x15c0] = 0x696b,
  [0x15c1] = 0x6954, [0x15c2] = 0x697e, [0x15c3] = 0x696e, [0x15c4] = 0x6939,
  [0x15c5] = 0x6974, [0x15c6] = 0x693d, [0x15c7] = 0x6959, [0x15c8] = 0x6930,
  [0x15c9] = 0x6961, [0x15ca] = 0x695e, [0x15cb] = 0x695d, [0x15cc] = 0x6981,
  [0x15cd] = 0x696a, [0x15ce] = 0x69b2, [0x15cf] = 0x69ae, [0x15d0] = 0x69d0,
  [0x15d1] = 0x69bf, [0x15d2] = 0x69c1, [0x15d3] = 0x69d3, [0x15d4] = 0x69be,
  [0x15d5] = 0x69ce, [0x15d6] = 0x5be8, [0x15d7] = 0x69ca, [0x15d8] = 0x69dd,
  [0x15d9] = 0x69bb, [0x15da] = 0x69c3, [0x15db] = 0x69a7, [0x15dc] = 0x6a2e,
  [0x15dd] = 0x6991, [0x15de] = 0x69a0, [0x15df] = 0x699c, [0x15e0] = 0x6995,
  [0x15e1] = 0x69b4, [0x15e2] = 0x69de, [0x15e3] = 0x69e8, [0x15e4] = 0x6a02,
  [0x15e5] = 0x6a1b, [0x15e6] = 0x69ff, [0x15e7] = 0x6b0a, [0x15e8] = 0x69f9,
  [0x15e9] = 0x69f2, [0x15ea] = 0x69e7, [0x15eb] = 0x6a05, [0x15ec] = 0x69b1,
  [0x15ed] = 0x6a1e, [0x15ee] = 0x69ed, [0x15ef] = 0x6a14, [0x15f0] = 0x69eb,
  [0x15f1] = 0x6a0a, [0x15f2] = 0x6a12, [0x15f3] = 0x6ac1, [0x15f4] = 0x6a23,
  [0x15f5] = 0x6a13, [0x15f6] = 0x6a44, [0x15f7] = 0x6a0c, [0x15f8] = 0x6a72,
  [0x15f9] = 0x6a36, [0x15fa] = 0x6a78, [0x15fb] = 0x6a47, [0x15fc] = 0x6a62,
  [0x15fd] = 0x6a59, [0x15fe] = 0x6a66, [0x15ff] = 0x6a48, [0x1600] = 0x6a38,
  [0x1601] = 0x6a22, [0x1602] = 0x6a90, [0x1603] = 0x6a8d, [0x1604] = 0x6aa0,
  [0x1605] = 0x6a84, [0x1606] = 0x6aa2, [0x1607] = 0x6aa3, [0x1608] = 0x6a97,
  [0x1609] = 0x8617, [0x160a] = 0x6abb, [0x160b] = 0x6ac3, [0x160c] = 0x6ac2,
  [0x160d] = 0x6ab8, [0x160e] = 0x6ab3, [0x160f] = 0x6aac, [0x1610] = 0x6ade,
  [0x1611] = 0x6ad1, [0x1612] = 0x6adf, [0x1613] = 0x6aaa, [0x1614] = 0x6ada,
  [0x1615] = 0x6aea, [0x1616] = 0x6afb, [0x1617] = 0x6b05, [0x1618] = 0x8616,
  [0x1619] = 0x6afa, [0x161a] = 0x6b12, [0x161b] = 0x6b16, [0x161c] = 0x9b31,
  [0x161d] = 0x6b1f, [0x161e] = 0x6b38, [0x161f] = 0x6b37, [0x1620] = 0x76dc,
  [0x1621] = 0x6b39, [0x1622] = 0x98ee, [0x1623] = 0x6b47, [0x1624] = 0x6b43,
  [0x1625] = 0x6b49, [0x1626] = 0x6b50, [0x1627] = 0x6b59, [0x1628] = 0x6b54,
  [0x1629] = 0x6b5b, [0x162a] = 0x6b5f, [0x162b] = 0x6b61, [0x162c] = 0x6b78,
  [0x162d] = 0x6b79, [0x162e] = 0x6b7f, [0x162f] = 0x6b80, [0x1630] = 0x6b84,
  [0x1631] = 0x6b83, [0x1632] = 0x6b8d, [0x1633] = 0x6b98, [0x1634] = 0x6b95,
  [0x1635] = 0x6b9e, [0x1636] = 0x6ba4, [0x1637] = 0x6baa, [0x1638] = 0x6bab,
  [0x1639] = 0x6baf, [0x163a] = 0x6bb2, [0x163b] = 0x6bb1, [0x163c] = 0x6bb3,
  [0x163d] = 0x6bb7, [0x163e] = 0x6bbc, [0x163f] = 0x6bc6, [0x1640] = 0x6bcb,
  [0x1641] = 0x6bd3, [0x1642] = 0x6bdf, [0x1643] = 0x6bec, [0x1644] = 0x6beb,
  [0x1645] = 0x6bf3, [0x1646] = 0x6bef, [0x1647] = 0x9ebe, [0x1648] = 0x6c08,
  [0x1649] = 0x6c13, [0x164a] = 0x6c14, [0x164b] = 0x6c1b, [0x164c] = 0x6c24,
  [0x164d] = 0x6c23, [0x164e] = 0x6c5e, [0x164f] = 0x6c55, [0x1650] = 0x6c62,
  [0x1651] = 0x6c6a, [0x1652] = 0x6c82, [0x1653] = 0x6c8d, [0x1654] = 0x6c9a,
  [0x1655] = 0x6c81, [0x1656] = 0x6c9b, [0x1657] = 0x6c7e, [0x1658] = 0x6c68,
  [0x1659] = 0x6c73, [0x165a] = 0x6c92, [0x165b] = 0x6c90, [0x165c] = 0x6cc4,
  [0x165d] = 0x6cf1, [0x165e] = 0x6cd3, [0x165f] = 0x6cbd, [0x1660] = 0x6cd7,
  [0x1661] = 0x6cc5, [0x1662] = 0x6cdd, [0x1663] = 0x6cae, [0x1664] = 0x6cb1,
  [0x1665] = 0x6cbe, [0x1666] = 0x6cba, [0x1667] = 0x6cdb, [0x1668] = 0x6cef,
  [0x1669] = 0x6cd9, [0x166a] = 0x6cea, [0x166b] = 0x6d1f, [0x166c] = 0x884d,
  [0x166d] = 0x6d36, [0x166e] = 0x6d2b, [0x166f] = 0x6d3d, [0x1670] = 0x6d38,
  [0x1671] = 0x6d19, [0x1672] = 0x6d35, [0x1673] = 0x6d33, [0x1674] = 0x6d12,
  [0x1675] = 0x6d0c, [0x1676] = 0x6d63, [0x1677] = 0x6d93, [0x1678] = 0x6d64,
  [0x1679] = 0x6d5a, [0x167a] = 0x6d79, [0x167b] = 0x6d59, [0x167c] = 0x6d8e,
  [0x167d] = 0x6d95, [0x167e] = 0x6fe4, [0x167f] = 0x6d85, [0x1680] = 0x6df9,
  [0x1681] = 0x6e15, [0x1682] = 0x6e0a, [0x1683] = 0x6db5, [0x1684] = 0x6dc7,
  [0x1685] = 0x6de6, [0x1686] = 0x6db8, [0x1687] = 0x6dc6, [0x1688] = 0x6dec,
  [0x1689] = 0x6dde, [0x168a] = 0x6dcc, [0x168b] = 0x6de8, [0x168c] = 0x6dd2,
  [0x168d] = 0x6dc5, [0x168e] = 0x6dfa, [0x168f] = 0x6dd9, [0x1690] = 0x6de4,
  [0x1691] = 0x6dd5, [0x1692] = 0x6dea, [0x1693] = 0x6dee, [0x1694] = 0x6e2d,
  [0x1695] = 0x6e6e, [0x1696] = 0x6e2e, [0x1697] = 0x6e19, [0x1698] = 0x6e72,
  [0x1699] = 0x6e5f, [0x169a] = 0x6e3e, [0x169b] = 0x6e23, [0x169c] = 0x6e6b,
  [0x169d] = 0x6e2b, [0x169e] = 0x6e76, [0x169f] = 0x6e4d, [0x16a0] = 0x6e1f,
  [0x16a1] = 0x6e43, [0x16a2] = 0x6e3a, [0x16a3] = 0x6e4e, [0x16a4] = 0x6e24,
  [0x16a5] = 0x6eff, [0x16a6] = 0x6e1d, [0x16a7] = 0x6e38, [0x16a8] = 0x6e82,
  [0x16a9] = 0x6eaa, [0x16aa] = 0x6e98, [0x16ab] = 0x6ec9, [0x16ac] = 0x6eb7,
  [0x16ad] = 0x6ed3, [0x16ae] = 0x6ebd, [0x16af] = 0x6eaf, [0x16b0] = 0x6ec4,
  [0x16b1] = 0x6eb2, [0x16b2] = 0x6ed4, [0x16b3] = 0x6ed5, [0x16b4] = 0x6e8f,
  [0x16b5] = 0x6ea5, [0x16b6] = 0x6ec2, [0x16b7] = 0x6e9f, [0x16b8] = 0x6f41,
  [0x16b9] = 0x6f11, [0x16ba] = 0x704c, [0x16bb] = 0x6eec, [0x16bc] = 0x6ef8,
  [0x16bd] = 0x6efe, [0x16be] = 0x6f3f, [0x16bf] = 0x6ef2, [0x16c0] = 0x6f31,
  [0x16c1] = 0x6eef, [0x16c2] = 0x6f32, [0x16c3] = 0x6ecc, [0x16c4] = 0x6f3e,
  [0x16c5] = 0x6f13, [0x16c6] = 0x6ef7, [0x16c7] = 0x6f86, [0x16c8] = 0x6f7a,
  [0x16c9] = 0x6f78, [0x16ca] = 0x6f81, [0x16cb] = 0x6f80, [0x16cc] = 0x6f6f,
  [0x16cd] = 0x6f5b, [0x16ce] = 0x6ff3, [0x16cf] = 0x6f6d, [0x16d0] = 0x6f82,
  [0x16d1] = 0x6f7c, [0x16d2] = 0x6f58, [0x16d3] = 0x6f8e, [0x16d4] = 0x6f91,
  [0x16d5] = 0x6fc2, [0x16d6] = 0x6f66, [0x16d7] = 0x6fb3, [0x16d8] = 0x6fa3,
  [0x16d9] = 0x6fa1, [0x16da] = 0x6fa4, [0x16db] = 0x6fb9, [0x16dc] = 0x6fc6,
  [0x16dd] = 0x6faa, [0x16de] = 0x6fdf, [0x16df] = 0x6fd5, [0x16e0] = 0x6fec,
  [0x16e1] = 0x6fd4, [0x16e2] = 0x6fd8, [0x16e3] = 0x6ff1, [0x16e4] = 0x6fee,
  [0x16e5] = 0x6fdb, [0x16e6] = 0x7009, [0x16e7] = 0x700b, [0x16e8] = 0x6ffa,
  [0x16e9] = 0x7011, [0x16ea] = 0x7001, [0x16eb] = 0x700f, [0x16ec] = 0x6ffe,
  [0x16ed] = 0x701b, [0x16ee] = 0x701a, [0x16ef] = 0x6f74, [0x16f0] = 0x701d,
  [0x16f1] = 0x7018, [0x16f2] = 0x701f, [0x16f3] = 0x7030, [0x16f4] = 0x703e,
  [0x16f5] = 0x7032, [0x16f6] = 0x7051, [0x16f7] = 0x7063, [0x16f8] = 0x7099,
  [0x16f9] = 0x7092, [0x16fa] = 0x70af, [0x16fb] = 0x70f1, [0x16fc] = 0x70ac,
  [0x16fd] = 0x70b8, [0x16fe] = 0x70b3, [0x16ff] = 0x70ae, [0x1700] = 0x70df,
  [0x1701] = 0x70cb, [0x1702] = 0x70dd, [0x1703] = 0x70d9, [0x1704] = 0x7109,
  [0x1705] = 0x70fd, [0x1706] = 0x711c, [0x1707] = 0x7119, [0x1708] = 0x7165,
  [0x1709] = 0x7155, [0x170a] = 0x7188, [0x170b] = 0x7166, [0x170c] = 0x7162,
  [0x170d] = 0x714c, [0x170e] = 0x7156, [0x170f] = 0x716c, [0x1710] = 0x718f,
  [0x1711] = 0x71fb, [0x1712] = 0x7184, [0x1713] = 0x7195, [0x1714] = 0x71a8,
  [0x1715] = 0x71ac, [0x1716] = 0x71d7, [0x1717] = 0x71b9, [0x1718] = 0x71be,
  [0x1719] = 0x71d2, [0x171a] = 0x71c9, [0x171b] = 0x71d4, [0x171c] = 0x71ce,
  [0x171d] = 0x71e0, [0x171e] = 0x71ec, [0x171f] = 0x71e7, [0x1720] = 0x71f5,
  [0x1721] = 0x71fc, [0x1722] = 0x71f9, [0x1723] = 0x71ff, [0x1724] = 0x720d,
  [0x1725] = 0x7210, [0x1726] = 0x721b, [0x1727] = 0x7228, [0x1728] = 0x722d,
  [0x1729] = 0x722c, [0x172a] = 0x7230, [0x172b] = 0x7232, [0x172c] = 0x723b,
  [0x172d] = 0x723c, [0x172e] = 0x723f, [0x172f] = 0x7240, [0x1730] = 0x7246,
  [0x1731] = 0x724b, [0x1732] = 0x7258, [0x1733] = 0x7274, [0x1734] = 0x727e,
  [0x1735] = 0x7282, [0x1736] = 0x7281, [0x1737] = 0x7287, [0x1738] = 0x7292,
  [0x1739] = 0x7296, [0x173a] = 0x72a2, [0x173b] = 0x72a7, [0x173c] = 0x72b9,
  [0x173d] = 0x72b2, [0x173e] = 0x72c3, [0x173f] = 0x72c6, [0x1740] = 0x72c4,
  [0x1741] = 0x72ce, [0x1742] = 0x72d2, [0x1743] = 0x72e2, [0x1744] = 0x72e0,
  [0x1745] = 0x72e1, [0x1746] = 0x72f9, [0x1747] = 0x72f7, [0x1748] = 0x500f,
  [0x1749] = 0x7317, [0x174a] = 0x730a, [0x174b] = 0x731c, [0x174c] = 0x7316,
  [0x174d] = 0x731d, [0x174e] = 0x7334, [0x174f] = 0x732f, [0x1750] = 0x7329,
  [0x1751] = 0x7325, [0x1752] = 0x733e, [0x1753] = 0x734e, [0x1754] = 0x734f,
  [0x1755] = 0x9ed8, [0x1756] = 0x7357, [0x1757] = 0x736a, [0x1758] = 0x7368,
  [0x1759] = 0x7370, [0x175a] = 0x7378, [0x175b] = 0x7375, [0x175c] = 0x737b,
  [0x175d] = 0x737a, [0x175e] = 0x73c8, [0x175f] = 0x73b3, [0x1760] = 0x73ce,
  [0x1761] = 0x73bb, [0x1762] = 0x73c0, [0x1763] = 0x73e5, [0x1764] = 0x73ee,
  [0x1765] = 0x73de, [0x1766] = 0x74a2, [0x1767] = 0x7405, [0x1768] = 0x746f,
  [0x1769] = 0x7425, [0x176a] = 0x73f8, [0x176b] = 0x7432, [0x176c] = 0x743a,
  [0x176d] = 0x7455, [0x176e] = 0x743f, [0x176f] = 0x745f, [0x1770] = 0x7459,
  [0x1771] = 0x7441, [0x1772] = 0x745c, [0x1773] = 0x7469, [0x1774] = 0x7470,
  [0x1775] = 0x7463, [0x1776] = 0x746a, [0x1777] = 0x7476, [0x1778] = 0x747e,
  [0x1779] = 0x748b, [0x177a] = 0x749e, [0x177b] = 0x74a7, [0x177c] = 0x74ca,
  [0x177d] = 0x74cf, [0x177e] = 0x74d4, [0x177f] = 0x73f1, [0x1780] = 0x74e0,
  [0x1781] = 0x74e3, [0x1782] = 0x74e7, [0x1783] = 0x74e9, [0x1784] = 0x74ee,
  [0x1785] = 0x74f2, [0x1786] = 0x74f0, [0x1787] = 0x74f1, [0x1788] = 0x74f8,
  [0x1789] = 0x74f7, [0x178a] = 0x7504, [0x178b] = 0x7503, [0x178c] = 0x7505,
  [0x178d] = 0x750c, [0x178e] = 0x750e, [0x178f] = 0x750d, [0x1790] = 0x7515,
  [0x1791] = 0x7513, [0x1792] = 0x751e, [0x1793] = 0x7526, [0x1794] = 0x752c,
  [0x1795] = 0x753c, [0x1796] = 0x7544, [0x1797] = 0x754d, [0x1798] = 0x754a,
  [0x1799] = 0x7549, [0x179a] = 0x755b, [0x179b] = 0x7546, [0x179c] = 0x755a,
  [0x179d] = 0x7569, [0x179e] = 0x7564, [0x179f] = 0x7567, [0x17a0] = 0x756b,
  [0x17a1] = 0x756d, [0x17a2] = 0x7578, [0x17a3] = 0x7576, [0x17a4] = 0x7586,
  [0x17a5] = 0x7587, [0x17a6] = 0x7574, [0x17a7] = 0x758a, [0x17a8] = 0x7589,
  [0x17a9] = 0x7582, [0x17aa] = 0x7594, [0x17ab] = 0x759a, [0x17ac] = 0x759d,
  [0x17ad] = 0x75a5, [0x17ae] = 0x75a3, [0x17af] = 0x75c2, [0x17b0] = 0x75b3,
  [0x17b1] = 0x75c3, [0x17b2] = 0x75b5, [0x17b3] = 0x75bd, [0x17b4] = 0x75b8,
  [0x17b5] = 0x75bc, [0x17b6] = 0x75b1, [0x17b7] = 0x75cd, [0x17b8] = 0x75ca,
  [0x17b9] = 0x75d2, [0x17ba] = 0x75d9, [0x17bb] = 0x75e3, [0x17bc] = 0x75de,
  [0x17bd] = 0x75fe, [0x17be] = 0x75ff, [0x17bf] = 0x75fc, [0x17c0] = 0x7601,
  [0x17c1] = 0x75f0, [0x17c2] = 0x75fa, [0x17c3] = 0x75f2, [0x17c4] = 0x75f3,
  [0x17c5] = 0x760b, [0x17c6] = 0x760d, [0x17c7] = 0x7609, [0x17c8] = 0x761f,
  [0x17c9] = 0x7627, [0x17ca] = 0x7620, [0x17cb] = 0x7621, [0x17cc] = 0x7622,
  [0x17cd] = 0x7624, [0x17ce] = 0x7634, [0x17cf] = 0x7630, [0x17d0] = 0x763b,
  [0x17d1] = 0x7647, [0x17d2] = 0x7648, [0x17d3] = 0x7646, [0x17d4] = 0x765c,
  [0x17d5] = 0x7658, [0x17d6] = 0x7661, [0x17d7] = 0x7662, [0x17d8] = 0x7668,
  [0x17d9] = 0x7669, [0x17da] = 0x766a, [0x17db] = 0x7667, [0x17dc] = 0x766c,
  [0x17dd] = 0x7670, [0x17de] = 0x7672, [0x17df] = 0x7676, [0x17e0] = 0x7678,
  [0x17e1] = 0x767c, [0x17e2] = 0x7680, [0x17e3] = 0x7683, [0x17e4] = 0x7688,
  [0x17e5] = 0x768b, [0x17e6] = 0x768e, [0x17e7] = 0x7696, [0x17e8] = 0x7693,
  [0x17e9] = 0x7699, [0x17ea] = 0x769a, [0x17eb] = 0x76b0, [0x17ec] = 0x76b4,
  [0x17ed] = 0x76b8, [0x17ee] = 0x76b9, [0x17ef] = 0x76ba, [0x17f0] = 0x76c2,
  [0x17f1] = 0x76cd, [0x17f2] = 0x76d6, [0x17f3] = 0x76d2, [0x17f4] = 0x76de,
  [0x17f5] = 0x76e1, [0x17f6] = 0x76e5, [0x17f7] = 0x76e7, [0x17f8] = 0x76ea,
  [0x17f9] = 0x862f, [0x17fa] = 0x76fb, [0x17fb] = 0x7708, [0x17fc] = 0x7707,
  [0x17fd] = 0x7704, [0x17fe] = 0x7729, [0x17ff] = 0x7724, [0x1800] = 0x771e,
  [0x1801] = 0x7725, [0x1802] = 0x7726, [0x1803] = 0x771b, [0x1804] = 0x7737,
  [0x1805] = 0x7738, [0x1806] = 0x7747, [0x1807] = 0x775a, [0x1808] = 0x7768,
  [0x1809] = 0x776b, [0x180a] = 0x775b, [0x180b] = 0x7765, [0x180c] = 0x777f,
  [0x180d] = 0x777e, [0x180e] = 0x7779, [0x180f] = 0x778e, [0x1810] = 0x778b,
  [0x1811] = 0x7791, [0x1812] = 0x77a0, [0x1813] = 0x779e, [0x1814] = 0x77b0,
  [0x1815] = 0x77b6, [0x1816] = 0x77b9, [0x1817] = 0x77bf, [0x1818] = 0x77bc,
  [0x1819] = 0x77bd, [0x181a] = 0x77bb, [0x181b] = 0x77c7, [0x181c] = 0x77cd,
  [0x181d] = 0x77d7, [0x181e] = 0x77da, [0x181f] = 0x77dc, [0x1820] = 0x77e3,
  [0x1821] = 0x77ee, [0x1822] = 0x77fc, [0x1823] = 0x780c, [0x1824] = 0x7812,
  [0x1825] = 0x7926, [0x1826] = 0x7820, [0x1827] = 0x792a, [0x1828] = 0x7845,
  [0x1829] = 0x788e, [0x182a] = 0x7874, [0x182b] = 0x7886, [0x182c] = 0x787c,
  [0x182d] = 0x789a, [0x182e] = 0x788c, [0x182f] = 0x78a3, [0x1830] = 0x78b5,
  [0x1831] = 0x78aa, [0x1832] = 0x78af, [0x1833] = 0x78d1, [0x1834] = 0x78c6,
  [0x1835] = 0x78cb, [0x1836] = 0x78d4, [0x1837] = 0x78be, [0x1838] = 0x78bc,
  [0x1839] = 0x78c5, [0x183a] = 0x78ca, [0x183b] = 0x78ec, [0x183c] = 0x78e7,
  [0x183d] = 0x78da, [0x183e] = 0x78fd, [0x183f] = 0x78f4, [0x1840] = 0x7907,
  [0x1841] = 0x7912, [0x1842] = 0x7911, [0x1843] = 0x7919, [0x1844] = 0x792c,
  [0x1845] = 0x792b, [0x1846] = 0x7940, [0x1847] = 0x7960, [0x1848] = 0x7957,
  [0x1849] = 0x795f, [0x184a] = 0x795a, [0x184b] = 0x7955, [0x184c] = 0x7953,
  [0x184d] = 0x797a, [0x184e] = 0x797f, [0x184f] = 0x798a, [0x1850] = 0x799d,
  [0x1851] = 0x79a7, [0x1852] = 0x9f4b, [0x1853] = 0x79aa, [0x1854] = 0x79ae,
  [0x1855] = 0x79b3, [0x1856] = 0x79b9, [0x1857] = 0x79ba, [0x1858] = 0x79c9,
  [0x1859] = 0x79d5, [0x185a] = 0x79e7, [0x185b] = 0x79ec, [0x185c] = 0x79e1,
  [0x185d] = 0x79e3, [0x185e] = 0x7a08, [0x185f] = 0x7a0d, [0x1860] = 0x7a18,
  [0x1861] = 0x7a19, [0x1862] = 0x7a20, [0x1863] = 0x7a1f, [0x1864] = 0x7980,
  [0x1865] = 0x7a31, [0x1866] = 0x7a3b, [0x1867] = 0x7a3e, [0x1868] = 0x7a37,
  [0x1869] = 0x7a43, [0x186a] = 0x7a57, [0x186b] = 0x7a49, [0x186c] = 0x7a61,
  [0x186d] = 0x7a62, [0x186e] = 0x7a69, [0x186f] = 0x9f9d, [0x1870] = 0x7a70,
  [0x1871] = 0x7a79, [0x1872] = 0x7a7d, [0x1873] = 0x7a88, [0x1874] = 0x7a97,
  [0x1875] = 0x7a95, [0x1876] = 0x7a98, [0x1877] = 0x7a96, [0x1878] = 0x7aa9,
  [0x1879] = 0x7ac8, [0x187a] = 0x7ab0, [0x187b] = 0x7ab6, [0x187c] = 0x7ac5,
  [0x187d] = 0x7ac4, [0x187e] = 0x7abf, [0x187f] = 0x9083, [0x1880] = 0x7ac7,
  [0x1881] = 0x7aca, [0x1882] = 0x7acd, [0x1883] = 0x7acf, [0x1884] = 0x7ad5,
  [0x1885] = 0x7ad3, [0x1886] = 0x7ad9, [0x1887] = 0x7ada, [0x1888] = 0x7add,
  [0x1889] = 0x7ae1, [0x188a] = 0x7ae2, [0x188b] = 0x7ae6, [0x188c] = 0x7aed,
  [0x188d] = 0x7af0, [0x188e] = 0x7b02, [0x188f] = 0x7b0f, [0x1890] = 0x7b0a,
  [0x1891] = 0x7b06, [0x1892] = 0x7b33, [0x1893] = 0x7b18, [0x1894] = 0x7b19,
  [0x1895] = 0x7b1e, [0x1896] = 0x7b35, [0x1897] = 0x7b28, [0x1898] = 0x7b36,
  [0x1899] = 0x7b50, [0x189a] = 0x7b7a, [0x189b] = 0x7b04, [0x189c] = 0x7b4d,
  [0x189d] = 0x7b0b, [0x189e] = 0x7b4c, [0x189f] = 0x7b45, [0x18a0] = 0x7b75,
  [0x18a1] = 0x7b65, [0x18a2] = 0x7b74, [0x18a3] = 0x7b67, [0x18a4] = 0x7b70,
  [0x18a5] = 0x7b71, [0x18a6] = 0x7b6c, [0x18a7] = 0x7b6e, [0x18a8] = 0x7b9d,
  [0x18a9] = 0x7b98, [0x18aa] = 0x7b9f, [0x18ab] = 0x7b8d, [0x18ac] = 0x7b9c,
  [0x18ad] = 0x7b9a, [0x18ae] = 0x7b8b, [0x18af] = 0x7b92, [0x18b0] = 0x7b8f,
  [0x18b1] = 0x7b5d, [0x18b2] = 0x7b99, [0x18b3] = 0x7bcb, [0x18b4] = 0x7bc1,
  [0x18b5] = 0x7bcc, [0x18b6] = 0x7bcf, [0x18b7] = 0x7bb4, [0x18b8] = 0x7bc6,
  [0x18b9] = 0x7bdd, [0x18ba] = 0x7be9, [0x18bb] = 0x7c11, [0x18bc] = 0x7c14,
  [0x18bd] = 0x7be6, [0x18be] = 0x7be5, [0x18bf] = 0x7c60, [0x18c0] = 0x7c00,
  [0x18c1] = 0x7c07, [0x18c2] = 0x7c13, [0x18c3] = 0x7bf3, [0x18c4] = 0x7bf7,
  [0x18c5] = 0x7c17, [0x18c6] = 0x7c0d, [0x18c7] = 0x7bf6, [0x18c8] = 0x7c23,
  [0x18c9] = 0x7c27, [0x18ca] = 0x7c2a, [0x18cb] = 0x7c1f, [0x18cc] = 0x7c37,
  [0x18cd] = 0x7c2b, [0x18ce] = 0x7c3d, [0x18cf] = 0x7c4c, [0x18d0] = 0x7c43,
  [0x18d1] = 0x7c54, [0x18d2] = 0x7c4f, [0x18d3] = 0x7c40, [0x18d4] = 0x7c50,
  [0x18d5] = 0x7c58, [0x18d6] = 0x7c5f, [0x18d7] = 0x7c64, [0x18d8] = 0x7c56,
  [0x18d9] = 0x7c65, [0x18da] = 0x7c6c, [0x18db] = 0x7c75, [0x18dc] = 0x7c83,
  [0x18dd] = 0x7c90, [0x18de] = 0x7ca4, [0x18df] = 0x7cad, [0x18e0] = 0x7ca2,
  [0x18e1] = 0x7cab, [0x18e2] = 0x7ca1, [0x18e3] = 0x7ca8, [0x18e4] = 0x7cb3,
  [0x18e5] = 0x7cb2, [0x18e6] = 0x7cb1, [0x18e7] = 0x7cae, [0x18e8] = 0x7cb9,
  [0x18e9] = 0x7cbd, [0x18ea] = 0x7cc0, [0x18eb] = 0x7cc5, [0x18ec] = 0x7cc2,
  [0x18ed] = 0x7cd8, [0x18ee] = 0x7cd2, [0x18ef] = 0x7cdc, [0x18f0] = 0x7ce2,
  [0x18f1] = 0x9b3b, [0x18f2] = 0x7cef, [0x18f3] = 0x7cf2, [0x18f4] = 0x7cf4,
  [0x18f5] = 0x7cf6, [0x18f6] = 0x7cfa, [0x18f7] = 0x7d06, [0x18f8] = 0x7d02,
  [0x18f9] = 0x7d1c, [0x18fa] = 0x7d15, [0x18fb] = 0x7d0a, [0x18fc] = 0x7d45,
  [0x18fd] = 0x7d4b, [0x18fe] = 0x7d2e, [0x18ff] = 0x7d32, [0x1900] = 0x7d3f,
  [0x1901] = 0x7d35, [0x1902] = 0x7d46, [0x1903] = 0x7d73, [0x1904] = 0x7d56,
  [0x1905] = 0x7d4e, [0x1906] = 0x7d72, [0x1907] = 0x7d68, [0x1908] = 0x7d6e,
  [0x1909] = 0x7d4f, [0x190a] = 0x7d63, [0x190b] = 0x7d93, [0x190c] = 0x7d89,
  [0x190d] = 0x7d5b, [0x190e] = 0x7d8f, [0x190f] = 0x7d7d, [0x1910] = 0x7d9b,
  [0x1911] = 0x7dba, [0x1912] = 0x7dae, [0x1913] = 0x7da3, [0x1914] = 0x7db5,
  [0x1915] = 0x7dc7, [0x1916] = 0x7dbd, [0x1917] = 0x7dab, [0x1918] = 0x7e3d,
  [0x1919] = 0x7da2, [0x191a] = 0x7daf, [0x191b] = 0x7ddc, [0x191c] = 0x7db8,
  [0x191d] = 0x7d9f, [0x191e] = 0x7db0, [0x191f] = 0x7dd8, [0x1920] = 0x7ddd,
  [0x1921] = 0x7de4, [0x1922] = 0x7dde, [0x1923] = 0x7dfb, [0x1924] = 0x7df2,
  [0x1925] = 0x7de1, [0x1926] = 0x7e05, [0x1927] = 0x7e0a, [0x1928] = 0x7e23,
  [0x1929] = 0x7e21, [0x192a] = 0x7e12, [0x192b] = 0x7e31, [0x192c] = 0x7e1f,
  [0x192d] = 0x7e09, [0x192e] = 0x7e0b, [0x192f] = 0x7e22, [0x1930] = 0x7e46,
  [0x1931] = 0x7e66, [0x1932] = 0x7e3b, [0x1933] = 0x7e35, [0x1934] = 0x7e39,
  [0x1935] = 0x7e43, [0x1936] = 0x7e37, [0x1937] = 0x7e32, [0x1938] = 0x7e3a,
  [0x1939] = 0x7e67, [0x193a] = 0x7e5d, [0x193b] = 0x7e56, [0x193c] = 0x7e5e,
  [0x193d] = 0x7e59, [0x193e] = 0x7e5a, [0x193f] = 0x7e79, [0x1940] = 0x7e6a,
  [0x1941] = 0x7e69, [0x1942] = 0x7e7c, [0x1943] = 0x7e7b, [0x1944] = 0x7e83,
  [0x1945] = 0x7dd5, [0x1946] = 0x7e7d, [0x1947] = 0x8fae, [0x1948] = 0x7e7f,
  [0x1949] = 0x7e88, [0x194a] = 0x7e89, [0x194b] = 0x7e8c, [0x194c] = 0x7e92,
  [0x194d] = 0x7e90, [0x194e] = 0x7e93, [0x194f] = 0x7e94, [0x1950] = 0x7e96,
  [0x1951] = 0x7e8e, [0x1952] = 0x7e9b, [0x1953] = 0x7e9c, [0x1954] = 0x7f38,
  [0x1955] = 0x7f3a, [0x1956] = 0x7f45, [0x1957] = 0x7f4c, [0x1958] = 0x7f4d,
  [0x1959] = 0x7f4e, [0x195a] = 0x7f50, [0x195b] = 0x7f51, [0x195c] = 0x7f55,
  [0x195d] = 0x7f54, [0x195e] = 0x7f58, [0x195f] = 0x7f5f, [0x1960] = 0x7f60,
  [0x1961] = 0x7f68, [0x1962] = 0x7f69, [0x1963] = 0x7f67, [0x1964] = 0x7f78,
  [0x1965] = 0x7f82, [0x1966] = 0x7f86, [0x1967] = 0x7f83, [0x1968] = 0x7f88,
  [0x1969] = 0x7f87, [0x196a] = 0x7f8c, [0x196b] = 0x7f94, [0x196c] = 0x7f9e,
  [0x196d] = 0x7f9d, [0x196e] = 0x7f9a, [0x196f] = 0x7fa3, [0x1970] = 0x7faf,
  [0x1971] = 0x7fb2, [0x1972] = 0x7fb9, [0x1973] = 0x7fae, [0x1974] = 0x7fb6,
  [0x1975] = 0x7fb8, [0x1976] = 0x8b71, [0x1977] = 0x7fc5, [0x1978] = 0x7fc6,
  [0x1979] = 0x7fca, [0x197a] = 0x7fd5, [0x197b] = 0x7fd4, [0x197c] = 0x7fe1,
  [0x197d] = 0x7fe6, [0x197e] = 0x7fe9, [0x197f] = 0x7ff3, [0x1980] = 0x7ff9,
  [0x1981] = 0x98dc, [0x1982] = 0x8006, [0x1983] = 0x8004, [0x1984] = 0x800b,
  [0x1985] = 0x8012, [0x1986] = 0x8018, [0x1987] = 0x8019, [0x1988] = 0x801c,
  [0x1989] = 0x8021, [0x198a] = 0x8028, [0x198b] = 0x803f, [0x198c] = 0x803b,
  [0x198d] = 0x804a, [0x198e] = 0x8046, [0x198f] = 0x8052, [0x1990] = 0x8058,
  [0x1991] = 0x805a, [0x1992] = 0x805f, [0x1993] = 0x8062, [0x1994] = 0x8068,
  [0x1995] = 0x8073, [0x1996] = 0x8072, [0x1997] = 0x8070, [0x1998] = 0x8076,
  [0x1999] = 0x8079, [0x199a] = 0x807d, [0x199b] = 0x807f, [0x199c] = 0x8084,
  [0x199d] = 0x8086, [0x199e] = 0x8085, [0x199f] = 0x809b, [0x19a0] = 0x8093,
  [0x19a1] = 0x809a, [0x19a2] = 0x80ad, [0x19a3] = 0x5190, [0x19a4] = 0x80ac,
  [0x19a5] = 0x80db, [0x19a6] = 0x80e5, [0x19a7] = 0x80d9, [0x19a8] = 0x80dd,
  [0x19a9] = 0x80c4, [0x19aa] = 0x80da, [0x19ab] = 0x80d6, [0x19ac] = 0x8109,
  [0x19ad] = 0x80ef, [0x19ae] = 0x80f1, [0x19af] = 0x811b, [0x19b0] = 0x8129,
  [0x19b1] = 0x8123, [0x19b2] = 0x812f, [0x19b3] = 0x814b, [0x19b4] = 0x968b,
  [0x19b5] = 0x8146, [0x19b6] = 0x813e, [0x19b7] = 0x8153, [0x19b8] = 0x8151,
  [0x19b9] = 0x80fc, [0x19ba] = 0x8171, [0x19bb] = 0x816e, [0x19bc] = 0x8165,
  [0x19bd] = 0x8166, [0x19be] = 0x8174, [0x19bf] = 0x8183, [0x19c0] = 0x8188,
  [0x19c1] = 0x818a, [0x19c2] = 0x8180, [0x19c3] = 0x8182, [0x19c4] = 0x81a0,
  [0x19c5] = 0x8195, [0x19c6] = 0x81a4, [0x19c7] = 0x81a3, [0x19c8] = 0x815f,
  [0x19c9] = 0x8193, [0x19ca] = 0x81a9, [0x19cb] = 0x81b0, [0x19cc] = 0x81b5,
  [0x19cd] = 0x81be, [0x19ce] = 0x81b8, [0x19cf] = 0x81bd, [0x19d0] = 0x81c0,
  [0x19d1] = 0x81c2, [0x19d2] = 0x81ba, [0x19d3] = 0x81c9, [0x19d4] = 0x81cd,
  [0x19d5] = 0x81d1, [0x19d6] = 0x81d9, [0x19d7] = 0x81d8, [0x19d8] = 0x81c8,
  [0x19d9] = 0x81da, [0x19da] = 0x81df, [0x19db] = 0x81e0, [0x19dc] = 0x81e7,
  [0x19dd] = 0x81fa, [0x19de] = 0x81fb, [0x19df] = 0x81fe, [0x19e0] = 0x8201,
  [0x19e1] = 0x8202, [0x19e2] = 0x8205, [0x19e3] = 0x8207, [0x19e4] = 0x820a,
  [0x19e5] = 0x820d, [0x19e6] = 0x8210, [0x19e7] = 0x8216, [0x19e8] = 0x8229,
  [0x19e9] = 0x822b, [0x19ea] = 0x8238, [0x19eb] = 0x8233, [0x19ec] = 0x8240,
  [0x19ed] = 0x8259, [0x19ee] = 0x8258, [0x19ef] = 0x825d, [0x19f0] = 0x825a,
  [0x19f1] = 0x825f, [0x19f2] = 0x8264, [0x19f3] = 0x8262, [0x19f4] = 0x8268,
  [0x19f5] = 0x826a, [0x19f6] = 0x826b, [0x19f7] = 0x822e, [0x19f8] = 0x8271,
  [0x19f9] = 0x8277, [0x19fa] = 0x8278, [0x19fb] = 0x827e, [0x19fc] = 0x828d,
  [0x19fd] = 0x8292, [0x19fe] = 0x82ab, [0x19ff] = 0x829f, [0x1a00] = 0x82bb,
  [0x1a01] = 0x82ac, [0x1a02] = 0x82e1, [0x1a03] = 0x82e3, [0x1a04] = 0x82df,
  [0x1a05] = 0x82d2, [0x1a06] = 0x82f4, [0x1a07] = 0x82f3, [0x1a08] = 0x82fa,
  [0x1a09] = 0x8393, [0x1a0a] = 0x8303, [0x1a0b] = 0x82fb, [0x1a0c] = 0x82f9,
  [0x1a0d] = 0x82de, [0x1a0e] = 0x8306, [0x1a0f] = 0x82dc, [0x1a10] = 0x8309,
  [0x1a11] = 0x82d9, [0x1a12] = 0x8335, [0x1a13] = 0x8334, [0x1a14] = 0x8316,
  [0x1a15] = 0x8332, [0x1a16] = 0x8331, [0x1a17] = 0x8340, [0x1a18] = 0x8339,
  [0x1a19] = 0x8350, [0x1a1a] = 0x8345, [0x1a1b] = 0x832f, [0x1a1c] = 0x832b,
  [0x1a1d] = 0x8317, [0x1a1e] = 0x8318, [0x1a1f] = 0x8385, [0x1a20] = 0x839a,
  [0x1a21] = 0x83aa, [0x1a22] = 0x839f, [0x1a23] = 0x83a2, [0x1a24] = 0x8396,
  [0x1a25] = 0x8323, [0x1a26] = 0x838e, [0x1a27] = 0x8387, [0x1a28] = 0x838a,
  [0x1a29] = 0x837c, [0x1a2a] = 0x83b5, [0x1a2b] = 0x8373, [0x1a2c] = 0x8375,
  [0x1a2d] = 0x83a0, [0x1a2e] = 0x8389, [0x1a2f] = 0x83a8, [0x1a30] = 0x83f4,
  [0x1a31] = 0x8413, [0x1a32] = 0x83eb, [0x1a33] = 0x83ce, [0x1a34] = 0x83fd,
  [0x1a35] = 0x8403, [0x1a36] = 0x83d8, [0x1a37] = 0x840b, [0x1a38] = 0x83c1,
  [0x1a39] = 0x83f7, [0x1a3a] = 0x8407, [0x1a3b] = 0x83e0, [0x1a3c] = 0x83f2,
  [0x1a3d] = 0x840d, [0x1a3e] = 0x8422, [0x1a3f] = 0x8420, [0x1a40] = 0x83bd,
  [0x1a41] = 0x8438, [0x1a42] = 0x8506, [0x1a43] = 0x83fb, [0x1a44] = 0x846d,
  [0x1a45] = 0x842a, [0x1a46] = 0x843c, [0x1a47] = 0x855a, [0x1a48] = 0x8484,
  [0x1a49] = 0x8477, [0x1a4a] = 0x846b, [0x1a4b] = 0x84ad, [0x1a4c] = 0x846e,
  [0x1a4d] = 0x8482, [0x1a4e] = 0x8469, [0x1a4f] = 0x8446, [0x1a50] = 0x842c,
  [0x1a51] = 0x846f, [0x1a52] = 0x8479, [0x1a53] = 0x8435, [0x1a54] = 0x84ca,
  [0x1a55] = 0x8462, [0x1a56] = 0x84b9, [0x1a57] = 0x84bf, [0x1a58] = 0x849f,
  [0x1a59] = 0x84d9, [0x1a5a] = 0x84cd, [0x1a5b] = 0x84bb, [0x1a5c] = 0x84da,
  [0x1a5d] = 0x84d0, [0x1a5e] = 0x84c1, [0x1a5f] = 0x84c6, [0x1a60] = 0x84d6,
  [0x1a61] = 0x84a1, [0x1a62] = 0x8521, [0x1a63] = 0x84ff, [0x1a64] = 0x84f4,
  [0x1a65] = 0x8517, [0x1a66] = 0x8518, [0x1a67] = 0x852c, [0x1a68] = 0x851f,
  [0x1a69] = 0x8515, [0x1a6a] = 0x8514, [0x1a6b] = 0x84fc, [0x1a6c] = 0x8540,
  [0x1a6d] = 0x8563, [0x1a6e] = 0x8558, [0x1a6f] = 0x8548, [0x1a70] = 0x8541,
  [0x1a71] = 0x8602, [0x1a72] = 0x854b, [0x1a73] = 0x8555, [0x1a74] = 0x8580,
  [0x1a75] = 0x85a4, [0x1a76] = 0x8588, [0x1a77] = 0x8591, [0x1a78] = 0x858a,
  [0x1a79] = 0x85a8, [0x1a7a] = 0x856d, [0x1a7b] = 0x8594, [0x1a7c] = 0x859b,
  [0x1a7d] = 0x85ea, [0x1a7e] = 0x8587, [0x1a7f] = 0x859c, [0x1a80] = 0x8577,
  [0x1a81] = 0x857e, [0x1a82] = 0x8590, [0x1a83] = 0x85c9, [0x1a84] = 0x85ba,
  [0x1a85] = 0x85cf, [0x1a86] = 0x85b9, [0x1a87] = 0x85d0, [0x1a88] = 0x85d5,
  [0x1a89] = 0x85dd, [0x1a8a] = 0x85e5, [0x1a8b] = 0x85dc, [0x1a8c] = 0x85f9,
  [0x1a8d] = 0x860a, [0x1a8e] = 0x8613, [0x1a8f] = 0x860b, [0x1a90] = 0x85fe,
  [0x1a91] = 0x85fa, [0x1a92] = 0x8606, [0x1a93] = 0x8622, [0x1a94] = 0x861a,
  [0x1a95] = 0x8630, [0x1a96] = 0x863f, [0x1a97] = 0x864d, [0x1a98] = 0x4e55,
  [0x1a99] = 0x8654, [0x1a9a] = 0x865f, [0x1a9b] = 0x8667, [0x1a9c] = 0x8671,
  [0x1a9d] = 0x8693, [0x1a9e] = 0x86a3, [0x1a9f] = 0x86a9, [0x1aa0] = 0x86aa,
  [0x1aa1] = 0x868b, [0x1aa2] = 0x868c, [0x1aa3] = 0x86b6, [0x1aa4] = 0x86af,
  [0x1aa5] = 0x86c4, [0x1aa6] = 0x86c6, [0x1aa7] = 0x86b0, [0x1aa8] = 0x86c9,
  [0x1aa9] = 0x8823, [0x1aaa] = 0x86ab, [0x1aab] = 0x86d4, [0x1aac] = 0x86de,
  [0x1aad] = 0x86e9, [0x1aae] = 0x86ec, [0x1aaf] = 0x86df, [0x1ab0] = 0x86db,
  [0x1ab1] = 0x86ef, [0x1ab2] = 0x8712, [0x1ab3] = 0x8706, [0x1ab4] = 0x8708,
  [0x1ab5] = 0x8700, [0x1ab6] = 0x8703, [0x1ab7] = 0x86fb, [0x1ab8] = 0x8711,
  [0x1ab9] = 0x8709, [0x1aba] = 0x870d, [0x1abb] = 0x86f9, [0x1abc] = 0x870a,
  [0x1abd] = 0x8734, [0x1abe] = 0x873f, [0x1abf] = 0x8737, [0x1ac0] = 0x873b,
  [0x1ac1] = 0x8725, [0x1ac2] = 0x8729, [0x1ac3] = 0x871a, [0x1ac4] = 0x8760,
  [0x1ac5] = 0x875f, [0x1ac6] = 0x8778, [0x1ac7] = 0x874c, [0x1ac8] = 0x874e,
  [0x1ac9] = 0x8774, [0x1aca] = 0x8757, [0x1acb] = 0x8768, [0x1acc] = 0x876e,
  [0x1acd] = 0x8759, [0x1ace] = 0x8753, [0x1acf] = 0x8763, [0x1ad0] = 0x876a,
  [0x1ad1] = 0x8805, [0x1ad2] = 0x87a2, [0x1ad3] = 0x879f, [0x1ad4] = 0x8782,
  [0x1ad5] = 0x87af, [0x1ad6] = 0x87cb, [0x1ad7] = 0x87bd, [0x1ad8] = 0x87c0,
  [0x1ad9] = 0x87d0, [0x1ada] = 0x96d6, [0x1adb] = 0x87ab, [0x1adc] = 0x87c4,
  [0x1add] = 0x87b3, [0x1ade] = 0x87c7, [0x1adf] = 0x87c6, [0x1ae0] = 0x87bb,
  [0x1ae1] = 0x87ef, [0x1ae2] = 0x87f2, [0x1ae3] = 0x87e0, [0x1ae4] = 0x880f,
  [0x1ae5] = 0x880d, [0x1ae6] = 0x87fe, [0x1ae7] = 0x87f6, [0x1ae8] = 0x87f7,
  [0x1ae9] = 0x880e, [0x1aea] = 0x87d2, [0x1aeb] = 0x8811, [0x1aec] = 0x8816,
  [0x1aed] = 0x8815, [0x1aee] = 0x8822, [0x1aef] = 0x8821, [0x1af0] = 0x8831,
  [0x1af1] = 0x8836, [0x1af2] = 0x8839, [0x1af3] = 0x8827, [0x1af4] = 0x883b,
  [0x1af5] = 0x8844, [0x1af6] = 0x8842, [0x1af7] = 0x8852, [0x1af8] = 0x8859,
  [0x1af9] = 0x885e, [0x1afa] = 0x8862, [0x1afb] = 0x886b, [0x1afc] = 0x8881,
  [0x1afd] = 0x887e, [0x1afe] = 0x889e, [0x1aff] = 0x8875, [0x1b00] = 0x887d,
  [0x1b01] = 0x88b5, [0x1b02] = 0x8872, [0x1b03] = 0x8882, [0x1b04] = 0x8897,
  [0x1b05] = 0x8892, [0x1b06] = 0x88ae, [0x1b07] = 0x8899, [0x1b08] = 0x88a2,
  [0x1b09] = 0x888d, [0x1b0a] = 0x88a4, [0x1b0b] = 0x88b0, [0x1b0c] = 0x88bf,
  [0x1b0d] = 0x88b1, [0x1b0e] = 0x88c3, [0x1b0f] = 0x88c4, [0x1b10] = 0x88d4,
  [0x1b11] = 0x88d8, [0x1b12] = 0x88d9, [0x1b13] = 0x88dd, [0x1b14] = 0x88f9,
  [0x1b15] = 0x8902, [0x1b16] = 0x88fc, [0x1b17] = 0x88f4, [0x1b18] = 0x88e8,
  [0x1b19] = 0x88f2, [0x1b1a] = 0x8904, [0x1b1b] = 0x890c, [0x1b1c] = 0x890a,
  [0x1b1d] = 0x8913, [0x1b1e] = 0x8943, [0x1b1f] = 0x891e, [0x1b20] = 0x8925,
  [0x1b21] = 0x892a, [0x1b22] = 0x892b, [0x1b23] = 0x8941, [0x1b24] = 0x8944,
  [0x1b25] = 0x893b, [0x1b26] = 0x8936, [0x1b27] = 0x8938, [0x1b28] = 0x894c,
  [0x1b29] = 0x891d, [0x1b2a] = 0x8960, [0x1b2b] = 0x895e, [0x1b2c] = 0x8966,
  [0x1b2d] = 0x8964, [0x1b2e] = 0x896d, [0x1b2f] = 0x896a, [0x1b30] = 0x896f,
  [0x1b31] = 0x8974, [0x1b32] = 0x8977, [0x1b33] = 0x897e, [0x1b34] = 0x8983,
  [0x1b35] = 0x8988, [0x1b36] = 0x898a, [0x1b37] = 0x8993, [0x1b38] = 0x8998,
  [0x1b39] = 0x89a1, [0x1b3a] = 0x89a9, [0x1b3b] = 0x89a6, [0x1b3c] = 0x89ac,
  [0x1b3d] = 0x89af, [0x1b3e] = 0x89b2, [0x1b3f] = 0x89ba, [0x1b40] = 0x89bd,
  [0x1b41] = 0x89bf, [0x1b42] = 0x89c0, [0x1b43] = 0x89da, [0x1b44] = 0x89dc,
  [0x1b45] = 0x89dd, [0x1b46] = 0x89e7, [0x1b47] = 0x89f4, [0x1b48] = 0x89f8,
  [0x1b49] = 0x8a03, [0x1b4a] = 0x8a16, [0x1b4b] = 0x8a10, [0x1b4c] = 0x8a0c,
  [0x1b4d] = 0x8a1b, [0x1b4e] = 0x8a1d, [0x1b4f] = 0x8a25, [0x1b50] = 0x8a36,
  [0x1b51] = 0x8a41, [0x1b52] = 0x8a5b, [0x1b53] = 0x8a52, [0x1b54] = 0x8a46,
  [0x1b55] = 0x8a48, [0x1b56] = 0x8a7c, [0x1b57] = 0x8a6d, [0x1b58] = 0x8a6c,
  [0x1b59] = 0x8a62, [0x1b5a] = 0x8a85, [0x1b5b] = 0x8a82, [0x1b5c] = 0x8a84,
  [0x1b5d] = 0x8aa8, [0x1b5e] = 0x8aa1, [0x1b5f] = 0x8a91, [0x1b60] = 0x8aa5,
  [0x1b61] = 0x8aa6, [0x1b62] = 0x8a9a, [0x1b63] = 0x8aa3, [0x1b64] = 0x8ac4,
  [0x1b65] = 0x8acd, [0x1b66] = 0x8ac2, [0x1b67] = 0x8ada, [0x1b68] = 0x8aeb,
  [0x1b69] = 0x8af3, [0x1b6a] = 0x8ae7, [0x1b6b] = 0x8ae4, [0x1b6c] = 0x8af1,
  [0x1b6d] = 0x8b14, [0x1b6e] = 0x8ae0, [0x1b6f] = 0x8ae2, [0x1b70] = 0x8af7,
  [0x1b71] = 0x8ade, [0x1b72] = 0x8adb, [0x1b73] = 0x8b0c, [0x1b74] = 0x8b07,
  [0x1b75] = 0x8b1a, [0x1b76] = 0x8ae1, [0x1b77] = 0x8b16, [0x1b78] = 0x8b10,
  [0x1b79] = 0x8b17, [0x1b7a] = 0x8b20, [0x1b7b] = 0x8b33, [0x1b7c] = 0x97ab,
  [0x1b7d] = 0x8b26, [0x1b7e] = 0x8b2b, [0x1b7f] = 0x8b3e, [0x1b80] = 0x8b28,
  [0x1b81] = 0x8b41, [0x1b82] = 0x8b4c, [0x1b83] = 0x8b4f, [0x1b84] = 0x8b4e,
  [0x1b85] = 0x8b49, [0x1b86] = 0x8b56, [0x1b87] = 0x8b5b, [0x1b88] = 0x8b5a,
  [0x1b89] = 0x8b6b, [0x1b8a] = 0x8b5f, [0x1b8b] = 0x8b6c, [0x1b8c] = 0x8b6f,
  [0x1b8d] = 0x8b74, [0x1b8e] = 0x8b7d, [0x1b8f] = 0x8b80, [0x1b90] = 0x8b8c,
  [0x1b91] = 0x8b8e, [0x1b92] = 0x8b92, [0x1b93] = 0x8b93, [0x1b94] = 0x8b96,
  [0x1b95] = 0x8b99, [0x1b96] = 0x8b9a, [0x1b97] = 0x8c3a, [0x1b98] = 0x8c41,
  [0x1b99] = 0x8c3f, [0x1b9a] = 0x8c48, [0x1b9b] = 0x8c4c, [0x1b9c] = 0x8c4e,
  [0x1b9d] = 0x8c50, [0x1b9e] = 0x8c55, [0x1b9f] = 0x8c62, [0x1ba0] = 0x8c6c,
  [0x1ba1] = 0x8c78, [0x1ba2] = 0x8c7a, [0x1ba3] = 0x8c82, [0x1ba4] = 0x8c89,
  [0x1ba5] = 0x8c85, [0x1ba6] = 0x8c8a, [0x1ba7] = 0x8c8d, [0x1ba8] = 0x8c8e,
  [0x1ba9] = 0x8c94, [0x1baa] = 0x8c7c, [0x1bab] = 0x8c98, [0x1bac] = 0x621d,
  [0x1bad] = 0x8cad, [0x1bae] = 0x8caa, [0x1baf] = 0x8cbd, [0x1bb0] = 0x8cb2,
  [0x1bb1] = 0x8cb3, [0x1bb2] = 0x8cae, [0x1bb3] = 0x8cb6, [0x1bb4] = 0x8cc8,
  [0x1bb5] = 0x8cc1, [0x1bb6] = 0x8ce4, [0x1bb7] = 0x8ce3, [0x1bb8] = 0x8cda,
  [0x1bb9] = 0x8cfd, [0x1bba] = 0x8cfa, [0x1bbb] = 0x8cfb, [0x1bbc] = 0x8d04,
  [0x1bbd] = 0x8d05, [0x1bbe] = 0x8d0a, [0x1bbf] = 0x8d07, [0x1bc0] = 0x8d0f,
  [0x1bc1] = 0x8d0d, [0x1bc2] = 0x8d10, [0x1bc3] = 0x9f4e, [0x1bc4] = 0x8d13,
  [0x1bc5] = 0x8ccd, [0x1bc6] = 0x8d14, [0x1bc7] = 0x8d16, [0x1bc8] = 0x8d67,
  [0x1bc9] = 0x8d6d, [0x1bca] = 0x8d71, [0x1bcb] = 0x8d73, [0x1bcc] = 0x8d81,
  [0x1bcd] = 0x8d99, [0x1bce] = 0x8dc2, [0x1bcf] = 0x8dbe, [0x1bd0] = 0x8dba,
  [0x1bd1] = 0x8dcf, [0x1bd2] = 0x8dda, [0x1bd3] = 0x8dd6, [0x1bd4] = 0x8dcc,
  [0x1bd5] = 0x8ddb, [0x1bd6] = 0x8dcb, [0x1bd7] = 0x8dea, [0x1bd8] = 0x8deb,
  [0x1bd9] = 0x8ddf, [0x1bda] = 0x8de3, [0x1bdb] = 0x8dfc, [0x1bdc] = 0x8e08,
  [0x1bdd] = 0x8e09, [0x1bde] = 0x8dff, [0x1bdf] = 0x8e1d, [0x1be0] = 0x8e1e,
  [0x1be1] = 0x8e10, [0x1be2] = 0x8e1f, [0x1be3] = 0x8e42, [0x1be4] = 0x8e35,
  [0x1be5] = 0x8e30, [0x1be6] = 0x8e34, [0x1be7] = 0x8e4a, [0x1be8] = 0x8e47,
  [0x1be9] = 0x8e49, [0x1bea] = 0x8e4c, [0x1beb] = 0x8e50, [0x1bec] = 0x8e48,
  [0x1bed] = 0x8e59, [0x1bee] = 0x8e64, [0x1bef] = 0x8e60, [0x1bf0] = 0x8e2a,
  [0x1bf1] = 0x8e63, [0x1bf2] = 0x8e55, [0x1bf3] = 0x8e76, [0x1bf4] = 0x8e72,
  [0x1bf5] = 0x8e7c, [0x1bf6] = 0x8e81, [0x1bf7] = 0x8e87, [0x1bf8] = 0x8e85,
  [0x1bf9] = 0x8e84, [0x1bfa] = 0x8e8b, [0x1bfb] = 0x8e8a, [0x1bfc] = 0x8e93,
  [0x1bfd] = 0x8e91, [0x1bfe] = 0x8e94, [0x1bff] = 0x8e99, [0x1c00] = 0x8eaa,
  [0x1c01] = 0x8ea1, [0x1c02] = 0x8eac, [0x1c03] = 0x8eb0, [0x1c04] = 0x8ec6,
  [0x1c05] = 0x8eb1, [0x1c06] = 0x8ebe, [0x1c07] = 0x8ec5, [0x1c08] = 0x8ec8,
  [0x1c09] = 0x8ecb, [0x1c0a] = 0x8edb, [0x1c0b] = 0x8ee3, [0x1c0c] = 0x8efc,
  [0x1c0d] = 0x8efb, [0x1c0e] = 0x8eeb, [0x1c0f] = 0x8efe, [0x1c10] = 0x8f0a,
  [0x1c11] = 0x8f05, [0x1c12] = 0x8f15, [0x1c13] = 0x8f12, [0x1c14] = 0x8f19,
  [0x1c15] = 0x8f13, [0x1c16] = 0x8f1c, [0x1c17] = 0x8f1f, [0x1c18] = 0x8f1b,
  [0x1c19] = 0x8f0c, [0x1c1a] = 0x8f26, [0x1c1b] = 0x8f33, [0x1c1c] = 0x8f3b,
  [0x1c1d] = 0x8f39, [0x1c1e] = 0x8f45, [0x1c1f] = 0x8f42, [0x1c20] = 0x8f3e,
  [0x1c21] = 0x8f4c, [0x1c22] = 0x8f49, [0x1c23] = 0x8f46, [0x1c24] = 0x8f4e,
  [0x1c25] = 0x8f57, [0x1c26] = 0x8f5c, [0x1c27] = 0x8f62, [0x1c28] = 0x8f63,
  [0x1c29] = 0x8f64, [0x1c2a] = 0x8f9c, [0x1c2b] = 0x8f9f, [0x1c2c] = 0x8fa3,
  [0x1c2d] = 0x8fad, [0x1c2e] = 0x8faf, [0x1c2f] = 0x8fb7, [0x1c30] = 0x8fda,
  [0x1c31] = 0x8fe5, [0x1c32] = 0x8fe2, [0x1c33] = 0x8fea, [0x1c34] = 0x8fef,
  [0x1c35] = 0x9087, [0x1c36] = 0x8ff4, [0x1c37] = 0x9005, [0x1c38] = 0x8ff9,
  [0x1c39] = 0x8ffa, [0x1c3a] = 0x9011, [0x1c3b] = 0x9015, [0x1c3c] = 0x9021,
  [0x1c3d] = 0x900d, [0x1c3e] = 0x901e, [0x1c3f] = 0x9016, [0x1c40] = 0x900b,
  [0x1c41] = 0x9027, [0x1c42] = 0x9036, [0x1c43] = 0x9035, [0x1c44] = 0x9039,
  [0x1c45] = 0x8ff8, [0x1c46] = 0x904f, [0x1c47] = 0x9050, [0x1c48] = 0x9051,
  [0x1c49] = 0x9052, [0x1c4a] = 0x900e, [0x1c4b] = 0x9049, [0x1c4c] = 0x903e,
  [0x1c4d] = 0x9056, [0x1c4e] = 0x9058, [0x1c4f] = 0x905e, [0x1c50] = 0x9068,
  [0x1c51] = 0x906f, [0x1c52] = 0x9076, [0x1c53] = 0x96a8, [0x1c54] = 0x9072,
  [0x1c55] = 0x9082, [0x1c56] = 0x907d, [0x1c57] = 0x9081, [0x1c58] = 0x9080,
  [0x1c59] = 0x908a, [0x1c5a] = 0x9089, [0x1c5b] = 0x908f, [0x1c5c] = 0x90a8,
  [0x1c5d] = 0x90af, [0x1c5e] = 0x90b1, [0x1c5f] = 0x90b5, [0x1c60] = 0x90e2,
  [0x1c61] = 0x90e4, [0x1c62] = 0x6248, [0x1c63] = 0x90db, [0x1c64] = 0x9102,
  [0x1c65] = 0x9112, [0x1c66] = 0x9119, [0x1c67] = 0x9132, [0x1c68] = 0x9130,
  [0x1c69] = 0x914a, [0x1c6a] = 0x9156, [0x1c6b] = 0x9158, [0x1c6c] = 0x9163,
  [0x1c6d] = 0x9165, [0x1c6e] = 0x9169, [0x1c6f] = 0x9173, [0x1c70] = 0x9172,
  [0x1c71] = 0x918b, [0x1c72] = 0x9189, [0x1c73] = 0x9182, [0x1c74] = 0x91a2,
  [0x1c75] = 0x91ab, [0x1c76] = 0x91af, [0x1c77] = 0x91aa, [0x1c78] = 0x91b5,
  [0x1c79] = 0x91b4, [0x1c7a] = 0x91ba, [0x1c7b] = 0x91c0, [0x1c7c] = 0x91c1,
  [0x1c7d] = 0x91c9, [0x1c7e] = 0x91cb, [0x1c7f] = 0x91d0, [0x1c80] = 0x91d6,
  [0x1c81] = 0x91df, [0x1c82] = 0x91e1, [0x1c83] = 0x91db, [0x1c84] = 0x91fc,
  [0x1c85] = 0x91f5, [0x1c86] = 0x91f6, [0x1c87] = 0x921e, [0x1c88] = 0x91ff,
  [0x1c89] = 0x9214, [0x1c8a] = 0x922c, [0x1c8b] = 0x9215, [0x1c8c] = 0x9211,
  [0x1c8d] = 0x925e, [0x1c8e] = 0x9257, [0x1c8f] = 0x9245, [0x1c90] = 0x9249,
  [0x1c91] = 0x9264, [0x1c92] = 0x9248, [0x1c93] = 0x9295, [0x1c94] = 0x923f,
  [0x1c95] = 0x924b, [0x1c96] = 0x9250, [0x1c97] = 0x929c, [0x1c98] = 0x9296,
  [0x1c99] = 0x9293, [0x1c9a] = 0x929b, [0x1c9b] = 0x925a, [0x1c9c] = 0x92cf,
  [0x1c9d] = 0x92b9, [0x1c9e] = 0x92b7, [0x1c9f] = 0x92e9, [0x1ca0] = 0x930f,
  [0x1ca1] = 0x92fa, [0x1ca2] = 0x9344, [0x1ca3] = 0x932e, [0x1ca4] = 0x9319,
  [0x1ca5] = 0x9322, [0x1ca6] = 0x931a, [0x1ca7] = 0x9323, [0x1ca8] = 0x933a,
  [0x1ca9] = 0x9335, [0x1caa] = 0x933b, [0x1cab] = 0x935c, [0x1cac] = 0x9360,
  [0x1cad] = 0x937c, [0x1cae] = 0x936e, [0x1caf] = 0x9356, [0x1cb0] = 0x93b0,
  [0x1cb1] = 0x93ac, [0x1cb2] = 0x93ad, [0x1cb3] = 0x9394, [0x1cb4] = 0x93b9,
  [0x1cb5] = 0x93d6, [0x1cb6] = 0x93d7, [0x1cb7] = 0x93e8, [0x1cb8] = 0x93e5,
  [0x1cb9] = 0x93d8, [0x1cba] = 0x93c3, [0x1cbb] = 0x93dd, [0x1cbc] = 0x93d0,
  [0x1cbd] = 0x93c8, [0x1cbe] = 0x93e4, [0x1cbf] = 0x941a, [0x1cc0] = 0x9414,
  [0x1cc1] = 0x9413, [0x1cc2] = 0x9403, [0x1cc3] = 0x9407, [0x1cc4] = 0x9410,
  [0x1cc5] = 0x9436, [0x1cc6] = 0x942b, [0x1cc7] = 0x9435, [0x1cc8] = 0x9421,
  [0x1cc9] = 0x943a, [0x1cca] = 0x9441, [0x1ccb] = 0x9452, [0x1ccc] = 0x9444,
  [0x1ccd] = 0x945b, [0x1cce] = 0x9460, [0x1ccf] = 0x9462, [0x1cd0] = 0x945e,
  [0x1cd1] = 0x946a, [0x1cd2] = 0x9229, [0x1cd3] = 0x9470, [0x1cd4] = 0x9475,
  [0x1cd5] = 0x9477, [0x1cd6] = 0x947d, [0x1cd7] = 0x945a, [0x1cd8] = 0x947c,
  [0x1cd9] = 0x947e, [0x1cda] = 0x9481, [0x1cdb] = 0x947f, [0x1cdc] = 0x9582,
  [0x1cdd] = 0x9587, [0x1cde] = 0x958a, [0x1cdf] = 0x9594, [0x1ce0] = 0x9596,
  [0x1ce1] = 0x9598, [0x1ce2] = 0x9599, [0x1ce3] = 0x95a0, [0x1ce4] = 0x95a8,
  [0x1ce5] = 0x95a7, [0x1ce6] = 0x95ad, [0x1ce7] = 0x95bc, [0x1ce8] = 0x95bb,
  [0x1ce9] = 0x95b9, [0x1cea] = 0x95be, [0x1ceb] = 0x95ca, [0x1cec] = 0x6ff6,
  [0x1ced] = 0x95c3, [0x1cee] = 0x95cd, [0x1cef] = 0x95cc, [0x1cf0] = 0x95d5,
  [0x1cf1] = 0x95d4, [0x1cf2] = 0x95d6, [0x1cf3] = 0x95dc, [0x1cf4] = 0x95e1,
  [0x1cf5] = 0x95e5, [0x1cf6] = 0x95e2, [0x1cf7] = 0x9621, [0x1cf8] = 0x9628,
  [0x1cf9] = 0x962e, [0x1cfa] = 0x962f, [0x1cfb] = 0x9642, [0x1cfc] = 0x964c,
  [0x1cfd] = 0x964f, [0x1cfe] = 0x964b, [0x1cff] = 0x9677, [0x1d00] = 0x965c,
  [0x1d01] = 0x965e, [0x1d02] = 0x965d, [0x1d03] = 0x965f, [0x1d04] = 0x9666,
  [0x1d05] = 0x9672, [0x1d06] = 0x966c, [0x1d07] = 0x968d, [0x1d08] = 0x9698,
  [0x1d09] = 0x9695, [0x1d0a] = 0x9697, [0x1d0b] = 0x96aa, [0x1d0c] = 0x96a7,
  [0x1d0d] = 0x96b1, [0x1d0e] = 0x96b2, [0x1d0f] = 0x96b0, [0x1d10] = 0x96b4,
  [0x1d11] = 0x96b6, [0x1d12] = 0x96b8, [0x1d13] = 0x96b9, [0x1d14] = 0x96ce,
  [0x1d15] = 0x96cb, [0x1d16] = 0x96c9, [0x1d17] = 0x96cd, [0x1d18] = 0x894d,
  [0x1d19] = 0x96dc, [0x1d1a] = 0x970d, [0x1d1b] = 0x96d5, [0x1d1c] = 0x96f9,
  [0x1d1d] = 0x9704, [0x1d1e] = 0x9706, [0x1d1f] = 0x9708, [0x1d20] = 0x9713,
  [0x1d21] = 0x970e, [0x1d22] = 0x9711, [0x1d23] = 0x970f, [0x1d24] = 0x9716,
  [0x1d25] = 0x9719, [0x1d26] = 0x9724, [0x1d27] = 0x972a, [0x1d28] = 0x9730,
  [0x1d29] = 0x9739, [0x1d2a] = 0x973d, [0x1d2b] = 0x973e, [0x1d2c] = 0x9744,
  [0x1d2d] = 0x9746, [0x1d2e] = 0x9748, [0x1d2f] = 0x9742, [0x1d30] = 0x9749,
  [0x1d31] = 0x975c, [0x1d32] = 0x9760, [0x1d33] = 0x9764, [0x1d34] = 0x9766,
  [0x1d35] = 0x9768, [0x1d36] = 0x52d2, [0x1d37] = 0x976b, [0x1d38] = 0x9771,
  [0x1d39] = 0x9779, [0x1d3a] = 0x9785, [0x1d3b] = 0x977c, [0x1d3c] = 0x9781,
  [0x1d3d] = 0x977a, [0x1d3e] = 0x9786, [0x1d3f] = 0x978b, [0x1d40] = 0x978f,
  [0x1d41] = 0x9790, [0x1d42] = 0x979c, [0x1d43] = 0x97a8, [0x1d44] = 0x97a6,
  [0x1d45] = 0x97a3, [0x1d46] = 0x97b3, [0x1d47] = 0x97b4, [0x1d48] = 0x97c3,
  [0x1d49] = 0x97c6, [0x1d4a] = 0x97c8, [0x1d4b] = 0x97cb, [0x1d4c] = 0x97dc,
  [0x1d4d] = 0x97ed, [0x1d4e] = 0x9f4f, [0x1d4f] = 0x97f2, [0x1d50] = 0x7adf,
  [0x1d51] = 0x97f6, [0x1d52] = 0x97f5, [0x1d53] = 0x980f, [0x1d54] = 0x980c,
  [0x1d55] = 0x9838, [0x1d56] = 0x9824, [0x1d57] = 0x9821, [0x1d58] = 0x9837,
  [0x1d59] = 0x983d, [0x1d5a] = 0x9846, [0x1d5b] = 0x984f, [0x1d5c] = 0x984b,
  [0x1d5d] = 0x986b, [0x1d5e] = 0x986f, [0x1d5f] = 0x9870, [0x1d60] = 0x9871,
  [0x1d61] = 0x9874, [0x1d62] = 0x9873, [0x1d63] = 0x98aa, [0x1d64] = 0x98af,
  [0x1d65] = 0x98b1, [0x1d66] = 0x98b6, [0x1d67] = 0x98c4, [0x1d68] = 0x98c3,
  [0x1d69] = 0x98c6, [0x1d6a] = 0x98e9, [0x1d6b] = 0x98eb, [0x1d6c] = 0x9903,
  [0x1d6d] = 0x9909, [0x1d6e] = 0x9912, [0x1d6f] = 0x9914, [0x1d70] = 0x9918,
  [0x1d71] = 0x9921, [0x1d72] = 0x991d, [0x1d73] = 0x991e, [0x1d74] = 0x9924,
  [0x1d75] = 0x9920, [0x1d76] = 0x992c, [0x1d77] = 0x992e, [0x1d78] = 0x993d,
  [0x1d79] = 0x993e, [0x1d7a] = 0x9942, [0x1d7b] = 0x9949, [0x1d7c] = 0x9945,
  [0x1d7d] = 0x9950, [0x1d7e] = 0x994b, [0x1d7f] = 0x9951, [0x1d80] = 0x9952,
  [0x1d81] = 0x994c, [0x1d82] = 0x9955, [0x1d83] = 0x9997, [0x1d84] = 0x9998,
  [0x1d85] = 0x99a5, [0x1d86] = 0x99ad, [0x1d87] = 0x99ae, [0x1d88] = 0x99bc,
  [0x1d89] = 0x99df, [0x1d8a] = 0x99db, [0x1d8b] = 0x99dd, [0x1d8c] = 0x99d8,
  [0x1d8d] = 0x99d1, [0x1d8e] = 0x99ed, [0x1d8f] = 0x99ee, [0x1d90] = 0x99f1,
  [0x1d91] = 0x99f2, [0x1d92] = 0x99fb, [0x1d93] = 0x99f8, [0x1d94] = 0x9a01,
  [0x1d95] = 0x9a0f, [0x1d96] = 0x9a05, [0x1d97] = 0x99e2, [0x1d98] = 0x9a19,
  [0x1d99] = 0x9a2b, [0x1d9a] = 0x9a37, [0x1d9b] = 0x9a45, [0x1d9c] = 0x9a42,
  [0x1d9d] = 0x9a40, [0x1d9e] = 0x9a43, [0x1d9f] = 0x9a3e, [0x1da0] = 0x9a55,
  [0x1da1] = 0x9a4d, [0x1da2] = 0x9a5b, [0x1da3] = 0x9a57, [0x1da4] = 0x9a5f,
  [0x1da5] = 0x9a62, [0x1da6] = 0x9a65, [0x1da7] = 0x9a64, [0x1da8] = 0x9a69,
  [0x1da9] = 0x9a6b, [0x1daa] = 0x9a6a, [0x1dab] = 0x9aad, [0x1dac] = 0x9ab0,
  [0x1dad] = 0x9abc, [0x1dae] = 0x9ac0, [0x1daf] = 0x9acf, [0x1db0] = 0x9ad1,
  [0x1db1] = 0x9ad3, [0x1db2] = 0x9ad4, [0x1db3] = 0x9ade, [0x1db4] = 0x9adf,
  [0x1db5] = 0x9ae2, [0x1db6] = 0x9ae3, [0x1db7] = 0x9ae6, [0x1db8] = 0x9aef,
  [0x1db9] = 0x9aeb, [0x1dba] = 0x9aee, [0x1dbb] = 0x9af4, [0x1dbc] = 0x9af1,
  [0x1dbd] = 0x9af7, [0x1dbe] = 0x9afb, [0x1dbf] = 0x9b06, [0x1dc0] = 0x9b18,
  [0x1dc1] = 0x9b1a, [0x1dc2] = 0x9b1f, [0x1dc3] = 0x9b22, [0x1dc4] = 0x9b23,
  [0x1dc5] = 0x9b25, [0x1dc6] = 0x9b27, [0x1dc7] = 0x9b28, [0x1dc8] = 0x9b29,
  [0x1dc9] = 0x9b2a, [0x1dca] = 0x9b2e, [0x1dcb] = 0x9b2f, [0x1dcc] = 0x9b32,
  [0x1dcd] = 0x9b44, [0x1dce] = 0x9b43, [0x1dcf] = 0x9b4f, [0x1dd0] = 0x9b4d,
  [0x1dd1] = 0x9b4e, [0x1dd2] = 0x9b51, [0x1dd3] = 0x9b58, [0x1dd4] = 0x9b74,
  [0x1dd5] = 0x9b93, [0x1dd6] = 0x9b83, [0x1dd7] = 0x9b91, [0x1dd8] = 0x9b96,
  [0x1dd9] = 0x9b97, [0x1dda] = 0x9b9f, [0x1ddb] = 0x9ba0, [0x1ddc] = 0x9ba8,
  [0x1ddd] = 0x9bb4, [0x1dde] = 0x9bc0, [0x1ddf] = 0x9bca, [0x1de0] = 0x9bb9,
  [0x1de1] = 0x9bc6, [0x1de2] = 0x9bcf, [0x1de3] = 0x9bd1, [0x1de4] = 0x9bd2,
  [0x1de5] = 0x9be3, [0x1de6] = 0x9be2, [0x1de7] = 0x9be4, [0x1de8] = 0x9bd4,
  [0x1de9] = 0x9be1, [0x1dea] = 0x9c3a, [0x1deb] = 0x9bf2, [0x1dec] = 0x9bf1,
  [0x1ded] = 0x9bf0, [0x1dee] = 0x9c15, [0x1def] = 0x9c14, [0x1df0] = 0x9c09,
  [0x1df1] = 0x9c13, [0x1df2] = 0x9c0c, [0x1df3] = 0x9c06, [0x1df4] = 0x9c08,
  [0x1df5] = 0x9c12, [0x1df6] = 0x9c0a, [0x1df7] = 0x9c04, [0x1df8] = 0x9c2e,
  [0x1df9] = 0x9c1b, [0x1dfa] = 0x9c25, [0x1dfb] = 0x9c24, [0x1dfc] = 0x9c21,
  [0x1dfd] = 0x9c30, [0x1dfe] = 0x9c47, [0x1dff] = 0x9c32, [0x1e00] = 0x9c46,
  [0x1e01] = 0x9c3e, [0x1e02] = 0x9c5a, [0x1e03] = 0x9c60, [0x1e04] = 0x9c67,
  [0x1e05] = 0x9c76, [0x1e06] = 0x9c78, [0x1e07] = 0x9ce7, [0x1e08] = 0x9cec,
  [0x1e09] = 0x9cf0, [0x1e0a] = 0x9d09, [0x1e0b] = 0x9d08, [0x1e0c] = 0x9ceb,
  [0x1e0d] = 0x9d03, [0x1e0e] = 0x9d06, [0x1e0f] = 0x9d2a, [0x1e10] = 0x9d26,
  [0x1e11] = 0x9daf, [0x1e12] = 0x9d23, [0x1e13] = 0x9d1f, [0x1e14] = 0x9d44,
  [0x1e15] = 0x9d15, [0x1e16] = 0x9d12, [0x1e17] = 0x9d41, [0x1e18] = 0x9d3f,
  [0x1e19] = 0x9d3e, [0x1e1a] = 0x9d46, [0x1e1b] = 0x9d48, [0x1e1c] = 0x9d5d,
  [0x1e1d] = 0x9d5e, [0x1e1e] = 0x9d64, [0x1e1f] = 0x9d51, [0x1e20] = 0x9d50,
  [0x1e21] = 0x9d59, [0x1e22] = 0x9d72, [0x1e23] = 0x9d89, [0x1e24] = 0x9d87,
  [0x1e25] = 0x9dab, [0x1e26] = 0x9d6f, [0x1e27] = 0x9d7a, [0x1e28] = 0x9d9a,
  [0x1e29] = 0x9da4, [0x1e2a] = 0x9da9, [0x1e2b] = 0x9db2, [0x1e2c] = 0x9dc4,
  [0x1e2d] = 0x9dc1, [0x1e2e] = 0x9dbb, [0x1e2f] = 0x9db8, [0x1e30] = 0x9dba,
  [0x1e31] = 0x9dc6, [0x1e32] = 0x9dcf, [0x1e33] = 0x9dc2, [0x1e34] = 0x9dd9,
  [0x1e35] = 0x9dd3, [0x1e36] = 0x9df8, [0x1e37] = 0x9de6, [0x1e38] = 0x9ded,
  [0x1e39] = 0x9def, [0x1e3a] = 0x9dfd, [0x1e3b] = 0x9e1a, [0x1e3c] = 0x9e1b,
  [0x1e3d] = 0x9e1e, [0x1e3e] = 0x9e75, [0x1e3f] = 0x9e79, [0x1e40] = 0x9e7d,
  [0x1e41] = 0x9e81, [0x1e42] = 0x9e88, [0x1e43] = 0x9e8b, [0x1e44] = 0x9e8c,
  [0x1e45] = 0x9e92, [0x1e46] = 0x9e95, [0x1e47] = 0x9e91, [0x1e48] = 0x9e9d,
  [0x1e49] = 0x9ea5, [0x1e4a] = 0x9ea9, [0x1e4b] = 0x9eb8, [0x1e4c] = 0x9eaa,
  [0x1e4d] = 0x9ead, [0x1e4e] = 0x9761, [0x1e4f] = 0x9ecc, [0x1e50] = 0x9ece,
  [0x1e51] = 0x9ecf, [0x1e52] = 0x9ed0, [0x1e53] = 0x9ed4, [0x1e54] = 0x9edc,
  [0x1e55] = 0x9ede, [0x1e56] = 0x9edd, [0x1e57] = 0x9ee0, [0x1e58] = 0x9ee5,
  [0x1e59] = 0x9ee8, [0x1e5a] = 0x9eef, [0x1e5b] = 0x9ef4, [0x1e5c] = 0x9ef6,
  [0x1e5d] = 0x9ef7, [0x1e5e] = 0x9ef9, [0x1e5f] = 0x9efb, [0x1e60] = 0x9efc,
  [0x1e61] = 0x9efd, [0x1e62] = 0x9f07, [0x1e63] = 0x9f08, [0x1e64] = 0x76b7,
  [0x1e65] = 0x9f15, [0x1e66] = 0x9f21, [0x1e67] = 0x9f2c, [0x1e68] = 0x9f3e,
  [0x1e69] = 0x9f4a, [0x1e6a] = 0x9f52, [0x1e6b] = 0x9f54, [0x1e6c] = 0x9f63,
  [0x1e6d] = 0x9f5f, [0x1e6e] = 0x9f60, [0x1e6f] = 0x9f61, [0x1e70] = 0x9f66,
  [0x1e71] = 0x9f67, [0x1e72] = 0x9f6c, [0x1e73] = 0x9f6a, [0x1e74] = 0x9f77,
  [0x1e75] = 0x9f72, [0x1e76] = 0x9f76, [0x1e77] = 0x9f95, [0x1e78] = 0x9f9c,
  [0x1e79] = 0x9fa0, [0x1e7a] = 0x582f, [0x1e7b] = 0x69c7, [0x1e7c] = 0x9059,
  [0x1e7d] = 0x7464, [0x1e7e] = 0x51dc, [0x1e7f] = 0x7199
};


const char __jisx0208_from_ucs4_lat1[256][2] =
{
  [0x005C] = "\x21\x40", [0x00A2] = "\x21\x71", [0x00A3] = "\x21\x72",
  [0x00A7] = "\x21\x78", [0x00A8] = "\x21\x2f", [0x00AC] = "\x22\x4c",
  [0x00B0] = "\x21\x6B", [0x00B1] = "\x21\x5e", [0x00B4] = "\x21\x2d",
  [0x00B6] = "\x22\x79", [0x00D7] = "\x21\x5f", [0x00F7] = "\x21\x60"
};


/* The following table can be generated using

   egrep '^0x' < .../eastasia/jis/jis0208.txt |
   awk '{ print $3, $2}' | egrep '^0x0[34]' | sort -u | perl tab.pl

   where tab.pl contains:

   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   $n=0;
   while (<>) {
     local($ucs4, $jis, %rest) = split;
     local($u)=hex($ucs4);
     local($s)=hex($jis);
     printf ("\n ") if ($n % 3 eq 0);
     ++$n;
     printf (" [0x%02x] = \"\\x%02x\\x%02x\",",
	     int(($u - 0x391) / 256) * 94 + (($u - 0x391) & 0xff),
	     int($s/256), $s % 256);
   }
   printf ("\n");
   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
const char __jisx0208_from_ucs4_greek[0xc1][2] =
{
  [0x00] = "\x26\x21", [0x01] = "\x26\x22", [0x02] = "\x26\x23",
  [0x03] = "\x26\x24", [0x04] = "\x26\x25", [0x05] = "\x26\x26",
  [0x06] = "\x26\x27", [0x07] = "\x26\x28", [0x08] = "\x26\x29",
  [0x09] = "\x26\x2a", [0x0a] = "\x26\x2b", [0x0b] = "\x26\x2c",
  [0x0c] = "\x26\x2d", [0x0d] = "\x26\x2e", [0x0e] = "\x26\x2f",
  [0x0f] = "\x26\x30", [0x10] = "\x26\x31", [0x12] = "\x26\x32",
  [0x13] = "\x26\x33", [0x14] = "\x26\x34", [0x15] = "\x26\x35",
  [0x16] = "\x26\x36", [0x17] = "\x26\x37", [0x18] = "\x26\x38",
  [0x20] = "\x26\x41", [0x21] = "\x26\x42", [0x22] = "\x26\x43",
  [0x23] = "\x26\x44", [0x24] = "\x26\x45", [0x25] = "\x26\x46",
  [0x26] = "\x26\x47", [0x27] = "\x26\x48", [0x28] = "\x26\x49",
  [0x29] = "\x26\x4a", [0x2a] = "\x26\x4b", [0x2b] = "\x26\x4c",
  [0x2c] = "\x26\x4d", [0x2d] = "\x26\x4e", [0x2e] = "\x26\x4f",
  [0x2f] = "\x26\x50", [0x30] = "\x26\x51", [0x32] = "\x26\x52",
  [0x33] = "\x26\x53", [0x34] = "\x26\x54", [0x35] = "\x26\x55",
  [0x36] = "\x26\x56", [0x37] = "\x26\x57", [0x38] = "\x26\x58",
  [0x70] = "\x27\x27", [0x7f] = "\x27\x21", [0x80] = "\x27\x22",
  [0x81] = "\x27\x23", [0x82] = "\x27\x24", [0x83] = "\x27\x25",
  [0x84] = "\x27\x26", [0x85] = "\x27\x28", [0x86] = "\x27\x29",
  [0x87] = "\x27\x2a", [0x88] = "\x27\x2b", [0x89] = "\x27\x2c",
  [0x8a] = "\x27\x2d", [0x8b] = "\x27\x2e", [0x8c] = "\x27\x2f",
  [0x8d] = "\x27\x30", [0x8e] = "\x27\x31", [0x8f] = "\x27\x32",
  [0x90] = "\x27\x33", [0x91] = "\x27\x34", [0x92] = "\x27\x35",
  [0x93] = "\x27\x36", [0x94] = "\x27\x37", [0x95] = "\x27\x38",
  [0x96] = "\x27\x39", [0x97] = "\x27\x3a", [0x98] = "\x27\x3b",
  [0x99] = "\x27\x3c", [0x9a] = "\x27\x3d", [0x9b] = "\x27\x3e",
  [0x9c] = "\x27\x3f", [0x9d] = "\x27\x40", [0x9e] = "\x27\x41",
  [0x9f] = "\x27\x51", [0xa0] = "\x27\x52", [0xa1] = "\x27\x53",
  [0xa2] = "\x27\x54", [0xa3] = "\x27\x55", [0xa4] = "\x27\x56",
  [0xa5] = "\x27\x58", [0xa6] = "\x27\x59", [0xa7] = "\x27\x5a",
  [0xa8] = "\x27\x5b", [0xa9] = "\x27\x5c", [0xaa] = "\x27\x5d",
  [0xab] = "\x27\x5e", [0xac] = "\x27\x5f", [0xad] = "\x27\x60",
  [0xae] = "\x27\x61", [0xaf] = "\x27\x62", [0xb0] = "\x27\x63",
  [0xb1] = "\x27\x64", [0xb2] = "\x27\x65", [0xb3] = "\x27\x66",
  [0xb4] = "\x27\x67", [0xb5] = "\x27\x68", [0xb6] = "\x27\x69",
  [0xb7] = "\x27\x6a", [0xb8] = "\x27\x6b", [0xb9] = "\x27\x6c",
  [0xba] = "\x27\x6d", [0xbb] = "\x27\x6e", [0xbc] = "\x27\x6f",
  [0xbd] = "\x27\x70", [0xbe] = "\x27\x71", [0xc0] = "\x27\x57"
};


/* The mapping of all the other UCS values is quite difficult since they
   are distributed from 0x2010 upto to 0xffe5.  There are many gaps with
   unpredictable place and size.  Therefore we use a table which maps
   ranges to idexes in a table.  This table ca be generated with:

   egrep '^0x' < .../eastasia/jis/jis0208.txt | awk '{ print $3, $2}' |
   egrep '^0x[^01]' | sort -u | perl tab.pl

   where tab.pl is

   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   $first=$last=$idx=0;
   while (<>) {
     local($ucs,$jis,%rest) = split;
     local($u)=hex($ucs);
     if ($u - $last > 6) {
       if ($last != 0) {
	 printf (" { start: %#06x, end: %#06x, idx: %5d },\n",
		 $first, $last, $idx);
	 $idx += $last - $first + 1;
       }
       $first=$u;
     }
     $last=$u;
   }
   printf (" { start: %#06x, end: %#06x, idx: %5d }\n",
	   $first, $last, $idx);
   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

const struct jisx0208_ucs_idx __jisx0208_from_ucs_idx[] =
{
  { start: 0x2010, end: 0x2026, idx:     0 },
  { start: 0x2030, end: 0x2033, idx:    23 },
  { start: 0x203b, end: 0x203b, idx:    27 },
  { start: 0x2103, end: 0x2103, idx:    28 },
  { start: 0x212b, end: 0x212b, idx:    29 },
  { start: 0x2190, end: 0x2193, idx:    30 },
  { start: 0x21d2, end: 0x21d4, idx:    34 },
  { start: 0x2200, end: 0x220b, idx:    37 },
  { start: 0x2212, end: 0x2212, idx:    49 },
  { start: 0x221a, end: 0x2220, idx:    50 },
  { start: 0x2227, end: 0x222c, idx:    57 },
  { start: 0x2234, end: 0x2235, idx:    63 },
  { start: 0x223d, end: 0x223d, idx:    65 },
  { start: 0x2252, end: 0x2252, idx:    66 },
  { start: 0x2260, end: 0x226b, idx:    67 },
  { start: 0x2282, end: 0x2287, idx:    79 },
  { start: 0x22a5, end: 0x22a5, idx:    85 },
  { start: 0x2312, end: 0x2312, idx:    86 },
  { start: 0x2500, end: 0x2503, idx:    87 },
  { start: 0x250c, end: 0x2542, idx:    91 },
  { start: 0x254b, end: 0x254b, idx:   146 },
  { start: 0x25a0, end: 0x25a1, idx:   147 },
  { start: 0x25b2, end: 0x25b3, idx:   149 },
  { start: 0x25bc, end: 0x25bd, idx:   151 },
  { start: 0x25c6, end: 0x25cf, idx:   153 },
  { start: 0x25ef, end: 0x25ef, idx:   163 },
  { start: 0x2605, end: 0x2606, idx:   164 },
  { start: 0x2640, end: 0x2642, idx:   166 },
  { start: 0x266a, end: 0x266f, idx:   169 },
  { start: 0x3000, end: 0x3015, idx:   175 },
  { start: 0x301c, end: 0x301c, idx:   197 },
  { start: 0x3041, end: 0x3093, idx:   198 },
  { start: 0x309b, end: 0x30fe, idx:   281 },
  { start: 0x4e00, end: 0x4e62, idx:   381 },
  { start: 0x4e71, end: 0x4e73, idx:   480 },
  { start: 0x4e7e, end: 0x4ee5, idx:   483 },
  { start: 0x4eed, end: 0x4f01, idx:   587 },
  { start: 0x4f09, end: 0x4f11, idx:   608 },
  { start: 0x4f1a, end: 0x4f1d, idx:   617 },
  { start: 0x4f2f, end: 0x4f5e, idx:   621 },
  { start: 0x4f69, end: 0x4fa1, idx:   669 },
  { start: 0x4fab, end: 0x4fb6, idx:   726 },
  { start: 0x4fbf, end: 0x4fe5, idx:   738 },
  { start: 0x4fee, end: 0x4ffe, idx:   777 },
  { start: 0x5005, end: 0x502d, idx:   794 },
  { start: 0x5036, end: 0x5039, idx:   835 },
  { start: 0x5043, end: 0x505c, idx:   839 },
  { start: 0x5065, end: 0x5065, idx:   865 },
  { start: 0x506c, end: 0x5085, idx:   866 },
  { start: 0x508d, end: 0x5091, idx:   892 },
  { start: 0x5098, end: 0x509a, idx:   897 },
  { start: 0x50ac, end: 0x50b7, idx:   900 },
  { start: 0x50be, end: 0x50ee, idx:   912 },
  { start: 0x50f5, end: 0x5109, idx:   961 },
  { start: 0x5112, end: 0x5121, idx:   982 },
  { start: 0x512a, end: 0x512a, idx:   998 },
  { start: 0x5132, end: 0x5199, idx:   999 },
  { start: 0x51a0, end: 0x51bd, idx:  1103 },
  { start: 0x51c4, end: 0x51cd, idx:  1133 },
  { start: 0x51d6, end: 0x521d, idx:  1143 },
  { start: 0x5224, end: 0x523b, idx:  1215 },
  { start: 0x5243, end: 0x5275, idx:  1239 },
  { start: 0x527d, end: 0x5294, idx:  1290 },
  { start: 0x529b, end: 0x52e7, idx:  1314 },
  { start: 0x52f2, end: 0x5323, idx:  1391 },
  { start: 0x532a, end: 0x5384, idx:  1441 },
  { start: 0x5396, end: 0x53bb, idx:  1532 },
  { start: 0x53c2, end: 0x53fa, idx:  1570 },
  { start: 0x5401, end: 0x5411, idx:  1627 },
  { start: 0x541b, end: 0x542e, idx:  1644 },
  { start: 0x5436, end: 0x5451, idx:  1664 },
  { start: 0x545f, end: 0x545f, idx:  1692 },
  { start: 0x5468, end: 0x5492, idx:  1693 },
  { start: 0x54a2, end: 0x54c9, idx:  1736 },
  { start: 0x54d8, end: 0x54d8, idx:  1776 },
  { start: 0x54e1, end: 0x54f2, idx:  1777 },
  { start: 0x54fa, end: 0x54fd, idx:  1795 },
  { start: 0x5504, end: 0x5507, idx:  1799 },
  { start: 0x550f, end: 0x5516, idx:  1803 },
  { start: 0x552e, end: 0x5563, idx:  1811 },
  { start: 0x557b, end: 0x558b, idx:  1865 },
  { start: 0x5598, end: 0x559f, idx:  1882 },
  { start: 0x55a7, end: 0x55b6, idx:  1890 },
  { start: 0x55c4, end: 0x55c7, idx:  1906 },
  { start: 0x55d4, end: 0x55e4, idx:  1910 },
  { start: 0x55f7, end: 0x55fe, idx:  1927 },
  { start: 0x5606, end: 0x5609, idx:  1935 },
  { start: 0x5614, end: 0x561b, idx:  1939 },
  { start: 0x5629, end: 0x5638, idx:  1947 },
  { start: 0x5642, end: 0x5642, idx:  1963 },
  { start: 0x564c, end: 0x5650, idx:  1964 },
  { start: 0x565b, end: 0x565b, idx:  1969 },
  { start: 0x5664, end: 0x566c, idx:  1970 },
  { start: 0x5674, end: 0x5694, idx:  1979 },
  { start: 0x56a0, end: 0x56a5, idx:  2012 },
  { start: 0x56ae, end: 0x56e3, idx:  2018 },
  { start: 0x56ee, end: 0x571f, idx:  2072 },
  { start: 0x5726, end: 0x5730, idx:  2122 },
  { start: 0x5737, end: 0x5751, idx:  2133 },
  { start: 0x5761, end: 0x576a, idx:  2160 },
  { start: 0x577f, end: 0x578b, idx:  2170 },
  { start: 0x5793, end: 0x5793, idx:  2183 },
  { start: 0x57a0, end: 0x57b3, idx:  2184 },
  { start: 0x57c0, end: 0x57e3, idx:  2204 },
  { start: 0x57f4, end: 0x580b, idx:  2240 },
  { start: 0x5815, end: 0x5841, idx:  2264 },
  { start: 0x584a, end: 0x5862, idx:  2309 },
  { start: 0x5869, end: 0x5885, idx:  2334 },
  { start: 0x5893, end: 0x589f, idx:  2363 },
  { start: 0x58a8, end: 0x5902, idx:  2376 },
  { start: 0x5909, end: 0x59b2, idx:  2467 },
  { start: 0x59b9, end: 0x59be, idx:  2637 },
  { start: 0x59c6, end: 0x59dc, idx:  2643 },
  { start: 0x59e5, end: 0x59eb, idx:  2666 },
  { start: 0x59f6, end: 0x5a09, idx:  2673 },
  { start: 0x5a11, end: 0x5a11, idx:  2693 },
  { start: 0x5a18, end: 0x5a49, idx:  2694 },
  { start: 0x5a5a, end: 0x5a5a, idx:  2744 },
  { start: 0x5a62, end: 0x5a6c, idx:  2745 },
  { start: 0x5a7f, end: 0x5a7f, idx:  2756 },
  { start: 0x5a92, end: 0x5a92, idx:  2757 },
  { start: 0x5a9a, end: 0x5a9b, idx:  2758 },
  { start: 0x5abc, end: 0x5ac2, idx:  2760 },
  { start: 0x5ac9, end: 0x5ad7, idx:  2767 },
  { start: 0x5ae1, end: 0x5ae9, idx:  2782 },
  { start: 0x5afa, end: 0x5afb, idx:  2791 },
  { start: 0x5b09, end: 0x5b0c, idx:  2793 },
  { start: 0x5b16, end: 0x5b16, idx:  2797 },
  { start: 0x5b22, end: 0x5b22, idx:  2798 },
  { start: 0x5b2a, end: 0x5b36, idx:  2799 },
  { start: 0x5b3e, end: 0x5b45, idx:  2812 },
  { start: 0x5b50, end: 0x5ba6, idx:  2820 },
  { start: 0x5bae, end: 0x5bd4, idx:  2907 },
  { start: 0x5bdb, end: 0x5c31, idx:  2946 },
  { start: 0x5c38, end: 0x5c55, idx:  3033 },
  { start: 0x5c5e, end: 0x5c65, idx:  3063 },
  { start: 0x5c6c, end: 0x5c79, idx:  3071 },
  { start: 0x5c8c, end: 0x5c94, idx:  3085 },
  { start: 0x5ca1, end: 0x5ca1, idx:  3094 },
  { start: 0x5ca8, end: 0x5cbe, idx:  3095 },
  { start: 0x5cc5, end: 0x5cc7, idx:  3118 },
  { start: 0x5cd9, end: 0x5cd9, idx:  3121 },
  { start: 0x5ce0, end: 0x5ce1, idx:  3122 },
  { start: 0x5ce8, end: 0x5cfd, idx:  3124 },
  { start: 0x5d07, end: 0x5d22, idx:  3146 },
  { start: 0x5d29, end: 0x5d29, idx:  3174 },
  { start: 0x5d4b, end: 0x5d52, idx:  3175 },
  { start: 0x5d5c, end: 0x5d5c, idx:  3183 },
  { start: 0x5d69, end: 0x5d76, idx:  3184 },
  { start: 0x5d82, end: 0x5d90, idx:  3198 },
  { start: 0x5d9d, end: 0x5da2, idx:  3213 },
  { start: 0x5dac, end: 0x5dae, idx:  3219 },
  { start: 0x5db7, end: 0x5dbd, idx:  3222 },
  { start: 0x5dc9, end: 0x5e1d, idx:  3229 },
  { start: 0x5e25, end: 0x5e57, idx:  3314 },
  { start: 0x5e5f, end: 0x5e64, idx:  3365 },
  { start: 0x5e72, end: 0x5ead, idx:  3371 },
  { start: 0x5eb5, end: 0x5eb8, idx:  3431 },
  { start: 0x5ec1, end: 0x5f41, idx:  3435 },
  { start: 0x5f48, end: 0x5fa1, idx:  3564 },
  { start: 0x5fa8, end: 0x5fc5, idx:  3654 },
  { start: 0x5fcc, end: 0x5fcd, idx:  3684 },
  { start: 0x5fd6, end: 0x5fe4, idx:  3686 },
  { start: 0x5feb, end: 0x5fff, idx:  3701 },
  { start: 0x600e, end: 0x6031, idx:  3722 },
  { start: 0x603a, end: 0x603a, idx:  3758 },
  { start: 0x6041, end: 0x6077, idx:  3759 },
  { start: 0x6081, end: 0x60aa, idx:  3814 },
  { start: 0x60b2, end: 0x60bd, idx:  3856 },
  { start: 0x60c5, end: 0x60c7, idx:  3868 },
  { start: 0x60d1, end: 0x60e8, idx:  3871 },
  { start: 0x60f0, end: 0x612c, idx:  3895 },
  { start: 0x6134, end: 0x6134, idx:  3956 },
  { start: 0x613c, end: 0x6177, idx:  3957 },
  { start: 0x617e, end: 0x619a, idx:  4017 },
  { start: 0x61a4, end: 0x61d0, idx:  4046 },
  { start: 0x61e3, end: 0x61e6, idx:  4091 },
  { start: 0x61f2, end: 0x6200, idx:  4095 },
  { start: 0x6208, end: 0x628a, idx:  4110 },
  { start: 0x6291, end: 0x629e, idx:  4241 },
  { start: 0x62ab, end: 0x62e1, idx:  4255 },
  { start: 0x62ec, end: 0x62f7, idx:  4310 },
  { start: 0x62fe, end: 0x6311, idx:  4322 },
  { start: 0x6319, end: 0x631f, idx:  4342 },
  { start: 0x6327, end: 0x632f, idx:  4349 },
  { start: 0x633a, end: 0x633f, idx:  4358 },
  { start: 0x6349, end: 0x635c, idx:  4364 },
  { start: 0x6367, end: 0x63fa, idx:  4384 },
  { start: 0x6406, end: 0x6406, idx:  4532 },
  { start: 0x640d, end: 0x641c, idx:  4533 },
  { start: 0x6426, end: 0x642d, idx:  4549 },
  { start: 0x6434, end: 0x6442, idx:  4557 },
  { start: 0x644e, end: 0x644e, idx:  4572 },
  { start: 0x6458, end: 0x6458, idx:  4573 },
  { start: 0x6467, end: 0x646f, idx:  4574 },
  { start: 0x6476, end: 0x647a, idx:  4583 },
  { start: 0x6483, end: 0x6488, idx:  4588 },
  { start: 0x6492, end: 0x64b2, idx:  4594 },
  { start: 0x64b9, end: 0x6505, idx:  4627 },
  { start: 0x6518, end: 0x6578, idx:  4704 },
  { start: 0x6582, end: 0x65b0, idx:  4801 },
  { start: 0x65b7, end: 0x65f1, idx:  4848 },
  { start: 0x65fa, end: 0x65fb, idx:  4907 },
  { start: 0x6602, end: 0x6614, idx:  4909 },
  { start: 0x661c, end: 0x6652, idx:  4928 },
  { start: 0x665d, end: 0x667a, idx:  4983 },
  { start: 0x6681, end: 0x66c9, idx:  5013 },
  { start: 0x66d6, end: 0x66e9, idx:  5086 },
  { start: 0x66f0, end: 0x671f, idx:  5106 },
  { start: 0x6726, end: 0x67b9, idx:  5154 },
  { start: 0x67c1, end: 0x6804, idx:  5302 },
  { start: 0x6813, end: 0x6817, idx:  5370 },
  { start: 0x681e, end: 0x6822, idx:  5375 },
  { start: 0x6829, end: 0x682b, idx:  5380 },
  { start: 0x6832, end: 0x6867, idx:  5383 },
  { start: 0x6874, end: 0x6877, idx:  5437 },
  { start: 0x687e, end: 0x6885, idx:  5441 },
  { start: 0x688d, end: 0x68bc, idx:  5449 },
  { start: 0x68c4, end: 0x68e7, idx:  5497 },
  { start: 0x68ee, end: 0x68f2, idx:  5533 },
  { start: 0x68f9, end: 0x6912, idx:  5538 },
  { start: 0x6919, end: 0x693f, idx:  5564 },
  { start: 0x694a, end: 0x694a, idx:  5603 },
  { start: 0x6953, end: 0x6962, idx:  5604 },
  { start: 0x696a, end: 0x6982, idx:  5620 },
  { start: 0x698a, end: 0x69a0, idx:  5645 },
  { start: 0x69a7, end: 0x69a7, idx:  5668 },
  { start: 0x69ae, end: 0x69b4, idx:  5669 },
  { start: 0x69bb, end: 0x69de, idx:  5676 },
  { start: 0x69e7, end: 0x69f2, idx:  5712 },
  { start: 0x69f9, end: 0x6a2e, idx:  5724 },
  { start: 0x6a35, end: 0x6a3d, idx:  5778 },
  { start: 0x6a44, end: 0x6a4b, idx:  5787 },
  { start: 0x6a58, end: 0x6a66, idx:  5795 },
  { start: 0x6a72, end: 0x6a78, idx:  5810 },
  { start: 0x6a7f, end: 0x6a84, idx:  5817 },
  { start: 0x6a8d, end: 0x6a90, idx:  5823 },
  { start: 0x6a97, end: 0x6aa3, idx:  5827 },
  { start: 0x6aaa, end: 0x6ac3, idx:  5840 },
  { start: 0x6ad1, end: 0x6ad3, idx:  5866 },
  { start: 0x6ada, end: 0x6adf, idx:  5869 },
  { start: 0x6ae8, end: 0x6aea, idx:  5875 },
  { start: 0x6afa, end: 0x6afb, idx:  5878 },
  { start: 0x6b04, end: 0x6b0a, idx:  5880 },
  { start: 0x6b12, end: 0x6b16, idx:  5887 },
  { start: 0x6b1d, end: 0x6b27, idx:  5892 },
  { start: 0x6b32, end: 0x6b8d, idx:  5903 },
  { start: 0x6b95, end: 0x6bdf, idx:  5995 },
  { start: 0x6beb, end: 0x6bf3, idx:  6070 },
  { start: 0x6c08, end: 0x6c08, idx:  6079 },
  { start: 0x6c0f, end: 0x6c1b, idx:  6080 },
  { start: 0x6c23, end: 0x6c24, idx:  6093 },
  { start: 0x6c34, end: 0x6c42, idx:  6095 },
  { start: 0x6c4e, end: 0x6c73, idx:  6110 },
  { start: 0x6c7a, end: 0x6ca2, idx:  6148 },
  { start: 0x6cab, end: 0x6ccc, idx:  6189 },
  { start: 0x6cd3, end: 0x6cf3, idx:  6223 },
  { start: 0x6d0b, end: 0x6d2b, idx:  6256 },
  { start: 0x6d32, end: 0x6d45, idx:  6289 },
  { start: 0x6d59, end: 0x6d5c, idx:  6309 },
  { start: 0x6d63, end: 0x6d79, idx:  6313 },
  { start: 0x6d85, end: 0x6d9c, idx:  6336 },
  { start: 0x6daf, end: 0x6dfb, idx:  6360 },
  { start: 0x6e05, end: 0x6e0b, idx:  6437 },
  { start: 0x6e13, end: 0x6e2f, idx:  6444 },
  { start: 0x6e38, end: 0x6e43, idx:  6473 },
  { start: 0x6e4a, end: 0x6e4e, idx:  6485 },
  { start: 0x6e56, end: 0x6e5f, idx:  6490 },
  { start: 0x6e67, end: 0x6e76, idx:  6500 },
  { start: 0x6e7e, end: 0x6e82, idx:  6516 },
  { start: 0x6e8c, end: 0x6ed5, idx:  6521 },
  { start: 0x6edd, end: 0x6ede, idx:  6595 },
  { start: 0x6eec, end: 0x6f15, idx:  6597 },
  { start: 0x6f20, end: 0x6f23, idx:  6639 },
  { start: 0x6f2b, end: 0x6f45, idx:  6643 },
  { start: 0x6f54, end: 0x6f66, idx:  6670 },
  { start: 0x6f6d, end: 0x6f86, idx:  6689 },
  { start: 0x6f8e, end: 0x6f97, idx:  6715 },
  { start: 0x6fa1, end: 0x6faa, idx:  6725 },
  { start: 0x6fb1, end: 0x6fb9, idx:  6735 },
  { start: 0x6fc0, end: 0x6fc6, idx:  6744 },
  { start: 0x6fd4, end: 0x6fe4, idx:  6751 },
  { start: 0x6feb, end: 0x7001, idx:  6768 },
  { start: 0x7009, end: 0x701f, idx:  6791 },
  { start: 0x7026, end: 0x7032, idx:  6814 },
  { start: 0x703e, end: 0x703e, idx:  6827 },
  { start: 0x704c, end: 0x7051, idx:  6828 },
  { start: 0x7058, end: 0x7058, idx:  6834 },
  { start: 0x7063, end: 0x7063, idx:  6835 },
  { start: 0x706b, end: 0x7070, idx:  6836 },
  { start: 0x7078, end: 0x707d, idx:  6842 },
  { start: 0x7089, end: 0x7092, idx:  6848 },
  { start: 0x7099, end: 0x7099, idx:  6858 },
  { start: 0x70ac, end: 0x70ba, idx:  6859 },
  { start: 0x70c8, end: 0x70cf, idx:  6874 },
  { start: 0x70d9, end: 0x70df, idx:  6882 },
  { start: 0x70f1, end: 0x70f1, idx:  6889 },
  { start: 0x70f9, end: 0x70fd, idx:  6890 },
  { start: 0x7109, end: 0x7109, idx:  6895 },
  { start: 0x7114, end: 0x7126, idx:  6896 },
  { start: 0x7136, end: 0x713c, idx:  6915 },
  { start: 0x7149, end: 0x714e, idx:  6922 },
  { start: 0x7155, end: 0x7159, idx:  6928 },
  { start: 0x7162, end: 0x716e, idx:  6933 },
  { start: 0x717d, end: 0x717d, idx:  6946 },
  { start: 0x7184, end: 0x719f, idx:  6947 },
  { start: 0x71a8, end: 0x71b1, idx:  6975 },
  { start: 0x71b9, end: 0x71d7, idx:  6985 },
  { start: 0x71df, end: 0x71ee, idx:  7016 },
  { start: 0x71f5, end: 0x71ff, idx:  7032 },
  { start: 0x7206, end: 0x7206, idx:  7043 },
  { start: 0x720d, end: 0x7210, idx:  7044 },
  { start: 0x721b, end: 0x721b, idx:  7048 },
  { start: 0x7228, end: 0x7269, idx:  7049 },
  { start: 0x7272, end: 0x7287, idx:  7115 },
  { start: 0x7292, end: 0x7296, idx:  7137 },
  { start: 0x72a0, end: 0x72b9, idx:  7142 },
  { start: 0x72c2, end: 0x72c6, idx:  7168 },
  { start: 0x72ce, end: 0x72e2, idx:  7173 },
  { start: 0x72e9, end: 0x72ed, idx:  7194 },
  { start: 0x72f7, end: 0x72fd, idx:  7199 },
  { start: 0x730a, end: 0x730a, idx:  7206 },
  { start: 0x7316, end: 0x7337, idx:  7207 },
  { start: 0x733e, end: 0x7345, idx:  7241 },
  { start: 0x734e, end: 0x734f, idx:  7249 },
  { start: 0x7357, end: 0x7357, idx:  7251 },
  { start: 0x7363, end: 0x737b, idx:  7252 },
  { start: 0x7384, end: 0x738b, idx:  7277 },
  { start: 0x7396, end: 0x7396, idx:  7285 },
  { start: 0x73a9, end: 0x73a9, idx:  7286 },
  { start: 0x73b2, end: 0x73b3, idx:  7287 },
  { start: 0x73bb, end: 0x73ce, idx:  7289 },
  { start: 0x73de, end: 0x73f1, idx:  7309 },
  { start: 0x73f8, end: 0x7409, idx:  7329 },
  { start: 0x7422, end: 0x7425, idx:  7347 },
  { start: 0x7432, end: 0x7441, idx:  7351 },
  { start: 0x7455, end: 0x7476, idx:  7367 },
  { start: 0x747e, end: 0x7483, idx:  7401 },
  { start: 0x748b, end: 0x748b, idx:  7407 },
  { start: 0x749e, end: 0x74a7, idx:  7408 },
  { start: 0x74b0, end: 0x74b0, idx:  7418 },
  { start: 0x74bd, end: 0x74bd, idx:  7419 },
  { start: 0x74ca, end: 0x74d4, idx:  7420 },
  { start: 0x74dc, end: 0x74f8, idx:  7431 },
  { start: 0x7503, end: 0x7505, idx:  7460 },
  { start: 0x750c, end: 0x753c, idx:  7463 },
  { start: 0x7544, end: 0x7578, idx:  7512 },
  { start: 0x757f, end: 0x75e9, idx:  7565 },
  { start: 0x75f0, end: 0x7601, idx:  7672 },
  { start: 0x7609, end: 0x760d, idx:  7690 },
  { start: 0x761f, end: 0x7627, idx:  7695 },
  { start: 0x7630, end: 0x7634, idx:  7704 },
  { start: 0x763b, end: 0x763b, idx:  7709 },
  { start: 0x7642, end: 0x769a, idx:  7710 },
  { start: 0x76ae, end: 0x770c, idx:  7799 },
  { start: 0x771b, end: 0x7729, idx:  7894 },
  { start: 0x7737, end: 0x7740, idx:  7909 },
  { start: 0x7747, end: 0x7747, idx:  7919 },
  { start: 0x775a, end: 0x776b, idx:  7920 },
  { start: 0x7779, end: 0x777f, idx:  7938 },
  { start: 0x778b, end: 0x7791, idx:  7945 },
  { start: 0x779e, end: 0x77a5, idx:  7952 },
  { start: 0x77ac, end: 0x77bf, idx:  7960 },
  { start: 0x77c7, end: 0x77cd, idx:  7980 },
  { start: 0x77d7, end: 0x77f3, idx:  7987 },
  { start: 0x77fc, end: 0x7802, idx:  8016 },
  { start: 0x780c, end: 0x7815, idx:  8023 },
  { start: 0x7820, end: 0x7827, idx:  8033 },
  { start: 0x7832, end: 0x7845, idx:  8041 },
  { start: 0x785d, end: 0x785d, idx:  8061 },
  { start: 0x786b, end: 0x7874, idx:  8062 },
  { start: 0x787c, end: 0x789a, idx:  8072 },
  { start: 0x78a3, end: 0x78da, idx:  8103 },
  { start: 0x78e7, end: 0x78f4, idx:  8159 },
  { start: 0x78fd, end: 0x7907, idx:  8173 },
  { start: 0x790e, end: 0x7912, idx:  8184 },
  { start: 0x7919, end: 0x7919, idx:  8189 },
  { start: 0x7926, end: 0x792c, idx:  8190 },
  { start: 0x793a, end: 0x7949, idx:  8197 },
  { start: 0x7950, end: 0x796d, idx:  8213 },
  { start: 0x7977, end: 0x798f, idx:  8243 },
  { start: 0x799d, end: 0x799d, idx:  8268 },
  { start: 0x79a6, end: 0x79c1, idx:  8269 },
  { start: 0x79c9, end: 0x79d8, idx:  8297 },
  { start: 0x79df, end: 0x79f0, idx:  8313 },
  { start: 0x79fb, end: 0x7a00, idx:  8331 },
  { start: 0x7a08, end: 0x7a20, idx:  8337 },
  { start: 0x7a2e, end: 0x7a50, idx:  8362 },
  { start: 0x7a57, end: 0x7a57, idx:  8397 },
  { start: 0x7a61, end: 0x7a88, idx:  8398 },
  { start: 0x7a92, end: 0x7a98, idx:  8438 },
  { start: 0x7a9f, end: 0x7a9f, idx:  8445 },
  { start: 0x7aa9, end: 0x7b11, idx:  8446 },
  { start: 0x7b18, end: 0x7b2c, idx:  8551 },
  { start: 0x7b33, end: 0x7b39, idx:  8572 },
  { start: 0x7b45, end: 0x7b56, idx:  8579 },
  { start: 0x7b5d, end: 0x7b5d, idx:  8597 },
  { start: 0x7b65, end: 0x7b7a, idx:  8598 },
  { start: 0x7b86, end: 0x7ba1, idx:  8620 },
  { start: 0x7baa, end: 0x7bb8, idx:  8648 },
  { start: 0x7bc0, end: 0x7bcf, idx:  8663 },
  { start: 0x7bdd, end: 0x7bf7, idx:  8679 },
  { start: 0x7c00, end: 0x7c00, idx:  8706 },
  { start: 0x7c07, end: 0x7c17, idx:  8707 },
  { start: 0x7c1f, end: 0x7c2b, idx:  8724 },
  { start: 0x7c37, end: 0x7c43, idx:  8737 },
  { start: 0x7c4c, end: 0x7c58, idx:  8750 },
  { start: 0x7c5f, end: 0x7c65, idx:  8763 },
  { start: 0x7c6c, end: 0x7c6c, idx:  8770 },
  { start: 0x7c73, end: 0x7c75, idx:  8771 },
  { start: 0x7c7e, end: 0x7ce7, idx:  8774 },
  { start: 0x7cef, end: 0x7d22, idx:  8880 },
  { start: 0x7d2b, end: 0x7d7d, idx:  8932 },
  { start: 0x7d89, end: 0x7da3, idx:  9015 },
  { start: 0x7dab, end: 0x7dbf, idx:  9042 },
  { start: 0x7dc7, end: 0x7df4, idx:  9063 },
  { start: 0x7dfb, end: 0x7e0b, idx:  9109 },
  { start: 0x7e12, end: 0x7e12, idx:  9126 },
  { start: 0x7e1b, end: 0x7e4d, idx:  9127 },
  { start: 0x7e54, end: 0x7e5e, idx:  9178 },
  { start: 0x7e66, end: 0x7e70, idx:  9189 },
  { start: 0x7e79, end: 0x7e9c, idx:  9200 },
  { start: 0x7f36, end: 0x7f3a, idx:  9236 },
  { start: 0x7f45, end: 0x7f45, idx:  9241 },
  { start: 0x7f4c, end: 0x7f58, idx:  9242 },
  { start: 0x7f5f, end: 0x7f60, idx:  9255 },
  { start: 0x7f67, end: 0x7f79, idx:  9257 },
  { start: 0x7f82, end: 0x7fd5, idx:  9276 },
  { start: 0x7fe0, end: 0x8021, idx:  9360 },
  { start: 0x8028, end: 0x8028, idx:  9426 },
  { start: 0x8033, end: 0x803f, idx:  9427 },
  { start: 0x8046, end: 0x804a, idx:  9440 },
  { start: 0x8052, end: 0x8068, idx:  9445 },
  { start: 0x806f, end: 0x808c, idx:  9468 },
  { start: 0x8093, end: 0x80ba, idx:  9498 },
  { start: 0x80c3, end: 0x80ce, idx:  9538 },
  { start: 0x80d6, end: 0x80e5, idx:  9550 },
  { start: 0x80ef, end: 0x810a, idx:  9566 },
  { start: 0x811a, end: 0x811b, idx:  9594 },
  { start: 0x8123, end: 0x813e, idx:  9596 },
  { start: 0x8146, end: 0x8155, idx:  9624 },
  { start: 0x815f, end: 0x81a9, idx:  9640 },
  { start: 0x81b0, end: 0x821f, idx:  9715 },
  { start: 0x8229, end: 0x8239, idx:  9827 },
  { start: 0x8240, end: 0x8240, idx:  9844 },
  { start: 0x8247, end: 0x8247, idx:  9845 },
  { start: 0x8258, end: 0x827e, idx:  9846 },
  { start: 0x828b, end: 0x8292, idx:  9885 },
  { start: 0x8299, end: 0x82bd, idx:  9893 },
  { start: 0x82c5, end: 0x82c5, idx:  9930 },
  { start: 0x82d1, end: 0x82fb, idx:  9931 },
  { start: 0x8302, end: 0x830e, idx:  9974 },
  { start: 0x8316, end: 0x831c, idx:  9987 },
  { start: 0x8323, end: 0x8339, idx:  9994 },
  { start: 0x8340, end: 0x8358, idx: 10017 },
  { start: 0x8373, end: 0x837c, idx: 10042 },
  { start: 0x8385, end: 0x83b5, idx: 10052 },
  { start: 0x83bd, end: 0x83e0, idx: 10101 },
  { start: 0x83e9, end: 0x8413, idx: 10137 },
  { start: 0x8420, end: 0x8422, idx: 10180 },
  { start: 0x8429, end: 0x843d, idx: 10183 },
  { start: 0x8446, end: 0x844e, idx: 10204 },
  { start: 0x8457, end: 0x847a, idx: 10213 },
  { start: 0x8482, end: 0x8484, idx: 10249 },
  { start: 0x848b, end: 0x84a1, idx: 10252 },
  { start: 0x84ad, end: 0x84da, idx: 10275 },
  { start: 0x84ec, end: 0x84f4, idx: 10321 },
  { start: 0x84fc, end: 0x8506, idx: 10330 },
  { start: 0x8511, end: 0x852d, idx: 10341 },
  { start: 0x8535, end: 0x8535, idx: 10370 },
  { start: 0x853d, end: 0x854e, idx: 10371 },
  { start: 0x8555, end: 0x855a, idx: 10389 },
  { start: 0x8563, end: 0x856d, idx: 10395 },
  { start: 0x8577, end: 0x8577, idx: 10406 },
  { start: 0x857e, end: 0x859c, idx: 10407 },
  { start: 0x85a4, end: 0x85af, idx: 10438 },
  { start: 0x85b9, end: 0x85ba, idx: 10450 },
  { start: 0x85c1, end: 0x85c1, idx: 10452 },
  { start: 0x85c9, end: 0x85d5, idx: 10453 },
  { start: 0x85dc, end: 0x85dd, idx: 10466 },
  { start: 0x85e4, end: 0x85ea, idx: 10468 },
  { start: 0x85f7, end: 0x860b, idx: 10475 },
  { start: 0x8613, end: 0x861a, idx: 10496 },
  { start: 0x8622, end: 0x8622, idx: 10504 },
  { start: 0x862d, end: 0x8630, idx: 10505 },
  { start: 0x863f, end: 0x863f, idx: 10509 },
  { start: 0x864d, end: 0x865f, idx: 10510 },
  { start: 0x8667, end: 0x8671, idx: 10529 },
  { start: 0x8679, end: 0x867b, idx: 10540 },
  { start: 0x868a, end: 0x868c, idx: 10543 },
  { start: 0x8693, end: 0x8695, idx: 10546 },
  { start: 0x86a3, end: 0x86b6, idx: 10549 },
  { start: 0x86c4, end: 0x86ef, idx: 10569 },
  { start: 0x86f8, end: 0x871c, idx: 10613 },
  { start: 0x8725, end: 0x8729, idx: 10650 },
  { start: 0x8734, end: 0x873f, idx: 10655 },
  { start: 0x8749, end: 0x8778, idx: 10667 },
  { start: 0x877f, end: 0x8782, idx: 10715 },
  { start: 0x878d, end: 0x878d, idx: 10719 },
  { start: 0x879f, end: 0x87a2, idx: 10720 },
  { start: 0x87ab, end: 0x87b3, idx: 10724 },
  { start: 0x87ba, end: 0x87d2, idx: 10733 },
  { start: 0x87e0, end: 0x87e0, idx: 10758 },
  { start: 0x87ef, end: 0x87fe, idx: 10759 },
  { start: 0x8805, end: 0x8805, idx: 10775 },
  { start: 0x880d, end: 0x8816, idx: 10776 },
  { start: 0x8821, end: 0x8827, idx: 10786 },
  { start: 0x8831, end: 0x88a4, idx: 10793 },
  { start: 0x88ab, end: 0x88b7, idx: 10909 },
  { start: 0x88bf, end: 0x88c5, idx: 10922 },
  { start: 0x88cf, end: 0x88e1, idx: 10929 },
  { start: 0x88e8, end: 0x88e8, idx: 10948 },
  { start: 0x88f2, end: 0x8913, idx: 10949 },
  { start: 0x891d, end: 0x891e, idx: 10983 },
  { start: 0x8925, end: 0x892b, idx: 10985 },
  { start: 0x8936, end: 0x8944, idx: 10992 },
  { start: 0x894c, end: 0x894d, idx: 11007 },
  { start: 0x8956, end: 0x8956, idx: 11009 },
  { start: 0x895e, end: 0x8977, idx: 11010 },
  { start: 0x897e, end: 0x899a, idx: 11036 },
  { start: 0x89a1, end: 0x89b3, idx: 11065 },
  { start: 0x89ba, end: 0x89c0, idx: 11084 },
  { start: 0x89d2, end: 0x89d2, idx: 11091 },
  { start: 0x89da, end: 0x89e7, idx: 11092 },
  { start: 0x89f4, end: 0x89f8, idx: 11106 },
  { start: 0x8a00, end: 0x8a48, idx: 11111 },
  { start: 0x8a50, end: 0x8a73, idx: 11184 },
  { start: 0x8a7c, end: 0x8ab2, idx: 11220 },
  { start: 0x8ab9, end: 0x8b2c, idx: 11275 },
  { start: 0x8b33, end: 0x8b41, idx: 11391 },
  { start: 0x8b49, end: 0x8b4f, idx: 11406 },
  { start: 0x8b56, end: 0x8b5f, idx: 11413 },
  { start: 0x8b66, end: 0x8b83, idx: 11423 },
  { start: 0x8b8a, end: 0x8b9a, idx: 11453 },
  { start: 0x8c37, end: 0x8c5a, idx: 11470 },
  { start: 0x8c61, end: 0x8c62, idx: 11506 },
  { start: 0x8c6a, end: 0x8c6c, idx: 11508 },
  { start: 0x8c78, end: 0x8cd3, idx: 11511 },
  { start: 0x8cda, end: 0x8ced, idx: 11603 },
  { start: 0x8cfa, end: 0x8cfd, idx: 11623 },
  { start: 0x8d04, end: 0x8d16, idx: 11627 },
  { start: 0x8d64, end: 0x8d77, idx: 11646 },
  { start: 0x8d81, end: 0x8d8a, idx: 11666 },
  { start: 0x8d99, end: 0x8d99, idx: 11676 },
  { start: 0x8da3, end: 0x8da8, idx: 11677 },
  { start: 0x8db3, end: 0x8db3, idx: 11683 },
  { start: 0x8dba, end: 0x8dc2, idx: 11684 },
  { start: 0x8dcb, end: 0x8dcf, idx: 11693 },
  { start: 0x8dd6, end: 0x8df5, idx: 11698 },
  { start: 0x8dfc, end: 0x8dff, idx: 11730 },
  { start: 0x8e08, end: 0x8e10, idx: 11734 },
  { start: 0x8e1d, end: 0x8e1f, idx: 11743 },
  { start: 0x8e2a, end: 0x8e35, idx: 11746 },
  { start: 0x8e42, end: 0x8e64, idx: 11758 },
  { start: 0x8e72, end: 0x8e99, idx: 11793 },
  { start: 0x8ea1, end: 0x8ea1, idx: 11833 },
  { start: 0x8eaa, end: 0x8eb1, idx: 11834 },
  { start: 0x8ebe, end: 0x8ebe, idx: 11842 },
  { start: 0x8ec5, end: 0x8ed2, idx: 11843 },
  { start: 0x8edb, end: 0x8ee3, idx: 11857 },
  { start: 0x8eeb, end: 0x8eeb, idx: 11866 },
  { start: 0x8ef8, end: 0x8f1f, idx: 11867 },
  { start: 0x8f26, end: 0x8f4e, idx: 11907 },
  { start: 0x8f57, end: 0x8f64, idx: 11948 },
  { start: 0x8f9b, end: 0x8fc5, idx: 11962 },
  { start: 0x8fce, end: 0x8fda, idx: 12005 },
  { start: 0x8fe2, end: 0x9027, idx: 12018 },
  { start: 0x902e, end: 0x9091, idx: 12088 },
  { start: 0x90a3, end: 0x90b8, idx: 12188 },
  { start: 0x90c1, end: 0x90c1, idx: 12210 },
  { start: 0x90ca, end: 0x90ce, idx: 12211 },
  { start: 0x90db, end: 0x90ed, idx: 12216 },
  { start: 0x90f5, end: 0x9102, idx: 12235 },
  { start: 0x9112, end: 0x9112, idx: 12249 },
  { start: 0x9119, end: 0x9119, idx: 12250 },
  { start: 0x912d, end: 0x9132, idx: 12251 },
  { start: 0x9149, end: 0x9158, idx: 12257 },
  { start: 0x9162, end: 0x9178, idx: 12273 },
  { start: 0x9182, end: 0x91e7, idx: 12296 },
  { start: 0x91f5, end: 0x91ff, idx: 12398 },
  { start: 0x920d, end: 0x9215, idx: 12409 },
  { start: 0x921e, end: 0x921e, idx: 12418 },
  { start: 0x9229, end: 0x922c, idx: 12419 },
  { start: 0x9234, end: 0x9237, idx: 12423 },
  { start: 0x923f, end: 0x9250, idx: 12427 },
  { start: 0x9257, end: 0x9266, idx: 12445 },
  { start: 0x9271, end: 0x9271, idx: 12461 },
  { start: 0x927e, end: 0x9285, idx: 12462 },
  { start: 0x9291, end: 0x929c, idx: 12470 },
  { start: 0x92ad, end: 0x92ad, idx: 12482 },
  { start: 0x92b7, end: 0x92b9, idx: 12483 },
  { start: 0x92cf, end: 0x92d2, idx: 12486 },
  { start: 0x92e4, end: 0x92fc, idx: 12490 },
  { start: 0x9306, end: 0x9306, idx: 12515 },
  { start: 0x930f, end: 0x9310, idx: 12516 },
  { start: 0x9318, end: 0x933b, idx: 12518 },
  { start: 0x9344, end: 0x9344, idx: 12554 },
  { start: 0x934b, end: 0x934d, idx: 12555 },
  { start: 0x9354, end: 0x9360, idx: 12558 },
  { start: 0x936c, end: 0x936e, idx: 12571 },
  { start: 0x9375, end: 0x9375, idx: 12574 },
  { start: 0x937c, end: 0x937e, idx: 12575 },
  { start: 0x938c, end: 0x938c, idx: 12578 },
  { start: 0x9394, end: 0x939a, idx: 12579 },
  { start: 0x93a7, end: 0x93b0, idx: 12586 },
  { start: 0x93b9, end: 0x93b9, idx: 12596 },
  { start: 0x93c3, end: 0x93c8, idx: 12597 },
  { start: 0x93d0, end: 0x93e8, idx: 12603 },
  { start: 0x9403, end: 0x9407, idx: 12628 },
  { start: 0x9410, end: 0x941a, idx: 12633 },
  { start: 0x9421, end: 0x9421, idx: 12644 },
  { start: 0x942b, end: 0x942b, idx: 12645 },
  { start: 0x9435, end: 0x943a, idx: 12646 },
  { start: 0x9441, end: 0x9444, idx: 12652 },
  { start: 0x9451, end: 0x9453, idx: 12656 },
  { start: 0x945a, end: 0x9462, idx: 12659 },
  { start: 0x946a, end: 0x9481, idx: 12668 },
  { start: 0x9577, end: 0x9577, idx: 12692 },
  { start: 0x9580, end: 0x9599, idx: 12693 },
  { start: 0x95a0, end: 0x95b2, idx: 12719 },
  { start: 0x95b9, end: 0x95cd, idx: 12738 },
  { start: 0x95d4, end: 0x95e5, idx: 12759 },
  { start: 0x961c, end: 0x9621, idx: 12777 },
  { start: 0x9628, end: 0x9632, idx: 12783 },
  { start: 0x963b, end: 0x9644, idx: 12794 },
  { start: 0x964b, end: 0x9650, idx: 12804 },
  { start: 0x965b, end: 0x967d, idx: 12810 },
  { start: 0x9685, end: 0x9749, idx: 12845 },
  { start: 0x9752, end: 0x9790, idx: 13042 },
  { start: 0x9798, end: 0x97b4, idx: 13105 },
  { start: 0x97c3, end: 0x97cb, idx: 13134 },
  { start: 0x97d3, end: 0x97d3, idx: 13143 },
  { start: 0x97dc, end: 0x97dc, idx: 13144 },
  { start: 0x97ed, end: 0x981a, idx: 13145 },
  { start: 0x9821, end: 0x9824, idx: 13191 },
  { start: 0x982c, end: 0x982d, idx: 13195 },
  { start: 0x9834, end: 0x983d, idx: 13197 },
  { start: 0x9846, end: 0x985e, idx: 13207 },
  { start: 0x9867, end: 0x9874, idx: 13232 },
  { start: 0x98a8, end: 0x98b6, idx: 13246 },
  { start: 0x98c3, end: 0x98c6, idx: 13261 },
  { start: 0x98db, end: 0x98e2, idx: 13265 },
  { start: 0x98e9, end: 0x98f4, idx: 13273 },
  { start: 0x98fc, end: 0x992e, idx: 13285 },
  { start: 0x993d, end: 0x9957, idx: 13336 },
  { start: 0x9996, end: 0x9999, idx: 13363 },
  { start: 0x99a5, end: 0x99b4, idx: 13367 },
  { start: 0x99bc, end: 0x99c8, idx: 13383 },
  { start: 0x99d0, end: 0x99e2, idx: 13396 },
  { start: 0x99ed, end: 0x9a05, idx: 13415 },
  { start: 0x9a0e, end: 0x9a19, idx: 13440 },
  { start: 0x9a28, end: 0x9a30, idx: 13452 },
  { start: 0x9a37, end: 0x9a37, idx: 13461 },
  { start: 0x9a3e, end: 0x9a45, idx: 13462 },
  { start: 0x9a4d, end: 0x9a4d, idx: 13470 },
  { start: 0x9a55, end: 0x9a6b, idx: 13471 },
  { start: 0x9aa8, end: 0x9ab0, idx: 13494 },
  { start: 0x9ab8, end: 0x9ac4, idx: 13503 },
  { start: 0x9acf, end: 0x9afb, idx: 13516 },
  { start: 0x9b06, end: 0x9b06, idx: 13561 },
  { start: 0x9b18, end: 0x9b32, idx: 13562 },
  { start: 0x9b3b, end: 0x9b45, idx: 13589 },
  { start: 0x9b4d, end: 0x9b5a, idx: 13600 },
  { start: 0x9b6f, end: 0x9b74, idx: 13614 },
  { start: 0x9b83, end: 0x9b83, idx: 13620 },
  { start: 0x9b8e, end: 0x9b97, idx: 13621 },
  { start: 0x9b9f, end: 0x9ba0, idx: 13631 },
  { start: 0x9ba8, end: 0x9bb9, idx: 13633 },
  { start: 0x9bc0, end: 0x9be8, idx: 13651 },
  { start: 0x9bf0, end: 0x9bf5, idx: 13692 },
  { start: 0x9c04, end: 0x9c25, idx: 13698 },
  { start: 0x9c2d, end: 0x9c32, idx: 13732 },
  { start: 0x9c39, end: 0x9c3e, idx: 13738 },
  { start: 0x9c46, end: 0x9c48, idx: 13744 },
  { start: 0x9c52, end: 0x9c60, idx: 13747 },
  { start: 0x9c67, end: 0x9c67, idx: 13762 },
  { start: 0x9c76, end: 0x9c78, idx: 13763 },
  { start: 0x9ce5, end: 0x9cf6, idx: 13766 },
  { start: 0x9d03, end: 0x9d2c, idx: 13784 },
  { start: 0x9d3b, end: 0x9d48, idx: 13826 },
  { start: 0x9d50, end: 0x9d51, idx: 13840 },
  { start: 0x9d59, end: 0x9d64, idx: 13842 },
  { start: 0x9d6c, end: 0x9d72, idx: 13854 },
  { start: 0x9d7a, end: 0x9d7a, idx: 13861 },
  { start: 0x9d87, end: 0x9d8f, idx: 13862 },
  { start: 0x9d9a, end: 0x9d9a, idx: 13871 },
  { start: 0x9da4, end: 0x9dc6, idx: 13872 },
  { start: 0x9dcf, end: 0x9dd9, idx: 13907 },
  { start: 0x9de6, end: 0x9de6, idx: 13918 },
  { start: 0x9ded, end: 0x9dfd, idx: 13919 },
  { start: 0x9e1a, end: 0x9e1e, idx: 13936 },
  { start: 0x9e75, end: 0x9e81, idx: 13941 },
  { start: 0x9e88, end: 0x9ead, idx: 13954 },
  { start: 0x9eb8, end: 0x9ec4, idx: 13992 },
  { start: 0x9ecc, end: 0x9ee8, idx: 14005 },
  { start: 0x9eef, end: 0x9efd, idx: 14034 },
  { start: 0x9f07, end: 0x9f15, idx: 14049 },
  { start: 0x9f20, end: 0x9f21, idx: 14064 },
  { start: 0x9f2c, end: 0x9f2c, idx: 14066 },
  { start: 0x9f3b, end: 0x9f3e, idx: 14067 },
  { start: 0x9f4a, end: 0x9f54, idx: 14071 },
  { start: 0x9f5f, end: 0x9f77, idx: 14082 },
  { start: 0x9f8d, end: 0x9f8d, idx: 14107 },
  { start: 0x9f95, end: 0x9f95, idx: 14108 },
  { start: 0x9f9c, end: 0x9fa0, idx: 14109 },
  { start: 0xff01, end: 0xff5d, idx: 14114 },
  { start: 0xffe3, end: 0xffe5, idx: 14207 },
  { start: 0xffff, end: 0xffff, idx:     0 }
};


/* The following table is indexed using the previous tables.  It is
   therefore indexed using UCS with leaving large gaps out.  It can
   be computed with

   egrep '^0x' < .../eastasia/jis/jis0208.txt | awk '{ print $3, $2}' |
   egrep '^0x[^01]' | sort -u | perl tag.pl

   where tab.pl is

   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   $n=$first=$last=$idx=0;

   sub fmt {
     printf ("\n ") if (($n % 6) == 0);
     ++$n;
     my($val) = pop(@_);
     printf (" \"\\x%02x\\x%02x\",", int($val / 256), $val % 256);
   }

   while (<>) {
     local($ucs,$jis,%rest) = split;
     local($u)=hex($ucs);
     if ($u - $last > 6) {
       if ($last != 0) {
         $idx += $last - $first + 1;
       }
       $first=$u;
     } else {
       for ($m = $last + 1; $m < $u; ++$m) {
	 fmt (0);
       }
     }
     fmt ($u);
     $last=$u;
   }
   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
const char __jisx0208_from_ucs_tab[14210][2] =
{
  "\x20\x10", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x20\x15",
  "\x20\x16", "\x00\x00", "\x20\x18", "\x20\x19", "\x00\x00", "\x00\x00",
  "\x20\x1c", "\x20\x1d", "\x00\x00", "\x00\x00", "\x20\x20", "\x20\x21",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x20\x25", "\x20\x26", "\x20\x30",
  "\x00\x00", "\x20\x32", "\x20\x33", "\x20\x3b", "\x21\x03", "\x21\x2b",
  "\x21\x90", "\x21\x91", "\x21\x92", "\x21\x93", "\x21\xd2", "\x00\x00",
  "\x21\xd4", "\x22\x00", "\x00\x00", "\x22\x02", "\x22\x03", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x22\x07", "\x22\x08", "\x00\x00", "\x00\x00",
  "\x22\x0b", "\x22\x12", "\x22\x1a", "\x00\x00", "\x00\x00", "\x22\x1d",
  "\x22\x1e", "\x00\x00", "\x22\x20", "\x22\x27", "\x22\x28", "\x22\x29",
  "\x22\x2a", "\x22\x2b", "\x22\x2c", "\x22\x34", "\x22\x35", "\x22\x3d",
  "\x22\x52", "\x22\x60", "\x22\x61", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x22\x66", "\x22\x67", "\x00\x00", "\x00\x00", "\x22\x6a",
  "\x22\x6b", "\x22\x82", "\x22\x83", "\x00\x00", "\x00\x00", "\x22\x86",
  "\x22\x87", "\x22\xa5", "\x23\x12", "\x25\x00", "\x25\x01", "\x25\x02",
  "\x25\x03", "\x25\x0c", "\x00\x00", "\x00\x00", "\x25\x0f", "\x25\x10",
  "\x00\x00", "\x00\x00", "\x25\x13", "\x25\x14", "\x00\x00", "\x00\x00",
  "\x25\x17", "\x25\x18", "\x00\x00", "\x00\x00", "\x25\x1b", "\x25\x1c",
  "\x25\x1d", "\x00\x00", "\x00\x00", "\x25\x20", "\x00\x00", "\x00\x00",
  "\x25\x23", "\x25\x24", "\x25\x25", "\x00\x00", "\x00\x00", "\x25\x28",
  "\x00\x00", "\x00\x00", "\x25\x2b", "\x25\x2c", "\x00\x00", "\x00\x00",
  "\x25\x2f", "\x25\x30", "\x00\x00", "\x00\x00", "\x25\x33", "\x25\x34",
  "\x00\x00", "\x00\x00", "\x25\x37", "\x25\x38", "\x00\x00", "\x00\x00",
  "\x25\x3b", "\x25\x3c", "\x00\x00", "\x00\x00", "\x25\x3f", "\x00\x00",
  "\x00\x00", "\x25\x42", "\x25\x4b", "\x25\xa0", "\x25\xa1", "\x25\xb2",
  "\x25\xb3", "\x25\xbc", "\x25\xbd", "\x25\xc6", "\x25\xc7", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x25\xcb", "\x00\x00", "\x00\x00", "\x25\xce",
  "\x25\xcf", "\x25\xef", "\x26\x05", "\x26\x06", "\x26\x40", "\x00\x00",
  "\x26\x42", "\x26\x6a", "\x00\x00", "\x00\x00", "\x26\x6d", "\x00\x00",
  "\x26\x6f", "\x30\x00", "\x30\x01", "\x30\x02", "\x30\x03", "\x00\x00",
  "\x30\x05", "\x30\x06", "\x30\x07", "\x30\x08", "\x30\x09", "\x30\x0a",
  "\x30\x0b", "\x30\x0c", "\x30\x0d", "\x30\x0e", "\x30\x0f", "\x30\x10",
  "\x30\x11", "\x30\x12", "\x30\x13", "\x30\x14", "\x30\x15", "\x30\x1c",
  "\x30\x41", "\x30\x42", "\x30\x43", "\x30\x44", "\x30\x45", "\x30\x46",
  "\x30\x47", "\x30\x48", "\x30\x49", "\x30\x4a", "\x30\x4b", "\x30\x4c",
  "\x30\x4d", "\x30\x4e", "\x30\x4f", "\x30\x50", "\x30\x51", "\x30\x52",
  "\x30\x53", "\x30\x54", "\x30\x55", "\x30\x56", "\x30\x57", "\x30\x58",
  "\x30\x59", "\x30\x5a", "\x30\x5b", "\x30\x5c", "\x30\x5d", "\x30\x5e",
  "\x30\x5f", "\x30\x60", "\x30\x61", "\x30\x62", "\x30\x63", "\x30\x64",
  "\x30\x65", "\x30\x66", "\x30\x67", "\x30\x68", "\x30\x69", "\x30\x6a",
  "\x30\x6b", "\x30\x6c", "\x30\x6d", "\x30\x6e", "\x30\x6f", "\x30\x70",
  "\x30\x71", "\x30\x72", "\x30\x73", "\x30\x74", "\x30\x75", "\x30\x76",
  "\x30\x77", "\x30\x78", "\x30\x79", "\x30\x7a", "\x30\x7b", "\x30\x7c",
  "\x30\x7d", "\x30\x7e", "\x30\x7f", "\x30\x80", "\x30\x81", "\x30\x82",
  "\x30\x83", "\x30\x84", "\x30\x85", "\x30\x86", "\x30\x87", "\x30\x88",
  "\x30\x89", "\x30\x8a", "\x30\x8b", "\x30\x8c", "\x30\x8d", "\x30\x8e",
  "\x30\x8f", "\x30\x90", "\x30\x91", "\x30\x92", "\x30\x93", "\x30\x9b",
  "\x30\x9c", "\x30\x9d", "\x30\x9e", "\x00\x00", "\x00\x00", "\x30\xa1",
  "\x30\xa2", "\x30\xa3", "\x30\xa4", "\x30\xa5", "\x30\xa6", "\x30\xa7",
  "\x30\xa8", "\x30\xa9", "\x30\xaa", "\x30\xab", "\x30\xac", "\x30\xad",
  "\x30\xae", "\x30\xaf", "\x30\xb0", "\x30\xb1", "\x30\xb2", "\x30\xb3",
  "\x30\xb4", "\x30\xb5", "\x30\xb6", "\x30\xb7", "\x30\xb8", "\x30\xb9",
  "\x30\xba", "\x30\xbb", "\x30\xbc", "\x30\xbd", "\x30\xbe", "\x30\xbf",
  "\x30\xc0", "\x30\xc1", "\x30\xc2", "\x30\xc3", "\x30\xc4", "\x30\xc5",
  "\x30\xc6", "\x30\xc7", "\x30\xc8", "\x30\xc9", "\x30\xca", "\x30\xcb",
  "\x30\xcc", "\x30\xcd", "\x30\xce", "\x30\xcf", "\x30\xd0", "\x30\xd1",
  "\x30\xd2", "\x30\xd3", "\x30\xd4", "\x30\xd5", "\x30\xd6", "\x30\xd7",
  "\x30\xd8", "\x30\xd9", "\x30\xda", "\x30\xdb", "\x30\xdc", "\x30\xdd",
  "\x30\xde", "\x30\xdf", "\x30\xe0", "\x30\xe1", "\x30\xe2", "\x30\xe3",
  "\x30\xe4", "\x30\xe5", "\x30\xe6", "\x30\xe7", "\x30\xe8", "\x30\xe9",
  "\x30\xea", "\x30\xeb", "\x30\xec", "\x30\xed", "\x30\xee", "\x30\xef",
  "\x30\xf0", "\x30\xf1", "\x30\xf2", "\x30\xf3", "\x30\xf4", "\x30\xf5",
  "\x30\xf6", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x30\xfb",
  "\x30\xfc", "\x30\xfd", "\x30\xfe", "\x4e\x00", "\x4e\x01", "\x00\x00",
  "\x4e\x03", "\x00\x00", "\x00\x00", "\x00\x00", "\x4e\x07", "\x4e\x08",
  "\x4e\x09", "\x4e\x0a", "\x4e\x0b", "\x00\x00", "\x4e\x0d", "\x4e\x0e",
  "\x00\x00", "\x4e\x10", "\x4e\x11", "\x00\x00", "\x00\x00", "\x4e\x14",
  "\x4e\x15", "\x4e\x16", "\x4e\x17", "\x4e\x18", "\x4e\x19", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x4e\x1e", "\x00\x00", "\x00\x00",
  "\x4e\x21", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x4e\x26",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x4e\x2a", "\x00\x00", "\x00\x00",
  "\x4e\x2d", "\x00\x00", "\x00\x00", "\x00\x00", "\x4e\x31", "\x4e\x32",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x4e\x36", "\x00\x00", "\x4e\x38",
  "\x4e\x39", "\x00\x00", "\x4e\x3b", "\x4e\x3c", "\x00\x00", "\x00\x00",
  "\x4e\x3f", "\x00\x00", "\x00\x00", "\x4e\x42", "\x4e\x43", "\x00\x00",
  "\x4e\x45", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x4e\x4b", "\x00\x00", "\x4e\x4d", "\x4e\x4e", "\x4e\x4f", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x4e\x55", "\x4e\x56",
  "\x4e\x57", "\x4e\x58", "\x4e\x59", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x4e\x5d", "\x4e\x5e", "\x4e\x5f", "\x00\x00", "\x00\x00", "\x4e\x62",
  "\x4e\x71", "\x00\x00", "\x4e\x73", "\x4e\x7e", "\x00\x00", "\x4e\x80",
  "\x00\x00", "\x4e\x82", "\x00\x00", "\x00\x00", "\x4e\x85", "\x4e\x86",
  "\x00\x00", "\x4e\x88", "\x4e\x89", "\x4e\x8a", "\x4e\x8b", "\x4e\x8c",
  "\x00\x00", "\x4e\x8e", "\x00\x00", "\x00\x00", "\x4e\x91", "\x4e\x92",
  "\x00\x00", "\x4e\x94", "\x4e\x95", "\x00\x00", "\x00\x00", "\x4e\x98",
  "\x4e\x99", "\x00\x00", "\x4e\x9b", "\x4e\x9c", "\x00\x00", "\x4e\x9e",
  "\x4e\x9f", "\x4e\xa0", "\x4e\xa1", "\x4e\xa2", "\x00\x00", "\x4e\xa4",
  "\x4e\xa5", "\x4e\xa6", "\x00\x00", "\x4e\xa8", "\x00\x00", "\x00\x00",
  "\x4e\xab", "\x4e\xac", "\x4e\xad", "\x4e\xae", "\x00\x00", "\x4e\xb0",
  "\x00\x00", "\x00\x00", "\x4e\xb3", "\x00\x00", "\x00\x00", "\x4e\xb6",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x4e\xba", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x4e\xc0", "\x4e\xc1", "\x4e\xc2",
  "\x00\x00", "\x4e\xc4", "\x00\x00", "\x4e\xc6", "\x4e\xc7", "\x00\x00",
  "\x00\x00", "\x4e\xca", "\x4e\xcb", "\x00\x00", "\x4e\xcd", "\x4e\xce",
  "\x4e\xcf", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x4e\xd4",
  "\x4e\xd5", "\x4e\xd6", "\x4e\xd7", "\x4e\xd8", "\x4e\xd9", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x4e\xdd", "\x4e\xde", "\x4e\xdf", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x4e\xe3", "\x4e\xe4", "\x4e\xe5", "\x4e\xed",
  "\x4e\xee", "\x00\x00", "\x4e\xf0", "\x00\x00", "\x4e\xf2", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x4e\xf6", "\x4e\xf7", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x4e\xfb", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x4f\x01", "\x4f\x09", "\x4f\x0a", "\x00\x00", "\x00\x00",
  "\x4f\x0d", "\x4f\x0e", "\x4f\x0f", "\x4f\x10", "\x4f\x11", "\x4f\x1a",
  "\x00\x00", "\x4f\x1c", "\x4f\x1d", "\x4f\x2f", "\x4f\x30", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x4f\x34", "\x00\x00", "\x4f\x36", "\x00\x00",
  "\x4f\x38", "\x00\x00", "\x4f\x3a", "\x00\x00", "\x4f\x3c", "\x4f\x3d",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x4f\x43",
  "\x00\x00", "\x00\x00", "\x4f\x46", "\x4f\x47", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x4f\x4d", "\x4f\x4e", "\x4f\x4f",
  "\x4f\x50", "\x4f\x51", "\x00\x00", "\x4f\x53", "\x00\x00", "\x4f\x55",
  "\x00\x00", "\x4f\x57", "\x00\x00", "\x4f\x59", "\x4f\x5a", "\x4f\x5b",
  "\x4f\x5c", "\x4f\x5d", "\x4f\x5e", "\x4f\x69", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x4f\x6f", "\x4f\x70", "\x00\x00",
  "\x00\x00", "\x4f\x73", "\x00\x00", "\x4f\x75", "\x4f\x76", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x4f\x7b", "\x4f\x7c", "\x00\x00",
  "\x00\x00", "\x4f\x7f", "\x00\x00", "\x00\x00", "\x00\x00", "\x4f\x83",
  "\x00\x00", "\x00\x00", "\x4f\x86", "\x00\x00", "\x4f\x88", "\x00\x00",
  "\x00\x00", "\x4f\x8b", "\x00\x00", "\x4f\x8d", "\x00\x00", "\x4f\x8f",
  "\x00\x00", "\x4f\x91", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x4f\x96", "\x00\x00", "\x4f\x98", "\x00\x00", "\x00\x00", "\x4f\x9b",
  "\x00\x00", "\x4f\x9d", "\x00\x00", "\x00\x00", "\x4f\xa0", "\x4f\xa1",
  "\x4f\xab", "\x00\x00", "\x4f\xad", "\x4f\xae", "\x4f\xaf", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x4f\xb5", "\x4f\xb6",
  "\x4f\xbf", "\x00\x00", "\x00\x00", "\x4f\xc2", "\x4f\xc3", "\x4f\xc4",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x4f\xca",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x4f\xce", "\x00\x00", "\x4f\xd0",
  "\x4f\xd1", "\x00\x00", "\x00\x00", "\x4f\xd4", "\x00\x00", "\x00\x00",
  "\x4f\xd7", "\x4f\xd8", "\x00\x00", "\x4f\xda", "\x4f\xdb", "\x00\x00",
  "\x4f\xdd", "\x00\x00", "\x4f\xdf", "\x00\x00", "\x4f\xe1", "\x00\x00",
  "\x4f\xe3", "\x4f\xe4", "\x4f\xe5", "\x4f\xee", "\x4f\xef", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x4f\xf3", "\x00\x00", "\x4f\xf5", "\x4f\xf6",
  "\x00\x00", "\x4f\xf8", "\x00\x00", "\x4f\xfa", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x4f\xfe", "\x50\x05", "\x50\x06", "\x00\x00", "\x00\x00",
  "\x50\x09", "\x00\x00", "\x50\x0b", "\x00\x00", "\x50\x0d", "\x00\x00",
  "\x50\x0f", "\x00\x00", "\x50\x11", "\x50\x12", "\x00\x00", "\x50\x14",
  "\x00\x00", "\x50\x16", "\x00\x00", "\x00\x00", "\x50\x19", "\x50\x1a",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x50\x1f", "\x00\x00",
  "\x50\x21", "\x00\x00", "\x50\x23", "\x50\x24", "\x50\x25", "\x50\x26",
  "\x00\x00", "\x50\x28", "\x50\x29", "\x50\x2a", "\x50\x2b", "\x50\x2c",
  "\x50\x2d", "\x50\x36", "\x00\x00", "\x00\x00", "\x50\x39", "\x50\x43",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x50\x47", "\x50\x48", "\x50\x49",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x50\x4f",
  "\x50\x50", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x50\x55",
  "\x50\x56", "\x00\x00", "\x00\x00", "\x00\x00", "\x50\x5a", "\x00\x00",
  "\x50\x5c", "\x50\x65", "\x50\x6c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x50\x72", "\x00\x00", "\x50\x74", "\x50\x75",
  "\x50\x76", "\x00\x00", "\x50\x78", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x50\x7d", "\x00\x00", "\x00\x00", "\x50\x80", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x50\x85", "\x50\x8d", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x50\x91", "\x50\x98", "\x50\x99", "\x50\x9a",
  "\x50\xac", "\x50\xad", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x50\xb2", "\x50\xb3", "\x50\xb4", "\x50\xb5", "\x00\x00", "\x50\xb7",
  "\x50\xbe", "\x00\x00", "\x00\x00", "\x00\x00", "\x50\xc2", "\x00\x00",
  "\x00\x00", "\x50\xc5", "\x00\x00", "\x00\x00", "\x00\x00", "\x50\xc9",
  "\x50\xca", "\x00\x00", "\x00\x00", "\x50\xcd", "\x00\x00", "\x50\xcf",
  "\x00\x00", "\x50\xd1", "\x00\x00", "\x00\x00", "\x00\x00", "\x50\xd5",
  "\x50\xd6", "\x00\x00", "\x00\x00", "\x00\x00", "\x50\xda", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x50\xde", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x50\xe3", "\x00\x00", "\x50\xe5", "\x00\x00", "\x50\xe7",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x50\xed",
  "\x50\xee", "\x50\xf5", "\x00\x00", "\x00\x00", "\x00\x00", "\x50\xf9",
  "\x00\x00", "\x50\xfb", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x51\x00", "\x51\x01", "\x51\x02", "\x00\x00", "\x51\x04", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x51\x09", "\x51\x12", "\x00\x00",
  "\x51\x14", "\x51\x15", "\x51\x16", "\x00\x00", "\x51\x18", "\x00\x00",
  "\x51\x1a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x51\x1f",
  "\x00\x00", "\x51\x21", "\x51\x2a", "\x51\x32", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x51\x37", "\x00\x00", "\x00\x00", "\x51\x3a",
  "\x51\x3b", "\x51\x3c", "\x00\x00", "\x00\x00", "\x51\x3f", "\x51\x40",
  "\x51\x41", "\x00\x00", "\x51\x43", "\x51\x44", "\x51\x45", "\x51\x46",
  "\x51\x47", "\x51\x48", "\x51\x49", "\x00\x00", "\x51\x4b", "\x51\x4c",
  "\x51\x4d", "\x51\x4e", "\x00\x00", "\x51\x50", "\x00\x00", "\x51\x52",
  "\x00\x00", "\x51\x54", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x51\x5a", "\x00\x00", "\x51\x5c", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x51\x62", "\x00\x00", "\x00\x00",
  "\x51\x65", "\x00\x00", "\x00\x00", "\x51\x68", "\x51\x69", "\x51\x6a",
  "\x51\x6b", "\x51\x6c", "\x51\x6d", "\x51\x6e", "\x00\x00", "\x00\x00",
  "\x51\x71", "\x00\x00", "\x00\x00", "\x00\x00", "\x51\x75", "\x51\x76",
  "\x51\x77", "\x51\x78", "\x00\x00", "\x00\x00", "\x00\x00", "\x51\x7c",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x51\x80", "\x00\x00", "\x51\x82",
  "\x00\x00", "\x00\x00", "\x51\x85", "\x51\x86", "\x00\x00", "\x00\x00",
  "\x51\x89", "\x51\x8a", "\x00\x00", "\x51\x8c", "\x51\x8d", "\x00\x00",
  "\x51\x8f", "\x51\x90", "\x51\x91", "\x51\x92", "\x51\x93", "\x00\x00",
  "\x51\x95", "\x51\x96", "\x51\x97", "\x00\x00", "\x51\x99", "\x51\xa0",
  "\x00\x00", "\x51\xa2", "\x00\x00", "\x51\xa4", "\x51\xa5", "\x51\xa6",
  "\x00\x00", "\x51\xa8", "\x51\xa9", "\x51\xaa", "\x51\xab", "\x51\xac",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x51\xb0", "\x51\xb1", "\x51\xb2",
  "\x51\xb3", "\x51\xb4", "\x51\xb5", "\x51\xb6", "\x51\xb7", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x51\xbd", "\x51\xc4",
  "\x51\xc5", "\x51\xc6", "\x00\x00", "\x00\x00", "\x51\xc9", "\x00\x00",
  "\x51\xcb", "\x51\xcc", "\x51\xcd", "\x51\xd6", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x51\xdb", "\x51\xdc", "\x51\xdd", "\x00\x00",
  "\x00\x00", "\x51\xe0", "\x51\xe1", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x51\xe6", "\x51\xe7", "\x00\x00", "\x51\xe9", "\x51\xea",
  "\x00\x00", "\x00\x00", "\x51\xed", "\x00\x00", "\x00\x00", "\x51\xf0",
  "\x51\xf1", "\x00\x00", "\x00\x00", "\x00\x00", "\x51\xf5", "\x51\xf6",
  "\x00\x00", "\x51\xf8", "\x51\xf9", "\x51\xfa", "\x00\x00", "\x00\x00",
  "\x51\xfd", "\x51\xfe", "\x00\x00", "\x52\x00", "\x00\x00", "\x00\x00",
  "\x52\x03", "\x52\x04", "\x00\x00", "\x52\x06", "\x52\x07", "\x52\x08",
  "\x00\x00", "\x52\x0a", "\x52\x0b", "\x00\x00", "\x00\x00", "\x52\x0e",
  "\x00\x00", "\x00\x00", "\x52\x11", "\x00\x00", "\x00\x00", "\x52\x14",
  "\x00\x00", "\x00\x00", "\x52\x17", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x52\x1d", "\x52\x24", "\x52\x25", "\x00\x00",
  "\x52\x27", "\x00\x00", "\x52\x29", "\x52\x2a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x52\x2e", "\x00\x00", "\x52\x30", "\x00\x00", "\x00\x00",
  "\x52\x33", "\x00\x00", "\x00\x00", "\x52\x36", "\x52\x37", "\x52\x38",
  "\x52\x39", "\x52\x3a", "\x52\x3b", "\x52\x43", "\x52\x44", "\x00\x00",
  "\x00\x00", "\x52\x47", "\x00\x00", "\x00\x00", "\x52\x4a", "\x52\x4b",
  "\x52\x4c", "\x52\x4d", "\x00\x00", "\x52\x4f", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x52\x54", "\x00\x00", "\x52\x56", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x52\x5b", "\x00\x00", "\x00\x00",
  "\x52\x5e", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x52\x63",
  "\x52\x64", "\x52\x65", "\x00\x00", "\x00\x00", "\x00\x00", "\x52\x69",
  "\x52\x6a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x52\x6f",
  "\x52\x70", "\x52\x71", "\x52\x72", "\x52\x73", "\x52\x74", "\x52\x75",
  "\x52\x7d", "\x00\x00", "\x52\x7f", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x52\x83", "\x00\x00", "\x00\x00", "\x00\x00", "\x52\x87", "\x52\x88",
  "\x52\x89", "\x00\x00", "\x00\x00", "\x00\x00", "\x52\x8d", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x52\x91", "\x52\x92", "\x00\x00", "\x52\x94",
  "\x52\x9b", "\x00\x00", "\x00\x00", "\x00\x00", "\x52\x9f", "\x52\xa0",
  "\x00\x00", "\x00\x00", "\x52\xa3", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x52\xa9", "\x52\xaa", "\x52\xab", "\x52\xac",
  "\x52\xad", "\x00\x00", "\x00\x00", "\x00\x00", "\x52\xb1", "\x00\x00",
  "\x00\x00", "\x52\xb4", "\x52\xb5", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x52\xb9", "\x00\x00", "\x00\x00", "\x52\xbc", "\x00\x00", "\x52\xbe",
  "\x00\x00", "\x00\x00", "\x52\xc1", "\x00\x00", "\x52\xc3", "\x00\x00",
  "\x52\xc5", "\x00\x00", "\x52\xc7", "\x00\x00", "\x52\xc9", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x52\xcd", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x52\xd2", "\x00\x00", "\x00\x00", "\x52\xd5", "\x00\x00",
  "\x52\xd7", "\x52\xd8", "\x52\xd9", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x52\xdd", "\x52\xde", "\x52\xdf", "\x52\xe0", "\x00\x00", "\x52\xe2",
  "\x52\xe3", "\x52\xe4", "\x00\x00", "\x52\xe6", "\x52\xe7", "\x52\xf2",
  "\x52\xf3", "\x00\x00", "\x52\xf5", "\x00\x00", "\x00\x00", "\x52\xf8",
  "\x52\xf9", "\x52\xfa", "\x00\x00", "\x00\x00", "\x00\x00", "\x52\xfe",
  "\x52\xff", "\x00\x00", "\x53\x01", "\x53\x02", "\x00\x00", "\x00\x00",
  "\x53\x05", "\x53\x06", "\x00\x00", "\x53\x08", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x53\x0d", "\x00\x00", "\x53\x0f", "\x53\x10",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x53\x15", "\x53\x16",
  "\x53\x17", "\x00\x00", "\x53\x19", "\x53\x1a", "\x00\x00", "\x00\x00",
  "\x53\x1d", "\x00\x00", "\x00\x00", "\x53\x20", "\x53\x21", "\x00\x00",
  "\x53\x23", "\x53\x2a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x53\x2f", "\x00\x00", "\x53\x31", "\x00\x00", "\x53\x33", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x53\x38", "\x53\x39", "\x53\x3a",
  "\x53\x3b", "\x00\x00", "\x00\x00", "\x00\x00", "\x53\x3f", "\x53\x40",
  "\x53\x41", "\x00\x00", "\x53\x43", "\x00\x00", "\x53\x45", "\x53\x46",
  "\x53\x47", "\x53\x48", "\x53\x49", "\x53\x4a", "\x00\x00", "\x00\x00",
  "\x53\x4d", "\x00\x00", "\x00\x00", "\x00\x00", "\x53\x51", "\x53\x52",
  "\x53\x53", "\x53\x54", "\x00\x00", "\x00\x00", "\x53\x57", "\x53\x58",
  "\x00\x00", "\x53\x5a", "\x00\x00", "\x53\x5c", "\x00\x00", "\x53\x5e",
  "\x00\x00", "\x53\x60", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x53\x66", "\x00\x00", "\x00\x00", "\x53\x69", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x53\x6e", "\x53\x6f", "\x53\x70",
  "\x53\x71", "\x00\x00", "\x53\x73", "\x53\x74", "\x53\x75", "\x00\x00",
  "\x53\x77", "\x53\x78", "\x00\x00", "\x00\x00", "\x53\x7b", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x53\x7f", "\x00\x00", "\x00\x00", "\x53\x82",
  "\x00\x00", "\x53\x84", "\x53\x96", "\x00\x00", "\x53\x98", "\x00\x00",
  "\x53\x9a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x53\x9f",
  "\x53\xa0", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x53\xa5",
  "\x53\xa6", "\x00\x00", "\x53\xa8", "\x53\xa9", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x53\xad", "\x53\xae", "\x00\x00", "\x53\xb0", "\x00\x00",
  "\x00\x00", "\x53\xb3", "\x00\x00", "\x00\x00", "\x53\xb6", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x53\xbb", "\x53\xc2", "\x53\xc3",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x53\xc8", "\x53\xc9",
  "\x53\xca", "\x53\xcb", "\x53\xcc", "\x53\xcd", "\x53\xce", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x53\xd4", "\x00\x00",
  "\x53\xd6", "\x53\xd7", "\x00\x00", "\x53\xd9", "\x00\x00", "\x53\xdb",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x53\xdf", "\x00\x00", "\x53\xe1",
  "\x53\xe2", "\x53\xe3", "\x53\xe4", "\x53\xe5", "\x00\x00", "\x00\x00",
  "\x53\xe8", "\x53\xe9", "\x53\xea", "\x53\xeb", "\x53\xec", "\x53\xed",
  "\x53\xee", "\x53\xef", "\x53\xf0", "\x53\xf1", "\x53\xf2", "\x53\xf3",
  "\x00\x00", "\x00\x00", "\x53\xf6", "\x53\xf7", "\x53\xf8", "\x00\x00",
  "\x53\xfa", "\x54\x01", "\x00\x00", "\x54\x03", "\x54\x04", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x54\x08", "\x54\x09", "\x54\x0a", "\x54\x0b",
  "\x54\x0c", "\x54\x0d", "\x54\x0e", "\x54\x0f", "\x54\x10", "\x54\x11",
  "\x54\x1b", "\x00\x00", "\x54\x1d", "\x00\x00", "\x54\x1f", "\x54\x20",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x54\x26",
  "\x00\x00", "\x00\x00", "\x54\x29", "\x00\x00", "\x54\x2b", "\x54\x2c",
  "\x54\x2d", "\x54\x2e", "\x54\x36", "\x00\x00", "\x54\x38", "\x54\x39",
  "\x00\x00", "\x54\x3b", "\x54\x3c", "\x54\x3d", "\x54\x3e", "\x00\x00",
  "\x54\x40", "\x00\x00", "\x54\x42", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x54\x46", "\x00\x00", "\x54\x48", "\x54\x49", "\x54\x4a", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x54\x4e", "\x00\x00", "\x00\x00", "\x54\x51",
  "\x54\x5f", "\x54\x68", "\x00\x00", "\x54\x6a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x54\x70", "\x54\x71", "\x00\x00",
  "\x54\x73", "\x00\x00", "\x54\x75", "\x54\x76", "\x54\x77", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x54\x7b", "\x54\x7c", "\x54\x7d", "\x00\x00",
  "\x00\x00", "\x54\x80", "\x00\x00", "\x00\x00", "\x00\x00", "\x54\x84",
  "\x00\x00", "\x54\x86", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x54\x8b", "\x54\x8c", "\x00\x00", "\x54\x8e", "\x54\x8f", "\x54\x90",
  "\x00\x00", "\x54\x92", "\x54\xa2", "\x00\x00", "\x54\xa4", "\x54\xa5",
  "\x00\x00", "\x00\x00", "\x54\xa8", "\x00\x00", "\x00\x00", "\x54\xab",
  "\x54\xac", "\x00\x00", "\x00\x00", "\x54\xaf", "\x00\x00", "\x00\x00",
  "\x54\xb2", "\x54\xb3", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x54\xb8", "\x00\x00", "\x00\x00", "\x00\x00", "\x54\xbc", "\x54\xbd",
  "\x54\xbe", "\x00\x00", "\x54\xc0", "\x54\xc1", "\x54\xc2", "\x00\x00",
  "\x54\xc4", "\x00\x00", "\x00\x00", "\x54\xc7", "\x54\xc8", "\x54\xc9",
  "\x54\xd8", "\x54\xe1", "\x54\xe2", "\x00\x00", "\x00\x00", "\x54\xe5",
  "\x54\xe6", "\x00\x00", "\x54\xe8", "\x54\xe9", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x54\xed", "\x54\xee", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x54\xf2", "\x54\xfa", "\x00\x00", "\x00\x00", "\x54\xfd", "\x55\x04",
  "\x00\x00", "\x55\x06", "\x55\x07", "\x55\x0f", "\x55\x10", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x55\x14", "\x00\x00", "\x55\x16", "\x55\x2e",
  "\x55\x2f", "\x00\x00", "\x55\x31", "\x00\x00", "\x55\x33", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x55\x38", "\x55\x39", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x55\x3e", "\x00\x00", "\x55\x40",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x55\x44", "\x55\x45", "\x55\x46",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x55\x4c",
  "\x00\x00", "\x00\x00", "\x55\x4f", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x55\x53", "\x00\x00", "\x00\x00", "\x55\x56", "\x55\x57", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x55\x5c", "\x55\x5d", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x55\x63", "\x55\x7b",
  "\x55\x7c", "\x00\x00", "\x55\x7e", "\x00\x00", "\x55\x80", "\x00\x00",
  "\x00\x00", "\x55\x83", "\x55\x84", "\x00\x00", "\x00\x00", "\x55\x87",
  "\x00\x00", "\x55\x89", "\x55\x8a", "\x55\x8b", "\x55\x98", "\x55\x99",
  "\x55\x9a", "\x00\x00", "\x55\x9c", "\x55\x9d", "\x55\x9e", "\x55\x9f",
  "\x55\xa7", "\x55\xa8", "\x55\xa9", "\x55\xaa", "\x55\xab", "\x55\xac",
  "\x00\x00", "\x55\xae", "\x00\x00", "\x55\xb0", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x55\xb6", "\x55\xc4", "\x55\xc5",
  "\x00\x00", "\x55\xc7", "\x55\xd4", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x55\xda", "\x00\x00", "\x55\xdc", "\x00\x00",
  "\x00\x00", "\x55\xdf", "\x00\x00", "\x00\x00", "\x00\x00", "\x55\xe3",
  "\x55\xe4", "\x55\xf7", "\x00\x00", "\x55\xf9", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x55\xfd", "\x55\xfe", "\x56\x06", "\x00\x00", "\x00\x00",
  "\x56\x09", "\x56\x14", "\x00\x00", "\x56\x16", "\x56\x17", "\x56\x18",
  "\x00\x00", "\x00\x00", "\x56\x1b", "\x56\x29", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x56\x2f", "\x00\x00", "\x56\x31",
  "\x56\x32", "\x00\x00", "\x56\x34", "\x00\x00", "\x56\x36", "\x00\x00",
  "\x56\x38", "\x56\x42", "\x56\x4c", "\x00\x00", "\x56\x4e", "\x00\x00",
  "\x56\x50", "\x56\x5b", "\x56\x64", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x56\x68", "\x00\x00", "\x56\x6a", "\x56\x6b", "\x56\x6c", "\x56\x74",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x56\x78", "\x00\x00", "\x56\x7a",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x56\x80",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x56\x86",
  "\x56\x87", "\x00\x00", "\x00\x00", "\x56\x8a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x56\x8f", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x56\x94", "\x56\xa0", "\x00\x00", "\x56\xa2", "\x00\x00",
  "\x00\x00", "\x56\xa5", "\x56\xae", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x56\xb4", "\x00\x00", "\x56\xb6", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x56\xbc", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x56\xc0", "\x56\xc1", "\x56\xc2", "\x56\xc3",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x56\xc8", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x56\xce", "\x00\x00",
  "\x00\x00", "\x56\xd1", "\x00\x00", "\x56\xd3", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x56\xd7", "\x56\xd8", "\x00\x00", "\x56\xda", "\x56\xdb",
  "\x00\x00", "\x00\x00", "\x56\xde", "\x00\x00", "\x56\xe0", "\x00\x00",
  "\x00\x00", "\x56\xe3", "\x56\xee", "\x00\x00", "\x56\xf0", "\x00\x00",
  "\x56\xf2", "\x56\xf3", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x56\xf9", "\x56\xfa", "\x00\x00", "\x00\x00", "\x56\xfd",
  "\x00\x00", "\x56\xff", "\x57\x00", "\x00\x00", "\x00\x00", "\x57\x03",
  "\x57\x04", "\x00\x00", "\x00\x00", "\x00\x00", "\x57\x08", "\x57\x09",
  "\x00\x00", "\x57\x0b", "\x00\x00", "\x57\x0d", "\x00\x00", "\x57\x0f",
  "\x00\x00", "\x00\x00", "\x57\x12", "\x57\x13", "\x00\x00", "\x00\x00",
  "\x57\x16", "\x00\x00", "\x57\x18", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x57\x1c", "\x00\x00", "\x00\x00", "\x57\x1f", "\x57\x26", "\x57\x27",
  "\x57\x28", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x57\x2d",
  "\x00\x00", "\x00\x00", "\x57\x30", "\x57\x37", "\x57\x38", "\x00\x00",
  "\x00\x00", "\x57\x3b", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x57\x40", "\x00\x00", "\x57\x42", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x57\x47", "\x00\x00", "\x00\x00", "\x57\x4a", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x57\x4e", "\x57\x4f", "\x57\x50", "\x57\x51",
  "\x57\x61", "\x00\x00", "\x00\x00", "\x57\x64", "\x00\x00", "\x57\x66",
  "\x00\x00", "\x00\x00", "\x57\x69", "\x57\x6a", "\x57\x7f", "\x00\x00",
  "\x00\x00", "\x57\x82", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x57\x88", "\x57\x89", "\x00\x00", "\x57\x8b", "\x57\x93",
  "\x57\xa0", "\x00\x00", "\x57\xa2", "\x57\xa3", "\x57\xa4", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x57\xaa", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x57\xb0", "\x00\x00",
  "\x00\x00", "\x57\xb3", "\x57\xc0", "\x00\x00", "\x00\x00", "\x57\xc3",
  "\x00\x00", "\x00\x00", "\x57\xc6", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x57\xcb", "\x00\x00", "\x00\x00", "\x57\xce", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x57\xd2", "\x57\xd3", "\x57\xd4", "\x00\x00",
  "\x57\xd6", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x57\xdc", "\x00\x00", "\x00\x00", "\x57\xdf", "\x57\xe0", "\x00\x00",
  "\x00\x00", "\x57\xe3", "\x57\xf4", "\x00\x00", "\x00\x00", "\x57\xf7",
  "\x00\x00", "\x57\xf9", "\x57\xfa", "\x00\x00", "\x57\xfc", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x58\x00", "\x00\x00", "\x58\x02", "\x00\x00",
  "\x00\x00", "\x58\x05", "\x58\x06", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x58\x0a", "\x58\x0b", "\x58\x15", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x58\x19", "\x00\x00", "\x00\x00", "\x00\x00", "\x58\x1d", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x58\x21", "\x00\x00", "\x00\x00", "\x58\x24",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x58\x2a",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x58\x2f", "\x58\x30",
  "\x58\x31", "\x00\x00", "\x00\x00", "\x58\x34", "\x58\x35", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x58\x3a", "\x00\x00", "\x00\x00",
  "\x58\x3d", "\x00\x00", "\x00\x00", "\x58\x40", "\x58\x41", "\x58\x4a",
  "\x58\x4b", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x58\x51", "\x58\x52", "\x00\x00", "\x58\x54", "\x00\x00", "\x00\x00",
  "\x58\x57", "\x58\x58", "\x58\x59", "\x58\x5a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x58\x5e", "\x00\x00", "\x00\x00", "\x00\x00", "\x58\x62",
  "\x58\x69", "\x00\x00", "\x58\x6b", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x58\x70", "\x00\x00", "\x58\x72", "\x00\x00", "\x00\x00",
  "\x58\x75", "\x00\x00", "\x00\x00", "\x00\x00", "\x58\x79", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x58\x7e", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x58\x83", "\x00\x00", "\x58\x85", "\x58\x93",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x58\x97", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x58\x9c", "\x00\x00", "\x00\x00", "\x58\x9f",
  "\x58\xa8", "\x00\x00", "\x00\x00", "\x58\xab", "\x00\x00", "\x00\x00",
  "\x58\xae", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x58\xb3",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x58\xb8", "\x58\xb9",
  "\x58\xba", "\x58\xbb", "\x00\x00", "\x00\x00", "\x58\xbe", "\x00\x00",
  "\x00\x00", "\x58\xc1", "\x00\x00", "\x00\x00", "\x00\x00", "\x58\xc5",
  "\x00\x00", "\x58\xc7", "\x00\x00", "\x00\x00", "\x58\xca", "\x00\x00",
  "\x58\xcc", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x58\xd1",
  "\x00\x00", "\x58\xd3", "\x00\x00", "\x58\xd5", "\x00\x00", "\x58\xd7",
  "\x58\xd8", "\x58\xd9", "\x00\x00", "\x00\x00", "\x58\xdc", "\x00\x00",
  "\x58\xde", "\x58\xdf", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x58\xe4", "\x58\xe5", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x58\xeb", "\x58\xec", "\x00\x00", "\x58\xee", "\x58\xef",
  "\x58\xf0", "\x58\xf1", "\x58\xf2", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x58\xf7", "\x00\x00", "\x58\xf9", "\x58\xfa", "\x58\xfb",
  "\x58\xfc", "\x58\xfd", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x59\x02", "\x59\x09", "\x59\x0a", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x59\x0f", "\x59\x10", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x59\x15", "\x59\x16", "\x00\x00", "\x59\x18", "\x59\x19",
  "\x59\x1a", "\x59\x1b", "\x59\x1c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x59\x22", "\x00\x00", "\x00\x00", "\x59\x25",
  "\x00\x00", "\x59\x27", "\x00\x00", "\x59\x29", "\x59\x2a", "\x59\x2b",
  "\x59\x2c", "\x59\x2d", "\x59\x2e", "\x00\x00", "\x00\x00", "\x59\x31",
  "\x59\x32", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x59\x37",
  "\x59\x38", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x59\x3e", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x59\x44", "\x00\x00", "\x00\x00", "\x59\x47", "\x59\x48", "\x59\x49",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x59\x4e", "\x59\x4f",
  "\x59\x50", "\x59\x51", "\x00\x00", "\x00\x00", "\x59\x54", "\x59\x55",
  "\x00\x00", "\x59\x57", "\x59\x58", "\x00\x00", "\x59\x5a", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x59\x60", "\x00\x00",
  "\x59\x62", "\x00\x00", "\x00\x00", "\x59\x65", "\x00\x00", "\x59\x67",
  "\x59\x68", "\x59\x69", "\x59\x6a", "\x00\x00", "\x59\x6c", "\x00\x00",
  "\x59\x6e", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x59\x73",
  "\x59\x74", "\x00\x00", "\x00\x00", "\x00\x00", "\x59\x78", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x59\x7d", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x59\x81", "\x59\x82", "\x59\x83", "\x59\x84", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x59\x8a", "\x00\x00",
  "\x00\x00", "\x59\x8d", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x59\x93", "\x00\x00", "\x00\x00", "\x59\x96", "\x00\x00",
  "\x00\x00", "\x59\x99", "\x00\x00", "\x59\x9b", "\x00\x00", "\x59\x9d",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x59\xa3",
  "\x00\x00", "\x59\xa5", "\x00\x00", "\x00\x00", "\x59\xa8", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x59\xac", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x59\xb2", "\x59\xb9", "\x00\x00", "\x59\xbb",
  "\x00\x00", "\x00\x00", "\x59\xbe", "\x59\xc6", "\x00\x00", "\x00\x00",
  "\x59\xc9", "\x00\x00", "\x59\xcb", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x59\xd0", "\x59\xd1", "\x00\x00", "\x59\xd3", "\x59\xd4",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x59\xd9", "\x59\xda",
  "\x00\x00", "\x59\xdc", "\x59\xe5", "\x59\xe6", "\x00\x00", "\x59\xe8",
  "\x00\x00", "\x59\xea", "\x59\xeb", "\x59\xf6", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x59\xfb", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x59\xff", "\x00\x00", "\x5a\x01", "\x00\x00", "\x5a\x03", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5a\x09", "\x5a\x11",
  "\x5a\x18", "\x00\x00", "\x5a\x1a", "\x00\x00", "\x5a\x1c", "\x00\x00",
  "\x00\x00", "\x5a\x1f", "\x5a\x20", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5a\x25", "\x00\x00", "\x00\x00", "\x00\x00", "\x5a\x29",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5a\x2f",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5a\x35",
  "\x5a\x36", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x5a\x3c", "\x00\x00", "\x00\x00", "\x00\x00", "\x5a\x40", "\x5a\x41",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5a\x46", "\x00\x00",
  "\x00\x00", "\x5a\x49", "\x5a\x5a", "\x5a\x62", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5a\x66", "\x00\x00", "\x00\x00", "\x00\x00", "\x5a\x6a",
  "\x00\x00", "\x5a\x6c", "\x5a\x7f", "\x5a\x92", "\x5a\x9a", "\x5a\x9b",
  "\x5a\xbc", "\x5a\xbd", "\x5a\xbe", "\x00\x00", "\x00\x00", "\x5a\xc1",
  "\x5a\xc2", "\x5a\xc9", "\x00\x00", "\x5a\xcb", "\x5a\xcc", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5a\xd0", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5a\xd6", "\x5a\xd7", "\x5a\xe1", "\x00\x00",
  "\x5a\xe3", "\x00\x00", "\x00\x00", "\x5a\xe6", "\x00\x00", "\x00\x00",
  "\x5a\xe9", "\x5a\xfa", "\x5a\xfb", "\x5b\x09", "\x00\x00", "\x5b\x0b",
  "\x5b\x0c", "\x5b\x16", "\x5b\x22", "\x5b\x2a", "\x00\x00", "\x5b\x2c",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5b\x30", "\x00\x00", "\x5b\x32",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5b\x36", "\x5b\x3e", "\x00\x00",
  "\x5b\x40", "\x00\x00", "\x00\x00", "\x5b\x43", "\x00\x00", "\x5b\x45",
  "\x5b\x50", "\x5b\x51", "\x00\x00", "\x00\x00", "\x5b\x54", "\x5b\x55",
  "\x00\x00", "\x5b\x57", "\x5b\x58", "\x00\x00", "\x5b\x5a", "\x5b\x5b",
  "\x5b\x5c", "\x5b\x5d", "\x00\x00", "\x5b\x5f", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5b\x63", "\x5b\x64", "\x5b\x65", "\x5b\x66", "\x00\x00",
  "\x00\x00", "\x5b\x69", "\x00\x00", "\x5b\x6b", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5b\x70", "\x5b\x71", "\x00\x00", "\x5b\x73",
  "\x00\x00", "\x5b\x75", "\x00\x00", "\x00\x00", "\x5b\x78", "\x00\x00",
  "\x5b\x7a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x5b\x80", "\x00\x00", "\x00\x00", "\x5b\x83", "\x00\x00", "\x5b\x85",
  "\x00\x00", "\x5b\x87", "\x5b\x88", "\x5b\x89", "\x00\x00", "\x5b\x8b",
  "\x5b\x8c", "\x5b\x8d", "\x00\x00", "\x5b\x8f", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5b\x95", "\x00\x00", "\x5b\x97",
  "\x5b\x98", "\x5b\x99", "\x5b\x9a", "\x5b\x9b", "\x5b\x9c", "\x5b\x9d",
  "\x00\x00", "\x5b\x9f", "\x00\x00", "\x00\x00", "\x5b\xa2", "\x5b\xa3",
  "\x5b\xa4", "\x5b\xa5", "\x5b\xa6", "\x5b\xae", "\x00\x00", "\x5b\xb0",
  "\x00\x00", "\x00\x00", "\x5b\xb3", "\x5b\xb4", "\x5b\xb5", "\x5b\xb6",
  "\x00\x00", "\x5b\xb8", "\x5b\xb9", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5b\xbf", "\x00\x00", "\x00\x00", "\x5b\xc2",
  "\x5b\xc3", "\x5b\xc4", "\x5b\xc5", "\x5b\xc6", "\x5b\xc7", "\x00\x00",
  "\x5b\xc9", "\x00\x00", "\x00\x00", "\x5b\xcc", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5b\xd0", "\x00\x00", "\x5b\xd2", "\x5b\xd3", "\x5b\xd4",
  "\x5b\xdb", "\x00\x00", "\x5b\xdd", "\x5b\xde", "\x5b\xdf", "\x00\x00",
  "\x5b\xe1", "\x5b\xe2", "\x00\x00", "\x5b\xe4", "\x5b\xe5", "\x5b\xe6",
  "\x5b\xe7", "\x5b\xe8", "\x5b\xe9", "\x00\x00", "\x5b\xeb", "\x00\x00",
  "\x00\x00", "\x5b\xee", "\x00\x00", "\x5b\xf0", "\x00\x00", "\x00\x00",
  "\x5b\xf3", "\x00\x00", "\x5b\xf5", "\x5b\xf6", "\x00\x00", "\x5b\xf8",
  "\x00\x00", "\x5b\xfa", "\x00\x00", "\x00\x00", "\x00\x00", "\x5b\xfe",
  "\x5b\xff", "\x00\x00", "\x5c\x01", "\x5c\x02", "\x00\x00", "\x5c\x04",
  "\x5c\x05", "\x5c\x06", "\x5c\x07", "\x5c\x08", "\x5c\x09", "\x5c\x0a",
  "\x5c\x0b", "\x00\x00", "\x5c\x0d", "\x5c\x0e", "\x5c\x0f", "\x00\x00",
  "\x5c\x11", "\x00\x00", "\x5c\x13", "\x00\x00", "\x00\x00", "\x5c\x16",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5c\x1a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5c\x20", "\x00\x00", "\x5c\x22",
  "\x00\x00", "\x5c\x24", "\x00\x00", "\x00\x00", "\x00\x00", "\x5c\x28",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5c\x2d", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5c\x31", "\x5c\x38", "\x5c\x39", "\x5c\x3a",
  "\x5c\x3b", "\x5c\x3c", "\x5c\x3d", "\x5c\x3e", "\x5c\x3f", "\x5c\x40",
  "\x5c\x41", "\x00\x00", "\x00\x00", "\x00\x00", "\x5c\x45", "\x5c\x46",
  "\x00\x00", "\x5c\x48", "\x00\x00", "\x5c\x4a", "\x5c\x4b", "\x00\x00",
  "\x5c\x4d", "\x5c\x4e", "\x5c\x4f", "\x5c\x50", "\x5c\x51", "\x00\x00",
  "\x5c\x53", "\x00\x00", "\x5c\x55", "\x5c\x5e", "\x00\x00", "\x5c\x60",
  "\x5c\x61", "\x00\x00", "\x00\x00", "\x5c\x64", "\x5c\x65", "\x5c\x6c",
  "\x00\x00", "\x5c\x6e", "\x5c\x6f", "\x00\x00", "\x5c\x71", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5c\x76", "\x00\x00", "\x00\x00",
  "\x5c\x79", "\x5c\x8c", "\x00\x00", "\x00\x00", "\x00\x00", "\x5c\x90",
  "\x5c\x91", "\x00\x00", "\x00\x00", "\x5c\x94", "\x5c\xa1", "\x5c\xa8",
  "\x5c\xa9", "\x00\x00", "\x5c\xab", "\x5c\xac", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5c\xb1", "\x00\x00", "\x5c\xb3", "\x00\x00",
  "\x00\x00", "\x5c\xb6", "\x5c\xb7", "\x5c\xb8", "\x00\x00", "\x00\x00",
  "\x5c\xbb", "\x5c\xbc", "\x00\x00", "\x5c\xbe", "\x5c\xc5", "\x00\x00",
  "\x5c\xc7", "\x5c\xd9", "\x5c\xe0", "\x5c\xe1", "\x5c\xe8", "\x5c\xe9",
  "\x5c\xea", "\x00\x00", "\x00\x00", "\x5c\xed", "\x00\x00", "\x5c\xef",
  "\x5c\xf0", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x5c\xf6", "\x00\x00", "\x00\x00", "\x00\x00", "\x5c\xfa", "\x5c\xfb",
  "\x00\x00", "\x5c\xfd", "\x5d\x07", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x5d\x0b", "\x00\x00", "\x00\x00", "\x5d\x0e", "\x00\x00", "\x00\x00",
  "\x5d\x11", "\x00\x00", "\x00\x00", "\x5d\x14", "\x5d\x15", "\x5d\x16",
  "\x5d\x17", "\x5d\x18", "\x5d\x19", "\x5d\x1a", "\x5d\x1b", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5d\x1f", "\x00\x00", "\x00\x00", "\x5d\x22",
  "\x5d\x29", "\x5d\x4b", "\x5d\x4c", "\x00\x00", "\x5d\x4e", "\x00\x00",
  "\x5d\x50", "\x00\x00", "\x5d\x52", "\x5d\x5c", "\x5d\x69", "\x00\x00",
  "\x00\x00", "\x5d\x6c", "\x00\x00", "\x00\x00", "\x5d\x6f", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5d\x73", "\x00\x00", "\x00\x00", "\x5d\x76",
  "\x5d\x82", "\x00\x00", "\x5d\x84", "\x00\x00", "\x00\x00", "\x5d\x87",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5d\x8b", "\x5d\x8c", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5d\x90", "\x5d\x9d", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5d\xa2", "\x5d\xac", "\x00\x00", "\x5d\xae",
  "\x5d\xb7", "\x00\x00", "\x00\x00", "\x5d\xba", "\x00\x00", "\x5d\xbc",
  "\x5d\xbd", "\x5d\xc9", "\x00\x00", "\x00\x00", "\x5d\xcc", "\x5d\xcd",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5d\xd2", "\x5d\xd3",
  "\x00\x00", "\x00\x00", "\x5d\xd6", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5d\xdb", "\x00\x00", "\x5d\xdd", "\x5d\xde", "\x00\x00",
  "\x00\x00", "\x5d\xe1", "\x00\x00", "\x5d\xe3", "\x00\x00", "\x5d\xe5",
  "\x5d\xe6", "\x5d\xe7", "\x5d\xe8", "\x00\x00", "\x00\x00", "\x5d\xeb",
  "\x00\x00", "\x00\x00", "\x5d\xee", "\x00\x00", "\x00\x00", "\x5d\xf1",
  "\x5d\xf2", "\x5d\xf3", "\x5d\xf4", "\x5d\xf5", "\x00\x00", "\x5d\xf7",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5d\xfb", "\x00\x00", "\x5d\xfd",
  "\x5d\xfe", "\x00\x00", "\x00\x00", "\x00\x00", "\x5e\x02", "\x5e\x03",
  "\x00\x00", "\x00\x00", "\x5e\x06", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5e\x0b", "\x5e\x0c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5e\x11", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x5e\x16", "\x00\x00", "\x00\x00", "\x5e\x19", "\x5e\x1a", "\x5e\x1b",
  "\x00\x00", "\x5e\x1d", "\x5e\x25", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5e\x2b", "\x00\x00", "\x5e\x2d", "\x00\x00",
  "\x5e\x2f", "\x5e\x30", "\x00\x00", "\x00\x00", "\x5e\x33", "\x00\x00",
  "\x00\x00", "\x5e\x36", "\x5e\x37", "\x5e\x38", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5e\x3d", "\x00\x00", "\x00\x00", "\x5e\x40",
  "\x00\x00", "\x00\x00", "\x5e\x43", "\x5e\x44", "\x5e\x45", "\x00\x00",
  "\x5e\x47", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5e\x4c",
  "\x00\x00", "\x5e\x4e", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5e\x54", "\x5e\x55", "\x00\x00", "\x5e\x57", "\x5e\x5f",
  "\x00\x00", "\x5e\x61", "\x5e\x62", "\x5e\x63", "\x5e\x64", "\x5e\x72",
  "\x5e\x73", "\x5e\x74", "\x5e\x75", "\x5e\x76", "\x00\x00", "\x5e\x78",
  "\x5e\x79", "\x5e\x7a", "\x5e\x7b", "\x5e\x7c", "\x5e\x7d", "\x5e\x7e",
  "\x5e\x7f", "\x00\x00", "\x5e\x81", "\x00\x00", "\x5e\x83", "\x5e\x84",
  "\x00\x00", "\x00\x00", "\x5e\x87", "\x00\x00", "\x00\x00", "\x5e\x8a",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5e\x8f", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5e\x95", "\x5e\x96",
  "\x5e\x97", "\x00\x00", "\x00\x00", "\x5e\x9a", "\x00\x00", "\x5e\x9c",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5e\xa0", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5e\xa6", "\x5e\xa7", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5e\xab", "\x00\x00", "\x5e\xad", "\x5e\xb5",
  "\x5e\xb6", "\x5e\xb7", "\x5e\xb8", "\x5e\xc1", "\x5e\xc2", "\x5e\xc3",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5e\xc8", "\x5e\xc9",
  "\x5e\xca", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5e\xcf",
  "\x5e\xd0", "\x00\x00", "\x00\x00", "\x5e\xd3", "\x00\x00", "\x00\x00",
  "\x5e\xd6", "\x00\x00", "\x00\x00", "\x00\x00", "\x5e\xda", "\x5e\xdb",
  "\x00\x00", "\x5e\xdd", "\x00\x00", "\x5e\xdf", "\x5e\xe0", "\x5e\xe1",
  "\x5e\xe2", "\x5e\xe3", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x5e\xe8", "\x5e\xe9", "\x00\x00", "\x00\x00", "\x5e\xec", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5e\xf0", "\x5e\xf1", "\x00\x00", "\x5e\xf3",
  "\x5e\xf4", "\x00\x00", "\x5e\xf6", "\x5e\xf7", "\x5e\xf8", "\x00\x00",
  "\x5e\xfa", "\x5e\xfb", "\x5e\xfc", "\x00\x00", "\x5e\xfe", "\x5e\xff",
  "\x00\x00", "\x5f\x01", "\x00\x00", "\x5f\x03", "\x5f\x04", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5f\x09", "\x5f\x0a", "\x5f\x0b",
  "\x5f\x0c", "\x5f\x0d", "\x00\x00", "\x5f\x0f", "\x5f\x10", "\x5f\x11",
  "\x00\x00", "\x5f\x13", "\x5f\x14", "\x5f\x15", "\x5f\x16", "\x5f\x17",
  "\x5f\x18", "\x00\x00", "\x00\x00", "\x5f\x1b", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5f\x1f", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5f\x25", "\x5f\x26", "\x5f\x27", "\x00\x00", "\x5f\x29",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5f\x2d", "\x00\x00", "\x5f\x2f",
  "\x00\x00", "\x5f\x31", "\x00\x00", "\x00\x00", "\x00\x00", "\x5f\x35",
  "\x00\x00", "\x5f\x37", "\x5f\x38", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x5f\x3c", "\x00\x00", "\x5f\x3e", "\x00\x00", "\x00\x00", "\x5f\x41",
  "\x5f\x48", "\x00\x00", "\x5f\x4a", "\x00\x00", "\x5f\x4c", "\x00\x00",
  "\x5f\x4e", "\x00\x00", "\x00\x00", "\x5f\x51", "\x00\x00", "\x5f\x53",
  "\x00\x00", "\x00\x00", "\x5f\x56", "\x5f\x57", "\x00\x00", "\x5f\x59",
  "\x00\x00", "\x00\x00", "\x5f\x5c", "\x5f\x5d", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5f\x61", "\x5f\x62", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x5f\x66", "\x00\x00", "\x00\x00", "\x5f\x69", "\x5f\x6a", "\x5f\x6b",
  "\x5f\x6c", "\x5f\x6d", "\x00\x00", "\x00\x00", "\x5f\x70", "\x5f\x71",
  "\x00\x00", "\x5f\x73", "\x00\x00", "\x00\x00", "\x00\x00", "\x5f\x77",
  "\x00\x00", "\x5f\x79", "\x00\x00", "\x00\x00", "\x5f\x7c", "\x00\x00",
  "\x00\x00", "\x5f\x7f", "\x5f\x80", "\x5f\x81", "\x5f\x82", "\x5f\x83",
  "\x5f\x84", "\x5f\x85", "\x00\x00", "\x5f\x87", "\x5f\x88", "\x00\x00",
  "\x5f\x8a", "\x5f\x8b", "\x5f\x8c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x5f\x90", "\x5f\x91", "\x5f\x92", "\x5f\x93", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x5f\x97", "\x5f\x98", "\x5f\x99", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x5f\x9e", "\x00\x00", "\x5f\xa0", "\x5f\xa1",
  "\x5f\xa8", "\x5f\xa9", "\x5f\xaa", "\x00\x00", "\x00\x00", "\x5f\xad",
  "\x5f\xae", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5f\xb3",
  "\x5f\xb4", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5f\xb9",
  "\x00\x00", "\x00\x00", "\x5f\xbc", "\x5f\xbd", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5f\xc3", "\x00\x00", "\x5f\xc5",
  "\x5f\xcc", "\x5f\xcd", "\x5f\xd6", "\x5f\xd7", "\x5f\xd8", "\x5f\xd9",
  "\x00\x00", "\x00\x00", "\x5f\xdc", "\x5f\xdd", "\x00\x00", "\x00\x00",
  "\x5f\xe0", "\x00\x00", "\x00\x00", "\x00\x00", "\x5f\xe4", "\x5f\xeb",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x5f\xf0", "\x5f\xf1",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x5f\xf5", "\x00\x00", "\x00\x00",
  "\x5f\xf8", "\x00\x00", "\x00\x00", "\x5f\xfb", "\x00\x00", "\x5f\xfd",
  "\x00\x00", "\x5f\xff", "\x60\x0e", "\x60\x0f", "\x60\x10", "\x00\x00",
  "\x60\x12", "\x00\x00", "\x00\x00", "\x60\x15", "\x60\x16", "\x00\x00",
  "\x00\x00", "\x60\x19", "\x00\x00", "\x60\x1b", "\x60\x1c", "\x60\x1d",
  "\x00\x00", "\x00\x00", "\x60\x20", "\x60\x21", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x60\x25", "\x60\x26", "\x60\x27", "\x60\x28", "\x60\x29",
  "\x60\x2a", "\x60\x2b", "\x00\x00", "\x00\x00", "\x00\x00", "\x60\x2f",
  "\x00\x00", "\x60\x31", "\x60\x3a", "\x60\x41", "\x60\x42", "\x60\x43",
  "\x00\x00", "\x00\x00", "\x60\x46", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x60\x4a", "\x60\x4b", "\x00\x00", "\x60\x4d", "\x00\x00", "\x00\x00",
  "\x60\x50", "\x00\x00", "\x60\x52", "\x00\x00", "\x00\x00", "\x60\x55",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x60\x59", "\x60\x5a", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x60\x5f", "\x60\x60", "\x00\x00",
  "\x60\x62", "\x60\x63", "\x60\x64", "\x60\x65", "\x00\x00", "\x00\x00",
  "\x60\x68", "\x60\x69", "\x60\x6a", "\x60\x6b", "\x60\x6c", "\x60\x6d",
  "\x00\x00", "\x60\x6f", "\x60\x70", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x60\x75", "\x00\x00", "\x60\x77", "\x60\x81", "\x00\x00",
  "\x60\x83", "\x60\x84", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x60\x89", "\x00\x00", "\x60\x8b", "\x60\x8c", "\x60\x8d", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x60\x92", "\x00\x00", "\x60\x94",
  "\x00\x00", "\x60\x96", "\x60\x97", "\x00\x00", "\x00\x00", "\x60\x9a",
  "\x60\x9b", "\x00\x00", "\x00\x00", "\x00\x00", "\x60\x9f", "\x60\xa0",
  "\x00\x00", "\x00\x00", "\x60\xa3", "\x00\x00", "\x00\x00", "\x60\xa6",
  "\x60\xa7", "\x00\x00", "\x60\xa9", "\x60\xaa", "\x60\xb2", "\x60\xb3",
  "\x60\xb4", "\x60\xb5", "\x60\xb6", "\x00\x00", "\x60\xb8", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x60\xbc", "\x60\xbd", "\x60\xc5", "\x60\xc6",
  "\x60\xc7", "\x60\xd1", "\x00\x00", "\x60\xd3", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x60\xd8", "\x00\x00", "\x60\xda", "\x00\x00",
  "\x60\xdc", "\x00\x00", "\x00\x00", "\x60\xdf", "\x60\xe0", "\x60\xe1",
  "\x00\x00", "\x60\xe3", "\x00\x00", "\x00\x00", "\x00\x00", "\x60\xe7",
  "\x60\xe8", "\x60\xf0", "\x60\xf1", "\x00\x00", "\x60\xf3", "\x60\xf4",
  "\x00\x00", "\x60\xf6", "\x60\xf7", "\x00\x00", "\x60\xf9", "\x60\xfa",
  "\x60\xfb", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x61\x00",
  "\x61\x01", "\x00\x00", "\x61\x03", "\x00\x00", "\x00\x00", "\x61\x06",
  "\x00\x00", "\x61\x08", "\x61\x09", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x61\x0d", "\x61\x0e", "\x61\x0f", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x61\x15", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x61\x1a", "\x61\x1b", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x61\x1f", "\x00\x00", "\x61\x21", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x61\x27", "\x61\x28", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x61\x2c", "\x61\x34", "\x61\x3c", "\x61\x3d", "\x61\x3e",
  "\x61\x3f", "\x00\x00", "\x00\x00", "\x61\x42", "\x00\x00", "\x61\x44",
  "\x00\x00", "\x00\x00", "\x61\x47", "\x61\x48", "\x00\x00", "\x61\x4a",
  "\x61\x4b", "\x61\x4c", "\x61\x4d", "\x61\x4e", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x61\x53", "\x00\x00", "\x61\x55", "\x00\x00",
  "\x00\x00", "\x61\x58", "\x61\x59", "\x61\x5a", "\x00\x00", "\x00\x00",
  "\x61\x5d", "\x00\x00", "\x61\x5f", "\x00\x00", "\x00\x00", "\x61\x62",
  "\x61\x63", "\x00\x00", "\x61\x65", "\x00\x00", "\x61\x67", "\x61\x68",
  "\x00\x00", "\x00\x00", "\x61\x6b", "\x00\x00", "\x00\x00", "\x61\x6e",
  "\x61\x6f", "\x61\x70", "\x61\x71", "\x00\x00", "\x61\x73", "\x61\x74",
  "\x61\x75", "\x61\x76", "\x61\x77", "\x61\x7e", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x61\x82", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x61\x87", "\x00\x00", "\x00\x00", "\x61\x8a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x61\x8e", "\x00\x00", "\x61\x90", "\x61\x91", "\x00\x00",
  "\x00\x00", "\x61\x94", "\x00\x00", "\x61\x96", "\x00\x00", "\x00\x00",
  "\x61\x99", "\x61\x9a", "\x61\xa4", "\x00\x00", "\x00\x00", "\x61\xa7",
  "\x00\x00", "\x61\xa9", "\x00\x00", "\x61\xab", "\x61\xac", "\x00\x00",
  "\x61\xae", "\x00\x00", "\x00\x00", "\x00\x00", "\x61\xb2", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x61\xb6", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x61\xba", "\x00\x00", "\x00\x00", "\x00\x00", "\x61\xbe", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x61\xc3", "\x00\x00", "\x00\x00",
  "\x61\xc6", "\x61\xc7", "\x61\xc8", "\x61\xc9", "\x61\xca", "\x61\xcb",
  "\x61\xcc", "\x61\xcd", "\x00\x00", "\x00\x00", "\x61\xd0", "\x61\xe3",
  "\x00\x00", "\x00\x00", "\x61\xe6", "\x61\xf2", "\x00\x00", "\x61\xf4",
  "\x00\x00", "\x61\xf6", "\x61\xf7", "\x61\xf8", "\x00\x00", "\x61\xfa",
  "\x00\x00", "\x61\xfc", "\x61\xfd", "\x61\xfe", "\x61\xff", "\x62\x00",
  "\x62\x08", "\x62\x09", "\x62\x0a", "\x00\x00", "\x62\x0c", "\x62\x0d",
  "\x62\x0e", "\x00\x00", "\x62\x10", "\x62\x11", "\x62\x12", "\x00\x00",
  "\x62\x14", "\x00\x00", "\x62\x16", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x62\x1a", "\x62\x1b", "\x00\x00", "\x62\x1d", "\x62\x1e", "\x62\x1f",
  "\x00\x00", "\x62\x21", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x62\x26", "\x00\x00", "\x00\x00", "\x00\x00", "\x62\x2a", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x62\x2e", "\x62\x2f", "\x62\x30", "\x00\x00",
  "\x62\x32", "\x62\x33", "\x62\x34", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x62\x38", "\x00\x00", "\x00\x00", "\x62\x3b", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x62\x3f", "\x62\x40", "\x62\x41", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x62\x47", "\x62\x48", "\x62\x49",
  "\x00\x00", "\x62\x4b", "\x00\x00", "\x62\x4d", "\x62\x4e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x62\x53", "\x00\x00", "\x62\x55",
  "\x00\x00", "\x00\x00", "\x62\x58", "\x00\x00", "\x00\x00", "\x62\x5b",
  "\x00\x00", "\x00\x00", "\x62\x5e", "\x00\x00", "\x62\x60", "\x00\x00",
  "\x00\x00", "\x62\x63", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x62\x68", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x62\x6e", "\x00\x00", "\x00\x00", "\x62\x71", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x62\x76", "\x00\x00", "\x00\x00", "\x62\x79",
  "\x00\x00", "\x00\x00", "\x62\x7c", "\x00\x00", "\x62\x7e", "\x62\x7f",
  "\x62\x80", "\x00\x00", "\x62\x82", "\x62\x83", "\x62\x84", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x62\x89", "\x62\x8a", "\x62\x91",
  "\x62\x92", "\x62\x93", "\x62\x94", "\x62\x95", "\x62\x96", "\x62\x97",
  "\x62\x98", "\x00\x00", "\x00\x00", "\x62\x9b", "\x62\x9c", "\x00\x00",
  "\x62\x9e", "\x62\xab", "\x62\xac", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x62\xb1", "\x00\x00", "\x00\x00", "\x00\x00", "\x62\xb5",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x62\xb9", "\x00\x00", "\x62\xbb",
  "\x62\xbc", "\x62\xbd", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x62\xc2", "\x00\x00", "\x00\x00", "\x62\xc5", "\x62\xc6", "\x62\xc7",
  "\x62\xc8", "\x62\xc9", "\x62\xca", "\x00\x00", "\x62\xcc", "\x62\xcd",
  "\x00\x00", "\x62\xcf", "\x62\xd0", "\x62\xd1", "\x62\xd2", "\x62\xd3",
  "\x62\xd4", "\x00\x00", "\x00\x00", "\x62\xd7", "\x62\xd8", "\x62\xd9",
  "\x00\x00", "\x62\xdb", "\x62\xdc", "\x62\xdd", "\x00\x00", "\x00\x00",
  "\x62\xe0", "\x62\xe1", "\x62\xec", "\x62\xed", "\x62\xee", "\x62\xef",
  "\x00\x00", "\x62\xf1", "\x00\x00", "\x62\xf3", "\x00\x00", "\x62\xf5",
  "\x62\xf6", "\x62\xf7", "\x62\xfe", "\x62\xff", "\x00\x00", "\x63\x01",
  "\x63\x02", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x63\x07",
  "\x63\x08", "\x63\x09", "\x00\x00", "\x00\x00", "\x63\x0c", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x63\x11", "\x63\x19", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x63\x1f", "\x63\x27",
  "\x63\x28", "\x00\x00", "\x00\x00", "\x63\x2b", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x63\x2f", "\x63\x3a", "\x00\x00", "\x00\x00", "\x63\x3d",
  "\x63\x3e", "\x63\x3f", "\x63\x49", "\x00\x00", "\x00\x00", "\x63\x4c",
  "\x63\x4d", "\x00\x00", "\x63\x4f", "\x63\x50", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x63\x55", "\x00\x00", "\x63\x57", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x63\x5c", "\x63\x67", "\x63\x68",
  "\x63\x69", "\x00\x00", "\x63\x6b", "\x00\x00", "\x00\x00", "\x63\x6e",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x63\x72", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x63\x76", "\x63\x77", "\x00\x00", "\x00\x00", "\x63\x7a",
  "\x63\x7b", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x63\x80",
  "\x00\x00", "\x00\x00", "\x63\x83", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x63\x88", "\x63\x89", "\x00\x00", "\x00\x00", "\x63\x8c",
  "\x00\x00", "\x63\x8e", "\x63\x8f", "\x00\x00", "\x00\x00", "\x63\x92",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x63\x96", "\x00\x00", "\x63\x98",
  "\x00\x00", "\x00\x00", "\x63\x9b", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x63\x9f", "\x63\xa0", "\x63\xa1", "\x63\xa2", "\x63\xa3", "\x00\x00",
  "\x63\xa5", "\x00\x00", "\x63\xa7", "\x63\xa8", "\x63\xa9", "\x63\xaa",
  "\x63\xab", "\x63\xac", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x63\xb2", "\x00\x00", "\x63\xb4", "\x63\xb5", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x63\xbb", "\x00\x00",
  "\x00\x00", "\x63\xbe", "\x00\x00", "\x63\xc0", "\x00\x00", "\x00\x00",
  "\x63\xc3", "\x63\xc4", "\x00\x00", "\x63\xc6", "\x00\x00", "\x00\x00",
  "\x63\xc9", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x63\xcf", "\x63\xd0", "\x00\x00", "\x63\xd2", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x63\xd6", "\x00\x00", "\x00\x00", "\x00\x00", "\x63\xda",
  "\x63\xdb", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x63\xe1", "\x00\x00", "\x63\xe3", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x63\xe9", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x63\xee", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x63\xf4", "\x00\x00", "\x63\xf6", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x63\xfa", "\x64\x06", "\x64\x0d", "\x00\x00", "\x64\x0f",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x64\x13", "\x00\x00", "\x00\x00",
  "\x64\x16", "\x64\x17", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x64\x1c", "\x64\x26", "\x00\x00", "\x64\x28", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x64\x2c", "\x64\x2d", "\x64\x34", "\x00\x00", "\x64\x36",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x64\x3a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x64\x3e", "\x00\x00", "\x00\x00", "\x00\x00", "\x64\x42",
  "\x64\x4e", "\x64\x58", "\x64\x67", "\x00\x00", "\x64\x69", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x64\x6f", "\x64\x76",
  "\x00\x00", "\x64\x78", "\x00\x00", "\x64\x7a", "\x64\x83", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x64\x88", "\x64\x92", "\x64\x93",
  "\x00\x00", "\x64\x95", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x64\x9a", "\x00\x00", "\x00\x00", "\x00\x00", "\x64\x9e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x64\xa4", "\x64\xa5",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x64\xa9", "\x00\x00", "\x64\xab",
  "\x00\x00", "\x64\xad", "\x64\xae", "\x00\x00", "\x64\xb0", "\x00\x00",
  "\x64\xb2", "\x64\xb9", "\x00\x00", "\x64\xbb", "\x64\xbc", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x64\xc1", "\x64\xc2", "\x00\x00",
  "\x00\x00", "\x64\xc5", "\x00\x00", "\x64\xc7", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x64\xcd", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x64\xd2", "\x00\x00", "\x64\xd4", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x64\xd8", "\x00\x00", "\x64\xda", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x64\xe0", "\x64\xe1",
  "\x64\xe2", "\x64\xe3", "\x00\x00", "\x00\x00", "\x64\xe6", "\x64\xe7",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x64\xec", "\x00\x00",
  "\x00\x00", "\x64\xef", "\x00\x00", "\x64\xf1", "\x64\xf2", "\x00\x00",
  "\x64\xf4", "\x00\x00", "\x64\xf6", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x64\xfa", "\x00\x00", "\x00\x00", "\x64\xfd", "\x64\xfe", "\x00\x00",
  "\x65\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x65\x05",
  "\x65\x18", "\x00\x00", "\x00\x00", "\x00\x00", "\x65\x1c", "\x65\x1d",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x65\x23",
  "\x65\x24", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x65\x2a", "\x65\x2b", "\x65\x2c", "\x00\x00", "\x00\x00", "\x65\x2f",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x65\x34", "\x65\x35",
  "\x65\x36", "\x65\x37", "\x65\x38", "\x65\x39", "\x00\x00", "\x65\x3b",
  "\x00\x00", "\x00\x00", "\x65\x3e", "\x65\x3f", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x65\x45", "\x00\x00", "\x00\x00",
  "\x65\x48", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x65\x4d",
  "\x00\x00", "\x65\x4f", "\x00\x00", "\x65\x51", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x65\x55", "\x65\x56", "\x65\x57", "\x65\x58", "\x65\x59",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x65\x5d", "\x65\x5e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x65\x62", "\x65\x63", "\x00\x00", "\x00\x00",
  "\x65\x66", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x65\x6c", "\x00\x00", "\x00\x00", "\x00\x00", "\x65\x70", "\x00\x00",
  "\x65\x72", "\x00\x00", "\x65\x74", "\x65\x75", "\x00\x00", "\x65\x77",
  "\x65\x78", "\x65\x82", "\x65\x83", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x65\x87", "\x65\x88", "\x65\x89", "\x00\x00", "\x00\x00", "\x65\x8c",
  "\x00\x00", "\x65\x8e", "\x00\x00", "\x65\x90", "\x65\x91", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x65\x97", "\x00\x00",
  "\x65\x99", "\x00\x00", "\x65\x9b", "\x65\x9c", "\x00\x00", "\x00\x00",
  "\x65\x9f", "\x00\x00", "\x65\xa1", "\x00\x00", "\x00\x00", "\x65\xa4",
  "\x65\xa5", "\x00\x00", "\x65\xa7", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x65\xab", "\x65\xac", "\x65\xad", "\x00\x00", "\x65\xaf", "\x65\xb0",
  "\x65\xb7", "\x00\x00", "\x65\xb9", "\x00\x00", "\x00\x00", "\x65\xbc",
  "\x65\xbd", "\x00\x00", "\x00\x00", "\x00\x00", "\x65\xc1", "\x00\x00",
  "\x65\xc3", "\x65\xc4", "\x65\xc5", "\x65\xc6", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x65\xcb", "\x65\xcc", "\x00\x00", "\x00\x00",
  "\x65\xcf", "\x00\x00", "\x00\x00", "\x65\xd2", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x65\xd7", "\x00\x00", "\x65\xd9", "\x00\x00",
  "\x65\xdb", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x65\xe0",
  "\x65\xe1", "\x65\xe2", "\x00\x00", "\x00\x00", "\x65\xe5", "\x65\xe6",
  "\x65\xe7", "\x65\xe8", "\x65\xe9", "\x00\x00", "\x00\x00", "\x65\xec",
  "\x65\xed", "\x00\x00", "\x00\x00", "\x00\x00", "\x65\xf1", "\x65\xfa",
  "\x65\xfb", "\x66\x02", "\x66\x03", "\x00\x00", "\x00\x00", "\x66\x06",
  "\x66\x07", "\x00\x00", "\x00\x00", "\x66\x0a", "\x00\x00", "\x66\x0c",
  "\x00\x00", "\x66\x0e", "\x66\x0f", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x66\x13", "\x66\x14", "\x66\x1c", "\x00\x00", "\x00\x00", "\x66\x1f",
  "\x66\x20", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x66\x25",
  "\x00\x00", "\x66\x27", "\x66\x28", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x66\x2d", "\x00\x00", "\x66\x2f", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x66\x34", "\x66\x35", "\x66\x36", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x66\x3c", "\x00\x00",
  "\x00\x00", "\x66\x3f", "\x00\x00", "\x66\x41", "\x66\x42", "\x66\x43",
  "\x66\x44", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x66\x49",
  "\x00\x00", "\x66\x4b", "\x00\x00", "\x00\x00", "\x00\x00", "\x66\x4f",
  "\x00\x00", "\x00\x00", "\x66\x52", "\x66\x5d", "\x66\x5e", "\x66\x5f",
  "\x00\x00", "\x00\x00", "\x66\x62", "\x00\x00", "\x66\x64", "\x00\x00",
  "\x66\x66", "\x66\x67", "\x66\x68", "\x66\x69", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x66\x6e", "\x66\x6f", "\x66\x70", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x66\x74", "\x00\x00", "\x66\x76", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x66\x7a", "\x66\x81", "\x00\x00", "\x66\x83",
  "\x66\x84", "\x00\x00", "\x00\x00", "\x66\x87", "\x66\x88", "\x66\x89",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x66\x8e", "\x00\x00",
  "\x00\x00", "\x66\x91", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x66\x96", "\x66\x97", "\x66\x98", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x66\x9d", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x66\xa2", "\x00\x00", "\x00\x00", "\x00\x00", "\x66\xa6", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x66\xab", "\x00\x00", "\x00\x00",
  "\x66\xae", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x66\xb4", "\x00\x00", "\x00\x00", "\x00\x00", "\x66\xb8", "\x66\xb9",
  "\x00\x00", "\x00\x00", "\x66\xbc", "\x00\x00", "\x66\xbe", "\x00\x00",
  "\x00\x00", "\x66\xc1", "\x00\x00", "\x00\x00", "\x66\xc4", "\x00\x00",
  "\x00\x00", "\x66\xc7", "\x00\x00", "\x66\xc9", "\x66\xd6", "\x00\x00",
  "\x00\x00", "\x66\xd9", "\x66\xda", "\x00\x00", "\x66\xdc", "\x66\xdd",
  "\x00\x00", "\x00\x00", "\x66\xe0", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x66\xe6", "\x00\x00", "\x00\x00", "\x66\xe9",
  "\x66\xf0", "\x00\x00", "\x66\xf2", "\x66\xf3", "\x66\xf4", "\x66\xf5",
  "\x00\x00", "\x66\xf7", "\x66\xf8", "\x66\xf9", "\x00\x00", "\x00\x00",
  "\x66\xfc", "\x66\xfd", "\x66\xfe", "\x66\xff", "\x67\x00", "\x00\x00",
  "\x00\x00", "\x67\x03", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x67\x08", "\x67\x09", "\x00\x00", "\x67\x0b", "\x00\x00", "\x67\x0d",
  "\x00\x00", "\x67\x0f", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x67\x14", "\x67\x15", "\x67\x16", "\x67\x17", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x67\x1b", "\x00\x00", "\x67\x1d", "\x67\x1e", "\x67\x1f",
  "\x67\x26", "\x67\x27", "\x67\x28", "\x00\x00", "\x67\x2a", "\x67\x2b",
  "\x67\x2c", "\x67\x2d", "\x67\x2e", "\x00\x00", "\x00\x00", "\x67\x31",
  "\x00\x00", "\x00\x00", "\x67\x34", "\x00\x00", "\x67\x36", "\x67\x37",
  "\x67\x38", "\x00\x00", "\x67\x3a", "\x00\x00", "\x00\x00", "\x67\x3d",
  "\x00\x00", "\x67\x3f", "\x00\x00", "\x67\x41", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x67\x46", "\x00\x00", "\x00\x00", "\x67\x49",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x67\x4e", "\x67\x4f",
  "\x67\x50", "\x67\x51", "\x00\x00", "\x67\x53", "\x00\x00", "\x00\x00",
  "\x67\x56", "\x00\x00", "\x00\x00", "\x67\x59", "\x00\x00", "\x00\x00",
  "\x67\x5c", "\x00\x00", "\x67\x5e", "\x67\x5f", "\x67\x60", "\x67\x61",
  "\x67\x62", "\x67\x63", "\x67\x64", "\x67\x65", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x67\x6a", "\x00\x00", "\x00\x00", "\x67\x6d",
  "\x00\x00", "\x67\x6f", "\x67\x70", "\x67\x71", "\x67\x72", "\x67\x73",
  "\x00\x00", "\x67\x75", "\x00\x00", "\x67\x77", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x67\x7c", "\x00\x00", "\x67\x7e", "\x67\x7f",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x67\x85",
  "\x00\x00", "\x67\x87", "\x00\x00", "\x67\x89", "\x00\x00", "\x67\x8b",
  "\x67\x8c", "\x00\x00", "\x00\x00", "\x00\x00", "\x67\x90", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x67\x95", "\x00\x00", "\x67\x97",
  "\x00\x00", "\x00\x00", "\x67\x9a", "\x00\x00", "\x67\x9c", "\x67\x9d",
  "\x00\x00", "\x00\x00", "\x67\xa0", "\x67\xa1", "\x67\xa2", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x67\xa6", "\x00\x00", "\x00\x00", "\x67\xa9",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x67\xaf",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x67\xb3", "\x67\xb4", "\x00\x00",
  "\x67\xb6", "\x67\xb7", "\x67\xb8", "\x67\xb9", "\x67\xc1", "\x00\x00",
  "\x00\x00", "\x67\xc4", "\x00\x00", "\x67\xc6", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x67\xca", "\x00\x00", "\x00\x00", "\x00\x00", "\x67\xce",
  "\x67\xcf", "\x67\xd0", "\x67\xd1", "\x00\x00", "\x67\xd3", "\x67\xd4",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x67\xd8", "\x00\x00", "\x67\xda",
  "\x00\x00", "\x00\x00", "\x67\xdd", "\x67\xde", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x67\xe2", "\x00\x00", "\x67\xe4", "\x00\x00", "\x00\x00",
  "\x67\xe7", "\x00\x00", "\x67\xe9", "\x00\x00", "\x00\x00", "\x67\xec",
  "\x00\x00", "\x67\xee", "\x67\xef", "\x00\x00", "\x67\xf1", "\x00\x00",
  "\x67\xf3", "\x67\xf4", "\x67\xf5", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x67\xfb", "\x00\x00", "\x00\x00", "\x67\xfe",
  "\x67\xff", "\x00\x00", "\x00\x00", "\x68\x02", "\x68\x03", "\x68\x04",
  "\x68\x13", "\x00\x00", "\x00\x00", "\x68\x16", "\x68\x17", "\x68\x1e",
  "\x00\x00", "\x00\x00", "\x68\x21", "\x68\x22", "\x68\x29", "\x68\x2a",
  "\x68\x2b", "\x68\x32", "\x00\x00", "\x68\x34", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x68\x38", "\x68\x39", "\x00\x00", "\x00\x00", "\x68\x3c",
  "\x68\x3d", "\x00\x00", "\x00\x00", "\x68\x40", "\x68\x41", "\x68\x42",
  "\x68\x43", "\x00\x00", "\x00\x00", "\x68\x46", "\x00\x00", "\x68\x48",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x68\x4d", "\x68\x4e",
  "\x00\x00", "\x68\x50", "\x68\x51", "\x00\x00", "\x68\x53", "\x68\x54",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x68\x59", "\x00\x00",
  "\x00\x00", "\x68\x5c", "\x68\x5d", "\x00\x00", "\x68\x5f", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x68\x63", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x68\x67", "\x68\x74", "\x00\x00", "\x68\x76", "\x68\x77", "\x68\x7e",
  "\x68\x7f", "\x00\x00", "\x68\x81", "\x00\x00", "\x68\x83", "\x00\x00",
  "\x68\x85", "\x68\x8d", "\x00\x00", "\x68\x8f", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x68\x93", "\x68\x94", "\x00\x00", "\x00\x00", "\x68\x97",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x68\x9b", "\x00\x00", "\x68\x9d",
  "\x00\x00", "\x68\x9f", "\x68\xa0", "\x00\x00", "\x68\xa2", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x68\xa6", "\x68\xa7", "\x68\xa8", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x68\xad", "\x00\x00", "\x68\xaf",
  "\x68\xb0", "\x68\xb1", "\x00\x00", "\x68\xb3", "\x00\x00", "\x68\xb5",
  "\x68\xb6", "\x00\x00", "\x00\x00", "\x68\xb9", "\x68\xba", "\x00\x00",
  "\x68\xbc", "\x68\xc4", "\x00\x00", "\x68\xc6", "\x00\x00", "\x00\x00",
  "\x68\xc9", "\x68\xca", "\x68\xcb", "\x00\x00", "\x68\xcd", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x68\xd2", "\x00\x00", "\x68\xd4",
  "\x68\xd5", "\x00\x00", "\x68\xd7", "\x68\xd8", "\x00\x00", "\x68\xda",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x68\xdf", "\x68\xe0",
  "\x68\xe1", "\x00\x00", "\x68\xe3", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x68\xe7", "\x68\xee", "\x68\xef", "\x00\x00", "\x00\x00", "\x68\xf2",
  "\x68\xf9", "\x68\xfa", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x69\x00", "\x69\x01", "\x00\x00", "\x00\x00", "\x69\x04",
  "\x69\x05", "\x00\x00", "\x00\x00", "\x69\x08", "\x00\x00", "\x00\x00",
  "\x69\x0b", "\x69\x0c", "\x69\x0d", "\x69\x0e", "\x69\x0f", "\x00\x00",
  "\x00\x00", "\x69\x12", "\x69\x19", "\x69\x1a", "\x69\x1b", "\x69\x1c",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x69\x21", "\x69\x22",
  "\x69\x23", "\x00\x00", "\x69\x25", "\x69\x26", "\x00\x00", "\x69\x28",
  "\x00\x00", "\x69\x2a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x69\x30", "\x00\x00", "\x00\x00", "\x00\x00", "\x69\x34",
  "\x00\x00", "\x69\x36", "\x00\x00", "\x00\x00", "\x69\x39", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x69\x3d", "\x00\x00", "\x69\x3f", "\x69\x4a",
  "\x69\x53", "\x69\x54", "\x69\x55", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x69\x59", "\x69\x5a", "\x00\x00", "\x69\x5c", "\x69\x5d", "\x69\x5e",
  "\x00\x00", "\x69\x60", "\x69\x61", "\x69\x62", "\x69\x6a", "\x69\x6b",
  "\x00\x00", "\x69\x6d", "\x69\x6e", "\x69\x6f", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x69\x73", "\x69\x74", "\x69\x75", "\x00\x00", "\x69\x77",
  "\x69\x78", "\x69\x79", "\x00\x00", "\x00\x00", "\x69\x7c", "\x69\x7d",
  "\x69\x7e", "\x00\x00", "\x00\x00", "\x69\x81", "\x69\x82", "\x69\x8a",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x69\x8e", "\x00\x00", "\x00\x00",
  "\x69\x91", "\x00\x00", "\x00\x00", "\x69\x94", "\x69\x95", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x69\x9b", "\x69\x9c",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x69\xa0", "\x69\xa7", "\x69\xae",
  "\x00\x00", "\x00\x00", "\x69\xb1", "\x69\xb2", "\x00\x00", "\x69\xb4",
  "\x69\xbb", "\x00\x00", "\x00\x00", "\x69\xbe", "\x69\xbf", "\x00\x00",
  "\x69\xc1", "\x00\x00", "\x69\xc3", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x69\xc7", "\x00\x00", "\x00\x00", "\x69\xca", "\x69\xcb", "\x69\xcc",
  "\x69\xcd", "\x69\xce", "\x00\x00", "\x69\xd0", "\x00\x00", "\x00\x00",
  "\x69\xd3", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x69\xd8",
  "\x69\xd9", "\x00\x00", "\x00\x00", "\x00\x00", "\x69\xdd", "\x69\xde",
  "\x69\xe7", "\x69\xe8", "\x00\x00", "\x00\x00", "\x69\xeb", "\x00\x00",
  "\x69\xed", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x69\xf2",
  "\x69\xf9", "\x00\x00", "\x69\xfb", "\x00\x00", "\x69\xfd", "\x00\x00",
  "\x69\xff", "\x00\x00", "\x00\x00", "\x6a\x02", "\x00\x00", "\x00\x00",
  "\x6a\x05", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6a\x0a",
  "\x6a\x0b", "\x6a\x0c", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x6a\x12", "\x6a\x13", "\x6a\x14", "\x00\x00", "\x00\x00",
  "\x6a\x17", "\x00\x00", "\x6a\x19", "\x00\x00", "\x6a\x1b", "\x00\x00",
  "\x00\x00", "\x6a\x1e", "\x6a\x1f", "\x00\x00", "\x6a\x21", "\x6a\x22",
  "\x6a\x23", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6a\x29", "\x6a\x2a", "\x6a\x2b", "\x00\x00", "\x00\x00", "\x6a\x2e",
  "\x6a\x35", "\x6a\x36", "\x00\x00", "\x6a\x38", "\x6a\x39", "\x6a\x3a",
  "\x00\x00", "\x00\x00", "\x6a\x3d", "\x6a\x44", "\x00\x00", "\x00\x00",
  "\x6a\x47", "\x6a\x48", "\x00\x00", "\x00\x00", "\x6a\x4b", "\x6a\x58",
  "\x6a\x59", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6a\x5f", "\x00\x00", "\x6a\x61", "\x6a\x62", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x6a\x66", "\x6a\x72", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6a\x78", "\x6a\x7f", "\x6a\x80", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6a\x84", "\x6a\x8d", "\x6a\x8e", "\x00\x00",
  "\x6a\x90", "\x6a\x97", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6a\x9c", "\x00\x00", "\x00\x00", "\x00\x00", "\x6a\xa0", "\x00\x00",
  "\x6a\xa2", "\x6a\xa3", "\x6a\xaa", "\x00\x00", "\x6a\xac", "\x00\x00",
  "\x6a\xae", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6a\xb3",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6a\xb8", "\x00\x00",
  "\x00\x00", "\x6a\xbb", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x6a\xc1", "\x6a\xc2", "\x6a\xc3", "\x6a\xd1", "\x00\x00",
  "\x6a\xd3", "\x6a\xda", "\x6a\xdb", "\x00\x00", "\x00\x00", "\x6a\xde",
  "\x6a\xdf", "\x6a\xe8", "\x00\x00", "\x6a\xea", "\x6a\xfa", "\x6a\xfb",
  "\x6b\x04", "\x6b\x05", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6b\x0a", "\x6b\x12", "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\x16",
  "\x6b\x1d", "\x00\x00", "\x6b\x1f", "\x6b\x20", "\x6b\x21", "\x00\x00",
  "\x6b\x23", "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\x27", "\x6b\x32",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\x37", "\x6b\x38",
  "\x6b\x39", "\x6b\x3a", "\x00\x00", "\x00\x00", "\x6b\x3d", "\x6b\x3e",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\x43", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6b\x47", "\x00\x00", "\x6b\x49", "\x00\x00",
  "\x00\x00", "\x6b\x4c", "\x00\x00", "\x6b\x4e", "\x00\x00", "\x6b\x50",
  "\x00\x00", "\x00\x00", "\x6b\x53", "\x6b\x54", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6b\x59", "\x00\x00", "\x6b\x5b", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6b\x5f", "\x00\x00", "\x6b\x61", "\x6b\x62",
  "\x6b\x63", "\x6b\x64", "\x00\x00", "\x6b\x66", "\x00\x00", "\x00\x00",
  "\x6b\x69", "\x6b\x6a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6b\x6f", "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\x73", "\x6b\x74",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\x78", "\x6b\x79", "\x00\x00",
  "\x6b\x7b", "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\x7f", "\x6b\x80",
  "\x00\x00", "\x00\x00", "\x6b\x83", "\x6b\x84", "\x00\x00", "\x6b\x86",
  "\x00\x00", "\x00\x00", "\x6b\x89", "\x6b\x8a", "\x6b\x8b", "\x00\x00",
  "\x6b\x8d", "\x6b\x95", "\x6b\x96", "\x00\x00", "\x6b\x98", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\x9e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\xa4", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\xaa", "\x6b\xab",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\xaf", "\x00\x00", "\x6b\xb1",
  "\x6b\xb2", "\x6b\xb3", "\x6b\xb4", "\x6b\xb5", "\x00\x00", "\x6b\xb7",
  "\x00\x00", "\x00\x00", "\x6b\xba", "\x6b\xbb", "\x6b\xbc", "\x00\x00",
  "\x00\x00", "\x6b\xbf", "\x6b\xc0", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x6b\xc5", "\x6b\xc6", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x6b\xcb", "\x00\x00", "\x6b\xcd", "\x6b\xce", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6b\xd2", "\x6b\xd3", "\x6b\xd4", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6b\xd8", "\x00\x00", "\x00\x00", "\x6b\xdb",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6b\xdf", "\x6b\xeb", "\x6b\xec",
  "\x00\x00", "\x00\x00", "\x6b\xef", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6b\xf3", "\x6c\x08", "\x6c\x0f", "\x00\x00", "\x6c\x11", "\x00\x00",
  "\x6c\x13", "\x6c\x14", "\x00\x00", "\x00\x00", "\x6c\x17", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6c\x1b", "\x6c\x23", "\x6c\x24", "\x6c\x34",
  "\x00\x00", "\x00\x00", "\x6c\x37", "\x6c\x38", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6c\x3e", "\x00\x00", "\x6c\x40",
  "\x6c\x41", "\x6c\x42", "\x6c\x4e", "\x00\x00", "\x6c\x50", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6c\x55", "\x00\x00", "\x6c\x57",
  "\x00\x00", "\x00\x00", "\x6c\x5a", "\x00\x00", "\x00\x00", "\x6c\x5d",
  "\x6c\x5e", "\x6c\x5f", "\x6c\x60", "\x00\x00", "\x6c\x62", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6c\x68", "\x00\x00",
  "\x6c\x6a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6c\x70", "\x00\x00", "\x6c\x72", "\x6c\x73", "\x6c\x7a", "\x00\x00",
  "\x00\x00", "\x6c\x7d", "\x6c\x7e", "\x00\x00", "\x00\x00", "\x6c\x81",
  "\x6c\x82", "\x6c\x83", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6c\x88", "\x00\x00", "\x00\x00", "\x00\x00", "\x6c\x8c", "\x6c\x8d",
  "\x00\x00", "\x00\x00", "\x6c\x90", "\x00\x00", "\x6c\x92", "\x6c\x93",
  "\x00\x00", "\x00\x00", "\x6c\x96", "\x00\x00", "\x00\x00", "\x6c\x99",
  "\x6c\x9a", "\x6c\x9b", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x6c\xa1", "\x6c\xa2", "\x6c\xab", "\x00\x00", "\x00\x00",
  "\x6c\xae", "\x00\x00", "\x00\x00", "\x6c\xb1", "\x00\x00", "\x6c\xb3",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6c\xb8", "\x6c\xb9",
  "\x6c\xba", "\x6c\xbb", "\x6c\xbc", "\x6c\xbd", "\x6c\xbe", "\x6c\xbf",
  "\x00\x00", "\x6c\xc1", "\x00\x00", "\x00\x00", "\x6c\xc4", "\x6c\xc5",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6c\xc9", "\x6c\xca", "\x00\x00",
  "\x6c\xcc", "\x6c\xd3", "\x00\x00", "\x6c\xd5", "\x00\x00", "\x6c\xd7",
  "\x00\x00", "\x6c\xd9", "\x00\x00", "\x6c\xdb", "\x00\x00", "\x6c\xdd",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6c\xe1", "\x6c\xe2", "\x6c\xe3",
  "\x00\x00", "\x6c\xe5", "\x00\x00", "\x00\x00", "\x6c\xe8", "\x00\x00",
  "\x6c\xea", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6c\xef",
  "\x6c\xf0", "\x6c\xf1", "\x00\x00", "\x6c\xf3", "\x6d\x0b", "\x6d\x0c",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6d\x12",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6d\x17", "\x00\x00",
  "\x6d\x19", "\x00\x00", "\x6d\x1b", "\x00\x00", "\x00\x00", "\x6d\x1e",
  "\x6d\x1f", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6d\x25", "\x00\x00", "\x00\x00", "\x00\x00", "\x6d\x29", "\x6d\x2a",
  "\x6d\x2b", "\x6d\x32", "\x6d\x33", "\x00\x00", "\x6d\x35", "\x6d\x36",
  "\x00\x00", "\x6d\x38", "\x00\x00", "\x00\x00", "\x6d\x3b", "\x00\x00",
  "\x6d\x3d", "\x6d\x3e", "\x00\x00", "\x00\x00", "\x6d\x41", "\x00\x00",
  "\x00\x00", "\x6d\x44", "\x6d\x45", "\x6d\x59", "\x6d\x5a", "\x00\x00",
  "\x6d\x5c", "\x6d\x63", "\x6d\x64", "\x00\x00", "\x6d\x66", "\x00\x00",
  "\x00\x00", "\x6d\x69", "\x6d\x6a", "\x00\x00", "\x6d\x6c", "\x00\x00",
  "\x6d\x6e", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6d\x74", "\x00\x00", "\x00\x00", "\x6d\x77", "\x6d\x78", "\x6d\x79",
  "\x6d\x85", "\x00\x00", "\x00\x00", "\x6d\x88", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x6d\x8c", "\x00\x00", "\x6d\x8e", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6d\x93", "\x00\x00", "\x6d\x95", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6d\x99", "\x00\x00", "\x6d\x9b", "\x6d\x9c",
  "\x6d\xaf", "\x00\x00", "\x00\x00", "\x6d\xb2", "\x00\x00", "\x00\x00",
  "\x6d\xb5", "\x00\x00", "\x00\x00", "\x6d\xb8", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x6d\xbc", "\x00\x00", "\x00\x00", "\x00\x00", "\x6d\xc0",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6d\xc5", "\x6d\xc6",
  "\x6d\xc7", "\x00\x00", "\x00\x00", "\x00\x00", "\x6d\xcb", "\x6d\xcc",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6d\xd1", "\x6d\xd2",
  "\x00\x00", "\x00\x00", "\x6d\xd5", "\x00\x00", "\x00\x00", "\x6d\xd8",
  "\x6d\xd9", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6d\xde",
  "\x00\x00", "\x00\x00", "\x6d\xe1", "\x00\x00", "\x00\x00", "\x6d\xe4",
  "\x00\x00", "\x6d\xe6", "\x00\x00", "\x6d\xe8", "\x00\x00", "\x6d\xea",
  "\x6d\xeb", "\x6d\xec", "\x00\x00", "\x6d\xee", "\x00\x00", "\x00\x00",
  "\x6d\xf1", "\x00\x00", "\x6d\xf3", "\x00\x00", "\x6d\xf5", "\x00\x00",
  "\x6d\xf7", "\x00\x00", "\x6d\xf9", "\x6d\xfa", "\x6d\xfb", "\x6e\x05",
  "\x00\x00", "\x6e\x07", "\x6e\x08", "\x6e\x09", "\x6e\x0a", "\x6e\x0b",
  "\x6e\x13", "\x00\x00", "\x6e\x15", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6e\x19", "\x6e\x1a", "\x6e\x1b", "\x00\x00", "\x6e\x1d", "\x00\x00",
  "\x6e\x1f", "\x6e\x20", "\x6e\x21", "\x00\x00", "\x6e\x23", "\x6e\x24",
  "\x6e\x25", "\x6e\x26", "\x00\x00", "\x00\x00", "\x6e\x29", "\x00\x00",
  "\x6e\x2b", "\x6e\x2c", "\x6e\x2d", "\x6e\x2e", "\x6e\x2f", "\x6e\x38",
  "\x00\x00", "\x6e\x3a", "\x00\x00", "\x00\x00", "\x00\x00", "\x6e\x3e",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6e\x43", "\x6e\x4a",
  "\x00\x00", "\x00\x00", "\x6e\x4d", "\x6e\x4e", "\x6e\x56", "\x00\x00",
  "\x6e\x58", "\x00\x00", "\x00\x00", "\x6e\x5b", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x6e\x5f", "\x6e\x67", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6e\x6b", "\x00\x00", "\x00\x00", "\x6e\x6e", "\x6e\x6f", "\x00\x00",
  "\x00\x00", "\x6e\x72", "\x00\x00", "\x00\x00", "\x00\x00", "\x6e\x76",
  "\x6e\x7e", "\x6e\x7f", "\x6e\x80", "\x00\x00", "\x6e\x82", "\x6e\x8c",
  "\x00\x00", "\x00\x00", "\x6e\x8f", "\x6e\x90", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6e\x96", "\x00\x00", "\x6e\x98",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6e\x9c", "\x6e\x9d", "\x00\x00",
  "\x6e\x9f", "\x00\x00", "\x00\x00", "\x6e\xa2", "\x00\x00", "\x00\x00",
  "\x6e\xa5", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6e\xaa",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6e\xaf", "\x00\x00",
  "\x00\x00", "\x6e\xb2", "\x00\x00", "\x00\x00", "\x00\x00", "\x6e\xb6",
  "\x6e\xb7", "\x00\x00", "\x00\x00", "\x6e\xba", "\x00\x00", "\x00\x00",
  "\x6e\xbd", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6e\xc2",
  "\x00\x00", "\x6e\xc4", "\x6e\xc5", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6e\xc9", "\x00\x00", "\x6e\xcb", "\x6e\xcc", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6e\xd1", "\x00\x00", "\x6e\xd3", "\x6e\xd4",
  "\x6e\xd5", "\x6e\xdd", "\x6e\xde", "\x6e\xec", "\x00\x00", "\x00\x00",
  "\x6e\xef", "\x00\x00", "\x00\x00", "\x6e\xf2", "\x00\x00", "\x6e\xf4",
  "\x00\x00", "\x00\x00", "\x6e\xf7", "\x6e\xf8", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6e\xfe", "\x6e\xff", "\x00\x00",
  "\x6f\x01", "\x6f\x02", "\x00\x00", "\x00\x00", "\x00\x00", "\x6f\x06",
  "\x00\x00", "\x00\x00", "\x6f\x09", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6f\x0f", "\x00\x00", "\x6f\x11", "\x00\x00",
  "\x6f\x13", "\x6f\x14", "\x6f\x15", "\x6f\x20", "\x00\x00", "\x6f\x22",
  "\x6f\x23", "\x6f\x2b", "\x6f\x2c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x6f\x31", "\x6f\x32", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6f\x38", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6f\x3e", "\x6f\x3f", "\x00\x00", "\x6f\x41",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6f\x45", "\x6f\x54", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6f\x58", "\x00\x00", "\x00\x00", "\x6f\x5b",
  "\x6f\x5c", "\x00\x00", "\x00\x00", "\x6f\x5f", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6f\x64", "\x00\x00", "\x6f\x66", "\x6f\x6d",
  "\x6f\x6e", "\x6f\x6f", "\x6f\x70", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6f\x74", "\x00\x00", "\x00\x00", "\x00\x00", "\x6f\x78", "\x00\x00",
  "\x6f\x7a", "\x00\x00", "\x6f\x7c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6f\x80", "\x6f\x81", "\x6f\x82", "\x00\x00", "\x6f\x84", "\x00\x00",
  "\x6f\x86", "\x6f\x8e", "\x00\x00", "\x00\x00", "\x6f\x91", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6f\x97", "\x6f\xa1",
  "\x00\x00", "\x6f\xa3", "\x6f\xa4", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x6f\xaa", "\x6f\xb1", "\x00\x00", "\x6f\xb3",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x6f\xb9",
  "\x6f\xc0", "\x6f\xc1", "\x6f\xc2", "\x6f\xc3", "\x00\x00", "\x00\x00",
  "\x6f\xc6", "\x6f\xd4", "\x6f\xd5", "\x00\x00", "\x00\x00", "\x6f\xd8",
  "\x00\x00", "\x00\x00", "\x6f\xdb", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x6f\xdf", "\x6f\xe0", "\x6f\xe1", "\x00\x00", "\x00\x00", "\x6f\xe4",
  "\x6f\xeb", "\x6f\xec", "\x00\x00", "\x6f\xee", "\x6f\xef", "\x00\x00",
  "\x6f\xf1", "\x00\x00", "\x6f\xf3", "\x00\x00", "\x00\x00", "\x6f\xf6",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x6f\xfa", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x6f\xfe", "\x00\x00", "\x00\x00", "\x70\x01", "\x70\x09",
  "\x00\x00", "\x70\x0b", "\x00\x00", "\x00\x00", "\x00\x00", "\x70\x0f",
  "\x00\x00", "\x70\x11", "\x00\x00", "\x00\x00", "\x00\x00", "\x70\x15",
  "\x00\x00", "\x00\x00", "\x70\x18", "\x00\x00", "\x70\x1a", "\x70\x1b",
  "\x00\x00", "\x70\x1d", "\x70\x1e", "\x70\x1f", "\x70\x26", "\x70\x27",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x70\x2c", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x70\x30", "\x00\x00", "\x70\x32", "\x70\x3e",
  "\x70\x4c", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x70\x51",
  "\x70\x58", "\x70\x63", "\x70\x6b", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x70\x6f", "\x70\x70", "\x70\x78", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x70\x7c", "\x70\x7d", "\x70\x89", "\x70\x8a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x70\x8e", "\x00\x00", "\x00\x00", "\x00\x00", "\x70\x92",
  "\x70\x99", "\x70\xac", "\x70\xad", "\x70\xae", "\x70\xaf", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x70\xb3", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x70\xb8", "\x70\xb9", "\x70\xba", "\x70\xc8", "\x00\x00",
  "\x00\x00", "\x70\xcb", "\x00\x00", "\x00\x00", "\x00\x00", "\x70\xcf",
  "\x70\xd9", "\x00\x00", "\x00\x00", "\x00\x00", "\x70\xdd", "\x00\x00",
  "\x70\xdf", "\x70\xf1", "\x70\xf9", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x70\xfd", "\x71\x09", "\x71\x14", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x71\x19", "\x71\x1a", "\x00\x00", "\x71\x1c", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x71\x21", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x71\x26", "\x71\x36", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x71\x3c", "\x71\x49", "\x00\x00",
  "\x00\x00", "\x71\x4c", "\x00\x00", "\x71\x4e", "\x71\x55", "\x71\x56",
  "\x00\x00", "\x00\x00", "\x71\x59", "\x71\x62", "\x00\x00", "\x71\x64",
  "\x71\x65", "\x71\x66", "\x71\x67", "\x00\x00", "\x71\x69", "\x00\x00",
  "\x00\x00", "\x71\x6c", "\x00\x00", "\x71\x6e", "\x71\x7d", "\x71\x84",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x71\x88", "\x00\x00", "\x71\x8a",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x71\x8f", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x71\x94", "\x71\x95", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x71\x99", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x71\x9f", "\x71\xa8", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x71\xac", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x71\xb1", "\x71\xb9", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x71\xbe", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x71\xc3",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x71\xc8", "\x71\xc9",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x71\xce", "\x00\x00",
  "\x71\xd0", "\x00\x00", "\x71\xd2", "\x00\x00", "\x71\xd4", "\x71\xd5",
  "\x00\x00", "\x71\xd7", "\x71\xdf", "\x71\xe0", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x71\xe5", "\x71\xe6", "\x71\xe7", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x71\xec", "\x71\xed", "\x71\xee",
  "\x71\xf5", "\x00\x00", "\x00\x00", "\x00\x00", "\x71\xf9", "\x00\x00",
  "\x71\xfb", "\x71\xfc", "\x00\x00", "\x00\x00", "\x71\xff", "\x72\x06",
  "\x72\x0d", "\x00\x00", "\x00\x00", "\x72\x10", "\x72\x1b", "\x72\x28",
  "\x00\x00", "\x72\x2a", "\x00\x00", "\x72\x2c", "\x72\x2d", "\x00\x00",
  "\x00\x00", "\x72\x30", "\x00\x00", "\x72\x32", "\x00\x00", "\x00\x00",
  "\x72\x35", "\x72\x36", "\x00\x00", "\x00\x00", "\x00\x00", "\x72\x3a",
  "\x72\x3b", "\x72\x3c", "\x72\x3d", "\x72\x3e", "\x72\x3f", "\x72\x40",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x72\x46",
  "\x72\x47", "\x72\x48", "\x00\x00", "\x00\x00", "\x72\x4b", "\x72\x4c",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x72\x52",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x72\x58",
  "\x72\x59", "\x00\x00", "\x72\x5b", "\x00\x00", "\x72\x5d", "\x00\x00",
  "\x72\x5f", "\x00\x00", "\x72\x61", "\x72\x62", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x72\x67", "\x00\x00", "\x72\x69", "\x72\x72",
  "\x00\x00", "\x72\x74", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x72\x79", "\x00\x00", "\x00\x00", "\x00\x00", "\x72\x7d", "\x72\x7e",
  "\x00\x00", "\x72\x80", "\x72\x81", "\x72\x82", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x72\x87", "\x72\x92", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x72\x96", "\x72\xa0", "\x00\x00", "\x72\xa2", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x72\xa7", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x72\xac", "\x00\x00", "\x00\x00", "\x72\xaf",
  "\x00\x00", "\x00\x00", "\x72\xb2", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x72\xb6", "\x00\x00", "\x00\x00", "\x72\xb9", "\x72\xc2", "\x72\xc3",
  "\x72\xc4", "\x00\x00", "\x72\xc6", "\x72\xce", "\x00\x00", "\x72\xd0",
  "\x00\x00", "\x72\xd2", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x72\xd7", "\x00\x00", "\x72\xd9", "\x00\x00", "\x72\xdb", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x72\xe0", "\x72\xe1", "\x72\xe2",
  "\x72\xe9", "\x00\x00", "\x00\x00", "\x72\xec", "\x72\xed", "\x72\xf7",
  "\x72\xf8", "\x72\xf9", "\x00\x00", "\x00\x00", "\x72\xfc", "\x72\xfd",
  "\x73\x0a", "\x73\x16", "\x73\x17", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x73\x1b", "\x73\x1c", "\x73\x1d", "\x00\x00", "\x73\x1f", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x73\x25", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x73\x29", "\x73\x2a", "\x73\x2b", "\x00\x00",
  "\x00\x00", "\x73\x2e", "\x73\x2f", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x73\x34", "\x00\x00", "\x73\x36", "\x73\x37", "\x73\x3e",
  "\x73\x3f", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x73\x44",
  "\x73\x45", "\x73\x4e", "\x73\x4f", "\x73\x57", "\x73\x63", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x73\x68", "\x00\x00", "\x73\x6a",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x73\x70",
  "\x00\x00", "\x73\x72", "\x00\x00", "\x00\x00", "\x73\x75", "\x00\x00",
  "\x00\x00", "\x73\x78", "\x00\x00", "\x73\x7a", "\x73\x7b", "\x73\x84",
  "\x00\x00", "\x00\x00", "\x73\x87", "\x00\x00", "\x73\x89", "\x00\x00",
  "\x73\x8b", "\x73\x96", "\x73\xa9", "\x73\xb2", "\x73\xb3", "\x73\xbb",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x73\xc0", "\x00\x00",
  "\x73\xc2", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x73\xc8", "\x00\x00", "\x73\xca", "\x00\x00", "\x00\x00", "\x73\xcd",
  "\x73\xce", "\x73\xde", "\x00\x00", "\x73\xe0", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x73\xe5", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x73\xea", "\x00\x00", "\x00\x00", "\x73\xed", "\x73\xee",
  "\x00\x00", "\x00\x00", "\x73\xf1", "\x73\xf8", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x73\xfe", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x74\x03", "\x00\x00", "\x74\x05", "\x74\x06",
  "\x00\x00", "\x00\x00", "\x74\x09", "\x74\x22", "\x00\x00", "\x00\x00",
  "\x74\x25", "\x74\x32", "\x74\x33", "\x74\x34", "\x74\x35", "\x74\x36",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x74\x3a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x74\x3f", "\x00\x00", "\x74\x41", "\x74\x55",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x74\x59", "\x74\x5a", "\x74\x5b",
  "\x74\x5c", "\x00\x00", "\x74\x5e", "\x74\x5f", "\x74\x60", "\x00\x00",
  "\x00\x00", "\x74\x63", "\x74\x64", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x74\x69", "\x74\x6a", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x74\x6f", "\x74\x70", "\x00\x00", "\x00\x00", "\x74\x73",
  "\x00\x00", "\x00\x00", "\x74\x76", "\x74\x7e", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x74\x83", "\x74\x8b", "\x74\x9e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x74\xa2", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x74\xa7", "\x74\xb0", "\x74\xbd", "\x74\xca", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x74\xcf", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x74\xd4", "\x74\xdc", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x74\xe0", "\x00\x00", "\x74\xe2", "\x74\xe3", "\x00\x00",
  "\x00\x00", "\x74\xe6", "\x74\xe7", "\x00\x00", "\x74\xe9", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x74\xee", "\x00\x00", "\x74\xf0",
  "\x74\xf1", "\x74\xf2", "\x00\x00", "\x00\x00", "\x00\x00", "\x74\xf6",
  "\x74\xf7", "\x74\xf8", "\x75\x03", "\x75\x04", "\x75\x05", "\x75\x0c",
  "\x75\x0d", "\x75\x0e", "\x00\x00", "\x00\x00", "\x75\x11", "\x00\x00",
  "\x75\x13", "\x00\x00", "\x75\x15", "\x00\x00", "\x00\x00", "\x75\x18",
  "\x00\x00", "\x75\x1a", "\x00\x00", "\x75\x1c", "\x00\x00", "\x75\x1e",
  "\x75\x1f", "\x00\x00", "\x00\x00", "\x00\x00", "\x75\x23", "\x00\x00",
  "\x75\x25", "\x75\x26", "\x00\x00", "\x75\x28", "\x00\x00", "\x00\x00",
  "\x75\x2b", "\x75\x2c", "\x00\x00", "\x00\x00", "\x00\x00", "\x75\x30",
  "\x75\x31", "\x75\x32", "\x75\x33", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x75\x37", "\x75\x38", "\x00\x00", "\x75\x3a", "\x75\x3b", "\x75\x3c",
  "\x75\x44", "\x00\x00", "\x75\x46", "\x00\x00", "\x00\x00", "\x75\x49",
  "\x75\x4a", "\x75\x4b", "\x75\x4c", "\x75\x4d", "\x00\x00", "\x75\x4f",
  "\x00\x00", "\x75\x51", "\x00\x00", "\x00\x00", "\x75\x54", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x75\x59", "\x75\x5a", "\x75\x5b",
  "\x75\x5c", "\x75\x5d", "\x00\x00", "\x00\x00", "\x75\x60", "\x00\x00",
  "\x75\x62", "\x00\x00", "\x75\x64", "\x75\x65", "\x75\x66", "\x75\x67",
  "\x00\x00", "\x75\x69", "\x75\x6a", "\x75\x6b", "\x00\x00", "\x75\x6d",
  "\x00\x00", "\x00\x00", "\x75\x70", "\x00\x00", "\x00\x00", "\x75\x73",
  "\x75\x74", "\x00\x00", "\x75\x76", "\x75\x77", "\x75\x78", "\x75\x7f",
  "\x00\x00", "\x00\x00", "\x75\x82", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x75\x86", "\x75\x87", "\x00\x00", "\x75\x89", "\x75\x8a", "\x75\x8b",
  "\x00\x00", "\x00\x00", "\x75\x8e", "\x75\x8f", "\x00\x00", "\x75\x91",
  "\x00\x00", "\x00\x00", "\x75\x94", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x75\x9a", "\x00\x00", "\x00\x00", "\x75\x9d",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x75\xa3",
  "\x00\x00", "\x75\xa5", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x75\xab", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x75\xb1", "\x75\xb2", "\x75\xb3", "\x00\x00", "\x75\xb5",
  "\x00\x00", "\x00\x00", "\x75\xb8", "\x75\xb9", "\x00\x00", "\x00\x00",
  "\x75\xbc", "\x75\xbd", "\x75\xbe", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x75\xc2", "\x75\xc3", "\x00\x00", "\x75\xc5", "\x00\x00", "\x75\xc7",
  "\x00\x00", "\x00\x00", "\x75\xca", "\x00\x00", "\x00\x00", "\x75\xcd",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x75\xd2", "\x00\x00",
  "\x75\xd4", "\x75\xd5", "\x00\x00", "\x00\x00", "\x75\xd8", "\x75\xd9",
  "\x00\x00", "\x75\xdb", "\x00\x00", "\x00\x00", "\x75\xde", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x75\xe2", "\x75\xe3", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x75\xe9", "\x75\xf0", "\x00\x00",
  "\x75\xf2", "\x75\xf3", "\x75\xf4", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x75\xfa", "\x00\x00", "\x75\xfc", "\x00\x00",
  "\x75\xfe", "\x75\xff", "\x00\x00", "\x76\x01", "\x76\x09", "\x00\x00",
  "\x76\x0b", "\x00\x00", "\x76\x0d", "\x76\x1f", "\x76\x20", "\x76\x21",
  "\x76\x22", "\x00\x00", "\x76\x24", "\x00\x00", "\x00\x00", "\x76\x27",
  "\x76\x30", "\x00\x00", "\x00\x00", "\x00\x00", "\x76\x34", "\x76\x3b",
  "\x76\x42", "\x00\x00", "\x00\x00", "\x00\x00", "\x76\x46", "\x76\x47",
  "\x76\x48", "\x00\x00", "\x00\x00", "\x00\x00", "\x76\x4c", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x76\x52", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x76\x56", "\x00\x00", "\x76\x58", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x76\x5c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x76\x61", "\x76\x62", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x76\x67", "\x76\x68", "\x76\x69", "\x76\x6a", "\x00\x00",
  "\x76\x6c", "\x00\x00", "\x00\x00", "\x00\x00", "\x76\x70", "\x00\x00",
  "\x76\x72", "\x00\x00", "\x00\x00", "\x00\x00", "\x76\x76", "\x00\x00",
  "\x76\x78", "\x00\x00", "\x76\x7a", "\x76\x7b", "\x76\x7c", "\x76\x7d",
  "\x76\x7e", "\x00\x00", "\x76\x80", "\x00\x00", "\x00\x00", "\x76\x83",
  "\x76\x84", "\x00\x00", "\x76\x86", "\x76\x87", "\x76\x88", "\x00\x00",
  "\x00\x00", "\x76\x8b", "\x00\x00", "\x00\x00", "\x76\x8e", "\x00\x00",
  "\x76\x90", "\x00\x00", "\x00\x00", "\x76\x93", "\x00\x00", "\x00\x00",
  "\x76\x96", "\x00\x00", "\x00\x00", "\x76\x99", "\x76\x9a", "\x76\xae",
  "\x00\x00", "\x76\xb0", "\x00\x00", "\x00\x00", "\x00\x00", "\x76\xb4",
  "\x00\x00", "\x00\x00", "\x76\xb7", "\x76\xb8", "\x76\xb9", "\x76\xba",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x76\xbf", "\x00\x00",
  "\x00\x00", "\x76\xc2", "\x76\xc3", "\x00\x00", "\x00\x00", "\x76\xc6",
  "\x00\x00", "\x76\xc8", "\x00\x00", "\x76\xca", "\x00\x00", "\x00\x00",
  "\x76\xcd", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x76\xd2",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x76\xd6", "\x76\xd7", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x76\xdb", "\x76\xdc", "\x00\x00", "\x76\xde",
  "\x76\xdf", "\x00\x00", "\x76\xe1", "\x00\x00", "\x76\xe3", "\x76\xe4",
  "\x76\xe5", "\x00\x00", "\x76\xe7", "\x00\x00", "\x00\x00", "\x76\xea",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x76\xee", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x76\xf2", "\x00\x00", "\x76\xf4", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x76\xf8", "\x00\x00", "\x00\x00", "\x76\xfb", "\x00\x00",
  "\x00\x00", "\x76\xfe", "\x00\x00", "\x00\x00", "\x77\x01", "\x00\x00",
  "\x00\x00", "\x77\x04", "\x00\x00", "\x00\x00", "\x77\x07", "\x77\x08",
  "\x77\x09", "\x00\x00", "\x77\x0b", "\x77\x0c", "\x77\x1b", "\x00\x00",
  "\x00\x00", "\x77\x1e", "\x77\x1f", "\x77\x20", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x77\x24", "\x77\x25", "\x77\x26", "\x00\x00", "\x00\x00",
  "\x77\x29", "\x77\x37", "\x77\x38", "\x00\x00", "\x77\x3a", "\x00\x00",
  "\x77\x3c", "\x00\x00", "\x00\x00", "\x00\x00", "\x77\x40", "\x77\x47",
  "\x77\x5a", "\x77\x5b", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x77\x61", "\x00\x00", "\x77\x63", "\x00\x00", "\x77\x65",
  "\x77\x66", "\x00\x00", "\x77\x68", "\x00\x00", "\x00\x00", "\x77\x6b",
  "\x77\x79", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x77\x7e",
  "\x77\x7f", "\x77\x8b", "\x00\x00", "\x00\x00", "\x77\x8e", "\x00\x00",
  "\x00\x00", "\x77\x91", "\x77\x9e", "\x00\x00", "\x77\xa0", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x77\xa5", "\x77\xac", "\x77\xad",
  "\x00\x00", "\x00\x00", "\x77\xb0", "\x00\x00", "\x00\x00", "\x77\xb3",
  "\x00\x00", "\x00\x00", "\x77\xb6", "\x00\x00", "\x00\x00", "\x77\xb9",
  "\x00\x00", "\x77\xbb", "\x77\xbc", "\x77\xbd", "\x00\x00", "\x77\xbf",
  "\x77\xc7", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x77\xcd", "\x77\xd7", "\x00\x00", "\x00\x00", "\x77\xda", "\x77\xdb",
  "\x77\xdc", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x77\xe2", "\x77\xe3", "\x00\x00", "\x77\xe5", "\x00\x00", "\x77\xe7",
  "\x00\x00", "\x77\xe9", "\x00\x00", "\x00\x00", "\x00\x00", "\x77\xed",
  "\x77\xee", "\x77\xef", "\x00\x00", "\x00\x00", "\x00\x00", "\x77\xf3",
  "\x77\xfc", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x78\x02", "\x78\x0c", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x78\x12", "\x00\x00", "\x78\x14", "\x78\x15", "\x78\x20",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x78\x25", "\x78\x26",
  "\x78\x27", "\x78\x32", "\x00\x00", "\x78\x34", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x78\x3a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x78\x3f", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x78\x45", "\x78\x5d", "\x78\x6b", "\x78\x6c",
  "\x00\x00", "\x00\x00", "\x78\x6f", "\x00\x00", "\x00\x00", "\x78\x72",
  "\x00\x00", "\x78\x74", "\x78\x7c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x78\x81", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x78\x86", "\x78\x87", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x78\x8c", "\x78\x8d", "\x78\x8e", "\x00\x00", "\x00\x00", "\x78\x91",
  "\x00\x00", "\x78\x93", "\x00\x00", "\x78\x95", "\x00\x00", "\x78\x97",
  "\x00\x00", "\x00\x00", "\x78\x9a", "\x78\xa3", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x78\xa7", "\x00\x00", "\x78\xa9", "\x78\xaa", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x78\xaf", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x78\xb5", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x78\xba", "\x00\x00", "\x78\xbc", "\x00\x00",
  "\x78\xbe", "\x00\x00", "\x00\x00", "\x78\xc1", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x78\xc5", "\x78\xc6", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x78\xca", "\x78\xcb", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x78\xd0", "\x78\xd1", "\x00\x00", "\x00\x00", "\x78\xd4", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x78\xda", "\x78\xe7",
  "\x78\xe8", "\x00\x00", "\x00\x00", "\x00\x00", "\x78\xec", "\x00\x00",
  "\x00\x00", "\x78\xef", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x78\xf4", "\x78\xfd", "\x00\x00", "\x00\x00", "\x00\x00", "\x79\x01",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x79\x07",
  "\x79\x0e", "\x00\x00", "\x00\x00", "\x79\x11", "\x79\x12", "\x79\x19",
  "\x79\x26", "\x00\x00", "\x00\x00", "\x00\x00", "\x79\x2a", "\x79\x2b",
  "\x79\x2c", "\x79\x3a", "\x00\x00", "\x79\x3c", "\x00\x00", "\x79\x3e",
  "\x00\x00", "\x79\x40", "\x79\x41", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x79\x47", "\x79\x48", "\x79\x49", "\x79\x50",
  "\x00\x00", "\x00\x00", "\x79\x53", "\x00\x00", "\x79\x55", "\x79\x56",
  "\x79\x57", "\x00\x00", "\x00\x00", "\x79\x5a", "\x00\x00", "\x00\x00",
  "\x79\x5d", "\x79\x5e", "\x79\x5f", "\x79\x60", "\x00\x00", "\x79\x62",
  "\x00\x00", "\x00\x00", "\x79\x65", "\x00\x00", "\x00\x00", "\x79\x68",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x79\x6d", "\x79\x77",
  "\x00\x00", "\x00\x00", "\x79\x7a", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x79\x7f", "\x79\x80", "\x79\x81", "\x00\x00", "\x00\x00",
  "\x79\x84", "\x79\x85", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x79\x8a", "\x00\x00", "\x00\x00", "\x79\x8d", "\x79\x8e", "\x79\x8f",
  "\x79\x9d", "\x79\xa6", "\x79\xa7", "\x00\x00", "\x00\x00", "\x79\xaa",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x79\xae", "\x00\x00", "\x79\xb0",
  "\x00\x00", "\x00\x00", "\x79\xb3", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x79\xb9", "\x79\xba", "\x00\x00", "\x00\x00",
  "\x79\xbd", "\x79\xbe", "\x79\xbf", "\x79\xc0", "\x79\xc1", "\x79\xc9",
  "\x00\x00", "\x79\xcb", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x79\xd1", "\x79\xd2", "\x00\x00", "\x00\x00", "\x79\xd5",
  "\x00\x00", "\x00\x00", "\x79\xd8", "\x79\xdf", "\x00\x00", "\x79\xe1",
  "\x00\x00", "\x79\xe3", "\x79\xe4", "\x00\x00", "\x79\xe6", "\x79\xe7",
  "\x00\x00", "\x79\xe9", "\x00\x00", "\x00\x00", "\x79\xec", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x79\xf0", "\x79\xfb", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x7a\x00", "\x7a\x08", "\x00\x00", "\x00\x00",
  "\x7a\x0b", "\x00\x00", "\x7a\x0d", "\x7a\x0e", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7a\x14", "\x00\x00", "\x00\x00",
  "\x7a\x17", "\x7a\x18", "\x7a\x19", "\x7a\x1a", "\x00\x00", "\x7a\x1c",
  "\x00\x00", "\x00\x00", "\x7a\x1f", "\x7a\x20", "\x7a\x2e", "\x00\x00",
  "\x00\x00", "\x7a\x31", "\x7a\x32", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7a\x37", "\x00\x00", "\x00\x00", "\x00\x00", "\x7a\x3b",
  "\x7a\x3c", "\x7a\x3d", "\x7a\x3e", "\x7a\x3f", "\x7a\x40", "\x00\x00",
  "\x7a\x42", "\x7a\x43", "\x00\x00", "\x00\x00", "\x7a\x46", "\x00\x00",
  "\x00\x00", "\x7a\x49", "\x00\x00", "\x00\x00", "\x00\x00", "\x7a\x4d",
  "\x7a\x4e", "\x7a\x4f", "\x7a\x50", "\x7a\x57", "\x7a\x61", "\x7a\x62",
  "\x7a\x63", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7a\x69", "\x00\x00", "\x7a\x6b", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7a\x70", "\x00\x00", "\x00\x00", "\x00\x00", "\x7a\x74",
  "\x00\x00", "\x7a\x76", "\x00\x00", "\x00\x00", "\x7a\x79", "\x7a\x7a",
  "\x00\x00", "\x00\x00", "\x7a\x7d", "\x00\x00", "\x7a\x7f", "\x00\x00",
  "\x7a\x81", "\x00\x00", "\x7a\x83", "\x7a\x84", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7a\x88", "\x7a\x92", "\x7a\x93", "\x00\x00", "\x7a\x95",
  "\x7a\x96", "\x7a\x97", "\x7a\x98", "\x7a\x9f", "\x7a\xa9", "\x7a\xaa",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7a\xae", "\x7a\xaf", "\x7a\xb0",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x7a\xb6",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7a\xba", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x7a\xbf", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7a\xc3", "\x7a\xc4", "\x7a\xc5", "\x00\x00", "\x7a\xc7", "\x7a\xc8",
  "\x00\x00", "\x7a\xca", "\x7a\xcb", "\x00\x00", "\x7a\xcd", "\x00\x00",
  "\x7a\xcf", "\x00\x00", "\x00\x00", "\x7a\xd2", "\x7a\xd3", "\x00\x00",
  "\x7a\xd5", "\x00\x00", "\x00\x00", "\x00\x00", "\x7a\xd9", "\x7a\xda",
  "\x00\x00", "\x7a\xdc", "\x7a\xdd", "\x00\x00", "\x7a\xdf", "\x7a\xe0",
  "\x7a\xe1", "\x7a\xe2", "\x7a\xe3", "\x00\x00", "\x7a\xe5", "\x7a\xe6",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7a\xea", "\x00\x00", "\x00\x00",
  "\x7a\xed", "\x00\x00", "\x7a\xef", "\x7a\xf0", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7a\xf6", "\x00\x00", "\x7a\xf8",
  "\x7a\xf9", "\x7a\xfa", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7a\xff", "\x00\x00", "\x00\x00", "\x7b\x02", "\x00\x00", "\x7b\x04",
  "\x00\x00", "\x7b\x06", "\x00\x00", "\x7b\x08", "\x00\x00", "\x7b\x0a",
  "\x7b\x0b", "\x00\x00", "\x00\x00", "\x00\x00", "\x7b\x0f", "\x00\x00",
  "\x7b\x11", "\x7b\x18", "\x7b\x19", "\x00\x00", "\x7b\x1b", "\x00\x00",
  "\x00\x00", "\x7b\x1e", "\x00\x00", "\x7b\x20", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x7b\x25", "\x7b\x26", "\x00\x00", "\x7b\x28",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7b\x2c", "\x7b\x33", "\x00\x00",
  "\x7b\x35", "\x7b\x36", "\x00\x00", "\x00\x00", "\x7b\x39", "\x7b\x45",
  "\x7b\x46", "\x00\x00", "\x7b\x48", "\x7b\x49", "\x00\x00", "\x7b\x4b",
  "\x7b\x4c", "\x7b\x4d", "\x00\x00", "\x7b\x4f", "\x7b\x50", "\x7b\x51",
  "\x7b\x52", "\x00\x00", "\x7b\x54", "\x00\x00", "\x7b\x56", "\x7b\x5d",
  "\x7b\x65", "\x00\x00", "\x7b\x67", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7b\x6c", "\x00\x00", "\x7b\x6e", "\x00\x00", "\x7b\x70",
  "\x7b\x71", "\x00\x00", "\x00\x00", "\x7b\x74", "\x7b\x75", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7b\x7a", "\x7b\x86", "\x7b\x87",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7b\x8b", "\x00\x00", "\x7b\x8d",
  "\x00\x00", "\x7b\x8f", "\x00\x00", "\x00\x00", "\x7b\x92", "\x00\x00",
  "\x7b\x94", "\x7b\x95", "\x00\x00", "\x7b\x97", "\x7b\x98", "\x7b\x99",
  "\x7b\x9a", "\x00\x00", "\x7b\x9c", "\x7b\x9d", "\x00\x00", "\x7b\x9f",
  "\x00\x00", "\x7b\xa1", "\x7b\xaa", "\x00\x00", "\x00\x00", "\x7b\xad",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7b\xb1", "\x00\x00", "\x00\x00",
  "\x7b\xb4", "\x00\x00", "\x00\x00", "\x00\x00", "\x7b\xb8", "\x7b\xc0",
  "\x7b\xc1", "\x00\x00", "\x00\x00", "\x7b\xc4", "\x00\x00", "\x7b\xc6",
  "\x7b\xc7", "\x00\x00", "\x7b\xc9", "\x00\x00", "\x7b\xcb", "\x7b\xcc",
  "\x00\x00", "\x00\x00", "\x7b\xcf", "\x7b\xdd", "\x00\x00", "\x00\x00",
  "\x7b\xe0", "\x00\x00", "\x00\x00", "\x00\x00", "\x7b\xe4", "\x7b\xe5",
  "\x7b\xe6", "\x00\x00", "\x00\x00", "\x7b\xe9", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7b\xed", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7b\xf3", "\x00\x00", "\x00\x00", "\x7b\xf6", "\x7b\xf7",
  "\x7c\x00", "\x7c\x07", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7c\x0d", "\x00\x00", "\x00\x00", "\x00\x00", "\x7c\x11",
  "\x7c\x12", "\x7c\x13", "\x7c\x14", "\x00\x00", "\x00\x00", "\x7c\x17",
  "\x7c\x1f", "\x00\x00", "\x7c\x21", "\x00\x00", "\x7c\x23", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x7c\x27", "\x00\x00", "\x00\x00", "\x7c\x2a",
  "\x7c\x2b", "\x7c\x37", "\x7c\x38", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7c\x3d", "\x7c\x3e", "\x7c\x3f", "\x7c\x40", "\x00\x00",
  "\x00\x00", "\x7c\x43", "\x7c\x4c", "\x7c\x4d", "\x00\x00", "\x7c\x4f",
  "\x7c\x50", "\x00\x00", "\x00\x00", "\x00\x00", "\x7c\x54", "\x00\x00",
  "\x7c\x56", "\x00\x00", "\x7c\x58", "\x7c\x5f", "\x7c\x60", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x7c\x64", "\x7c\x65", "\x7c\x6c", "\x7c\x73",
  "\x00\x00", "\x7c\x75", "\x7c\x7e", "\x00\x00", "\x00\x00", "\x7c\x81",
  "\x7c\x82", "\x7c\x83", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7c\x89", "\x00\x00", "\x7c\x8b", "\x00\x00", "\x7c\x8d",
  "\x00\x00", "\x00\x00", "\x7c\x90", "\x00\x00", "\x7c\x92", "\x00\x00",
  "\x00\x00", "\x7c\x95", "\x00\x00", "\x7c\x97", "\x7c\x98", "\x00\x00",
  "\x00\x00", "\x7c\x9b", "\x00\x00", "\x00\x00", "\x00\x00", "\x7c\x9f",
  "\x00\x00", "\x7c\xa1", "\x7c\xa2", "\x00\x00", "\x7c\xa4", "\x7c\xa5",
  "\x00\x00", "\x7c\xa7", "\x7c\xa8", "\x00\x00", "\x00\x00", "\x7c\xab",
  "\x00\x00", "\x7c\xad", "\x7c\xae", "\x00\x00", "\x00\x00", "\x7c\xb1",
  "\x7c\xb2", "\x7c\xb3", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7c\xb9", "\x00\x00", "\x00\x00", "\x00\x00", "\x7c\xbd",
  "\x7c\xbe", "\x00\x00", "\x7c\xc0", "\x00\x00", "\x7c\xc2", "\x00\x00",
  "\x00\x00", "\x7c\xc5", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7c\xca", "\x00\x00", "\x00\x00", "\x00\x00", "\x7c\xce", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x7c\xd2", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7c\xd6", "\x00\x00", "\x7c\xd8", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7c\xdc", "\x00\x00", "\x7c\xde", "\x7c\xdf", "\x7c\xe0", "\x00\x00",
  "\x7c\xe2", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x7c\xe7",
  "\x7c\xef", "\x00\x00", "\x00\x00", "\x7c\xf2", "\x00\x00", "\x7c\xf4",
  "\x00\x00", "\x7c\xf6", "\x00\x00", "\x7c\xf8", "\x00\x00", "\x7c\xfa",
  "\x7c\xfb", "\x00\x00", "\x00\x00", "\x7c\xfe", "\x00\x00", "\x7d\x00",
  "\x00\x00", "\x7d\x02", "\x00\x00", "\x7d\x04", "\x7d\x05", "\x7d\x06",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7d\x0a", "\x7d\x0b", "\x00\x00",
  "\x7d\x0d", "\x00\x00", "\x00\x00", "\x7d\x10", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7d\x14", "\x7d\x15", "\x00\x00", "\x7d\x17", "\x7d\x18",
  "\x7d\x19", "\x7d\x1a", "\x7d\x1b", "\x7d\x1c", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7d\x20", "\x7d\x21", "\x7d\x22", "\x7d\x2b", "\x7d\x2c",
  "\x00\x00", "\x7d\x2e", "\x7d\x2f", "\x7d\x30", "\x00\x00", "\x7d\x32",
  "\x7d\x33", "\x00\x00", "\x7d\x35", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7d\x39", "\x7d\x3a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7d\x3f", "\x00\x00", "\x00\x00", "\x7d\x42", "\x7d\x43", "\x7d\x44",
  "\x7d\x45", "\x7d\x46", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7d\x4b", "\x7d\x4c", "\x00\x00", "\x7d\x4e", "\x7d\x4f", "\x7d\x50",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x7d\x56",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x7d\x5b", "\x00\x00",
  "\x00\x00", "\x7d\x5e", "\x00\x00", "\x00\x00", "\x7d\x61", "\x7d\x62",
  "\x7d\x63", "\x00\x00", "\x00\x00", "\x7d\x66", "\x00\x00", "\x7d\x68",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x7d\x6e",
  "\x00\x00", "\x00\x00", "\x7d\x71", "\x7d\x72", "\x7d\x73", "\x00\x00",
  "\x7d\x75", "\x7d\x76", "\x00\x00", "\x00\x00", "\x7d\x79", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x7d\x7d", "\x7d\x89", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7d\x8f", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7d\x93", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7d\x99", "\x7d\x9a", "\x7d\x9b", "\x7d\x9c", "\x00\x00",
  "\x00\x00", "\x7d\x9f", "\x00\x00", "\x00\x00", "\x7d\xa2", "\x7d\xa3",
  "\x7d\xab", "\x7d\xac", "\x7d\xad", "\x7d\xae", "\x7d\xaf", "\x7d\xb0",
  "\x7d\xb1", "\x7d\xb2", "\x00\x00", "\x7d\xb4", "\x7d\xb5", "\x00\x00",
  "\x00\x00", "\x7d\xb8", "\x00\x00", "\x7d\xba", "\x7d\xbb", "\x00\x00",
  "\x7d\xbd", "\x7d\xbe", "\x7d\xbf", "\x7d\xc7", "\x00\x00", "\x00\x00",
  "\x7d\xca", "\x7d\xcb", "\x00\x00", "\x00\x00", "\x00\x00", "\x7d\xcf",
  "\x00\x00", "\x7d\xd1", "\x7d\xd2", "\x00\x00", "\x00\x00", "\x7d\xd5",
  "\x00\x00", "\x00\x00", "\x7d\xd8", "\x00\x00", "\x7d\xda", "\x00\x00",
  "\x7d\xdc", "\x7d\xdd", "\x7d\xde", "\x00\x00", "\x7d\xe0", "\x7d\xe1",
  "\x00\x00", "\x00\x00", "\x7d\xe4", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7d\xe8", "\x7d\xe9", "\x00\x00", "\x00\x00", "\x7d\xec", "\x00\x00",
  "\x00\x00", "\x7d\xef", "\x00\x00", "\x00\x00", "\x7d\xf2", "\x00\x00",
  "\x7d\xf4", "\x7d\xfb", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7e\x01", "\x00\x00", "\x00\x00", "\x7e\x04", "\x7e\x05",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7e\x09", "\x7e\x0a", "\x7e\x0b",
  "\x7e\x12", "\x7e\x1b", "\x00\x00", "\x00\x00", "\x7e\x1e", "\x7e\x1f",
  "\x00\x00", "\x7e\x21", "\x7e\x22", "\x7e\x23", "\x00\x00", "\x00\x00",
  "\x7e\x26", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x7e\x2b",
  "\x00\x00", "\x00\x00", "\x7e\x2e", "\x00\x00", "\x00\x00", "\x7e\x31",
  "\x7e\x32", "\x00\x00", "\x00\x00", "\x7e\x35", "\x00\x00", "\x7e\x37",
  "\x00\x00", "\x7e\x39", "\x7e\x3a", "\x7e\x3b", "\x00\x00", "\x7e\x3d",
  "\x7e\x3e", "\x00\x00", "\x00\x00", "\x7e\x41", "\x00\x00", "\x7e\x43",
  "\x00\x00", "\x00\x00", "\x7e\x46", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7e\x4a", "\x7e\x4b", "\x00\x00", "\x7e\x4d", "\x7e\x54", "\x7e\x55",
  "\x7e\x56", "\x00\x00", "\x00\x00", "\x7e\x59", "\x7e\x5a", "\x00\x00",
  "\x00\x00", "\x7e\x5d", "\x7e\x5e", "\x7e\x66", "\x7e\x67", "\x00\x00",
  "\x7e\x69", "\x7e\x6a", "\x00\x00", "\x00\x00", "\x7e\x6d", "\x00\x00",
  "\x00\x00", "\x7e\x70", "\x7e\x79", "\x00\x00", "\x7e\x7b", "\x7e\x7c",
  "\x7e\x7d", "\x00\x00", "\x7e\x7f", "\x00\x00", "\x00\x00", "\x7e\x82",
  "\x7e\x83", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x7e\x88",
  "\x7e\x89", "\x00\x00", "\x00\x00", "\x7e\x8c", "\x00\x00", "\x7e\x8e",
  "\x7e\x8f", "\x7e\x90", "\x00\x00", "\x7e\x92", "\x7e\x93", "\x7e\x94",
  "\x00\x00", "\x7e\x96", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7e\x9b", "\x7e\x9c", "\x7f\x36", "\x00\x00", "\x7f\x38", "\x00\x00",
  "\x7f\x3a", "\x7f\x45", "\x7f\x4c", "\x7f\x4d", "\x7f\x4e", "\x00\x00",
  "\x7f\x50", "\x7f\x51", "\x00\x00", "\x00\x00", "\x7f\x54", "\x7f\x55",
  "\x00\x00", "\x00\x00", "\x7f\x58", "\x7f\x5f", "\x7f\x60", "\x7f\x67",
  "\x7f\x68", "\x7f\x69", "\x7f\x6a", "\x7f\x6b", "\x00\x00", "\x00\x00",
  "\x7f\x6e", "\x00\x00", "\x7f\x70", "\x00\x00", "\x7f\x72", "\x00\x00",
  "\x00\x00", "\x7f\x75", "\x00\x00", "\x7f\x77", "\x7f\x78", "\x7f\x79",
  "\x7f\x82", "\x7f\x83", "\x00\x00", "\x7f\x85", "\x7f\x86", "\x7f\x87",
  "\x7f\x88", "\x00\x00", "\x7f\x8a", "\x00\x00", "\x7f\x8c", "\x00\x00",
  "\x7f\x8e", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7f\x94", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7f\x9a", "\x00\x00", "\x00\x00", "\x7f\x9d", "\x7f\x9e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7f\xa3", "\x7f\xa4", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x7f\xa8", "\x7f\xa9", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x7f\xae", "\x7f\xaf", "\x00\x00", "\x00\x00",
  "\x7f\xb2", "\x00\x00", "\x00\x00", "\x00\x00", "\x7f\xb6", "\x00\x00",
  "\x7f\xb8", "\x7f\xb9", "\x00\x00", "\x00\x00", "\x00\x00", "\x7f\xbd",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x7f\xc1", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7f\xc5", "\x7f\xc6", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7f\xca", "\x00\x00", "\x7f\xcc", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x7f\xd2", "\x00\x00", "\x7f\xd4", "\x7f\xd5",
  "\x7f\xe0", "\x7f\xe1", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x7f\xe6", "\x00\x00", "\x00\x00", "\x7f\xe9", "\x00\x00", "\x7f\xeb",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x7f\xf0", "\x00\x00",
  "\x00\x00", "\x7f\xf3", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x7f\xf9", "\x00\x00", "\x7f\xfb", "\x7f\xfc", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x80\x00", "\x80\x01", "\x00\x00", "\x80\x03",
  "\x80\x04", "\x80\x05", "\x80\x06", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x80\x0b", "\x80\x0c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x80\x10", "\x00\x00", "\x80\x12", "\x00\x00", "\x00\x00", "\x80\x15",
  "\x00\x00", "\x80\x17", "\x80\x18", "\x80\x19", "\x00\x00", "\x00\x00",
  "\x80\x1c", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x80\x21",
  "\x80\x28", "\x80\x33", "\x00\x00", "\x00\x00", "\x80\x36", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x80\x3b", "\x00\x00", "\x80\x3d",
  "\x00\x00", "\x80\x3f", "\x80\x46", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x80\x4a", "\x80\x52", "\x00\x00", "\x00\x00", "\x00\x00", "\x80\x56",
  "\x00\x00", "\x80\x58", "\x00\x00", "\x80\x5a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x80\x5e", "\x80\x5f", "\x00\x00", "\x80\x61", "\x80\x62",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x80\x68",
  "\x80\x6f", "\x80\x70", "\x00\x00", "\x80\x72", "\x80\x73", "\x80\x74",
  "\x00\x00", "\x80\x76", "\x80\x77", "\x00\x00", "\x80\x79", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x80\x7d", "\x80\x7e", "\x80\x7f", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x80\x84", "\x80\x85", "\x80\x86",
  "\x80\x87", "\x00\x00", "\x80\x89", "\x00\x00", "\x80\x8b", "\x80\x8c",
  "\x80\x93", "\x00\x00", "\x00\x00", "\x80\x96", "\x00\x00", "\x80\x98",
  "\x00\x00", "\x80\x9a", "\x80\x9b", "\x00\x00", "\x80\x9d", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x80\xa1", "\x80\xa2", "\x00\x00", "\x00\x00",
  "\x80\xa5", "\x00\x00", "\x00\x00", "\x00\x00", "\x80\xa9", "\x80\xaa",
  "\x00\x00", "\x80\xac", "\x80\xad", "\x00\x00", "\x80\xaf", "\x00\x00",
  "\x80\xb1", "\x80\xb2", "\x00\x00", "\x80\xb4", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x80\xba", "\x80\xc3", "\x80\xc4",
  "\x00\x00", "\x80\xc6", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x80\xcc", "\x00\x00", "\x80\xce", "\x80\xd6", "\x00\x00",
  "\x00\x00", "\x80\xd9", "\x80\xda", "\x80\xdb", "\x00\x00", "\x80\xdd",
  "\x80\xde", "\x00\x00", "\x00\x00", "\x80\xe1", "\x00\x00", "\x00\x00",
  "\x80\xe4", "\x80\xe5", "\x80\xef", "\x00\x00", "\x80\xf1", "\x00\x00",
  "\x00\x00", "\x80\xf4", "\x00\x00", "\x00\x00", "\x00\x00", "\x80\xf8",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x80\xfc", "\x80\xfd", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x81\x02", "\x00\x00", "\x00\x00",
  "\x81\x05", "\x81\x06", "\x81\x07", "\x81\x08", "\x81\x09", "\x81\x0a",
  "\x81\x1a", "\x81\x1b", "\x81\x23", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x81\x29", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x81\x2f", "\x00\x00", "\x81\x31", "\x00\x00",
  "\x81\x33", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x81\x39", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x81\x3e",
  "\x81\x46", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x81\x4b",
  "\x00\x00", "\x00\x00", "\x81\x4e", "\x00\x00", "\x81\x50", "\x81\x51",
  "\x00\x00", "\x81\x53", "\x81\x54", "\x81\x55", "\x81\x5f", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x81\x65", "\x81\x66",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x81\x6b", "\x00\x00",
  "\x00\x00", "\x81\x6e", "\x00\x00", "\x81\x70", "\x81\x71", "\x00\x00",
  "\x00\x00", "\x81\x74", "\x00\x00", "\x00\x00", "\x00\x00", "\x81\x78",
  "\x81\x79", "\x81\x7a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x81\x7f", "\x81\x80", "\x00\x00", "\x81\x82", "\x81\x83", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x81\x88", "\x00\x00", "\x81\x8a",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x81\x8f", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x81\x93", "\x00\x00", "\x81\x95", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x81\x9a", "\x00\x00", "\x81\x9c",
  "\x81\x9d", "\x00\x00", "\x00\x00", "\x81\xa0", "\x00\x00", "\x00\x00",
  "\x81\xa3", "\x81\xa4", "\x00\x00", "\x00\x00", "\x00\x00", "\x81\xa8",
  "\x81\xa9", "\x81\xb0", "\x00\x00", "\x00\x00", "\x81\xb3", "\x00\x00",
  "\x81\xb5", "\x00\x00", "\x00\x00", "\x81\xb8", "\x00\x00", "\x81\xba",
  "\x00\x00", "\x00\x00", "\x81\xbd", "\x81\xbe", "\x81\xbf", "\x81\xc0",
  "\x00\x00", "\x81\xc2", "\x00\x00", "\x00\x00", "\x00\x00", "\x81\xc6",
  "\x00\x00", "\x81\xc8", "\x81\xc9", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x81\xcd", "\x00\x00", "\x00\x00", "\x00\x00", "\x81\xd1", "\x00\x00",
  "\x81\xd3", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x81\xd8",
  "\x81\xd9", "\x81\xda", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x81\xdf", "\x81\xe0", "\x00\x00", "\x00\x00", "\x81\xe3", "\x00\x00",
  "\x81\xe5", "\x00\x00", "\x81\xe7", "\x81\xe8", "\x00\x00", "\x81\xea",
  "\x00\x00", "\x00\x00", "\x81\xed", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x81\xf3", "\x81\xf4", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x81\xfa", "\x81\xfb", "\x81\xfc",
  "\x00\x00", "\x81\xfe", "\x00\x00", "\x00\x00", "\x82\x01", "\x82\x02",
  "\x00\x00", "\x00\x00", "\x82\x05", "\x00\x00", "\x82\x07", "\x82\x08",
  "\x82\x09", "\x82\x0a", "\x00\x00", "\x82\x0c", "\x82\x0d", "\x82\x0e",
  "\x00\x00", "\x82\x10", "\x00\x00", "\x82\x12", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x82\x16", "\x82\x17", "\x82\x18", "\x00\x00", "\x00\x00",
  "\x82\x1b", "\x82\x1c", "\x00\x00", "\x82\x1e", "\x82\x1f", "\x82\x29",
  "\x82\x2a", "\x82\x2b", "\x82\x2c", "\x00\x00", "\x82\x2e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x82\x33", "\x00\x00", "\x82\x35",
  "\x82\x36", "\x82\x37", "\x82\x38", "\x82\x39", "\x82\x40", "\x82\x47",
  "\x82\x58", "\x82\x59", "\x82\x5a", "\x00\x00", "\x00\x00", "\x82\x5d",
  "\x00\x00", "\x82\x5f", "\x00\x00", "\x00\x00", "\x82\x62", "\x00\x00",
  "\x82\x64", "\x00\x00", "\x82\x66", "\x00\x00", "\x82\x68", "\x00\x00",
  "\x82\x6a", "\x82\x6b", "\x00\x00", "\x00\x00", "\x82\x6e", "\x82\x6f",
  "\x00\x00", "\x82\x71", "\x82\x72", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x82\x76", "\x82\x77", "\x82\x78", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x82\x7e", "\x82\x8b", "\x00\x00", "\x82\x8d",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x82\x92", "\x82\x99",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x82\x9d", "\x00\x00", "\x82\x9f",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x82\xa5",
  "\x82\xa6", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x82\xab",
  "\x82\xac", "\x82\xad", "\x00\x00", "\x82\xaf", "\x00\x00", "\x82\xb1",
  "\x00\x00", "\x82\xb3", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x82\xb8", "\x82\xb9", "\x00\x00", "\x82\xbb", "\x00\x00", "\x82\xbd",
  "\x82\xc5", "\x82\xd1", "\x82\xd2", "\x82\xd3", "\x82\xd4", "\x00\x00",
  "\x00\x00", "\x82\xd7", "\x00\x00", "\x82\xd9", "\x00\x00", "\x82\xdb",
  "\x82\xdc", "\x00\x00", "\x82\xde", "\x82\xdf", "\x00\x00", "\x82\xe1",
  "\x00\x00", "\x82\xe3", "\x00\x00", "\x82\xe5", "\x82\xe6", "\x82\xe7",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x82\xeb", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x82\xf1", "\x00\x00", "\x82\xf3",
  "\x82\xf4", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x82\xf9",
  "\x82\xfa", "\x82\xfb", "\x83\x02", "\x83\x03", "\x83\x04", "\x83\x05",
  "\x83\x06", "\x00\x00", "\x00\x00", "\x83\x09", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x83\x0e", "\x83\x16", "\x83\x17", "\x83\x18",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x83\x1c", "\x83\x23", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x83\x28", "\x00\x00", "\x00\x00",
  "\x83\x2b", "\x00\x00", "\x00\x00", "\x00\x00", "\x83\x2f", "\x00\x00",
  "\x83\x31", "\x83\x32", "\x00\x00", "\x83\x34", "\x83\x35", "\x83\x36",
  "\x00\x00", "\x83\x38", "\x83\x39", "\x83\x40", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x83\x45", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x83\x49", "\x83\x4a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x83\x4f", "\x83\x50", "\x00\x00", "\x83\x52", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x83\x58", "\x83\x73", "\x00\x00",
  "\x83\x75", "\x00\x00", "\x83\x77", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x83\x7b", "\x83\x7c", "\x83\x85", "\x00\x00", "\x83\x87", "\x00\x00",
  "\x83\x89", "\x83\x8a", "\x00\x00", "\x00\x00", "\x00\x00", "\x83\x8e",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x83\x93", "\x00\x00",
  "\x00\x00", "\x83\x96", "\x00\x00", "\x00\x00", "\x00\x00", "\x83\x9a",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x83\x9e", "\x83\x9f", "\x83\xa0",
  "\x00\x00", "\x83\xa2", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x83\xa8", "\x00\x00", "\x83\xaa", "\x83\xab", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x83\xb1", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x83\xb5", "\x83\xbd", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x83\xc1", "\x00\x00", "\x00\x00", "\x00\x00", "\x83\xc5",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x83\xca", "\x00\x00",
  "\x83\xcc", "\x00\x00", "\x83\xce", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x83\xd3", "\x00\x00", "\x00\x00", "\x83\xd6", "\x00\x00",
  "\x83\xd8", "\x00\x00", "\x00\x00", "\x00\x00", "\x83\xdc", "\x00\x00",
  "\x00\x00", "\x83\xdf", "\x83\xe0", "\x83\xe9", "\x00\x00", "\x83\xeb",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x83\xef", "\x83\xf0", "\x83\xf1",
  "\x83\xf2", "\x00\x00", "\x83\xf4", "\x00\x00", "\x00\x00", "\x83\xf7",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x83\xfb", "\x00\x00", "\x83\xfd",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x84\x03",
  "\x84\x04", "\x00\x00", "\x00\x00", "\x84\x07", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x84\x0b", "\x84\x0c", "\x84\x0d", "\x84\x0e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x84\x13", "\x84\x20", "\x00\x00",
  "\x84\x22", "\x84\x29", "\x84\x2a", "\x00\x00", "\x84\x2c", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x84\x31", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x84\x35", "\x00\x00", "\x00\x00", "\x84\x38", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x84\x3c", "\x84\x3d", "\x84\x46", "\x00\x00",
  "\x00\x00", "\x84\x49", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x84\x4e", "\x84\x57", "\x00\x00", "\x00\x00", "\x00\x00", "\x84\x5b",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x84\x61",
  "\x84\x62", "\x84\x63", "\x00\x00", "\x00\x00", "\x84\x66", "\x00\x00",
  "\x00\x00", "\x84\x69", "\x00\x00", "\x84\x6b", "\x84\x6c", "\x84\x6d",
  "\x84\x6e", "\x84\x6f", "\x00\x00", "\x84\x71", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x84\x75", "\x00\x00", "\x84\x77", "\x00\x00", "\x84\x79",
  "\x84\x7a", "\x84\x82", "\x00\x00", "\x84\x84", "\x84\x8b", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x84\x90", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x84\x94", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x84\x99", "\x00\x00", "\x00\x00", "\x84\x9c", "\x00\x00", "\x00\x00",
  "\x84\x9f", "\x00\x00", "\x84\xa1", "\x84\xad", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x84\xb2", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x84\xb8", "\x84\xb9", "\x00\x00", "\x84\xbb",
  "\x84\xbc", "\x00\x00", "\x00\x00", "\x84\xbf", "\x00\x00", "\x84\xc1",
  "\x00\x00", "\x00\x00", "\x84\xc4", "\x00\x00", "\x84\xc6", "\x00\x00",
  "\x00\x00", "\x84\xc9", "\x84\xca", "\x84\xcb", "\x00\x00", "\x84\xcd",
  "\x00\x00", "\x00\x00", "\x84\xd0", "\x84\xd1", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x84\xd6", "\x00\x00", "\x00\x00", "\x84\xd9",
  "\x84\xda", "\x84\xec", "\x00\x00", "\x84\xee", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x84\xf4", "\x84\xfc", "\x00\x00",
  "\x00\x00", "\x84\xff", "\x85\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x85\x06", "\x85\x11", "\x00\x00", "\x85\x13",
  "\x85\x14", "\x85\x15", "\x00\x00", "\x85\x17", "\x85\x18", "\x00\x00",
  "\x85\x1a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x85\x1f",
  "\x00\x00", "\x85\x21", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x85\x26", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x85\x2c", "\x85\x2d", "\x85\x35", "\x85\x3d", "\x00\x00", "\x00\x00",
  "\x85\x40", "\x85\x41", "\x00\x00", "\x85\x43", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x85\x48", "\x85\x49", "\x85\x4a", "\x85\x4b",
  "\x00\x00", "\x00\x00", "\x85\x4e", "\x85\x55", "\x00\x00", "\x85\x57",
  "\x85\x58", "\x00\x00", "\x85\x5a", "\x85\x63", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x85\x68", "\x85\x69", "\x85\x6a", "\x00\x00",
  "\x00\x00", "\x85\x6d", "\x85\x77", "\x85\x7e", "\x00\x00", "\x85\x80",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x85\x84", "\x00\x00", "\x00\x00",
  "\x85\x87", "\x85\x88", "\x00\x00", "\x85\x8a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x85\x90", "\x85\x91", "\x00\x00",
  "\x00\x00", "\x85\x94", "\x00\x00", "\x00\x00", "\x85\x97", "\x00\x00",
  "\x85\x99", "\x00\x00", "\x85\x9b", "\x85\x9c", "\x85\xa4", "\x00\x00",
  "\x85\xa6", "\x00\x00", "\x85\xa8", "\x85\xa9", "\x85\xaa", "\x85\xab",
  "\x85\xac", "\x00\x00", "\x85\xae", "\x85\xaf", "\x85\xb9", "\x85\xba",
  "\x85\xc1", "\x85\xc9", "\x00\x00", "\x00\x00", "\x00\x00", "\x85\xcd",
  "\x00\x00", "\x85\xcf", "\x85\xd0", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x85\xd5", "\x85\xdc", "\x85\xdd", "\x85\xe4", "\x85\xe5",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x85\xe9", "\x85\xea", "\x85\xf7",
  "\x00\x00", "\x85\xf9", "\x85\xfa", "\x85\xfb", "\x00\x00", "\x00\x00",
  "\x85\xfe", "\x00\x00", "\x00\x00", "\x00\x00", "\x86\x02", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x86\x06", "\x86\x07", "\x00\x00", "\x00\x00",
  "\x86\x0a", "\x86\x0b", "\x86\x13", "\x00\x00", "\x00\x00", "\x86\x16",
  "\x86\x17", "\x00\x00", "\x00\x00", "\x86\x1a", "\x86\x22", "\x86\x2d",
  "\x00\x00", "\x86\x2f", "\x86\x30", "\x86\x3f", "\x86\x4d", "\x86\x4e",
  "\x00\x00", "\x86\x50", "\x00\x00", "\x00\x00", "\x00\x00", "\x86\x54",
  "\x86\x55", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x86\x5a",
  "\x00\x00", "\x86\x5c", "\x00\x00", "\x86\x5e", "\x86\x5f", "\x86\x67",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x86\x6b", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x86\x71", "\x86\x79", "\x00\x00",
  "\x86\x7b", "\x86\x8a", "\x86\x8b", "\x86\x8c", "\x86\x93", "\x00\x00",
  "\x86\x95", "\x86\xa3", "\x86\xa4", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x86\xa9", "\x86\xaa", "\x86\xab", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x86\xaf", "\x86\xb0", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x86\xb6", "\x86\xc4", "\x00\x00", "\x86\xc6",
  "\x86\xc7", "\x00\x00", "\x86\xc9", "\x00\x00", "\x86\xcb", "\x00\x00",
  "\x86\xcd", "\x86\xce", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x86\xd4", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x86\xd9", "\x00\x00", "\x86\xdb", "\x00\x00", "\x00\x00", "\x86\xde",
  "\x86\xdf", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x86\xe4",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x86\xe9", "\x00\x00",
  "\x00\x00", "\x86\xec", "\x86\xed", "\x86\xee", "\x86\xef", "\x86\xf8",
  "\x86\xf9", "\x00\x00", "\x86\xfb", "\x00\x00", "\x00\x00", "\x86\xfe",
  "\x00\x00", "\x87\x00", "\x00\x00", "\x87\x02", "\x87\x03", "\x00\x00",
  "\x00\x00", "\x87\x06", "\x00\x00", "\x87\x08", "\x87\x09", "\x87\x0a",
  "\x00\x00", "\x00\x00", "\x87\x0d", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x87\x11", "\x87\x12", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x87\x18", "\x00\x00", "\x87\x1a", "\x00\x00", "\x87\x1c",
  "\x87\x25", "\x00\x00", "\x00\x00", "\x00\x00", "\x87\x29", "\x87\x34",
  "\x00\x00", "\x00\x00", "\x87\x37", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x87\x3b", "\x00\x00", "\x00\x00", "\x00\x00", "\x87\x3f", "\x87\x49",
  "\x00\x00", "\x87\x4b", "\x87\x4c", "\x00\x00", "\x87\x4e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x87\x53", "\x00\x00", "\x87\x55",
  "\x00\x00", "\x87\x57", "\x00\x00", "\x87\x59", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x87\x5f", "\x87\x60", "\x00\x00",
  "\x00\x00", "\x87\x63", "\x00\x00", "\x00\x00", "\x87\x66", "\x00\x00",
  "\x87\x68", "\x00\x00", "\x87\x6a", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x87\x6e", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x87\x74", "\x00\x00", "\x87\x76", "\x00\x00", "\x87\x78", "\x87\x7f",
  "\x00\x00", "\x00\x00", "\x87\x82", "\x87\x8d", "\x87\x9f", "\x00\x00",
  "\x00\x00", "\x87\xa2", "\x87\xab", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x87\xaf", "\x00\x00", "\x00\x00", "\x00\x00", "\x87\xb3", "\x87\xba",
  "\x87\xbb", "\x00\x00", "\x87\xbd", "\x00\x00", "\x00\x00", "\x87\xc0",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x87\xc4", "\x00\x00", "\x87\xc6",
  "\x87\xc7", "\x00\x00", "\x00\x00", "\x00\x00", "\x87\xcb", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x87\xd0", "\x00\x00", "\x87\xd2",
  "\x87\xe0", "\x87\xef", "\x00\x00", "\x00\x00", "\x87\xf2", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x87\xf6", "\x87\xf7", "\x00\x00", "\x87\xf9",
  "\x00\x00", "\x87\xfb", "\x00\x00", "\x00\x00", "\x87\xfe", "\x88\x05",
  "\x88\x0d", "\x88\x0e", "\x88\x0f", "\x00\x00", "\x88\x11", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x88\x15", "\x88\x16", "\x88\x21", "\x88\x22",
  "\x88\x23", "\x00\x00", "\x00\x00", "\x00\x00", "\x88\x27", "\x88\x31",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x88\x36", "\x00\x00",
  "\x00\x00", "\x88\x39", "\x00\x00", "\x88\x3b", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x88\x40", "\x00\x00", "\x88\x42", "\x00\x00",
  "\x88\x44", "\x00\x00", "\x88\x46", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x88\x4c", "\x88\x4d", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x88\x52", "\x88\x53", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x88\x57", "\x00\x00", "\x88\x59", "\x00\x00", "\x88\x5b",
  "\x00\x00", "\x88\x5d", "\x88\x5e", "\x00\x00", "\x00\x00", "\x88\x61",
  "\x88\x62", "\x88\x63", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x88\x68", "\x00\x00", "\x00\x00", "\x88\x6b", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x88\x70", "\x00\x00", "\x88\x72", "\x00\x00",
  "\x00\x00", "\x88\x75", "\x00\x00", "\x88\x77", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x88\x7d", "\x88\x7e", "\x88\x7f",
  "\x00\x00", "\x88\x81", "\x88\x82", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x88\x88", "\x00\x00", "\x00\x00", "\x88\x8b",
  "\x00\x00", "\x88\x8d", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x88\x92", "\x00\x00", "\x00\x00", "\x00\x00", "\x88\x96", "\x88\x97",
  "\x00\x00", "\x88\x99", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x88\x9e", "\x00\x00", "\x00\x00", "\x00\x00", "\x88\xa2", "\x00\x00",
  "\x88\xa4", "\x88\xab", "\x00\x00", "\x00\x00", "\x88\xae", "\x00\x00",
  "\x88\xb0", "\x88\xb1", "\x00\x00", "\x00\x00", "\x88\xb4", "\x88\xb5",
  "\x00\x00", "\x88\xb7", "\x88\xbf", "\x00\x00", "\x88\xc1", "\x88\xc2",
  "\x88\xc3", "\x88\xc4", "\x88\xc5", "\x88\xcf", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x88\xd4", "\x88\xd5", "\x00\x00", "\x00\x00",
  "\x88\xd8", "\x88\xd9", "\x00\x00", "\x00\x00", "\x88\xdc", "\x88\xdd",
  "\x00\x00", "\x88\xdf", "\x00\x00", "\x88\xe1", "\x88\xe8", "\x88\xf2",
  "\x88\xf3", "\x88\xf4", "\x00\x00", "\x00\x00", "\x00\x00", "\x88\xf8",
  "\x88\xf9", "\x00\x00", "\x00\x00", "\x88\xfc", "\x88\xfd", "\x88\xfe",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x89\x02", "\x00\x00", "\x89\x04",
  "\x00\x00", "\x00\x00", "\x89\x07", "\x00\x00", "\x00\x00", "\x89\x0a",
  "\x00\x00", "\x89\x0c", "\x00\x00", "\x00\x00", "\x00\x00", "\x89\x10",
  "\x00\x00", "\x89\x12", "\x89\x13", "\x89\x1d", "\x89\x1e", "\x89\x25",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x89\x2a", "\x89\x2b",
  "\x89\x36", "\x00\x00", "\x89\x38", "\x00\x00", "\x00\x00", "\x89\x3b",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x89\x41",
  "\x00\x00", "\x89\x43", "\x89\x44", "\x89\x4c", "\x89\x4d", "\x89\x56",
  "\x89\x5e", "\x89\x5f", "\x89\x60", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x89\x64", "\x00\x00", "\x89\x66", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x89\x6a", "\x00\x00", "\x00\x00", "\x89\x6d", "\x00\x00", "\x89\x6f",
  "\x00\x00", "\x00\x00", "\x89\x72", "\x00\x00", "\x89\x74", "\x00\x00",
  "\x00\x00", "\x89\x77", "\x89\x7e", "\x89\x7f", "\x00\x00", "\x89\x81",
  "\x00\x00", "\x89\x83", "\x00\x00", "\x00\x00", "\x89\x86", "\x89\x87",
  "\x89\x88", "\x00\x00", "\x89\x8a", "\x89\x8b", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x89\x8f", "\x00\x00", "\x00\x00", "\x00\x00", "\x89\x93",
  "\x00\x00", "\x00\x00", "\x89\x96", "\x89\x97", "\x89\x98", "\x00\x00",
  "\x89\x9a", "\x89\xa1", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x89\xa6", "\x89\xa7", "\x00\x00", "\x89\xa9", "\x89\xaa", "\x00\x00",
  "\x89\xac", "\x00\x00", "\x00\x00", "\x89\xaf", "\x00\x00", "\x00\x00",
  "\x89\xb2", "\x89\xb3", "\x89\xba", "\x00\x00", "\x00\x00", "\x89\xbd",
  "\x00\x00", "\x89\xbf", "\x89\xc0", "\x89\xd2", "\x89\xda", "\x00\x00",
  "\x89\xdc", "\x89\xdd", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x89\xe3", "\x00\x00", "\x00\x00", "\x89\xe6", "\x89\xe7",
  "\x89\xf4", "\x00\x00", "\x00\x00", "\x00\x00", "\x89\xf8", "\x8a\x00",
  "\x00\x00", "\x8a\x02", "\x8a\x03", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x8a\x08", "\x00\x00", "\x8a\x0a", "\x00\x00", "\x8a\x0c",
  "\x00\x00", "\x8a\x0e", "\x00\x00", "\x8a\x10", "\x00\x00", "\x00\x00",
  "\x8a\x13", "\x00\x00", "\x00\x00", "\x8a\x16", "\x8a\x17", "\x8a\x18",
  "\x00\x00", "\x00\x00", "\x8a\x1b", "\x00\x00", "\x8a\x1d", "\x00\x00",
  "\x8a\x1f", "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\x23", "\x00\x00",
  "\x8a\x25", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\x2a",
  "\x00\x00", "\x00\x00", "\x8a\x2d", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8a\x31", "\x00\x00", "\x8a\x33", "\x8a\x34", "\x00\x00", "\x8a\x36",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\x3a", "\x8a\x3b", "\x8a\x3c",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\x41", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\x46", "\x00\x00", "\x8a\x48",
  "\x8a\x50", "\x8a\x51", "\x8a\x52", "\x00\x00", "\x8a\x54", "\x8a\x55",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\x5b",
  "\x00\x00", "\x00\x00", "\x8a\x5e", "\x00\x00", "\x8a\x60", "\x00\x00",
  "\x8a\x62", "\x8a\x63", "\x00\x00", "\x00\x00", "\x8a\x66", "\x00\x00",
  "\x00\x00", "\x8a\x69", "\x00\x00", "\x8a\x6b", "\x8a\x6c", "\x8a\x6d",
  "\x8a\x6e", "\x00\x00", "\x8a\x70", "\x8a\x71", "\x8a\x72", "\x8a\x73",
  "\x8a\x7c", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8a\x82", "\x00\x00", "\x8a\x84", "\x8a\x85", "\x00\x00", "\x8a\x87",
  "\x00\x00", "\x8a\x89", "\x00\x00", "\x00\x00", "\x8a\x8c", "\x8a\x8d",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\x91", "\x00\x00", "\x8a\x93",
  "\x00\x00", "\x8a\x95", "\x00\x00", "\x00\x00", "\x8a\x98", "\x00\x00",
  "\x8a\x9a", "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\x9e", "\x00\x00",
  "\x8a\xa0", "\x8a\xa1", "\x00\x00", "\x8a\xa3", "\x8a\xa4", "\x8a\xa5",
  "\x8a\xa6", "\x00\x00", "\x8a\xa8", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8a\xac", "\x8a\xad", "\x00\x00", "\x00\x00", "\x8a\xb0", "\x00\x00",
  "\x8a\xb2", "\x8a\xb9", "\x00\x00", "\x00\x00", "\x8a\xbc", "\x00\x00",
  "\x00\x00", "\x8a\xbf", "\x00\x00", "\x00\x00", "\x8a\xc2", "\x00\x00",
  "\x8a\xc4", "\x00\x00", "\x00\x00", "\x8a\xc7", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x8a\xcb", "\x8a\xcc", "\x8a\xcd", "\x00\x00", "\x8a\xcf",
  "\x00\x00", "\x00\x00", "\x8a\xd2", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8a\xd6", "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\xda", "\x8a\xdb",
  "\x8a\xdc", "\x00\x00", "\x8a\xde", "\x00\x00", "\x8a\xe0", "\x8a\xe1",
  "\x8a\xe2", "\x00\x00", "\x8a\xe4", "\x00\x00", "\x8a\xe6", "\x8a\xe7",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\xeb", "\x00\x00", "\x8a\xed",
  "\x8a\xee", "\x00\x00", "\x00\x00", "\x8a\xf1", "\x00\x00", "\x8a\xf3",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\xf7", "\x8a\xf8", "\x00\x00",
  "\x8a\xfa", "\x00\x00", "\x00\x00", "\x00\x00", "\x8a\xfe", "\x00\x00",
  "\x8b\x00", "\x8b\x01", "\x8b\x02", "\x00\x00", "\x8b\x04", "\x00\x00",
  "\x00\x00", "\x8b\x07", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8b\x0c", "\x00\x00", "\x8b\x0e", "\x00\x00", "\x8b\x10", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x8b\x14", "\x00\x00", "\x8b\x16", "\x8b\x17",
  "\x00\x00", "\x8b\x19", "\x8b\x1a", "\x8b\x1b", "\x00\x00", "\x8b\x1d",
  "\x00\x00", "\x00\x00", "\x8b\x20", "\x8b\x21", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x8b\x26", "\x00\x00", "\x8b\x28", "\x00\x00",
  "\x00\x00", "\x8b\x2b", "\x8b\x2c", "\x8b\x33", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8b\x39", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x8b\x3e", "\x00\x00", "\x00\x00", "\x8b\x41",
  "\x8b\x49", "\x00\x00", "\x00\x00", "\x8b\x4c", "\x00\x00", "\x8b\x4e",
  "\x8b\x4f", "\x8b\x56", "\x00\x00", "\x8b\x58", "\x00\x00", "\x8b\x5a",
  "\x8b\x5b", "\x8b\x5c", "\x00\x00", "\x00\x00", "\x8b\x5f", "\x8b\x66",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8b\x6b", "\x8b\x6c",
  "\x00\x00", "\x00\x00", "\x8b\x6f", "\x8b\x70", "\x8b\x71", "\x8b\x72",
  "\x00\x00", "\x8b\x74", "\x00\x00", "\x00\x00", "\x8b\x77", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8b\x7d", "\x00\x00",
  "\x00\x00", "\x8b\x80", "\x00\x00", "\x00\x00", "\x8b\x83", "\x8b\x8a",
  "\x00\x00", "\x8b\x8c", "\x00\x00", "\x8b\x8e", "\x00\x00", "\x8b\x90",
  "\x00\x00", "\x8b\x92", "\x8b\x93", "\x00\x00", "\x00\x00", "\x8b\x96",
  "\x00\x00", "\x00\x00", "\x8b\x99", "\x8b\x9a", "\x8c\x37", "\x00\x00",
  "\x00\x00", "\x8c\x3a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8c\x3f", "\x00\x00", "\x8c\x41", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x8c\x46", "\x00\x00", "\x8c\x48", "\x00\x00", "\x8c\x4a",
  "\x00\x00", "\x8c\x4c", "\x00\x00", "\x8c\x4e", "\x00\x00", "\x8c\x50",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8c\x55", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8c\x5a", "\x8c\x61", "\x8c\x62",
  "\x8c\x6a", "\x8c\x6b", "\x8c\x6c", "\x8c\x78", "\x8c\x79", "\x8c\x7a",
  "\x00\x00", "\x8c\x7c", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x8c\x82", "\x00\x00", "\x00\x00", "\x8c\x85", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x8c\x89", "\x8c\x8a", "\x00\x00", "\x8c\x8c",
  "\x8c\x8d", "\x8c\x8e", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x8c\x94", "\x00\x00", "\x00\x00", "\x00\x00", "\x8c\x98",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8c\x9d", "\x8c\x9e",
  "\x00\x00", "\x8c\xa0", "\x8c\xa1", "\x8c\xa2", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x8c\xa7", "\x8c\xa8", "\x8c\xa9", "\x8c\xaa",
  "\x8c\xab", "\x8c\xac", "\x8c\xad", "\x8c\xae", "\x8c\xaf", "\x8c\xb0",
  "\x00\x00", "\x8c\xb2", "\x8c\xb3", "\x8c\xb4", "\x00\x00", "\x8c\xb6",
  "\x8c\xb7", "\x8c\xb8", "\x00\x00", "\x00\x00", "\x8c\xbb", "\x8c\xbc",
  "\x8c\xbd", "\x00\x00", "\x8c\xbf", "\x8c\xc0", "\x8c\xc1", "\x8c\xc2",
  "\x8c\xc3", "\x8c\xc4", "\x00\x00", "\x00\x00", "\x8c\xc7", "\x8c\xc8",
  "\x00\x00", "\x8c\xca", "\x00\x00", "\x00\x00", "\x8c\xcd", "\x8c\xce",
  "\x00\x00", "\x00\x00", "\x8c\xd1", "\x00\x00", "\x8c\xd3", "\x8c\xda",
  "\x8c\xdb", "\x8c\xdc", "\x00\x00", "\x8c\xde", "\x00\x00", "\x8c\xe0",
  "\x00\x00", "\x8c\xe2", "\x8c\xe3", "\x8c\xe4", "\x00\x00", "\x8c\xe6",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8c\xea", "\x00\x00", "\x00\x00",
  "\x8c\xed", "\x8c\xfa", "\x8c\xfb", "\x8c\xfc", "\x8c\xfd", "\x8d\x04",
  "\x8d\x05", "\x00\x00", "\x8d\x07", "\x8d\x08", "\x00\x00", "\x8d\x0a",
  "\x8d\x0b", "\x00\x00", "\x8d\x0d", "\x00\x00", "\x8d\x0f", "\x8d\x10",
  "\x00\x00", "\x00\x00", "\x8d\x13", "\x8d\x14", "\x00\x00", "\x8d\x16",
  "\x8d\x64", "\x00\x00", "\x8d\x66", "\x8d\x67", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x8d\x6b", "\x00\x00", "\x8d\x6d", "\x00\x00", "\x00\x00",
  "\x8d\x70", "\x8d\x71", "\x00\x00", "\x8d\x73", "\x8d\x74", "\x00\x00",
  "\x00\x00", "\x8d\x77", "\x8d\x81", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8d\x85", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8d\x8a",
  "\x8d\x99", "\x8d\xa3", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8d\xa8", "\x8d\xb3", "\x8d\xba", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8d\xbe", "\x00\x00", "\x00\x00", "\x00\x00", "\x8d\xc2", "\x8d\xcb",
  "\x8d\xcc", "\x00\x00", "\x00\x00", "\x8d\xcf", "\x8d\xd6", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x8d\xda", "\x8d\xdb", "\x00\x00", "\x8d\xdd",
  "\x00\x00", "\x8d\xdf", "\x00\x00", "\x8d\xe1", "\x00\x00", "\x8d\xe3",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8d\xe8", "\x00\x00",
  "\x8d\xea", "\x8d\xeb", "\x00\x00", "\x00\x00", "\x00\x00", "\x8d\xef",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8d\xf3", "\x00\x00", "\x8d\xf5",
  "\x8d\xfc", "\x00\x00", "\x00\x00", "\x8d\xff", "\x8e\x08", "\x8e\x09",
  "\x8e\x0a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8e\x0f",
  "\x8e\x10", "\x8e\x1d", "\x8e\x1e", "\x8e\x1f", "\x8e\x2a", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8e\x30", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x8e\x34", "\x8e\x35", "\x8e\x42", "\x00\x00",
  "\x8e\x44", "\x00\x00", "\x00\x00", "\x8e\x47", "\x8e\x48", "\x8e\x49",
  "\x8e\x4a", "\x00\x00", "\x8e\x4c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8e\x50", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8e\x55",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8e\x59", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8e\x5f", "\x8e\x60", "\x00\x00",
  "\x00\x00", "\x8e\x63", "\x8e\x64", "\x8e\x72", "\x00\x00", "\x8e\x74",
  "\x00\x00", "\x8e\x76", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x8e\x7c", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8e\x81", "\x00\x00", "\x00\x00", "\x8e\x84", "\x8e\x85", "\x00\x00",
  "\x8e\x87", "\x00\x00", "\x00\x00", "\x8e\x8a", "\x8e\x8b", "\x00\x00",
  "\x8e\x8d", "\x00\x00", "\x00\x00", "\x00\x00", "\x8e\x91", "\x00\x00",
  "\x8e\x93", "\x8e\x94", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8e\x99", "\x8e\xa1", "\x8e\xaa", "\x8e\xab", "\x8e\xac", "\x00\x00",
  "\x00\x00", "\x8e\xaf", "\x8e\xb0", "\x8e\xb1", "\x8e\xbe", "\x8e\xc5",
  "\x8e\xc6", "\x00\x00", "\x8e\xc8", "\x00\x00", "\x8e\xca", "\x8e\xcb",
  "\x8e\xcc", "\x8e\xcd", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8e\xd2", "\x8e\xdb", "\x00\x00", "\x00\x00", "\x00\x00", "\x8e\xdf",
  "\x00\x00", "\x00\x00", "\x8e\xe2", "\x8e\xe3", "\x8e\xeb", "\x8e\xf8",
  "\x00\x00", "\x00\x00", "\x8e\xfb", "\x8e\xfc", "\x8e\xfd", "\x8e\xfe",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8f\x03", "\x00\x00",
  "\x8f\x05", "\x00\x00", "\x00\x00", "\x00\x00", "\x8f\x09", "\x8f\x0a",
  "\x00\x00", "\x8f\x0c", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x8f\x12", "\x8f\x13", "\x8f\x14", "\x8f\x15", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x8f\x19", "\x00\x00", "\x8f\x1b", "\x8f\x1c",
  "\x8f\x1d", "\x00\x00", "\x8f\x1f", "\x8f\x26", "\x00\x00", "\x00\x00",
  "\x8f\x29", "\x8f\x2a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8f\x2f", "\x00\x00", "\x00\x00", "\x00\x00", "\x8f\x33", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x8f\x38", "\x8f\x39", "\x00\x00",
  "\x8f\x3b", "\x00\x00", "\x00\x00", "\x8f\x3e", "\x8f\x3f", "\x00\x00",
  "\x00\x00", "\x8f\x42", "\x00\x00", "\x8f\x44", "\x8f\x45", "\x8f\x46",
  "\x00\x00", "\x00\x00", "\x8f\x49", "\x00\x00", "\x00\x00", "\x8f\x4c",
  "\x8f\x4d", "\x8f\x4e", "\x8f\x57", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x8f\x5c", "\x00\x00", "\x00\x00", "\x8f\x5f", "\x00\x00",
  "\x8f\x61", "\x8f\x62", "\x8f\x63", "\x8f\x64", "\x8f\x9b", "\x8f\x9c",
  "\x00\x00", "\x8f\x9e", "\x8f\x9f", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8f\xa3", "\x00\x00", "\x00\x00", "\x00\x00", "\x8f\xa7", "\x8f\xa8",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8f\xad", "\x8f\xae",
  "\x8f\xaf", "\x8f\xb0", "\x8f\xb1", "\x8f\xb2", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x8f\xb7", "\x00\x00", "\x00\x00", "\x8f\xba",
  "\x8f\xbb", "\x8f\xbc", "\x00\x00", "\x00\x00", "\x8f\xbf", "\x00\x00",
  "\x00\x00", "\x8f\xc2", "\x00\x00", "\x8f\xc4", "\x8f\xc5", "\x8f\xce",
  "\x00\x00", "\x00\x00", "\x8f\xd1", "\x00\x00", "\x00\x00", "\x8f\xd4",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x8f\xda",
  "\x8f\xe2", "\x00\x00", "\x00\x00", "\x8f\xe5", "\x8f\xe6", "\x00\x00",
  "\x00\x00", "\x8f\xe9", "\x8f\xea", "\x8f\xeb", "\x00\x00", "\x8f\xed",
  "\x00\x00", "\x8f\xef", "\x8f\xf0", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x8f\xf4", "\x00\x00", "\x00\x00", "\x8f\xf7", "\x8f\xf8", "\x8f\xf9",
  "\x8f\xfa", "\x00\x00", "\x00\x00", "\x8f\xfd", "\x00\x00", "\x00\x00",
  "\x90\x00", "\x90\x01", "\x00\x00", "\x90\x03", "\x00\x00", "\x90\x05",
  "\x90\x06", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x90\x0b",
  "\x00\x00", "\x90\x0d", "\x90\x0e", "\x90\x0f", "\x90\x10", "\x90\x11",
  "\x00\x00", "\x90\x13", "\x90\x14", "\x90\x15", "\x90\x16", "\x90\x17",
  "\x00\x00", "\x90\x19", "\x90\x1a", "\x00\x00", "\x00\x00", "\x90\x1d",
  "\x90\x1e", "\x90\x1f", "\x90\x20", "\x90\x21", "\x90\x22", "\x90\x23",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x90\x27", "\x90\x2e", "\x00\x00",
  "\x00\x00", "\x90\x31", "\x90\x32", "\x00\x00", "\x00\x00", "\x90\x35",
  "\x90\x36", "\x00\x00", "\x90\x38", "\x90\x39", "\x00\x00", "\x00\x00",
  "\x90\x3c", "\x00\x00", "\x90\x3e", "\x00\x00", "\x00\x00", "\x90\x41",
  "\x90\x42", "\x00\x00", "\x00\x00", "\x90\x45", "\x00\x00", "\x90\x47",
  "\x00\x00", "\x90\x49", "\x90\x4a", "\x90\x4b", "\x00\x00", "\x90\x4d",
  "\x90\x4e", "\x90\x4f", "\x90\x50", "\x90\x51", "\x90\x52", "\x90\x53",
  "\x90\x54", "\x90\x55", "\x90\x56", "\x00\x00", "\x90\x58", "\x90\x59",
  "\x00\x00", "\x00\x00", "\x90\x5c", "\x00\x00", "\x90\x5e", "\x00\x00",
  "\x90\x60", "\x90\x61", "\x00\x00", "\x90\x63", "\x00\x00", "\x90\x65",
  "\x00\x00", "\x00\x00", "\x90\x68", "\x90\x69", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x90\x6d", "\x90\x6e", "\x90\x6f", "\x00\x00", "\x00\x00",
  "\x90\x72", "\x00\x00", "\x00\x00", "\x90\x75", "\x90\x76", "\x90\x77",
  "\x90\x78", "\x00\x00", "\x90\x7a", "\x00\x00", "\x90\x7c", "\x90\x7d",
  "\x00\x00", "\x90\x7f", "\x90\x80", "\x90\x81", "\x90\x82", "\x90\x83",
  "\x90\x84", "\x00\x00", "\x00\x00", "\x90\x87", "\x00\x00", "\x90\x89",
  "\x90\x8a", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x90\x8f",
  "\x00\x00", "\x90\x91", "\x90\xa3", "\x00\x00", "\x00\x00", "\x90\xa6",
  "\x00\x00", "\x90\xa8", "\x00\x00", "\x90\xaa", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x90\xaf", "\x00\x00", "\x90\xb1", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x90\xb5", "\x00\x00", "\x00\x00", "\x90\xb8",
  "\x90\xc1", "\x90\xca", "\x00\x00", "\x00\x00", "\x00\x00", "\x90\xce",
  "\x90\xdb", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x90\xe1", "\x90\xe2", "\x00\x00", "\x90\xe4", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x90\xe8", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x90\xed", "\x90\xf5", "\x00\x00", "\x90\xf7", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x90\xfd", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x91\x02", "\x91\x12", "\x91\x19", "\x91\x2d",
  "\x00\x00", "\x00\x00", "\x91\x30", "\x00\x00", "\x91\x32", "\x91\x49",
  "\x91\x4a", "\x91\x4b", "\x91\x4c", "\x91\x4d", "\x91\x4e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x91\x52", "\x00\x00", "\x91\x54", "\x00\x00",
  "\x91\x56", "\x00\x00", "\x91\x58", "\x91\x62", "\x91\x63", "\x00\x00",
  "\x91\x65", "\x00\x00", "\x00\x00", "\x00\x00", "\x91\x69", "\x91\x6a",
  "\x00\x00", "\x91\x6c", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x91\x72", "\x91\x73", "\x00\x00", "\x91\x75", "\x00\x00",
  "\x91\x77", "\x91\x78", "\x91\x82", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x91\x87", "\x00\x00", "\x91\x89", "\x00\x00", "\x91\x8b",
  "\x00\x00", "\x91\x8d", "\x00\x00", "\x00\x00", "\x91\x90", "\x00\x00",
  "\x91\x92", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x91\x97",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x91\x9c", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x91\xa2", "\x00\x00",
  "\x91\xa4", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x91\xaa", "\x91\xab", "\x00\x00", "\x00\x00", "\x00\x00", "\x91\xaf",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x91\xb4", "\x91\xb5",
  "\x00\x00", "\x00\x00", "\x91\xb8", "\x00\x00", "\x91\xba", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x91\xc0", "\x91\xc1",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x91\xc6", "\x91\xc7",
  "\x91\xc8", "\x91\xc9", "\x00\x00", "\x91\xcb", "\x91\xcc", "\x91\xcd",
  "\x91\xce", "\x91\xcf", "\x91\xd0", "\x91\xd1", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x91\xd6", "\x00\x00", "\x91\xd8", "\x00\x00",
  "\x00\x00", "\x91\xdb", "\x91\xdc", "\x91\xdd", "\x00\x00", "\x91\xdf",
  "\x00\x00", "\x91\xe1", "\x00\x00", "\x91\xe3", "\x00\x00", "\x00\x00",
  "\x91\xe6", "\x91\xe7", "\x91\xf5", "\x91\xf6", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x91\xfc", "\x00\x00", "\x00\x00",
  "\x91\xff", "\x92\x0d", "\x92\x0e", "\x00\x00", "\x00\x00", "\x92\x11",
  "\x00\x00", "\x00\x00", "\x92\x14", "\x92\x15", "\x92\x1e", "\x92\x29",
  "\x00\x00", "\x00\x00", "\x92\x2c", "\x92\x34", "\x00\x00", "\x00\x00",
  "\x92\x37", "\x92\x3f", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x92\x44", "\x92\x45", "\x00\x00", "\x00\x00", "\x92\x48", "\x92\x49",
  "\x00\x00", "\x92\x4b", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x92\x50", "\x92\x57", "\x00\x00", "\x00\x00", "\x92\x5a", "\x92\x5b",
  "\x00\x00", "\x00\x00", "\x92\x5e", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x92\x62", "\x00\x00", "\x92\x64", "\x00\x00", "\x92\x66", "\x92\x71",
  "\x92\x7e", "\x00\x00", "\x92\x80", "\x00\x00", "\x00\x00", "\x92\x83",
  "\x00\x00", "\x92\x85", "\x92\x91", "\x00\x00", "\x92\x93", "\x00\x00",
  "\x92\x95", "\x92\x96", "\x00\x00", "\x92\x98", "\x00\x00", "\x92\x9a",
  "\x92\x9b", "\x92\x9c", "\x92\xad", "\x92\xb7", "\x00\x00", "\x92\xb9",
  "\x92\xcf", "\x00\x00", "\x00\x00", "\x92\xd2", "\x92\xe4", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x92\xe9", "\x92\xea", "\x00\x00",
  "\x00\x00", "\x92\xed", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x92\xf2", "\x92\xf3", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x92\xf8", "\x00\x00", "\x92\xfa", "\x00\x00", "\x92\xfc", "\x93\x06",
  "\x93\x0f", "\x93\x10", "\x93\x18", "\x93\x19", "\x93\x1a", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x93\x20", "\x00\x00",
  "\x93\x22", "\x93\x23", "\x00\x00", "\x00\x00", "\x93\x26", "\x00\x00",
  "\x93\x28", "\x00\x00", "\x00\x00", "\x93\x2b", "\x93\x2c", "\x00\x00",
  "\x93\x2e", "\x93\x2f", "\x00\x00", "\x00\x00", "\x93\x32", "\x00\x00",
  "\x00\x00", "\x93\x35", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x93\x3a", "\x93\x3b", "\x93\x44", "\x93\x4b", "\x00\x00", "\x93\x4d",
  "\x93\x54", "\x00\x00", "\x93\x56", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x93\x5b", "\x93\x5c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x93\x60", "\x93\x6c", "\x00\x00", "\x93\x6e", "\x93\x75", "\x93\x7c",
  "\x00\x00", "\x93\x7e", "\x93\x8c", "\x93\x94", "\x00\x00", "\x93\x96",
  "\x93\x97", "\x00\x00", "\x00\x00", "\x93\x9a", "\x93\xa7", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x93\xac", "\x93\xad", "\x93\xae",
  "\x00\x00", "\x93\xb0", "\x93\xb9", "\x93\xc3", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x93\xc8", "\x93\xd0", "\x93\xd1", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x93\xd6", "\x93\xd7", "\x93\xd8",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x93\xdd", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x93\xe1", "\x00\x00", "\x00\x00", "\x93\xe4",
  "\x93\xe5", "\x00\x00", "\x00\x00", "\x93\xe8", "\x94\x03", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x94\x07", "\x94\x10", "\x00\x00", "\x00\x00",
  "\x94\x13", "\x94\x14", "\x00\x00", "\x00\x00", "\x00\x00", "\x94\x18",
  "\x94\x19", "\x94\x1a", "\x94\x21", "\x94\x2b", "\x94\x35", "\x94\x36",
  "\x00\x00", "\x94\x38", "\x00\x00", "\x94\x3a", "\x94\x41", "\x00\x00",
  "\x00\x00", "\x94\x44", "\x94\x51", "\x94\x52", "\x94\x53", "\x94\x5a",
  "\x94\x5b", "\x00\x00", "\x00\x00", "\x94\x5e", "\x00\x00", "\x94\x60",
  "\x00\x00", "\x94\x62", "\x94\x6a", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x94\x70", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x94\x75", "\x00\x00", "\x94\x77", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x94\x7c", "\x94\x7d", "\x94\x7e", "\x94\x7f",
  "\x00\x00", "\x94\x81", "\x95\x77", "\x95\x80", "\x00\x00", "\x95\x82",
  "\x95\x83", "\x00\x00", "\x00\x00", "\x00\x00", "\x95\x87", "\x00\x00",
  "\x95\x89", "\x95\x8a", "\x95\x8b", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x95\x8f", "\x00\x00", "\x95\x91", "\x00\x00", "\x95\x93", "\x95\x94",
  "\x00\x00", "\x95\x96", "\x00\x00", "\x95\x98", "\x95\x99", "\x95\xa0",
  "\x00\x00", "\x95\xa2", "\x95\xa3", "\x95\xa4", "\x95\xa5", "\x00\x00",
  "\x95\xa7", "\x95\xa8", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x95\xad", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x95\xb2",
  "\x95\xb9", "\x00\x00", "\x95\xbb", "\x95\xbc", "\x00\x00", "\x95\xbe",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x95\xc3", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x95\xc7", "\x00\x00", "\x00\x00", "\x95\xca",
  "\x00\x00", "\x95\xcc", "\x95\xcd", "\x95\xd4", "\x95\xd5", "\x95\xd6",
  "\x00\x00", "\x95\xd8", "\x00\x00", "\x00\x00", "\x00\x00", "\x95\xdc",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x95\xe1", "\x95\xe2",
  "\x00\x00", "\x00\x00", "\x95\xe5", "\x96\x1c", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x96\x21", "\x96\x28", "\x00\x00", "\x96\x2a",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x96\x2e", "\x96\x2f", "\x00\x00",
  "\x00\x00", "\x96\x32", "\x96\x3b", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x96\x3f", "\x96\x40", "\x00\x00", "\x96\x42", "\x00\x00", "\x96\x44",
  "\x96\x4b", "\x96\x4c", "\x96\x4d", "\x00\x00", "\x96\x4f", "\x96\x50",
  "\x96\x5b", "\x96\x5c", "\x96\x5d", "\x96\x5e", "\x96\x5f", "\x00\x00",
  "\x00\x00", "\x96\x62", "\x96\x63", "\x96\x64", "\x96\x65", "\x96\x66",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x96\x6a", "\x00\x00", "\x96\x6c",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x96\x70", "\x00\x00", "\x96\x72",
  "\x96\x73", "\x00\x00", "\x96\x75", "\x96\x76", "\x96\x77", "\x96\x78",
  "\x00\x00", "\x96\x7a", "\x00\x00", "\x00\x00", "\x96\x7d", "\x96\x85",
  "\x96\x86", "\x00\x00", "\x96\x88", "\x00\x00", "\x96\x8a", "\x96\x8b",
  "\x00\x00", "\x96\x8d", "\x96\x8e", "\x96\x8f", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x96\x94", "\x96\x95", "\x00\x00", "\x96\x97",
  "\x96\x98", "\x96\x99", "\x00\x00", "\x96\x9b", "\x96\x9c", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x96\xa0", "\x00\x00", "\x00\x00", "\x96\xa3",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x96\xa7", "\x96\xa8", "\x00\x00",
  "\x96\xaa", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x96\xb0", "\x96\xb1", "\x96\xb2", "\x00\x00", "\x96\xb4", "\x00\x00",
  "\x96\xb6", "\x96\xb7", "\x96\xb8", "\x96\xb9", "\x00\x00", "\x96\xbb",
  "\x96\xbc", "\x00\x00", "\x00\x00", "\x00\x00", "\x96\xc0", "\x96\xc1",
  "\x00\x00", "\x00\x00", "\x96\xc4", "\x96\xc5", "\x96\xc6", "\x96\xc7",
  "\x00\x00", "\x96\xc9", "\x00\x00", "\x96\xcb", "\x96\xcc", "\x96\xcd",
  "\x96\xce", "\x00\x00", "\x00\x00", "\x96\xd1", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x96\xd5", "\x96\xd6", "\x00\x00", "\x00\x00", "\x96\xd9",
  "\x00\x00", "\x96\xdb", "\x96\xdc", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x96\xe2", "\x96\xe3", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x96\xe8", "\x00\x00", "\x96\xea", "\x96\xeb",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x96\xf0", "\x00\x00",
  "\x96\xf2", "\x00\x00", "\x00\x00", "\x00\x00", "\x96\xf6", "\x96\xf7",
  "\x00\x00", "\x96\xf9", "\x00\x00", "\x96\xfb", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x97\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x97\x04", "\x00\x00", "\x97\x06", "\x97\x07", "\x97\x08", "\x00\x00",
  "\x97\x0a", "\x00\x00", "\x00\x00", "\x97\x0d", "\x97\x0e", "\x97\x0f",
  "\x00\x00", "\x97\x11", "\x00\x00", "\x97\x13", "\x00\x00", "\x00\x00",
  "\x97\x16", "\x00\x00", "\x00\x00", "\x97\x19", "\x00\x00", "\x00\x00",
  "\x97\x1c", "\x00\x00", "\x97\x1e", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x97\x24", "\x00\x00", "\x00\x00", "\x97\x27",
  "\x00\x00", "\x00\x00", "\x97\x2a", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x97\x30", "\x00\x00", "\x97\x32", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x97\x38", "\x97\x39",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x97\x3d", "\x97\x3e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x97\x42", "\x00\x00", "\x97\x44", "\x00\x00",
  "\x97\x46", "\x00\x00", "\x97\x48", "\x97\x49", "\x97\x52", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x97\x56", "\x00\x00", "\x00\x00", "\x97\x59",
  "\x00\x00", "\x00\x00", "\x97\x5c", "\x00\x00", "\x97\x5e", "\x00\x00",
  "\x97\x60", "\x97\x61", "\x97\x62", "\x00\x00", "\x97\x64", "\x00\x00",
  "\x97\x66", "\x00\x00", "\x97\x68", "\x97\x69", "\x00\x00", "\x97\x6b",
  "\x00\x00", "\x97\x6d", "\x00\x00", "\x00\x00", "\x00\x00", "\x97\x71",
  "\x00\x00", "\x00\x00", "\x97\x74", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x97\x79", "\x97\x7a", "\x00\x00", "\x97\x7c", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x97\x81", "\x00\x00", "\x00\x00",
  "\x97\x84", "\x97\x85", "\x97\x86", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x97\x8b", "\x00\x00", "\x97\x8d", "\x00\x00", "\x97\x8f",
  "\x97\x90", "\x97\x98", "\x00\x00", "\x00\x00", "\x00\x00", "\x97\x9c",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x97\xa0", "\x00\x00", "\x00\x00",
  "\x97\xa3", "\x00\x00", "\x00\x00", "\x97\xa6", "\x00\x00", "\x97\xa8",
  "\x00\x00", "\x00\x00", "\x97\xab", "\x00\x00", "\x97\xad", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x97\xb3", "\x97\xb4",
  "\x97\xc3", "\x00\x00", "\x00\x00", "\x97\xc6", "\x00\x00", "\x97\xc8",
  "\x00\x00", "\x00\x00", "\x97\xcb", "\x97\xd3", "\x97\xdc", "\x97\xed",
  "\x97\xee", "\x00\x00", "\x00\x00", "\x00\x00", "\x97\xf2", "\x97\xf3",
  "\x00\x00", "\x97\xf5", "\x97\xf6", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x97\xfb", "\x00\x00", "\x00\x00", "\x00\x00", "\x97\xff",
  "\x00\x00", "\x98\x01", "\x98\x02", "\x98\x03", "\x00\x00", "\x98\x05",
  "\x98\x06", "\x00\x00", "\x98\x08", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x98\x0c", "\x00\x00", "\x00\x00", "\x98\x0f", "\x98\x10", "\x98\x11",
  "\x98\x12", "\x98\x13", "\x00\x00", "\x00\x00", "\x00\x00", "\x98\x17",
  "\x98\x18", "\x00\x00", "\x98\x1a", "\x98\x21", "\x00\x00", "\x00\x00",
  "\x98\x24", "\x98\x2c", "\x98\x2d", "\x98\x34", "\x00\x00", "\x00\x00",
  "\x98\x37", "\x98\x38", "\x00\x00", "\x00\x00", "\x98\x3b", "\x98\x3c",
  "\x98\x3d", "\x98\x46", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x98\x4b", "\x98\x4c", "\x98\x4d", "\x98\x4e", "\x98\x4f", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x98\x54", "\x98\x55", "\x00\x00",
  "\x00\x00", "\x98\x58", "\x00\x00", "\x00\x00", "\x98\x5b", "\x00\x00",
  "\x00\x00", "\x98\x5e", "\x98\x67", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x98\x6b", "\x00\x00", "\x00\x00", "\x00\x00", "\x98\x6f", "\x98\x70",
  "\x98\x71", "\x00\x00", "\x98\x73", "\x98\x74", "\x98\xa8", "\x00\x00",
  "\x98\xaa", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x98\xaf",
  "\x00\x00", "\x98\xb1", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x98\xb6", "\x98\xc3", "\x98\xc4", "\x00\x00", "\x98\xc6", "\x98\xdb",
  "\x98\xdc", "\x00\x00", "\x00\x00", "\x98\xdf", "\x00\x00", "\x00\x00",
  "\x98\xe2", "\x98\xe9", "\x00\x00", "\x98\xeb", "\x00\x00", "\x98\xed",
  "\x98\xee", "\x98\xef", "\x00\x00", "\x00\x00", "\x98\xf2", "\x00\x00",
  "\x98\xf4", "\x98\xfc", "\x98\xfd", "\x98\xfe", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x99\x03", "\x00\x00", "\x99\x05", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x99\x09", "\x99\x0a", "\x00\x00", "\x99\x0c",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x99\x10", "\x00\x00", "\x99\x12",
  "\x99\x13", "\x99\x14", "\x00\x00", "\x00\x00", "\x00\x00", "\x99\x18",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x99\x1d", "\x99\x1e",
  "\x00\x00", "\x99\x20", "\x99\x21", "\x00\x00", "\x00\x00", "\x99\x24",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x99\x28", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x99\x2c", "\x00\x00", "\x99\x2e", "\x99\x3d", "\x99\x3e",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x99\x42", "\x00\x00", "\x00\x00",
  "\x99\x45", "\x00\x00", "\x00\x00", "\x00\x00", "\x99\x49", "\x00\x00",
  "\x99\x4b", "\x99\x4c", "\x00\x00", "\x00\x00", "\x00\x00", "\x99\x50",
  "\x99\x51", "\x99\x52", "\x00\x00", "\x00\x00", "\x99\x55", "\x00\x00",
  "\x99\x57", "\x99\x96", "\x99\x97", "\x99\x98", "\x99\x99", "\x99\xa5",
  "\x00\x00", "\x00\x00", "\x99\xa8", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x99\xac", "\x99\xad", "\x99\xae", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x99\xb3", "\x99\xb4", "\x99\xbc", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x99\xc1", "\x00\x00", "\x00\x00", "\x99\xc4",
  "\x99\xc5", "\x99\xc6", "\x00\x00", "\x99\xc8", "\x99\xd0", "\x99\xd1",
  "\x99\xd2", "\x00\x00", "\x00\x00", "\x99\xd5", "\x00\x00", "\x00\x00",
  "\x99\xd8", "\x00\x00", "\x00\x00", "\x99\xdb", "\x00\x00", "\x99\xdd",
  "\x00\x00", "\x99\xdf", "\x00\x00", "\x00\x00", "\x99\xe2", "\x99\xed",
  "\x99\xee", "\x00\x00", "\x00\x00", "\x99\xf1", "\x99\xf2", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x99\xf8", "\x00\x00",
  "\x00\x00", "\x99\xfb", "\x00\x00", "\x00\x00", "\x00\x00", "\x99\xff",
  "\x00\x00", "\x9a\x01", "\x00\x00", "\x00\x00", "\x00\x00", "\x9a\x05",
  "\x9a\x0e", "\x9a\x0f", "\x00\x00", "\x00\x00", "\x9a\x12", "\x9a\x13",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9a\x19",
  "\x9a\x28", "\x00\x00", "\x00\x00", "\x9a\x2b", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x9a\x30", "\x9a\x37", "\x9a\x3e", "\x00\x00",
  "\x9a\x40", "\x00\x00", "\x9a\x42", "\x9a\x43", "\x00\x00", "\x9a\x45",
  "\x9a\x4d", "\x9a\x55", "\x00\x00", "\x9a\x57", "\x00\x00", "\x00\x00",
  "\x9a\x5a", "\x9a\x5b", "\x00\x00", "\x00\x00", "\x00\x00", "\x9a\x5f",
  "\x00\x00", "\x00\x00", "\x9a\x62", "\x00\x00", "\x9a\x64", "\x9a\x65",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x9a\x69", "\x9a\x6a", "\x9a\x6b",
  "\x9a\xa8", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9a\xad",
  "\x00\x00", "\x00\x00", "\x9a\xb0", "\x9a\xb8", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9a\xbc", "\x00\x00", "\x00\x00", "\x00\x00", "\x9a\xc0",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x9a\xc4", "\x9a\xcf", "\x00\x00",
  "\x9a\xd1", "\x00\x00", "\x9a\xd3", "\x9a\xd4", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9a\xd8", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9a\xde", "\x9a\xdf", "\x00\x00", "\x00\x00", "\x9a\xe2",
  "\x9a\xe3", "\x00\x00", "\x00\x00", "\x9a\xe6", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9a\xea", "\x9a\xeb", "\x00\x00", "\x9a\xed", "\x9a\xee",
  "\x9a\xef", "\x00\x00", "\x9a\xf1", "\x00\x00", "\x00\x00", "\x9a\xf4",
  "\x00\x00", "\x00\x00", "\x9a\xf7", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x9a\xfb", "\x9b\x06", "\x9b\x18", "\x00\x00", "\x9b\x1a", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x9b\x1f", "\x00\x00", "\x00\x00",
  "\x9b\x22", "\x9b\x23", "\x00\x00", "\x9b\x25", "\x00\x00", "\x9b\x27",
  "\x9b\x28", "\x9b\x29", "\x9b\x2a", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x9b\x2e", "\x9b\x2f", "\x00\x00", "\x9b\x31", "\x9b\x32", "\x9b\x3b",
  "\x9b\x3c", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9b\x41",
  "\x9b\x42", "\x9b\x43", "\x9b\x44", "\x9b\x45", "\x9b\x4d", "\x9b\x4e",
  "\x9b\x4f", "\x00\x00", "\x9b\x51", "\x00\x00", "\x00\x00", "\x9b\x54",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x9b\x58", "\x00\x00", "\x9b\x5a",
  "\x9b\x6f", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9b\x74",
  "\x9b\x83", "\x9b\x8e", "\x00\x00", "\x00\x00", "\x9b\x91", "\x9b\x92",
  "\x9b\x93", "\x00\x00", "\x00\x00", "\x9b\x96", "\x9b\x97", "\x9b\x9f",
  "\x9b\xa0", "\x9b\xa8", "\x00\x00", "\x9b\xaa", "\x9b\xab", "\x00\x00",
  "\x9b\xad", "\x9b\xae", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9b\xb4", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x9b\xb9", "\x9b\xc0", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9b\xc6", "\x00\x00", "\x00\x00", "\x9b\xc9", "\x9b\xca",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9b\xcf", "\x00\x00",
  "\x9b\xd1", "\x9b\xd2", "\x00\x00", "\x9b\xd4", "\x00\x00", "\x9b\xd6",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9b\xdb", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9b\xe1", "\x9b\xe2",
  "\x9b\xe3", "\x9b\xe4", "\x00\x00", "\x00\x00", "\x00\x00", "\x9b\xe8",
  "\x9b\xf0", "\x9b\xf1", "\x9b\xf2", "\x00\x00", "\x00\x00", "\x9b\xf5",
  "\x9c\x04", "\x00\x00", "\x9c\x06", "\x00\x00", "\x9c\x08", "\x9c\x09",
  "\x9c\x0a", "\x00\x00", "\x9c\x0c", "\x9c\x0d", "\x00\x00", "\x00\x00",
  "\x9c\x10", "\x00\x00", "\x9c\x12", "\x9c\x13", "\x9c\x14", "\x9c\x15",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9c\x1b",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9c\x21",
  "\x00\x00", "\x00\x00", "\x9c\x24", "\x9c\x25", "\x9c\x2d", "\x9c\x2e",
  "\x9c\x2f", "\x9c\x30", "\x00\x00", "\x9c\x32", "\x9c\x39", "\x9c\x3a",
  "\x9c\x3b", "\x00\x00", "\x00\x00", "\x9c\x3e", "\x9c\x46", "\x9c\x47",
  "\x9c\x48", "\x9c\x52", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x9c\x57", "\x00\x00", "\x00\x00", "\x9c\x5a", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x9c\x60", "\x9c\x67", "\x9c\x76",
  "\x00\x00", "\x9c\x78", "\x9c\xe5", "\x00\x00", "\x9c\xe7", "\x00\x00",
  "\x9c\xe9", "\x00\x00", "\x9c\xeb", "\x9c\xec", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9c\xf0", "\x00\x00", "\x00\x00", "\x9c\xf3", "\x9c\xf4",
  "\x00\x00", "\x9c\xf6", "\x9d\x03", "\x00\x00", "\x00\x00", "\x9d\x06",
  "\x9d\x07", "\x9d\x08", "\x9d\x09", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9d\x0e", "\x00\x00", "\x00\x00", "\x00\x00", "\x9d\x12",
  "\x00\x00", "\x00\x00", "\x9d\x15", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x9d\x1b", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x9d\x1f", "\x00\x00", "\x00\x00", "\x00\x00", "\x9d\x23", "\x00\x00",
  "\x00\x00", "\x9d\x26", "\x00\x00", "\x9d\x28", "\x00\x00", "\x9d\x2a",
  "\x9d\x2b", "\x9d\x2c", "\x9d\x3b", "\x00\x00", "\x00\x00", "\x9d\x3e",
  "\x9d\x3f", "\x00\x00", "\x9d\x41", "\x00\x00", "\x00\x00", "\x9d\x44",
  "\x00\x00", "\x9d\x46", "\x00\x00", "\x9d\x48", "\x9d\x50", "\x9d\x51",
  "\x9d\x59", "\x00\x00", "\x00\x00", "\x9d\x5c", "\x9d\x5d", "\x9d\x5e",
  "\x00\x00", "\x9d\x60", "\x9d\x61", "\x00\x00", "\x00\x00", "\x9d\x64",
  "\x9d\x6c", "\x00\x00", "\x00\x00", "\x9d\x6f", "\x00\x00", "\x00\x00",
  "\x9d\x72", "\x9d\x7a", "\x9d\x87", "\x00\x00", "\x9d\x89", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9d\x8f", "\x9d\x9a",
  "\x9d\xa4", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9d\xa9",
  "\x00\x00", "\x9d\xab", "\x00\x00", "\x00\x00", "\x00\x00", "\x9d\xaf",
  "\x00\x00", "\x00\x00", "\x9d\xb2", "\x00\x00", "\x9d\xb4", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x9d\xb8", "\x00\x00", "\x9d\xba", "\x9d\xbb",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9d\xc1",
  "\x9d\xc2", "\x00\x00", "\x9d\xc4", "\x00\x00", "\x9d\xc6", "\x9d\xcf",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x9d\xd3", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x9d\xd9", "\x9d\xe6", "\x9d\xed",
  "\x00\x00", "\x9d\xef", "\x00\x00", "\x00\x00", "\x9d\xf2", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9d\xf8", "\x9d\xf9",
  "\x9d\xfa", "\x00\x00", "\x00\x00", "\x9d\xfd", "\x9e\x1a", "\x9e\x1b",
  "\x00\x00", "\x00\x00", "\x9e\x1e", "\x9e\x75", "\x00\x00", "\x00\x00",
  "\x9e\x78", "\x9e\x79", "\x00\x00", "\x00\x00", "\x00\x00", "\x9e\x7d",
  "\x00\x00", "\x9e\x7f", "\x00\x00", "\x9e\x81", "\x9e\x88", "\x00\x00",
  "\x00\x00", "\x9e\x8b", "\x9e\x8c", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9e\x91", "\x9e\x92", "\x9e\x93", "\x00\x00", "\x9e\x95",
  "\x00\x00", "\x9e\x97", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9e\x9d", "\x00\x00", "\x9e\x9f", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x9e\xa5", "\x9e\xa6", "\x00\x00",
  "\x00\x00", "\x9e\xa9", "\x9e\xaa", "\x00\x00", "\x00\x00", "\x9e\xad",
  "\x9e\xb8", "\x9e\xb9", "\x9e\xba", "\x9e\xbb", "\x9e\xbc", "\x00\x00",
  "\x9e\xbe", "\x9e\xbf", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x9e\xc4", "\x9e\xcc", "\x9e\xcd", "\x9e\xce", "\x9e\xcf", "\x9e\xd0",
  "\x00\x00", "\x9e\xd2", "\x00\x00", "\x9e\xd4", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9e\xd8", "\x9e\xd9", "\x00\x00", "\x9e\xdb", "\x9e\xdc",
  "\x9e\xdd", "\x9e\xde", "\x00\x00", "\x9e\xe0", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x9e\xe5", "\x00\x00", "\x00\x00", "\x9e\xe8",
  "\x9e\xef", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9e\xf4",
  "\x00\x00", "\x9e\xf6", "\x9e\xf7", "\x00\x00", "\x9e\xf9", "\x00\x00",
  "\x9e\xfb", "\x9e\xfc", "\x9e\xfd", "\x9f\x07", "\x9f\x08", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00", "\x9f\x0e", "\x00\x00",
  "\x00\x00", "\x00\x00", "\x00\x00", "\x9f\x13", "\x00\x00", "\x9f\x15",
  "\x9f\x20", "\x9f\x21", "\x9f\x2c", "\x9f\x3b", "\x00\x00", "\x00\x00",
  "\x9f\x3e", "\x9f\x4a", "\x9f\x4b", "\x00\x00", "\x00\x00", "\x9f\x4e",
  "\x9f\x4f", "\x00\x00", "\x00\x00", "\x9f\x52", "\x00\x00", "\x9f\x54",
  "\x9f\x5f", "\x9f\x60", "\x9f\x61", "\x9f\x62", "\x9f\x63", "\x00\x00",
  "\x00\x00", "\x9f\x66", "\x9f\x67", "\x00\x00", "\x00\x00", "\x9f\x6a",
  "\x00\x00", "\x9f\x6c", "\x00\x00", "\x00\x00", "\x00\x00", "\x00\x00",
  "\x00\x00", "\x9f\x72", "\x00\x00", "\x00\x00", "\x00\x00", "\x9f\x76",
  "\x9f\x77", "\x9f\x8d", "\x9f\x95", "\x9f\x9c", "\x9f\x9d", "\x00\x00",
  "\x00\x00", "\x9f\xa0", "\xff\x01", "\x00\x00", "\xff\x03", "\xff\x04",
  "\xff\x05", "\xff\x06", "\x00\x00", "\xff\x08", "\xff\x09", "\xff\x0a",
  "\xff\x0b", "\xff\x0c", "\x00\x00", "\xff\x0e", "\xff\x0f", "\xff\x10",
  "\xff\x11", "\xff\x12", "\xff\x13", "\xff\x14", "\xff\x15", "\xff\x16",
  "\xff\x17", "\xff\x18", "\xff\x19", "\xff\x1a", "\xff\x1b", "\xff\x1c",
  "\xff\x1d", "\xff\x1e", "\xff\x1f", "\xff\x20", "\xff\x21", "\xff\x22",
  "\xff\x23", "\xff\x24", "\xff\x25", "\xff\x26", "\xff\x27", "\xff\x28",
  "\xff\x29", "\xff\x2a", "\xff\x2b", "\xff\x2c", "\xff\x2d", "\xff\x2e",
  "\xff\x2f", "\xff\x30", "\xff\x31", "\xff\x32", "\xff\x33", "\xff\x34",
  "\xff\x35", "\xff\x36", "\xff\x37", "\xff\x38", "\xff\x39", "\xff\x3a",
  "\xff\x3b", "\x00\x00", "\xff\x3d", "\xff\x3e", "\xff\x3f", "\xff\x40",
  "\xff\x41", "\xff\x42", "\xff\x43", "\xff\x44", "\xff\x45", "\xff\x46",
  "\xff\x47", "\xff\x48", "\xff\x49", "\xff\x4a", "\xff\x4b", "\xff\x4c",
  "\xff\x4d", "\xff\x4e", "\xff\x4f", "\xff\x50", "\xff\x51", "\xff\x52",
  "\xff\x53", "\xff\x54", "\xff\x55", "\xff\x56", "\xff\x57", "\xff\x58",
  "\xff\x59", "\xff\x5a", "\xff\x5b", "\xff\x5c", "\xff\x5d", "\xff\xe3",
  "\x00\x00", "\xff\xe5"
};
